﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the AIReadBuffer project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using AIReadBuffer.Utility;
using ALLDAQ.Boards;
using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;

namespace AIReadBuffer.ALLDAQ
{
    public class AnalogInputModule
    {
        private readonly AdqDriver.ADQAIChannel[] _channelList;

        public uint AdqError { get; set; }
        public event EventHandler<InfoTextChangedEventArgs> OnInfoTextChanged;

        public AnalogInputModule(string boardName, uint boardIndex, uint moduleIndex, 
            ref AdqDriver.ADQGeneralModuleInfo generalInfo, ref AdqDriver.ADQAIModuleInfo aiModuleInfo)
        {
            ModelName = AdqCommon.GetModelName(boardName);
            BoardIndex = boardIndex;
            ModuleIndex = moduleIndex;
            NumberOfChannels = aiModuleInfo.NumberOfChannels;
            NumberOfRanges = aiModuleInfo.NumberOfRanges;
            MaxFrequencyHz = generalInfo.ClockBaseFrequencyHz / generalInfo.ClockMinimumDivider;
            DesiredSampleFrequency = MaxFrequencyHz;
            BuildModuleName();
            _channelList = new AdqDriver.ADQAIChannel[NumberOfChannels];
            NumberOfFrames = AdqCommon.DefaultNumberOfFrames;

            AddRanges(ref aiModuleInfo);
            DefaultRange = Ranges[0];
        }

        private void AddRanges(ref AdqDriver.ADQAIModuleInfo adqAiModuleInfo)
        {
            for (uint rangeIdx = 0; rangeIdx < adqAiModuleInfo.NumberOfRanges; rangeIdx++)
            {
                AdqDriver.ADQAIRange range = adqAiModuleInfo.Ranges[rangeIdx];

                var analogRange = new AnalogRange(
                    rangeIdx,
                    range.PhysicalUnit,
                    range.DigitalMin,
                    range.DigitalMax,
                    range.PhysicalMin,
                    range.PhysicalMax);
                Ranges.Add(analogRange);
            }
        }

        public string ModelName { get; }
        public uint BoardIndex { get; }
        public uint ModuleIndex { get; }
        public string ModuleName { get; private set; }
        public uint NumberOfChannels { get; }
        public uint NumberOfRanges { get; }
        public List<ChannelSetting> ChannelSettings { get; } = new List<ChannelSetting>();
        public double MaxFrequencyHz { get; }

        private void BuildModuleName()
        {
            ModuleName = string.Format("{0}, Board {1}, Module {2}, {3} {4}, {5} {6}, max. {7}",
                ModelName,
                BoardIndex,
                ModuleIndex,
                NumberOfChannels,
                NumberOfChannels == 1 ? "Channel" : "Channels",
                NumberOfRanges,
                NumberOfRanges == 1 ? "Range" : "Ranges",
                AdqCommon.FormatFrequency(MaxFrequencyHz));
        }

        public void AddChannelSetting(uint channelIndex, AnalogRange range)
        {
            ChannelSettings.Add(new ChannelSetting(channelIndex, range));
        }

        public void ConfigureModule()
        {
            if (NumberOfFrames == 0)
            {
                throw new ApplicationException("Please enter a valid number of frames.");
            }

            if (FrameLength == 0)
            {
                throw new ApplicationException("Please select channels in the channel list.");    
            }

            if (DesiredSampleFrequency <= 0)
            {
                throw new ApplicationException("Please enter a valid sample frequency.");    
            }

            SetChannelListAndFrequency();
            SampleTime = 1.0 / _actualSampleFrequency;
            AdqError = AdqErrors.ADQ_ERROR_NONE;
            uint numberOfTotalValues = NumberOfFrames * FrameLength;
            if (Frames == null || Frames.Length != numberOfTotalValues)
            {
                Frames = new uint[numberOfTotalValues];
                PhysicalFrames = new double[numberOfTotalValues];
            }
            _numberOfFramesRead = 0;
        }

        public void UpdateChannelList()
        {
            for (int chIdx = 0; chIdx < _channelList.Length; chIdx++)
            {
                _channelList[chIdx].AcquireChannel = false;
                _channelList[chIdx].RangeIndex = 0;
            }

            MinResolution = double.MaxValue;
            foreach (ChannelSetting chSetting in ChannelSettings)
            {
                _channelList[chSetting.ChannelIndex].AcquireChannel = true;
                _channelList[chSetting.ChannelIndex].RangeIndex = chSetting.Range.RangeIndex;
                if (MinResolution > chSetting.Range.PhysicalResolution)
                {
                    MinResolution = chSetting.Range.PhysicalResolution;
                }
            }
            FrameLength = (uint)ChannelSettings.Count;
        }

        private void SetChannelListAndFrequency()
        {
            uint adqError = AdqDriver.adqAISetChannelListAndFrequency(BoardIndex, ModuleIndex,
                 _channelList, NumberOfChannels, DesiredSampleFrequency, out _actualSampleFrequency);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqAISetChannelListAndFrequency", adqError);
            }
        }

        public void ReadFramesFromModule()
        {
            TotalNumberOfValuesRead = 0;
            GCHandle gch = GCHandle.Alloc(Frames, GCHandleType.Pinned);
            AdqError = AdqDriver.adqAIReadBuffer(BoardIndex, ModuleIndex,
                FrameLength, NumberOfFrames, Frames, out _numberOfFramesRead);
            TotalNumberOfValuesRead = FrameLength * _numberOfFramesRead;
            gch.Free();
            CalculatePhysicalFrames();
        }

        private void CalculatePhysicalFrames()
        {
            ChannelSetting channelSetting;
            AnalogRange range;
            for (uint idx = 0; idx < FrameLength; idx++)
            {
                channelSetting = ChannelSettings[(int)idx];
                range = channelSetting.Range;

                uint valIdx = idx;
                while (valIdx < TotalNumberOfValuesRead)
                {
                    PhysicalFrames[valIdx] = range.DigitalToPhysical(Frames[valIdx]);
                    valIdx += FrameLength;
                }
            }
        }

        public uint StopAcquisition()
        {
            uint adqError = AdqDriver.adqStop(BoardIndex, ModuleIndex);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqStop", adqError);
            }
            return adqError;
        }

        public uint Reset()
        {
            uint adqError = AdqDriver.adqResetModule(BoardIndex, ModuleIndex);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqResetModule", adqError);
            }
            return adqError;
        }

        private void SetFunctionErrorInfo(string functionName, uint adqError)
        {
            InfoText = string.Format("API-function {0} returned Error number: 0x{1:X}", functionName, adqError);
        }

        protected virtual void RaiseModuleInfoTextChanged(InfoTextChangedEventArgs e)
        {
            OnInfoTextChanged?.Invoke(this, e);
        }

        private string _infoText = string.Empty;
        public string InfoText
        {
            get { return _infoText; }
            set
            {
                if (_infoText != value)
                {
                    _infoText = value;
                    var args = new InfoTextChangedEventArgs
                    {
                        InfoText = _infoText
                    };
                    RaiseModuleInfoTextChanged(args);
                }
            }
        }

        public uint NumberOfFrames { get; set; }
        public uint FrameLength { get; set; }

        private uint _numberOfFramesRead;
        public uint NumberOfFramesRead
        {
            get { return _numberOfFramesRead; }
        }
        public uint TotalNumberOfValuesRead { get; private set; }
        public uint[] Frames { get; private set; }
        public double[] PhysicalFrames { get; private set; }
        public double MinResolution { get; private set; }

        private double _actualSampleFrequency;
        public double ActualSampleFrequency
        {
            get { return _actualSampleFrequency; }
        }
        public double SampleTime { get; private set; }
        public double DesiredSampleFrequency { get; set; }

        public void ClearFrameBuffers()
        {
            Frames = null;
            PhysicalFrames = null;
        }

        public double GetSecondsToWait()
        {
            if (_actualSampleFrequency == 0) return 0.0;

            return NumberOfFrames / _actualSampleFrequency;
        }

        public AnalogRange DefaultRange { get; }

        public List<AnalogRange> Ranges { get; } = new List<AnalogRange>();
    }
}
