﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the AIReadBuffer project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using AIReadBuffer.ALLDAQ;
using AIReadBuffer.Utility;
using ALLDAQ.Boards;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace AIReadBuffer
{
    public partial class FormMain : Form
    {
        private readonly Color _gridLineColor = Color.DarkSlateGray;
        private readonly Color _labelColor = Color.LightSkyBlue;
        private const int _rangeSubItemIndex = 1;
        private const int _idxChartTypeLine = 0;
        private const int _idxChartTypeLineAndMarkers = 1;
        private const int _idxXAxisLabelsTime = 0;
        private const string _fmtStringPhysValues = "{0:0.000000}";

        private string _selectedUnitSymbol;
        private ModuleLocator _moduleLocator;
        private readonly List<AnalogInputModule> _aiModules = new List<AnalogInputModule>();
        private AnalogInputModule _selectedAiModule;
        private int _selectedModuleItemIndex = -1;
        private bool _isFormClosing;
        private bool _moduleIsBusy;
        private ListViewHitTestInfo _hitinfoChannelList;
        private readonly ComboBox _coBoxRanges = new ComboBox();

        public FormMain()
        {
            InitializeComponent();
            AdqCommon.SetDoubleBuffered(lvChannelList);
            InitRangeComboBox();
            coBoxChartType.SelectedIndex = _idxChartTypeLine;
            coBoxXAxisLabels.SelectedIndex = _idxXAxisLabelsTime;
        }

        private void FrmMain_Load(object sender, EventArgs e)
        {
            EnableUI(false);
            InitChart();
            try
            {
                _moduleLocator = new ModuleLocator(_aiModules, OnInfoTextChanged);
                _moduleLocator.FindModules();

                FillModuleCombobox();
                SelectFirstComboboxItem(coBoxModules);
                textBoxInfo.AppendText(string.Format("Number of analog input modules found: {0}" + Environment.NewLine, _aiModules.Count));
            }
            catch (DllNotFoundException ex)
            {
                textBoxInfo.AppendText("The device driver is not properly installed." + Environment.NewLine);
                textBoxInfo.AppendText(ex.Message + Environment.NewLine);
            }
        }

        private void InitRangeComboBox()
        {
            _coBoxRanges.DropDownStyle = ComboBoxStyle.DropDownList;
            _coBoxRanges.Parent = lvChannelList;
            _coBoxRanges.Visible = false;
            _coBoxRanges.DropDownClosed += CoBoxRanges_DropDownClosed;
            _coBoxRanges.Leave += CoBoxRanges_Leave;
            _coBoxRanges.KeyPress += CoBoxRanges_KeyPress;
        }

        private void FillModuleCombobox()
        {
            coBoxModules.Items.Clear();
            foreach (AnalogInputModule aiModule in _aiModules)
            {
                coBoxModules.Items.Add(aiModule.ModuleName);
            }
        }

        private void OnInfoTextChanged(object sender, InfoTextChangedEventArgs e)
        {
            textBoxInfo.AppendText(e.InfoText + Environment.NewLine);
        }

        private void DisplayErrorText(string functionName, uint adqError)
        {
            textBoxInfo.AppendText(
                string.Format("Function {0} returned Error number: 0x{1:X}" + Environment.NewLine,
                    functionName,
                    adqError));
        }

        private void SelectFirstComboboxItem(ComboBox coBox)
        {
            if (coBox.Items.Count > 0)
            {
                coBox.SelectedIndex = 0;
            }
        }

        private void CoBoxModules_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (coBoxModules.SelectedIndex == _selectedModuleItemIndex) return;

            _selectedModuleItemIndex = coBoxModules.SelectedIndex;
            ClearDisplayedValues();

            EnableUI(coBoxModules.SelectedIndex >= 0);
            btnStartStop.Enabled = coBoxModules.SelectedIndex >= 0;

            if (_selectedAiModule != null)
            {
                _selectedAiModule.OnInfoTextChanged -= OnInfoTextChanged;
                _selectedAiModule.ClearFrameBuffers();
            }

            if (coBoxModules.SelectedIndex < 0)
            {
                _selectedAiModule = null;
                textBoxSampleFrequency.Text = string.Empty;
                _selectedUnitSymbol = string.Empty;
            }
            else
            {
                _selectedAiModule = _aiModules[coBoxModules.SelectedIndex];
                _selectedAiModule.OnInfoTextChanged += OnInfoTextChanged;
                coBoxNumOfFrames.Text = _selectedAiModule.NumberOfFrames.ToString();
                textBoxSampleFrequency.Text = string.Format("{0}", _selectedAiModule.DesiredSampleFrequency);
                _selectedUnitSymbol = _selectedAiModule.Ranges[0].UnitSymbol;
            }

            textBoxActualFrequency.Text = string.Empty;
            FillChannelsAndRanges(_selectedAiModule);
            UpdateNumberOfValues();
        }

        private void FillChannelsAndRanges(AnalogInputModule aiModule)
        {
            FillRanges(aiModule);
            FillChannels(aiModule);
            UpdateRangesToolstripMenuItems(aiModule);
        }

        private void FillRanges(AnalogInputModule aiModule)
        {
            _coBoxRanges.Items.Clear();
            if (aiModule == null) return;

            foreach (AnalogRange range in aiModule.Ranges)
            {
                _coBoxRanges.Items.Add(new RangeComboBoxItem(range));
            }
            SelectFirstComboboxItem(_coBoxRanges);
        }

        private void FillChannels(AnalogInputModule aiModule)
        {
            lvChannelList.Items.Clear();
            if (aiModule == null) return;

            for (uint chIdx = 0; chIdx < aiModule.NumberOfChannels; chIdx++)
            {
                ListViewItem lvItem = lvChannelList.Items.Add("Channel " + chIdx.ToString());
                lvItem.SubItems.Add(aiModule.DefaultRange.RangeText);
                lvItem.Tag = aiModule.DefaultRange;
                lvItem.BackColor = Color.Black;
                lvItem.ForeColor = AdqCommon.ChannelColors[chIdx % AdqCommon.ChannelColors.Length];
            }

            CheckFirstChannels(2);
        }

        private void CheckFirstChannels(int count)
        {
            int idx = 0;
            while (idx < lvChannelList.Items.Count)
            {
                lvChannelList.Items[idx].Checked = idx < count;
                idx++;
            }
        }

        private void UpdateRangesToolstripMenuItems(AnalogInputModule aiModule)
        {
            tsMnuItemSetRange.DropDownItems.Clear();

            if (aiModule != null && aiModule.NumberOfRanges > 1)
            {
                foreach (AnalogRange range in aiModule.Ranges)
                {
                    var item = new ToolStripMenuItem(range.RangeText, null, TsMnuItemSetRange_Click)
                    {
                        Tag = range
                    };
                    tsMnuItemSetRange.DropDownItems.Add(item);
                }
            }
            else
            {
                var item = new ToolStripMenuItem("No selection available")
                {
                    Enabled = false
                };
                tsMnuItemSetRange.DropDownItems.Add(item);
            }
        }

        private void TsMnuItemSetRange_Click(object sender, EventArgs e)
        {
            if (sender is ToolStripMenuItem)
            {
                AnalogRange range = (AnalogRange)((ToolStripMenuItem)sender).Tag;
                SetChannelsToRange(range);
            }
        }

        private void SetChannelsToRange(AnalogRange range)
        {
            foreach (ListViewItem lvItem in lvChannelList.Items)
            {
                lvItem.Tag = range;
                lvItem.SubItems[_rangeSubItemIndex].Text = range.RangeText;
            }
        }

        private void LvChannelList_MouseDown(object sender, MouseEventArgs e)
        {
            _hitinfoChannelList = lvChannelList.HitTest(e.X, e.Y);
        }

        private void LvChannelList_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button != MouseButtons.Left || _moduleIsBusy || _selectedAiModule == null || _selectedAiModule.Ranges.Count <= 1 ||
            _hitinfoChannelList == null || _hitinfoChannelList.SubItem == null) return;

            ListViewHitTestInfo hitInfo = lvChannelList.HitTest(e.X, e.Y);
            if (_hitinfoChannelList.Item == hitInfo.Item && _hitinfoChannelList.Item.SubItems[_rangeSubItemIndex] == hitInfo.SubItem)
            {
                AnalogRange range = (AnalogRange)_hitinfoChannelList.Item.Tag;
                _coBoxRanges.Bounds = hitInfo.SubItem.Bounds;
                _coBoxRanges.SelectedIndex = (int)range.RangeIndex;
                _coBoxRanges.Visible = true;
                _coBoxRanges.BringToFront();
                _coBoxRanges.Focus();
            }
        }

        private void CoBoxRanges_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Return)
            {
                UpdateListViewItemChannelList();
            }
            else if (e.KeyChar == (char)Keys.Escape)
            {
                AnalogRange range = (AnalogRange)_hitinfoChannelList.Item.Tag;
                _coBoxRanges.SelectedIndex = (int)range.RangeIndex;
                _coBoxRanges.Visible = false;
            }
        }

        private void CoBoxRanges_DropDownClosed(object sender, EventArgs e)
        {
            UpdateListViewItemChannelList();
        }

        private void CoBoxRanges_Leave(object sender, EventArgs e)
        {
            UpdateListViewItemChannelList();
        }

        private void UpdateListViewItemChannelList()
        {
            if (_hitinfoChannelList == null || _hitinfoChannelList.SubItem == null || _coBoxRanges.SelectedItem == null) return;

            RangeComboBoxItem rangeItem = (RangeComboBoxItem)_coBoxRanges.SelectedItem;
            _hitinfoChannelList.Item.Tag = rangeItem.Range;
            _hitinfoChannelList.Item.SubItems[_rangeSubItemIndex].Text = rangeItem.ToString();

            _coBoxRanges.Visible = false;
            lvChannelList.Focus();
        }

        private void ContextMnuStripChanList_Opening(object sender, CancelEventArgs e)
        {
            if (_coBoxRanges.Visible)
            {
                _coBoxRanges.Visible = false;
            }
        }

        private void EnableUI(bool enabled)
        {
            coBoxModules.Enabled = enabled;
            textBoxSampleFrequency.Enabled = enabled;
            coBoxNumOfFrames.Enabled = enabled;
            coBoxChartType.Enabled = enabled;
            coBoxXAxisLabels.Enabled = enabled;
            btnSetConfig.Enabled = enabled;
            btnReset.Enabled = enabled;
            chBoxDisplayTextValues.Enabled = enabled;
            lvChannelList.Enabled = enabled;
            tsMnuItemSelectAll.Enabled = enabled;
            tsMnuItemSelectNone.Enabled = enabled;
            tsMnuItemInvertSelection.Enabled = enabled;
            tsMenuItemSaveGraphic.Enabled = enabled;
        }

        private void CoBoxNumOfFrames_TextChanged(object sender, EventArgs e)
        {
            if (_selectedAiModule == null) return;

            _selectedAiModule.NumberOfFrames = ParseNumberOfFrames(coBoxNumOfFrames.Text);
            UpdateNumberOfValues();
        }

        private void UpdateNumberOfValues()
        {
            if (_selectedAiModule == null)
            {
                textBoxNumberOfValues.Text = string.Empty;
                return;
            }

            textBoxNumberOfValues.Text = (_selectedAiModule.NumberOfFrames * _selectedAiModule.FrameLength).ToString();
        }

        private void TextBoxSampleFrequency_TextChanged(object sender, EventArgs e)
        {
            if (_selectedAiModule == null) return;

            _selectedAiModule.DesiredSampleFrequency = ParseFrequency(textBoxSampleFrequency.Text);
        }

        private double ParseFrequency(string sampleFrequency)
        {
            bool isValid = double.TryParse(sampleFrequency, out double actualFrequency);
            if (!isValid || actualFrequency <= 0) return 0.0;

            return actualFrequency;
        }

        private uint ParseNumberOfFrames(string numberOfFrames)
        {
            bool isValid = uint.TryParse(numberOfFrames, out uint numValues);

            return isValid ? numValues : 0;
        }

        private void LvChannelList_ItemChecked(object sender, ItemCheckedEventArgs e)
        {
            _selectedAiModule.FrameLength = (uint)lvChannelList.CheckedItems.Count;
            textBoxNumberOfSelectedChannels.Text = _selectedAiModule.FrameLength.ToString();
            UpdateNumberOfValues();
        }

        private void BtnReset_Click(object sender, EventArgs e)
        {
            uint adqError = _selectedAiModule.Reset();
            if (adqError == AdqErrors.ADQ_ERROR_NONE)
            {
                textBoxInfo.AppendText(string.Format("Reset board {0} module {1}" + Environment.NewLine,
                    _selectedAiModule.BoardIndex, _selectedAiModule.ModuleIndex));
            }
        }

        private void BtnSetConfig_Click(object sender, EventArgs e)
        {
            try
            {
                SetAiModuleConfiguration();
            }
            catch (Exception ex)
            {
                textBoxInfo.AppendText(ex.Message + Environment.NewLine);
            }
        }

        private void BtnStartStop_Click(object sender, EventArgs e)
        {
            if (_selectedAiModule == null) return;

            if (_moduleIsBusy)
            {
                _selectedAiModule.StopAcquisition();
                btnStartStop.Text = "Start";
                return;
            }

            ClearDisplayedValues();

            EnableUI(false);
            btnStartStop.Text = "Stop";
            try
            {
                SetAiModuleConfiguration();
                _moduleIsBusy = true;
                AddTextToCombobox(coBoxNumOfFrames, coBoxNumOfFrames.Text);
                backgroundWorker1.RunWorkerAsync(_selectedAiModule);
            }
            catch (Exception ex)
            {
                _moduleIsBusy = false;
                EnableUI(true);
                btnStartStop.Text = "Start";
                textBoxInfo.AppendText(ex.Message + Environment.NewLine);
            }
        }

        private void SetAiModuleConfiguration()
        {
            UpdateChannelSettings();
            _selectedAiModule.ConfigureModule();
            UpdateUI();
        }

        private void UpdateUI()
        {
            textBoxActualFrequency.Text = AdqCommon.FormatFrequency(_selectedAiModule.ActualSampleFrequency);

            double seconds = _selectedAiModule.GetSecondsToWait();
            lblDuration.Text = AdqCommon.FormatTimeToWait(seconds);
        }

        private void UpdateChannelSettings()
        {
            _selectedAiModule.ChannelSettings.Clear();
            foreach (ListViewItem item in lvChannelList.CheckedItems)
            {
                AnalogRange range = (AnalogRange)item.Tag;
                _selectedAiModule.AddChannelSetting((uint)item.Index, range);
            }
            _selectedAiModule.UpdateChannelList();
        }

        private void DisplayValues(AnalogInputModule aiModule)
        {
            DisplayAnalogValuesAsText(aiModule);
            DisplayAnalogValuesAsChart(aiModule);
        }

        private void DisplayAnalogValuesAsText(AnalogInputModule aiModule)
        {
            if (!chBoxDisplayTextValues.Checked) return;
            UseWaitCursor = true;
            try
            {
                var sb = new StringBuilder();
                sb.AppendLine(string.Format("Frames read: {0}", aiModule.NumberOfFramesRead));
                uint frameLength = aiModule.FrameLength;
                uint currentFrameOffset = 0;
                double time = 0;
                for (uint frameIdx = 0; frameIdx < aiModule.NumberOfFramesRead; frameIdx++)
                {
                    sb.Append(string.Format("{0:000}; {1:0.0000000sec}", frameIdx, time));
                    for (int idx = 0; idx < frameLength; idx++)
                    {
                        ChannelSetting chanSetting = aiModule.ChannelSettings[idx];
                        uint digitalValue = aiModule.Frames[currentFrameOffset + idx];
                        double physicalValue = aiModule.PhysicalFrames[currentFrameOffset + idx];
                        sb.Append(string.Format("; Ch{0}: 0x{1:X8} {1} --> {2:+0.00000000;-0.00000000;0}{3}",
                            chanSetting.ChannelIndex,
                            digitalValue,
                            physicalValue,
                            chanSetting.Range.UnitSymbol));
                    }
                    sb.AppendLine();
                    time += aiModule.SampleTime;
                    currentFrameOffset += frameLength;
                }
                textBoxInfo.AppendText(sb.ToString());
            }
            finally
            {
                UseWaitCursor = false;
            }
        }

        private void DisplayAnalogValuesAsChart(AnalogInputModule aiModule)
        {
            uint frameLength = aiModule.FrameLength;
            UseWaitCursor = true;
            chart1.SuspendLayout();
            try
            {
                UpdateChart(aiModule);
                InitSeriesSubMenus();
                double xValueDelta = GetXValueDelta(aiModule);

                DataPointCollection points0 = null;
                for (uint chIdx = 0; chIdx < frameLength; chIdx++)
                {
                    Series series = chart1.Series[(int)chIdx];
                    uint valIdx = chIdx;
                    int pointIdx = 0;
                    double xValue = 0.0;
                    string tsLabel;
                    while (valIdx < aiModule.TotalNumberOfValuesRead)
                    {
                        double physicalValue = aiModule.PhysicalFrames[valIdx];
                        var dp = new DataPoint(pointIdx, physicalValue);
                        if (points0 == null)
                        {
                            tsLabel = GetXAsixLabel(xValue);
                            xValue += xValueDelta;
                        }
                        else
                        {
                            tsLabel = points0[pointIdx].AxisLabel;
                        }
                        dp.AxisLabel = tsLabel;
                        series.Points.Add(dp);
                        valIdx += frameLength;
                        pointIdx++;
                    }
                    if (chIdx == 0)
                    {
                        points0 = series.Points;
                    }
                }
                chart1.ChartAreas[0].RecalculateAxesScale();
            }
            finally
            {
                chart1.ResumeLayout();
                UseWaitCursor = false;
            }
        }

        private string GetXAsixLabel(double xValue)
        {
            if (coBoxXAxisLabels.SelectedIndex == _idxXAxisLabelsTime)
            {
                return AdqCommon.FormatSampleTime(xValue);
            }
            return ((int)xValue).ToString();
        }

        private double GetXValueDelta(AnalogInputModule aiModule)
        {
            return (coBoxXAxisLabels.SelectedIndex == _idxXAxisLabelsTime) ? aiModule.SampleTime : 1.0;
        }

        private void InitSeriesSubMenus()
        {
            tsMnuItemShowSeries.DropDownItems.Clear();

            if (chart1.Series.Count == 0)
            {
                var item = new ToolStripMenuItem("No series present")
                {
                    Enabled = false
                };
                tsMnuItemShowSeries.DropDownItems.Add(item);
                return;
            }

            foreach (Series series in chart1.Series)
            {
                var item = new ToolStripMenuItem(series.LegendText, null, TsMnuItemShowSeries_Click)
                {
                    Checked = true,
                    CheckOnClick = true,
                    Tag = series
                };
                tsMnuItemShowSeries.DropDownItems.Add(item);
            }

            tsMnuItemShowSeries.DropDownItems.Add(new ToolStripSeparator());
            var itemShowAll = new ToolStripMenuItem("Show all Series", null, TsMnuItemShowAllSeries_Click);
            tsMnuItemShowSeries.DropDownItems.Add(itemShowAll);
            var itemHideAll = new ToolStripMenuItem("Hide all Series", null, TsMnuItemHideAllSeries_Click);
            tsMnuItemShowSeries.DropDownItems.Add(itemHideAll);
        }

        private void TsMnuItemShowSeries_Click(object sender, EventArgs e)
        {
            if (sender is ToolStripMenuItem)
            {
                Series series = (Series)((ToolStripMenuItem)sender).Tag;
                series.Enabled = ((ToolStripMenuItem)sender).Checked;
            }
        }

        private void TsMnuItemShowAllSeries_Click(object sender, EventArgs e)
        {
            ShowAllSeries(true);
        }

        private void TsMnuItemHideAllSeries_Click(object sender, EventArgs e)
        {
            ShowAllSeries(false);
        }

        private void ShowAllSeries(bool showSeries)
        {
            foreach (ToolStripItem item in tsMnuItemShowSeries.DropDownItems)
            {
                if (item is ToolStripMenuItem && item.Tag is Series)
                {
                    Series series = (Series)(item).Tag;
                    ((ToolStripMenuItem)item).Checked = showSeries;
                    series.Enabled = showSeries;
                }
            }
        }

        private void InitChart()
        {
            chart1.BackColor = Color.Black;
            ChartArea ca = chart1.ChartAreas[0];
            ca.BackColor = Color.Black;

            ca.AxisX.MajorGrid.LineColor = _gridLineColor;
            ca.AxisX.MajorTickMark.LineColor = _gridLineColor;
            ca.AxisX.MinorGrid.LineColor = _gridLineColor;
            ca.AxisX.MinorTickMark.LineColor = _gridLineColor;
            ca.AxisX.LineColor = _gridLineColor;
            ca.AxisX.LineWidth = 1;
            ca.AxisX.LabelStyle.ForeColor = _labelColor;
            ca.AxisX.TitleForeColor = _labelColor;
            ca.AxisX.ScrollBar.ButtonColor = Color.DarkGray;
            ca.AxisX.ScrollBar.BackColor = Color.Gray;
            ca.CursorX.IsUserSelectionEnabled = true;

            ca.AxisY.MajorGrid.LineColor = _gridLineColor;
            ca.AxisY.MajorTickMark.LineColor = _gridLineColor;
            ca.AxisY.MinorGrid.LineColor = _gridLineColor;
            ca.AxisY.MinorTickMark.LineColor = _gridLineColor;
            ca.AxisY.LineColor = _gridLineColor;
            ca.AxisY.LineWidth = 1;
            ca.AxisY.LabelStyle.ForeColor = _labelColor;
            ca.AxisY.LabelStyle.Format = _fmtStringPhysValues;
            ca.AxisY.TitleForeColor = _labelColor;
            ca.AxisY.ScrollBar.ButtonColor = Color.DarkGray;
            ca.AxisY.ScrollBar.BackColor = Color.Gray;
            ca.CursorY.IsUserSelectionEnabled = true;
        }

        private void UpdateChart(AnalogInputModule aiModule)
        {
            ChartArea ca = chart1.ChartAreas[0];
            ca.AxisX.Title = coBoxXAxisLabels.Text;
            ca.AxisY.Title = aiModule.ChannelSettings[0].Range.UnitName;

            double scrollSize = aiModule.MinResolution / 4.0;
            ca.AxisY.ScaleView.SmallScrollMinSize = scrollSize;
            ca.AxisY.ScaleView.SmallScrollSize = scrollSize;
            ca.CursorY.Interval = scrollSize;
            ca.AxisY.Maximum = double.NaN;
            ca.AxisY.Minimum = double.NaN;

            foreach (ChannelSetting chanSetting in aiModule.ChannelSettings)
            {
                InitSeries(chanSetting.ChannelIndex, chart1, coBoxChartType.SelectedIndex);
            }
        }

        private void InitSeries(uint channelIndex, Chart chart, int chartTypeIndex)
        {
            chart.GetToolTipText -= Chart1_GetToolTipText;
            Series series = new Series("Channel " + channelIndex.ToString());
            series.LegendText = series.Name;
            if (chartTypeIndex == _idxChartTypeLine)
            {
                series.ChartType = SeriesChartType.Line;
            }
            else if (chartTypeIndex == _idxChartTypeLineAndMarkers)
            {
                series.ChartType = SeriesChartType.Line;
                series.MarkerStyle = AdqCommon.MarkerStyles[channelIndex % AdqCommon.MarkerStyles.Length];
            }
            else
            {
                series.ChartType = SeriesChartType.Point;
            }

            chart.GetToolTipText += Chart1_GetToolTipText;
            series.Color = AdqCommon.ChannelColors[channelIndex % AdqCommon.ChannelColors.Length];
            series.YValueType = ChartValueType.Double;
            series.XValueType = ChartValueType.UInt32;
            series.IsXValueIndexed = false;
            chart.Series.Add(series);
        }

        private void Chart1_GetToolTipText(object sender, ToolTipEventArgs e)
        {
            if (e.HitTestResult.ChartElementType == ChartElementType.DataPoint)
            {
                var series = e.HitTestResult.Series;
                if (series != null)
                {
                    DataPoint dp = series.Points[e.HitTestResult.PointIndex];
                    e.Text = FormatTooltipText(dp);
                }
            }
            else if (e.HitTestResult.ChartElementType == ChartElementType.Axis
                || e.HitTestResult.ChartElementType == ChartElementType.TickMarks)
            {
                if (sender is Chart)
                {
                    HitTestResult result = ((Chart)sender).HitTest(e.X, e.Y, ChartElementType.DataPoint);
                    if (result.ChartElementType == ChartElementType.DataPoint)
                    {
                        DataPoint dp = result.Series.Points[result.PointIndex];
                        e.Text = FormatTooltipText(dp);
                    }
                }
            }
        }

        private string FormatTooltipText(DataPoint dp)
        {
            return string.Format("X={0}, Y={1}",
                        dp.XValue,
                        AdqCommon.FormatPhysicalValue(_fmtStringPhysValues, dp.YValues[0]) + _selectedUnitSymbol);
        }

        private void BackgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            AnalogInputModule aiModule = e.Argument as AnalogInputModule;
            aiModule.ReadFramesFromModule();
            e.Result = aiModule;
        }

        private void BackgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (_isFormClosing) return;

            AnalogInputModule aiModule = e.Result as AnalogInputModule;
            _moduleIsBusy = false;
            if (aiModule.AdqError == AdqErrors.ADQ_ERROR_NONE)
            {
                DisplayValues(aiModule);
            }
            else
            {
                DisplayErrorText("adqAIReadBuffer", aiModule.AdqError);
            }
            btnStartStop.Text = "Start";
            EnableUI(true);
        }

        private void FormMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_moduleIsBusy)
            {
                DialogResult result = MessageBox.Show("The acquisition is still running."
                                         + Environment.NewLine
                                         + "Do you want to close the program?",
                                         "Module busy",
                                         MessageBoxButtons.YesNo,
                                         MessageBoxIcon.Question);
                if (result == DialogResult.Yes)
                {
                    _isFormClosing = true;
                    _selectedAiModule.StopAcquisition();
                    e.Cancel = false;
                }
                else
                {
                    e.Cancel = true;
                }
                return;
            }
            _isFormClosing = true;
        }

        private void BtnClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void FormMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            CloseAdqDriver();
        }

        private void CloseAdqDriver()
        {
            if (_moduleLocator != null)
            {
                _moduleLocator.CloseAdqDriver();
            }
        }

        private void BtnClear_Click(object sender, EventArgs e)
        {
            ClearDisplayedValues();
        }

        private void ClearDisplayedValues()
        {
            chart1.Series.Clear();
            InitSeriesSubMenus();
            textBoxInfo.Clear();
        }

        private void TextBoxInfo_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Modifiers == Keys.Control && e.KeyCode == Keys.A)
            {
                textBoxInfo.SelectionStart = 0;
                textBoxInfo.SelectionLength = textBoxInfo.Text.Length;
            }
        }

        private void TsMnuItemSelectAll_Click(object sender, EventArgs e)
        {
            CheckListViewItems(true);
        }

        private void TsMnuItemInvertSelection_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < lvChannelList.Items.Count; i++)
            {
                lvChannelList.Items[i].Checked = !lvChannelList.Items[i].Checked;
            }
        }

        private void TsMnuItemSelectNone_Click(object sender, EventArgs e)
        {
            CheckListViewItems(false);
        }

        private void CheckListViewItems(bool check)
        {
            for (int i = 0; i < lvChannelList.Items.Count; i++)
            {
                lvChannelList.Items[i].Checked = check;
            }
        }

        private string GetImageFileName(AnalogInputModule aiModule, string fileExt)
        {
            string s = string.Empty;
            if (aiModule != null)
            {
                s = string.Format("{0}, Board {1}, Module {2}, ",
                    aiModule.ModelName,
                    aiModule.BoardIndex,
                    aiModule.ModuleIndex);
            }
            return s + DateTime.Now.ToString("yyyy-MM-dd HH.mm.ss") + fileExt;
        }

        private void TsMenuItemSaveGraphic_Click(object sender, EventArgs e)
        {
            saveFileDialog1.FileName = GetImageFileName(_selectedAiModule, ".png");
            saveFileDialog1.DefaultExt = "png";
            saveFileDialog1.ShowDialog();
        }

        private ChartImageFormat GetImageFormat(int filterIndex)
        {
            ChartImageFormat imgFormat;
            switch (filterIndex)
            {
                case 1:
                    imgFormat = ChartImageFormat.Png;
                    break;
                case 2:
                    imgFormat = ChartImageFormat.Gif;
                    break;
                case 3:
                    imgFormat = ChartImageFormat.Bmp;
                    break;
                case 4:
                    imgFormat = ChartImageFormat.Tiff;
                    break;
                case 5:
                    imgFormat = ChartImageFormat.Jpeg;
                    break;
                default:
                    imgFormat = ChartImageFormat.Png;
                    break;
            }
            return imgFormat;
        }

        private void SaveFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            ChartImageFormat imgFormat = GetImageFormat(saveFileDialog1.FilterIndex);
            chart1.SaveImage(saveFileDialog1.FileName, imgFormat);
        }

        private void TsMnuItemCopyToClipboard_Click(object sender, EventArgs e)
        {
            using (MemoryStream ms = new MemoryStream())
            {
                chart1.SaveImage(ms, ChartImageFormat.Png);
                var bitmap = new Bitmap(ms);
                Clipboard.SetImage(bitmap);
            }
        }

        private void AddTextToCombobox(ComboBox coBox, string text)
        {
            if (!coBox.Items.Contains(text))
            {
                coBox.Items.Insert(0, text);
            }
        }

        private void BtnInfo_Click(object sender, EventArgs e)
        {
            var aboutBox = new AboutBox();
            aboutBox.ShowDialog();
        }
    }
}
