﻿//-----------------------------------------------------------------------------
// adqDevice.cs - Constants, types and function prototypes
//      Copyright (c) ALLDAQ, a division of ALLNET GmbH
//      https://www.alldaq.com
//
// Purpose:
//      This file has defines for constants, types and function prototypes
//      used by ALLDAQ measurement boards
// class AdqDriver:
//      types used by unmanaged library
//      import function prototypes
// class AdqConstants:
//      constants used by the API
// class AdqErrors:
//      error codes returned by the API functions
//
// Note:
//      Native import library 32bit: adqDevice.dll
//                            64bit: adqDevice_64.dll
//-----------------------------------------------------------------------------

using System;
using System.Runtime.InteropServices;

namespace ALLDAQ.Boards
{
    public sealed class AdqDriver
    {
#if WIN64
        public const string ADQDEVICE_DLL = "adqDevice_64.dll";
#else
        public const string ADQDEVICE_DLL = "adqDevice.dll";
#endif
        #region Types

        //---------------------------------------------------------------------
        //  ADQ System information
        //---------------------------------------------------------------------
        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct ADQSystemInfo
        {
            public uint Size;
            public uint ID;
            public uint DLLVersion;
            public uint NumberOfBoards;

            public ADQSystemInfo(uint ID)
            {
                Size = (uint)Marshal.SizeOf(typeof(ADQSystemInfo));
                this.ID = ID;
                DLLVersion = 0;
                NumberOfBoards = 0;
            }
        }
        public const uint ADQSystemInfo_ID = 0x00010000;
        public static uint SizeOfSystemInfo()
        {
            return (uint)Marshal.SizeOf(typeof(ADQSystemInfo));
        }


        //---------------------------------------------------------------------
        //  Board information
        //---------------------------------------------------------------------
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode, Pack = 4)]
        public struct ADQBoardInfo
        {
            public uint Size;
            public uint ID;
            public ushort VendorID;
            public ushort DeviceID;
            public uint SerialNumber;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 64)]
            public string BoardName;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256)]
            public string BoardDescription;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 16)]
            public string DriverName;
            public uint DriverVersion;
            public uint TTLVoltageLevel;
            public uint BusType;
            [MarshalAs(UnmanagedType.U1)]
            public bool Remote;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256)]
            public string RemoteIPAddressOrHostName;
            public uint FirmwareVersion;
            public uint NumberOfModules;

            public ADQBoardInfo(uint ID)
            {
                Size = (uint)Marshal.SizeOf(typeof(ADQBoardInfo));
                this.ID = ID;
                VendorID = 0;
                DeviceID = 0;
                SerialNumber = 0;
                BoardName = "";
                BoardDescription = "";
                DriverName = "";
                DriverVersion = 0;
                TTLVoltageLevel = 0;
                BusType = 0;
                Remote = false;
                RemoteIPAddressOrHostName = "";
                FirmwareVersion = 0;
                NumberOfModules = 0;
            }
        }
        public const uint ADQBoardInfo_ID = 0x00020001;
        public static uint SizeOfBoardInfo()
        {
            return (uint)Marshal.SizeOf(typeof(ADQBoardInfo));
        }


        //---------------------------------------------------------------------
        //  Board PCI information structure, passed to adqGetBoardPCIInfo
        //---------------------------------------------------------------------
        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct ADQBoardPCIInfo
        {
            public uint Size;
            public uint ID;
            public uint BusNumber;
            public uint DeviceNumber;
            public uint FunctionNumber;

            public ADQBoardPCIInfo(uint ID)
            {
                Size = (uint)Marshal.SizeOf(typeof(ADQBoardPCIInfo));
                this.ID = ID;
                BusNumber = 0;
                DeviceNumber = 0;
                FunctionNumber = 0;
            }
        }
        public const uint ADQBoardPCIInfo_ID = 0x000D0000;
        public static uint SizeOfBoardPCIInfo()
        {
            return (uint)Marshal.SizeOf(typeof(ADQBoardPCIInfo));
        }


        //---------------------------------------------------------------------
        //  General module information
        //---------------------------------------------------------------------
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode, Pack = 4)]
        public struct ADQGeneralModuleInfo
        {
            public uint Size;
            public uint ID;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 1024)]
            public string ModuleDescription;
            public uint ModuleType;
            [MarshalAs(UnmanagedType.U1)]
            public bool IsIRQSource;
            public uint ClockType;
            public double ClockBaseFrequencyHz;
            public uint ClockMinimumDivider;
            public uint InternalDataBufferSizeValues;

            public ADQGeneralModuleInfo(uint ID)
            {
                Size = (uint)Marshal.SizeOf(typeof(ADQGeneralModuleInfo));
                this.ID = ID;
                ModuleDescription = "";
                ModuleType = 0;
                IsIRQSource = false;
                ClockType = AdqConstants.ADQ_CLOCK_TYPE_NONE;
                ClockBaseFrequencyHz = 0;
                ClockMinimumDivider = 0;
                InternalDataBufferSizeValues = 0;
            }
        }
        public const uint ADQGeneralModuleInfo_ID = 0x00030001;
        public static uint SizeOfGeneralModuleInfo()
        {
            return (uint)Marshal.SizeOf(typeof(ADQGeneralModuleInfo));
        }


        //---------------------------------------------------------------------
        //  Structures for trigger routines
        //---------------------------------------------------------------------
        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct ADQHWTriggerInfo
        {
            public uint Size;
            public uint ID;
            public uint HWTriggerStartSource;
            public uint HWTriggerStopSource;
            [MarshalAs(UnmanagedType.U1)]
            public bool SupportsChannelListPerTrigger;

            public ADQHWTriggerInfo(uint ID)
            {
                Size = (uint)Marshal.SizeOf(typeof(ADQHWTriggerInfo));
                this.ID = ID;
                HWTriggerStartSource = 0;
                HWTriggerStopSource = 0;
                SupportsChannelListPerTrigger = false;
            }
        }
        public const uint ADQHWTriggerInfo_ID = 0x000B0000;
        public static uint SizeOfHWTriggerInfo()
        {
            return (uint)Marshal.SizeOf(typeof(ADQHWTriggerInfo));
        }


        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct ADQHWTriggerEnable
        {
            public uint Size;
            public uint ID;
            public uint HWTriggerStartRisingEdge;
            public uint HWTriggerStartFallingEdge;
            public uint HWTriggerStopRisingEdge;
            public uint HWTriggerStopFallingEdge;
            [MarshalAs(UnmanagedType.U1)]
            public bool ChannelListPerTrigger;
            [MarshalAs(UnmanagedType.U1)]
            public bool AutomaticRearm;

            public ADQHWTriggerEnable(uint ID)
            {
                Size = (uint)Marshal.SizeOf(typeof(ADQHWTriggerEnable));
                this.ID = ID;
                HWTriggerStartRisingEdge = 0;
                HWTriggerStartFallingEdge = 0;
                HWTriggerStopRisingEdge = 0;
                HWTriggerStopFallingEdge = 0;
                ChannelListPerTrigger = false;
                AutomaticRearm = false;
            }
        }
        public const uint ADQHWTriggerEnable_ID = 0x000C0000;
        public static uint SizeOfHWTriggerEnable()
        {
            return (uint)Marshal.SizeOf(typeof(ADQHWTriggerEnable));
        }


        //---------------------------------------------------------------------
        //  Module information for DI, DIO and DO modules
        //---------------------------------------------------------------------
        public const uint ADQ_MAX_DI_FILTERS = 4;

        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct ADQDIModuleInfo
        {
            public uint Size;
            public uint ID;
            public uint PortWidthBits;
            [MarshalAs(UnmanagedType.U1)]
            public bool IsOptoIsolated;
            [MarshalAs(UnmanagedType.U1)]
            public bool IsTTL;
            public uint NumberOfFilters;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = (int)ADQ_MAX_DI_FILTERS)]
            public ulong[] FilterTimePicoseconds;

            public ADQDIModuleInfo(uint ID)
            {
                Size = (uint)Marshal.SizeOf(typeof(ADQDIModuleInfo));
                this.ID = ID;
                PortWidthBits = 0;
                IsOptoIsolated = false;
                IsTTL = false;
                NumberOfFilters = 0;
                FilterTimePicoseconds = new ulong[ADQ_MAX_DI_FILTERS];
            }
        }
        public const uint ADQDIModuleInfo_ID = 0x00040001;
        public static uint SizeOfDIModuleInfo()
        {
            return (uint)Marshal.SizeOf(typeof(ADQDIModuleInfo));
        }


        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct ADQDOModuleInfo
        {
            public uint Size;
            public uint ID;
            public uint PortWidthBits;
            [MarshalAs(UnmanagedType.U1)]
            public bool IsOptoIsolated;
            [MarshalAs(UnmanagedType.U1)]
            public bool IsTTL;

            public ADQDOModuleInfo(uint ID)
            {
                Size = (uint)Marshal.SizeOf(typeof(ADQDOModuleInfo));
                this.ID = ID;
                PortWidthBits = 0;
                IsOptoIsolated = false;
                IsTTL = false;
            }
        }
        public const uint ADQDOModuleInfo_ID = 0x00040000;
        public static uint SizeOfDOModuleInfo()
        {
            return (uint)Marshal.SizeOf(typeof(ADQDOModuleInfo));
        }

        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct ADQDIOModuleInfo
        {
            public uint Size;
            public uint ID;
            public uint PortWidthBits;
            [MarshalAs(UnmanagedType.U1)]
            public bool IsOptoIsolated;
            [MarshalAs(UnmanagedType.U1)]
            public bool IsTTL;

            public ADQDIOModuleInfo(uint ID)
            {
                Size = (uint)Marshal.SizeOf(typeof(ADQDIOModuleInfo));
                this.ID = ID;
                PortWidthBits = 0;
                IsOptoIsolated = false;
                IsTTL = false;
            }
        }
        public const uint ADQDIOModuleInfo_ID = 0x00040000;
        public static uint SizeOfDIOModuleInfo()
        {
            return (uint)Marshal.SizeOf(typeof(ADQDIOModuleInfo));
        }


        //---------------------------------------------------------------------
        //  Module information for Counter modules
        //---------------------------------------------------------------------
        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct ADQCNTRModuleInfo
        {
            public uint Size;
            public uint ID;
            public uint CounterType;
            public uint WidthBits;
            public uint ClockSources;

            public ADQCNTRModuleInfo(uint ID)
            {
                Size = (uint)Marshal.SizeOf(typeof(ADQCNTRModuleInfo));
                this.ID = ID;
                CounterType = 0;
                WidthBits = 0;
                ClockSources = 0;
            }
        }
        public const uint ADQCNTRModuleInfo_ID = 0x00050000;
        public static uint SizeOfADQCNTRModuleInfo()
        {
            return (uint)Marshal.SizeOf(typeof(ADQCNTRModuleInfo));
        }


        //---------------------------------------------------------------------
        //  Module information for Analog input modules
        //---------------------------------------------------------------------
        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct ADQAIRange
        {
            public uint DigitalMin;
            public uint DigitalMax;
            public uint PhysicalUnit;
            public double PhysicalMin;
            public double PhysicalMax;
        }

        public const uint ADQ_MAX_AI_RANGES = 8;


        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct ADQAIModuleInfo
        {
            public uint Size;
            public uint ID;
            public uint NumberOfChannels;
            [MarshalAs(UnmanagedType.U1)]
            public bool SharedDataRate;
            [MarshalAs(UnmanagedType.U1)]
            public bool RangeInfoPerChannel;
            public uint NumberOfRanges;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = (int)ADQ_MAX_AI_RANGES)]
            public ADQAIRange[] Ranges;

            public ADQAIModuleInfo(uint ID)
            {
                Size = (uint)Marshal.SizeOf(typeof(ADQAIModuleInfo));
                this.ID = ID;
                NumberOfChannels = 0;
                SharedDataRate = false;
                RangeInfoPerChannel = false;
                NumberOfRanges = 0;
                Ranges = new ADQAIRange[ADQ_MAX_AI_RANGES];
            }
        }
        public const uint ADQAIModuleInfo_ID = 0x00060000;
        public static uint SizeOfADQAIModuleInfo()
        {
            return (uint)Marshal.SizeOf(typeof(ADQAIModuleInfo));
        }


        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct ADQAIChannel
        {
            [MarshalAs(UnmanagedType.U1)]
            public bool AcquireChannel;
            public uint RangeIndex;
        }


        //---------------------------------------------------------------------
        //  Module information for Analog output modules
        //---------------------------------------------------------------------
        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct ADQAORange
        {
            public uint DigitalMin;
            public uint DigitalMax;
            public uint PhysicalUnit;
            public double PhysicalMin;
            public double PhysicalMax;
        }

        public const uint ADQ_MAX_AO_RANGES = 8;

        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct ADQAOModuleInfo
        {
            public uint Size;
            public uint ID;
            public uint NumberOfRanges;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = (int)ADQ_MAX_AO_RANGES)]
            public ADQAORange[] Ranges;

            public ADQAOModuleInfo(uint ID)
            {
                Size = (uint)Marshal.SizeOf(typeof(ADQAOModuleInfo));
                this.ID = ID;
                NumberOfRanges = 0;
                Ranges = new ADQAORange[ADQ_MAX_AO_RANGES];
            }
        }
        public const uint ADQAOModuleInfo_ID = 0x000A0000;
        public static uint SizeOfADQAOModuleInfo()
        {
            return (uint)Marshal.SizeOf(typeof(ADQAOModuleInfo));
        }


        //---------------------------------------------------------------------
        //  Module information for I2C modules
        //---------------------------------------------------------------------
        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct ADQI2CModuleInfo
        {
            public uint Size;
            public uint ID;
            public uint SpeedBitsPerSecond;
            [MarshalAs(UnmanagedType.U1)]
            public bool Supports7BitAddresses;
            [MarshalAs(UnmanagedType.U1)]
            public bool Supports10BitAddresses;

            public ADQI2CModuleInfo(uint ID)
            {
                Size = (uint)Marshal.SizeOf(typeof(ADQI2CModuleInfo));
                this.ID = ID;
                SpeedBitsPerSecond = 0;
                Supports7BitAddresses = false;
                Supports10BitAddresses = false;
            }
        }
        public const uint ADQI2CModuleInfo_ID = 0x000E0000;
        public static uint SizeOfADQI2CModuleInfo()
        {
            return (uint)Marshal.SizeOf(typeof(ADQI2CModuleInfo));
        }


        //---------------------------------------------------------------------
        //  Module information for PWM modules
        //---------------------------------------------------------------------
        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct ADQPWMModuleInfo
        {
            public uint Size;
            public uint ID;
            public double BaseFrequencyHz;
            public uint MinimumDivider;

            public ADQPWMModuleInfo(uint ID)
            {
                Size = (uint)Marshal.SizeOf(typeof(ADQPWMModuleInfo));
                this.ID = ID;
                BaseFrequencyHz = 0.0;
                MinimumDivider = 0;
            }
        }
        public const uint ADQPWMModuleInfo_ID = 0x000F0000;
        public static uint SizeOfADQPWMModuleInfo()
        {
            return (uint)Marshal.SizeOf(typeof(ADQPWMModuleInfo));
        }


        //---------------------------------------------------------------------
        //  Structures for IRQ routines
        //---------------------------------------------------------------------
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode, Pack = 4)]
        public struct ADQIRQInfo
        {
            public uint Size;
            public uint ID;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 1024)]
            public string IRQDescription;
            [MarshalAs(UnmanagedType.U1)]
            public bool SupportsBitChange;
            [MarshalAs(UnmanagedType.U1)]
            public bool SupportsBitPatternCompare;
            public uint SupportsExternal;
            public uint SupportsPortStatusError;
            public uint SupportsFaultRecovery;

            public ADQIRQInfo(uint ID)
            {
                Size = (uint)Marshal.SizeOf(typeof(ADQIRQInfo));
                this.ID = ID;
                IRQDescription = "";
                SupportsBitChange = false;
                SupportsBitPatternCompare = false;
                SupportsExternal = 0;
                SupportsPortStatusError = 0;
                SupportsFaultRecovery = 0;
            }
        }
        public const uint ADQIRQInfo_ID = 0x00070001;
        public static uint SizeOfADQIRQInfo()
        {
            return (uint)Marshal.SizeOf(typeof(ADQIRQInfo));
        }


        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct ADQIRQEnable
        {
            public uint Size;
            public uint ID;
            public uint PortBitChangeRisingEdge;
            public uint PortBitChangeFallingEdge;
            [MarshalAs(UnmanagedType.U1)]
            public bool PortBitPatternCompare;
            public uint PortBitPatternValue;
            public uint ExternalRisingEdge;
            public uint ExternalFallingEdge;
            public uint PortStatusError;
            public uint FaultRecovery;
            [MarshalAs(UnmanagedType.U1)]
            public bool ResetIRQCount;

            public ADQIRQEnable(uint ID)
            {
                Size = (uint)Marshal.SizeOf(typeof(ADQIRQEnable));
                this.ID = ID;
                PortBitChangeRisingEdge = 0;
                PortBitChangeFallingEdge = 0;
                PortBitPatternCompare = false;
                PortBitPatternValue = 0;
                ExternalRisingEdge = 0;
                ExternalFallingEdge = 0;
                PortStatusError = 0;
                FaultRecovery = 0;
                ResetIRQCount = false;
            }
        }
        public const uint ADQIRQEnable_ID = 0x00080001;
        public static uint SizeOfADQIRQEnable()
        {
            return (uint)Marshal.SizeOf(typeof(ADQIRQEnable));
        }


        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct ADQIRQStatus
        {
            public uint Size;
            public uint ID;
            public uint IRQCount;
            public uint PortBitChangeRisingEdge;
            public uint PortBitChangeFallingEdge;
            [MarshalAs(UnmanagedType.U1)]
            public bool PortBitPatternCompare;
            public uint ExternalRisingEdge;
            public uint ExternalFallingEdge;
            public uint PortStatusError;
            public uint FaultRecovery;
            public uint TriggerValue;

            public ADQIRQStatus(uint ID)
            {
                Size = (uint)Marshal.SizeOf(typeof(ADQIRQStatus));
                this.ID = ID;
                IRQCount = 0;
                PortBitChangeRisingEdge = 0;
                PortBitChangeFallingEdge = 0;
                PortBitPatternCompare = false;
                ExternalRisingEdge = 0;
                ExternalFallingEdge = 0;
                PortStatusError = 0;
                FaultRecovery = 0;
                TriggerValue = 0;
            }
        }
        public const uint ADQIRQStatus_ID = 0x00090001;
        public static uint SizeOfADQIRQStatus()
        {
            return (uint)Marshal.SizeOf(typeof(ADQIRQStatus));
        }


        //---------------------------------------------------------------------
        //  Module configuration for counter modules of type 8254 (ADQ_CNTR_TYPE_8254)
        //---------------------------------------------------------------------
        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct ADQConfigCNTR8254
        {
            public uint Size;
            public uint ID;
            public uint ClockSource;
            public uint CNTR8254Mode;
            public uint StartValue;

            public ADQConfigCNTR8254(uint ID)
            {
                Size = (uint)Marshal.SizeOf(typeof(ADQConfigCNTR8254));
                this.ID = ID;
                ClockSource = 0;
                CNTR8254Mode = 0;
                StartValue = 0;
            }
        }
        public const uint ADQConfigCNTR8254_ID = 0x00100000;
        public static uint SizeOfADQConfigCNTR8254()
        {
            return (uint)Marshal.SizeOf(typeof(ADQConfigCNTR8254));
        }


        //---------------------------------------------------------------------
        //  Module configuration for counter modules of type count down 32-Bit (ADQ_CNTR_TYPE_COUNT_DOWN_32_bit)
        //---------------------------------------------------------------------
        [StructLayout(LayoutKind.Sequential, Pack = 4)]
        public struct ADQConfigCNTRCountDown32Bit
        {
            public uint Size;
            public uint ID;
            public uint ClockSource;
            public uint StartValue;
            public uint ThresholdValue;
            [MarshalAs(UnmanagedType.U1)]
            public bool OneShot;
            public uint StrobeValue;
            [MarshalAs(UnmanagedType.U1)]
            public bool StrobeInvert;
            [MarshalAs(UnmanagedType.U1)]
            public bool HardwareGate;
            [MarshalAs(UnmanagedType.U1)]
            public bool HardwareTrigger;

            public ADQConfigCNTRCountDown32Bit(uint ID)
            {
                Size = (uint)Marshal.SizeOf(typeof(ADQConfigCNTRCountDown32Bit));
                this.ID = ID;
                ClockSource = 0;
                StartValue = 0;
                ThresholdValue = 0;
                OneShot = false;
                StrobeValue = 0;
                StrobeInvert = false;
                HardwareGate = false;
                HardwareTrigger = false;
            }
        }
        public const uint ADQConfigCNTRCountDown32Bit_ID = 0x00200000;
        public static uint SizeOfADQConfigCNTRCountDown32Bit()
        {
            return (uint)Marshal.SizeOf(typeof(ADQConfigCNTRCountDown32Bit));
        }


        //---------------------------------------------------------------------
        //  Callback routines
        //---------------------------------------------------------------------

        /// <summary>
        /// Delegate for the function passed to adqEnableIRQ
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="irqStatus">
        /// Reference to a structure ADQIRQStatus
        /// </param>
        /// <param name="adqError">
        /// Error code
        /// </param>
        /// <param name="userContext">
        /// Pointer to user defined context that was passed to function adqEnableIRQ
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        public delegate uint adqIrqCallback(
            uint boardIndex,
            uint moduleIndex,
            ref ADQIRQStatus irqStatus,
            uint adqError,
            IntPtr userContext
        );


        /// <summary>
        /// Delegate for the function passed to adqReadContinuousStart
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="numberOfValuesAvailable">
        /// Number of values that can be read
        /// </param>
        /// <param name="values">
        /// Pointer to values buffer
        /// </param>
        /// <param name="adqError">
        /// Error code
        /// </param>
        /// <param name="userContext">
        /// Pointer to user defined context that was passed to function adqReadContinuousStart
        /// </param>
        /// <param name="reasonForCall">
        /// One of the constants for start, data or stop of the operation
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        public delegate uint adqReadContinuousCallback(
            uint boardIndex,
            uint moduleIndex,
            uint numberOfValuesAvailable,
            IntPtr values,
            uint adqError,
            IntPtr userContext,
            uint reasonForCall
        );


        /// <summary>
        /// Delegate for the function passed to adqWriteContinuousStart
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="numberOfValuesAcceptable">
        /// Number of values that can be written to the buffer
        /// </param>
        /// <param name="values">
        /// Pointer to values buffer
        /// </param>
        /// <param name="numberOfValuesWritten">
        /// Number of values written to the module
        /// </param>
        /// <param name="adqError">
        /// Error code
        /// </param>
        /// <param name="userContext">
        /// Pointer to user defined context that was passed to function adqWriteContinuousStart
        /// </param>
        /// <param name="reasonForCall">
        /// One of the constants for start, data or stop of the operation
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        public delegate uint adqWriteContinuousCallback(
            uint boardIndex,
            uint moduleIndex,
            uint numberOfValuesAcceptable,
            IntPtr values,
            out uint numberOfValuesWritten,
            uint adqError,
            IntPtr userContext,
            uint reasonForCall
        );


        /// <summary>
        /// Delegate for the function passed to adqAIReadContinuousStart
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="numberOfFramesAvailable">
        /// Number of frames that can be read
        /// </param>
        /// <param name="frames">
        /// Pointer to frames in data buffer
        /// </param>
        /// <param name="adqError">
        /// Error code
        /// </param>
        /// <param name="userContext">
        /// Pointer to user defined context that was passed to function adqAIReadContinuousStart
        /// </param>
        /// <param name="reasonForCall">
        /// One of the constants for start, data or stop of the operation
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        public delegate uint adqAIReadContinuousCallback(
            uint boardIndex,
            uint moduleIndex,
            uint numberOfFramesAvailable,
            IntPtr frames,
            uint adqError,
            IntPtr userContext,
            uint reasonForCall
        );


        #endregion


        #region Import Functions
  #pragma warning disable IDE1006 // Naming Styles

        //---------------------------------------------------------------------
        //  API routines
        //---------------------------------------------------------------------

        /// <summary>
        /// Opens the ALLDAQ driver system
        /// </summary>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqOpen();


        /// <summary>
        /// Opens the ALLDAQ driver system by given names or IP-addresses
        /// </summary>
        /// <param name="hostNameOrIPAddress">
        /// Array of host names or IP-addresses of remote computers
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall,
            CharSet = CharSet.Unicode)]
        public static extern uint adqOpenEx(string[] hostNameOrIPAddress);


        /// <summary>
        /// Close the ALLDAQ driver system
        /// </summary>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqClose();


        /// <summary>
        /// Get information about an internal error. For internal use.
        /// </summary>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqGetInternalError(
            ref uint component,
            ref uint file,
            ref uint lineNumber,
            ref uint aType,
            ref uint errorParameter1,
            ref uint errorParameter2
        );


        /// <summary>
        /// Get Info about the ALLDAQ driver system
        /// </summary>
        /// <param name="systemInfo">
        /// Struct of type ADQSystemInfo to receive the information
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqGetSystemInfo(ref ADQSystemInfo systemInfo);


        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqQuery(
            uint boardIndex,
            uint moduleIndex,
            uint Id,
            uint subId,
            uint bufferSize,
            [Out] byte[] buffer,
            out uint infoSize
        );


        /// <summary>
        /// Determine the number of boards recognised by the driver system
        /// </summary>
        /// <param name="numberOfBoards">
        /// Contains the number of boards
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqGetNumberOfBoards(
            out uint numberOfBoards
        );


        /// <summary>
        /// Get info about the given board
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="boardInfo">
        /// Struct that contains the information about the board
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqGetBoardInfo(
            uint boardIndex,
            ref ADQBoardInfo boardInfo
        );


        /// <summary>
        /// Get PCI location information information about a board if BusType is ADQ_BUS_TYPE_PCI
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="boardPCIInfo">
        /// Struct that contains the PCI location information
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqGetBoardPCIInfo(
            uint boardIndex,
            ref ADQBoardPCIInfo boardPCIInfo
        );

        /// <summary>
        /// Get the voltage level of all TTL ports on the given board
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="voltage">
        /// Contains the current voltage level as one of the defined constants
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqGetBoardTTLVoltage(
            uint boardIndex,
            out uint voltage
        );

        /// <summary>
        /// Set the voltage level of all TTL ports on the given board
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="voltage">
        /// The new voltage level as one of the defined constants
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqSetBoardTTLVoltage(
            uint boardIndex,
            uint voltage
        );

        /// <summary>
        /// Determine the number of modules of the given board
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="numberOfModules">
        /// Contains the number of modules
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqGetNumberOfModules(
            uint boardIndex,
            out uint numberOfModules
        );


        /// <summary>
        /// Retrieve the type of the module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module to be queried
        /// </param>
        /// <param name="moduleType">
        /// Contains the type of the module
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqGetModuleType(
            uint boardIndex,
            uint moduleIndex,
            out uint moduleType
        );


        /// <summary>
        /// Get info about the given module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module to be queried
        /// </param>
        /// <param name="generalInfo">
        /// Struct of type ADQGeneralModuleInfo to receive the information
        /// </param>
        /// <param name="moduleInfoSize">
        /// Size of moduleInfo in bytes
        /// </param>
        /// <param name="moduleInfo">
        /// Pointer to a structure that receives the information about the module
        /// This parameter can be null
        /// </param>
        /// <param name="infoSize">
        /// Contains the size in bytes of the information in moduleInfo
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqGetModuleInfo(
            uint boardIndex,
            uint moduleIndex,
            ref ADQGeneralModuleInfo generalInfo,
            uint moduleInfoSize,
            IntPtr moduleInfo,
            out uint infoSize
        );


        /// <summary>
        /// Read a single value from the given module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module to read from
        /// </param>
        /// <param name="value">
        /// Contains the value read from the module
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqReadSingle(
            uint boardIndex,
            uint moduleIndex,
            out uint value
        );


        /// <summary>
        /// Read a predefined number of values from a module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module to read from
        /// </param>
        /// <param name="numberOfValuesRequired">
        /// Number of values to read
        /// </param>
        /// <param name="values">
        /// Data buffer that receives the values
        /// </param>
        /// <param name="numberOfValuesReturned">
        /// Number of values successfully read
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqReadBuffer(
            uint boardIndex,
            uint moduleIndex,
            uint numberOfValuesRequired,
            uint[] values,
            out uint numberOfValuesReturned
        );


        /// <summary>
        /// Start a conitinuous data acquisition from a module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module to read from
        /// </param>
        /// <param name="notifyAfterValues">
        /// Number of values after the application will get notified
        /// </param>
        /// <param name="stopAfterValues">
        /// Number of values after which the acquisition will stop automatically
        /// </param>
        /// <param name="adqReadContinuousCallback">
        /// Callback function, called on start, new data available, and stop of the acquisition
        /// </param>
        /// <param name="readContinuousUserContext">
        /// Pointer to user defined context
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqReadContinuousStart(
            uint boardIndex,
            uint moduleIndex,
            uint notifyAfterValues,
            ulong stopAfterValues,
            adqReadContinuousCallback readContinuousCallback,
            IntPtr readContinuousUserContext
        );


        /// <summary>
        /// Read values during a conitinuous data acquisition from a module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module to read from
        /// </param>
        /// <param name="numberOfValuesRequired">
        /// Number of values required
        /// </param>
        /// <param name="values">
        /// Reference to the data buffer to read from
        /// </param>
        /// <param name="waitForData">
        /// true, if the function should wait till the number of required values are acquired
        /// false, if the function should return immediately
        /// </param>
        /// <param name="numberOfValuesReturned">
        /// Number of values available in values buffer
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqReadContinuous(
            uint boardIndex,
            uint moduleIndex,
            uint numberOfValuesRequired,
            uint[] values,
            bool waitForData,
            out uint numberOfValuesReturned
        );


        /// <summary>
        /// Read the status of a continuous data acquisition
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module to read the status
        /// </param>
        /// <param name="waitForIdle">
        /// true, if the function should wait till the module will be in idle state
        /// false, if the function should return immediately
        /// </param>
        /// <param name="numberOfValuesAvailable">
        /// Number of values that can be read
        /// </param>
        /// <param name="running">
        /// true, if the module is still acquiring data
        /// false, if the module is in idle state
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqReadContinuousStatus(
            uint boardIndex,
            uint moduleIndex,
            bool waitForIdle,
            out uint numberOfValuesAvailable,
            out bool running
        );


        /// <summary>
        /// Writes a single value to the given module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module to write to
        /// </param>
        /// <param name="numberOfValuesToWrite">
        /// Number of values to write to the data buffer
        /// </param>
        /// <param name="values">
        /// Reference to buffer that contains the values to be written
        /// </param>
        /// <param name="numberOfValuesWritten">
        /// Number of values written to the module
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqWriteSingle(
            uint boardIndex,
            uint moduleIndex,
            uint value
        );


        /// <summary>
        /// Write a predefined number of values to a module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module to write to
        /// </param>
        /// <param name="bufferSizeValues">
        /// Size of the data buffer in values
        /// </param>
        /// <param name="values">
        /// Reference to buffer that contains the values to be written
        /// </param>
        /// <param name="repeatBuffer">
        /// Number of repetitions to write the values
        /// </param>
        /// <param name="numberOfValuesWritten">
        /// Number of values written to the module
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqWriteBuffer(
            uint boardIndex,
            uint moduleIndex,
            uint bufferSizeValues,
            uint[] values,
            uint repeatBuffer,
            out uint numberOfValuesWritten
        );


        /// <summary>
        /// Start a continuous output operation
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module to write to
        /// </param>
        /// <param name="bufferSizeValues">
        /// Size of the data buffer in values
        /// </param>
        /// <param name="values">
        /// Reference to buffer that contains the values to be written
        /// </param>
        /// <param name="repeatBuffer">
        /// Number of repetitions to write the values
        /// </param>
        /// <param name="waitForWrite">
        /// true, if the function should wait till the values are written to the buffer of the module
        /// false, if the function should return immediately
        /// </param>
        /// <param name="notifyAfterValues">
        /// Number of values after the application will get notified
        /// </param>
        /// <param name="numberOfValuesWritten">
        /// Number of values written to the module
        /// </param>
        /// <param name="writeContinuousCallback">
        /// Delegate of the callback function, called on start, data written, and stop of the output operation
        /// </param>
        /// <param name="writeContinuousUserContext">
        /// Pointer to user defined context
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqWriteContinuousStart(
            uint boardIndex,
            uint moduleIndex,
            uint bufferSizeValues,
            uint[] values,
            uint repeatBuffer,
            bool waitForWrite,
            uint notifyAfterValues,
            out uint numberOfValuesWritten,
            adqWriteContinuousCallback writeContinuousCallback,
            IntPtr writeContinuousUserContext
        );


        /// <summary>
        /// Write values for a continuous output operation
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="bufferSizeValues">
        /// Size of the data buffer in values
        /// </param>
        /// <param name="values">
        /// Reference to the buffer that contains the values to be written
        /// </param>
        /// <param name="repeatBuffer">
        /// Number of repetitions to write the values
        /// </param>
        /// <param name="waitForWrite">
        /// true, if the function should wait till the values are written to the buffer of the module
        /// false, if the function should return immediately
        /// </param>
        /// <param name="numberOfValuesWritten">
        /// Number of values written to the module
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqWriteContinuous(
            uint boardIndex,
            uint moduleIndex,
            uint bufferSizeValues,
            uint[] values,
            uint repeatBuffer,
            bool waitForWrite,
            out uint numberOfValuesWritten
        );


        /// <summary>
        /// Read the status of a continuous output operation
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="waitForIdle">
        /// true, if the function should wait till the module is in idle state
        /// false, if the function should return immediately
        /// </param>
        /// <param name="numberOfValuesAcceptable">
        /// Number of values that can be be written
        /// </param>
        /// <param name="running">
        /// true, if the module is still writing data
        /// false, if the module is in idle state
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqWriteContinuousStatus(
            uint boardIndex,
            uint moduleIndex,
            bool waitForIdle,
            out uint numberOfValuesAcceptable,
            out bool running
        );


        /// <summary>
        /// Stop a clock controlled acquisition or output
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqStop(
            uint boardIndex,
            uint moduleIndex
        );


        /// <summary>
        /// Get the current configured direction of a DIO module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="direction">
        /// Contains the direction of the module (Input, Output)
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqDIOGetDirection(
            uint boardIndex,
            uint moduleIndex,
            out uint direction
        );


        /// <summary>
        /// Set the direction of a DIO module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="direction">
        /// Direction of the module (Input, Output)
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqDIOSetDirection(
            uint boardIndex,
            uint moduleIndex,
            uint direction
        );


        /// <summary>
        /// Get the current digital filter value in picoseconds
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="filterPicoseconds">
        /// Contains the current digital filter value
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqDIOGetFilterPicoseconds(uint boardIndex, uint moduleIndex, out ulong filterPicoseconds);


        /// <summary>
        /// Set the digital filter value in picoseconds
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="filterPicoseconds">
        /// The digital filter value to be set
        ///</param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqDIOSetFilterPicoseconds(uint boardIndex, uint moduleIndex, ulong filterPicoseconds);


        /// <summary>
        /// Reset all modules of the board
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqResetBoard(
            uint boardIndex
        );


        /// <summary>
        /// Reset the specified module of the board
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqResetModule(
            uint boardIndex,
            uint moduleIndex
        );


        /// <summary>
        /// Set the requested Frequency for a module for acquisition or output
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="nominalFrequency">
        /// Desired frequency in Hertz
        /// </param>
        /// <param name="actualFrequency">
        /// Actual frequency in Hertz
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        /// <remarks>
        /// For analog input modules use the function adqAISetChannelListAndFrequency if more than one channel is used
        /// </remarks>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqSetFrequency(
            uint boardIndex,
            uint moduleIndex,
            double nominalFrequency,
            out double actualFrequency
        );


        /// <summary>
        /// Get the maximum sample frequency in Hertz for a given number of channels for analog input modules,
        /// digital input or counter modules
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="numberOfChannels">
        /// The number of channels
        /// </param>
        /// <param name="maximumSampleFrequency">
        /// The maximum sample frequency per channel in Hertz
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqGetMaximumSampleFrequency(
            uint boardIndex,
            uint moduleIndex,
            uint numberOfChannels,
            out double maximumSampleFrequency
        );


        /// <summary>
        /// Get info about supported hardware triggers of a module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="hwTriggerInfo">
        /// Struct ADQHWTriggerInfo to receive the information
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqGetHWTriggerInfo(
            uint boardIndex,
            uint moduleIndex,
            ref ADQHWTriggerInfo hwTriggerInfo
        );


        /// <summary>
        /// Enable hardware trigger for a module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="hwTriggerEnable">
        /// Struct ADQHWTriggerEnable with enabled trigger settings
        /// </param>
        /// <param name="waitForTrigger">
        /// true, if the function should wait till the trigger event occurs
        /// false, if the function should return immediately
        /// </param>
        /// <param name="timeoutMilliseconds">
        /// Timeout value in milliseconds until the function returns.
        /// If the constant ADQ_HW_START_TRIGGER_TIMEOUT_INFINITE is passed the function waits infinitely.
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqEnableHWTrigger(
            uint boardIndex,
            uint moduleIndex,
            ref ADQHWTriggerEnable hwTriggerEnable,
            bool waitForTrigger,
            uint timeoutMilliseconds
        );


        /// <summary>
        /// Disable the hardware trigger for a module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqDisableHWTrigger(
            uint boardIndex,
            uint moduleIndex
        );


        /// <summary>
        /// Get info about supported IRQ-sources of a module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="irqInfo">
        /// Struct ADQIRQInfo to receive the information
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqGetIRQInfo(
            uint boardIndex,
            uint moduleIndex,
            ref ADQIRQInfo irqInfo
        );


        /// <summary>
        /// Intall a callback function and enable interrupt
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="irqEnable">
        /// Struct ADQIRQEnable with IRQ enabled settings
        /// </param>
        /// <param name="irqCallback">
        /// Delegate of the callback function, can be null
        /// </param>
        /// <param name="adqIRQStatusID">
        /// ID of the struct ADQIRQEnable
        /// </param>
        /// <param name="irqUserContext">
        /// User defined reference, will be passed to delegate adqIrqCallback, can be null
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqEnableIRQ(
            uint boardIndex,
            uint moduleIndex,
            ref ADQIRQEnable irqEnable,
            adqIrqCallback irqCallback,
            uint adqIRQStatusID,
            IntPtr userContext
        );


        /// <summary>
        /// Wait for an Interrupt to occur
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="irqStatus">
        /// Struct ADQIRQStatus that contains the status of the
        /// </param>
        /// <param name="timeoutMilliseconds">
        /// Timeout value in milliseconds until the function returns.
        /// If the constant ADQ_TIMEOUT_INFINITE is passed the function waits infinitely
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqWaitIRQ(
            uint boardIndex,
            uint moduleIndex,
            ref ADQIRQStatus irqStatus,
            uint timeoutMilliseconds
        );


        /// <summary>
        /// Get the number of Interrupts that occured since the IRQ functionality was enabled.
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="irqCount">
        /// Number of interrupts
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqGetIRQCount(
            uint boardIndex,
            uint moduleIndex,
            out uint irqCount
        );


        /// <summary>
        /// Disable Interrupts on the given module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqDisableIRQ(
            uint boardIndex,
            uint moduleIndex
        );


        /// <summary>
        /// Configures the counter 8254 and enables it at the same time
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="clockSource">
        /// Clock source of the counter
        /// </param>
        /// <param name="counterMode">
        /// One of the defined modes the counter supports
        /// </param>
        /// <param name="startValue">
        /// Start value of the counter
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqCNTRConfig(
            uint boardIndex,
            uint moduleIndex,
            uint clockSource,
            uint counterMode,
            uint startValue
        );


        /// <summary>
        /// Configures a counter module with the given settings
        /// This function does not enable the counter at the same time
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="counterConfigSize">
        /// Size of counterConfig in bytes
        /// </param>
        /// <param name="counterConfig">
        /// Pointer to the structure that contains the configuration
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqCNTRConfigEx(
            uint boardIndex,
            uint moduleIndex,
            uint counterConfigSize,
            IntPtr counterConfig
        );


        /// <summary>
        /// Enables the counter module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqCNTREnable(
            uint boardIndex,
            uint moduleIndex
        );


        /// <summary>
        /// Disables the counter module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqCNTRDisable(
            uint boardIndex,
            uint moduleIndex
        );


        /// <summary>
        /// Configures the channel and range of an analog input module
        /// Use the functions adqReadSingle, adqReadBuffer, adqReadContinuous to read values
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="channelIndex">
        /// Index of the channel
        /// </param>
        /// <param name="rangeIndex">
        /// Index of the range
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqAISetChannelAndRange(
            uint boardIndex,
            uint moduleIndex,
            uint channelIndex,
            uint rangeIndex
        );


        /// <summary>
        /// Get the range for analog input modules that is currently set on the hardware
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="channelIndex">
        /// Index of the channel
        /// </param>
        /// <param name="rangeIndex">
        /// Contains the index of the range currently set
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        /// <remarks>
        /// Use this function for modules that have a safety mechanism to switch to the highest range on excessive range overflow e.g. ADQ-412
        /// </remarks>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqAIGetChannelHardwareRange(
            uint boardIndex,
            uint moduleIndex,
            uint channelIndex,
            out uint rangeIndex
        );


        /// <summary>
        /// Set the channel list for an analog input module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="channels">
        /// Array of structures of type ADQAIChannel
        /// </param>
        /// <param name="numberOfChannels">
        /// Number of elements in parameter channels
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [Obsolete("adqAISetChannelList is deprecated, please use adqAISetChannelListAndFrequency instead.")]
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqAISetChannelList(
            uint boardIndex,
            uint moduleIndex,
            ADQAIChannel[] channels,
            uint numberOfChannels
        );


        /// <summary>
        /// Set the channel list and sample frequency for an analog input module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="channels">
        /// Array of structures of type ADQAIChannel
        /// </param>
        /// <param name="numberOfChannels">
        /// Number of elements in parameter channels
        /// </param>
        /// <param name="nominalFrequency">
        /// Desired sample frequency in Hertz
        /// </param>
        /// <param name="actualFrequency">
        /// Actual sample frequency in Hertz
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqAISetChannelListAndFrequency(
            uint boardIndex,
            uint moduleIndex,
            ADQAIChannel[] channels,
            uint numberOfChannels,
            double nominalFrequency,
            out double actualFrequency
        );


        /// <summary>
        /// Read a single frame from an analog input module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="frameLength">
        /// Number of elements in one frame
        /// </param>
        /// <param name="frame">
        /// Reference to the data buffer to read from
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqAIReadSingle(
            uint boardIndex,
            uint moduleIndex,
            uint frameLength,
            uint[] frame
        );


        /// <summary>
        /// Read a predefined number of frames from an analog input module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="frameLength">
        /// Number of elements in one frame
        /// </param>
        /// <param name="numberOfFramesRequired">
        /// Number of frames to read
        /// </param>
        /// <param name="frames">
        /// Reference to the data buffer that receives the values
        /// </param>
        /// <param name="numberOfFramesReturned">
        /// Number of frames successfully read
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqAIReadBuffer(
            uint boardIndex,
            uint moduleIndex,
            uint frameLength,
            uint numberOfFramesRequired,
            uint[] frames,
            out uint numberOfFramesReturned
        );


        /// <summary>
        /// Start a continuous acquisition of frames on an analog input module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="notifyAfterFrames">
        /// Number of frames after the application will get notified
        /// </param>
        /// <param name="stopAfterFrames">
        /// Number of frames after which the acquisition will stop automatically
        /// </param>
        /// <param name="aiReadContinuousCallback">
        /// Delegate of the callback function, called on start, data read, and stop of the acquisition
        /// </param>
        /// <param name="aIReadContinuousUserContext">
        /// Pointer to user defined context
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqAIReadContinuousStart(
            uint boardIndex,
            uint moduleIndex,
            uint notifyAfterFrames,
            ulong stopAfterFrames,
            adqAIReadContinuousCallback aiReadContinuousCallback,
            IntPtr readContinuousUserContext
        );


        /// <summary>
        /// Read frames during a conitinuous data acquisition from a module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="frameLength">
        /// Number of elements in one frame
        /// </param>
        /// <param name="numberOfFramesRequired">
        /// Number of frames to read
        /// </param>
        /// <param name="frames">
        /// Array of the data buffer to read from
        /// </param>
        /// <param name="waitForData">
        /// true, if the function should wait till the number of required values are acquired
        /// false, if the function should return immediately
        /// </param>
        /// <param name="numberOfFramesReturned">
        /// Number of frames successfully read
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqAIReadContinuous(
            uint boardIndex,
            uint moduleIndex,
            uint frameLength,
            uint numberOfFramesRequired,
            uint[] frames,
            bool waitForData,
            out uint numberOfFramesReturned
        );


        /// <summary>
        /// Read the status of a continuous data acquisition
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="waitForIdle">
        /// true, if the function should wait till the module will be in idle state
        /// false, if the function should return immediately
        /// </param>
        /// <param name="numberOfFramesAvailable">
        /// Number of frames that can be read
        /// </param>
        /// <param name="running">
        /// true, if the module is still acquiring data
        /// false, if the module is in idle state
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqAIReadContinuousStatus(
            uint boardIndex,
            uint moduleIndex,
            bool waitForIdle,
            out uint numberOfFramesAvailable,
            out bool running
        );


        /// <summary>
        /// Get the selected calibration of the given module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="calibrationID">
        /// Contains one of the defined calibration constants
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqAnalogGetSelectedCalibration(
            uint boardIndex,
            uint moduleIndex,
            out uint calibrationID
        );


        /// <summary>
        /// Set the calibration to be used for the given module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="calibrationID">
        /// One of the defined calibration constants
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqAnalogSelectCalibration(
            uint boardIndex,
            uint moduleIndex,
            uint calibrationID
        );


        /// <summary>
        /// Set the address of the I2C slave device for a write or read operation
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="slaveAddress">
        /// The address of the slave device
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqI2CSetSlaveAddress(
            uint boardIndex,
            uint moduleIndex,
            uint slaveAddress
        );


        /// <summary>
        /// Read a given number of values from an I2C module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module to read from
        /// </param>
        /// <param name="numberOfBytesRequired">
        /// Number of bytes to read
        /// </param>
        /// <param name="data">
        /// Data buffer that receives the values
        /// </param>
        /// <param name="numberOfBytesReturned">
        /// Number of bytes successfully read
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqI2CRead(
            uint boardIndex,
            uint moduleIndex,
            uint numberOfBytesRequired,
            byte[] data,
            out uint numberOfBytesReturned
        );


        /// <summary>
        /// Write a given number of bytes to an I2C module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module to write to
        /// </param>
        /// <param name="numberOfBytesToWrite">
        /// The number of bytes to write to the module
        /// </param>
        /// <param name="data">
        /// The data buffer that contains the values
        /// </param>
        /// <param name="numberOfBytesWritten">
        /// The number of bytes successfully written to the module
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqI2CWrite(
            uint boardIndex,
            uint moduleIndex,
            uint numberOfBytesToWrite,
            byte[] data,
            out uint numberOfBytesWritten
        );


        /// <summary>
        /// Performs a write operation of a byte array, followed by a read operation
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module to write to
        /// </param>
        /// <param name="numberOfBytesToWrite">
        /// The number of bytes to write to the module
        /// </param>
        /// <param name="writeData">
        /// The data buffer that contains the output values
        /// </param>
        /// <param name="numberOfBytesWritten">
        /// The number of bytes successfully written to the module
        /// </param>
        /// <param name="numberOfBytesRequired">
        /// Number of bytes to read
        /// </param>
        /// <param name="readData">
        /// Data buffer that receives the values
        /// </param>
        /// <param name="numberOfBytesReturned">
        /// Number of bytes successfully read
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqI2CWriteThenRead(
            uint boardIndex,
            uint moduleIndex,
            uint numberOfBytesToWrite,
            byte[] writeData,
            out uint numberOfBytesWritten,
            uint numberOfBytesRequired,
            byte[] readData,
            out uint numberOfBytesReturned
        );


        /// <summary>
        /// Get the maximum frequency that can be generated by the PWM module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="maximumFrequency">
        /// The maximum frequency the module can generate
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqPWMGetMaximumFrequency(
            uint boardIndex,
            uint moduleIndex,
            out double maximumFrequency
        );


        /// <summary>
        /// Configure a PWM module for the output operation
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="nominalFrequency">
        /// The desired output frequency of the signal in Hertz
        /// </param>
        /// <param name="nominalOnTime">
        /// The part of one period the output signal will have high level in seconds
        /// </param>
        /// <param name="invert">
        /// True, if the output signal will be inverted, false otherwise
        /// By default (false) the signal starts with the high level part
        /// </param>
        /// <param name="hardwareTrigger">
        /// True, if the output of the signal will wait for an hardware trigger
        /// </param>
        /// <param name="actualFrequency">
        /// The actual output frequency in Hertz
        /// </param>
        /// <param name="actualOnTime">
        /// The actual time of the high level part of one period in seconds
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqPWMConfig(
            uint boardIndex,
            uint moduleIndex,
            double nominalFrequency,
            double nominalOnTime,
            bool invert,
            bool hardwareTrigger,
            out double actualFrequency,
            out double actualOnTime
        );


        /// <summary>
        /// Enables the output of the PWM module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="timeoutMilliseconds">
        /// </param>
        /// The time-out value in Milliseconds to wait for the hardware trigger, if it is enabled.
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqPWMEnable(
            uint boardIndex,
            uint moduleIndex,
            uint timeoutMilliseconds
        );


        /// <summary>
        /// Disables the output of the PWM module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqPWMDisable(
            uint boardIndex,
            uint moduleIndex
        );


        /// <summary>
        /// Measures a digital signal period
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="frequency">
        /// The frequency of the signal period in Hertz
        /// </param>
        /// <param name="onTime">
        /// The high-time of the signal period in seconds
        /// </param>
        /// <param name="timeoutMilliseconds">
        /// Timeout value in milliseconds the function will wait for a rising edge of the digital signal.
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqFINRead(
            uint boardIndex,
            uint moduleIndex,
            out double frequency,
            out double onTime,
            uint timeoutMilliseconds
        );


        /// <summary>
        /// Configures the output of a sine wave signal for an I2S output module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="nominalOutputFrequency">
        /// The desired output frequency of the module in Hertz
        /// </param>
        /// <param name="signalLeftPrecisionBits">
        /// The precision in bits of the signal for the left channel
        /// </param>
        /// <param name="signalLeftTwosComplement">
        /// True, if the output signal values for the left channel should be generated as two's complement
        /// </param>
        /// <param name="signalLeftSignalFrequency">
        /// The signal frequency of the left channel in Hertz
        /// </param>
        /// <param name="signalLeftAmplitude">
        /// The signal amplitude of the left channel as value between 0 and 1
        /// </param>
        /// <param name="signalRightPrecisionBits">
        /// The precision in bits of the signal for the right channel
        /// </param>
        /// <param name="signalRightTwosComplement">
        /// True, if the output signal values for the right channel should be generated as two's complement
        /// </param>
        /// <param name="signalRightSignalFrequency">
        /// The signal frequency of the right channel in Hertz
        /// </param>
        /// <param name="signalRightAmplitude">
        /// The signal amplitude of the right channel as value between 0 and 1
        /// </param>
        /// <param name="actualOutputFrequency">
        /// Actual output frequency of the module in Hertz
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqI2SOSineConfig(
            uint boardIndex,
            uint moduleIndex,
            double nominalOutputFrequency,
            uint signalLeftPrecisionBits,
            bool signalLeftTwosComplement,
            double signalLeftSignalFrequency,
            double signalLeftAmplitude,
            uint signalRightPrecisionBits,
            bool signalRightTwosComplement,
            double signalRightSignalFrequency,
            double signalRightAmplitude,
            out double actualOutputFrequency
        );


        /// <summary>
        /// Starts the output operation on an I2S output module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqI2SOSineStart(
            uint boardIndex,
            uint moduleIndex
        );


        /// <summary>
        /// Modify the sine wave signal for an I2SO module during an output operation
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <param name="signalLeftSignalFrequency">
        /// The new signal frequency of the left channel in Hertz
        /// </param>
        /// <param name="signalLeftAmplitude">
        /// The new signal amplitude of the left channel as value between 0 and 1
        /// </param>
        /// <param name="signalRightSignalFrequency">
        /// The new signal frequency of the right channel in Hertz
        /// </param>
        /// <param name="signalRightAmplitude">
        /// The new signal amplitude of the right channel as value between 0 and 1
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqI2SOSineModify(
            uint boardIndex, 
            uint moduleIndex, 
            double signalLeftSignalFrequency, 
            double signalLeftAmplitude, 
            double signalRightSignalFrequency, 
            double signalRightAmplitude
        );


        /// <summary>
        /// Stops the output operation on an I2S module
        /// </summary>
        /// <param name="boardIndex">
        /// Index of the board
        /// </param>
        /// <param name="moduleIndex">
        /// Index of the module
        /// </param>
        /// <returns>
        /// Error code
        /// </returns>
        [DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)]
        public static extern uint adqI2SOSineStop(
            uint boardIndex,
            uint moduleIndex
        );


#pragma warning restore IDE1006 // Naming Styles
        #endregion
    }

    public sealed class AdqConstants
    {
        /// <summary>
        ///  Used for any parameter in adqQuery which is not required in a particular context
        /// </summary>
        public const uint ADQ_QUERY_UNUSED_PARAMETER = 0xFFFFFFFFU;

        public const uint ADQ_QUERY_SYSTEM_NUMBER_OF_BOARDS = 0x00000001;
        public const uint ADQ_QUERY_SYSTEM_BOARD_HANDLE = 0x00000002;

        /// <summary>
        /// Board constants for the parameter 'ID' in adqQuery.
        /// Used with the board handle as the first parameter.
        /// The second parameter 'IndexModule' should always be ADQ_QUERY_UNUSED_PARAMETER.
        /// </summary>
        public const uint ADO_QUERY_BOARD_NUMBER_OF_MODULES = 0x00010001;
        public const uint ADQ_QUERY_BOARD_INFO = 0x00010002;

        /// <summary>
        /// Module constants for the parameter 'ID' in adqQuery.
        /// Used with the board handle as the first parameter.
        /// The second parameter 'IndexModule' is the 0 based module index.
        /// </summary>
        public const uint ADO_QUERY_MODULE_TYPE = 0x00020001;
        public const uint ADQ_QUERY_MODULE_INFO = 0x00020002;

        /// <summary>
        /// Module types
        /// </summary>
        public const uint ADQ_MODULE_TYPE_DI = 0x00030000;      // Digital input.
        public const uint ADQ_MODULE_TYPE_DO = 0x00030001;      // Digital output.
        public const uint ADQ_MODULE_TYPE_DIO = 0x00030002;     // Digital input / output
        public const uint ADQ_MODULE_TYPE_CNTR = 0x00030003;    // Counter
        public const uint ADQ_MODULE_TYPE_EXTINT = 0x00030004;  // External interrupt
        public const uint ADQ_MODULE_TYPE_AI = 0x00030005;      // Analog input
        public const uint ADQ_MODULE_TYPE_AO = 0x00030006;      // Analog output
        public const uint ADQ_MODULE_TYPE_I2C = 0x00030007;     // I2C
        public const uint ADQ_MODULE_TYPE_PWM = 0x00030008;     // PWM
        public const uint ADQ_MODULE_TYPE_FIN = 0x00030009;     // FIN
        public const uint ADQ_MODULE_TYPE_I2SO = 0x0003000A;    // I2S output

        /// <summary>
        /// Direction of DIO modules
        /// </summary>
        public const uint ADQ_DIO_INPUT = 0x00040000;
        public const uint ADQ_DIO_OUTPUT = 0x00040001;

        /// <summary>
        /// Clock source used by function adqCNTRConfig and structure ADQCNTRModuleInfo
        /// </summary>
        public const uint ADQ_CNTR_CLOCK_SOURCE_DEFAULT = 0x00050000;
        public const uint ADQ_CNTR_CLOCK_SOURCE_EXTERNAL = 0x00050001;
        public const uint ADQ_CNTR_CLOCK_SOURCE_INTERNAL_10_MHZ = 0x00050002;
        public const uint ADQ_CNTR_CLOCK_SOURCE_INTERNAL_66_MHZ = 0x00050004;

        /// <summary>
        /// Counter 8254 modes
        /// </summary>
        public const uint ADQ_CNTR_8254_MODE_0 = 0x00060000;  // Mode 0: Change state at zero
        public const uint ADQ_CNTR_8254_MODE_1 = 0x00060001;  // Mode 1: Retriggerable "One Shot"
        public const uint ADQ_CNTR_8254_MODE_2 = 0x00060002;  // Mode 2: Asymmetric divider
        public const uint ADQ_CNTR_8254_MODE_3 = 0x00060003;  // Mode 3: Symmetric divider
        public const uint ADQ_CNTR_8254_MODE_4 = 0x00060004;  // Mode 4: Counter start by software trigger
        public const uint ADQ_CNTR_8254_MODE_5 = 0x00060005;  // Mode 5: Counter start by hardware trigger

        /// <summary>
        /// System constants for the parameter CalibbrationID
        /// returned by adqAnalogGetSelectedCalibration and passed to adqAnalogSelectCalibration
        /// </summary>
        public const uint ADQ_CALIBRATION_FACTORY = 0x00070000;
        public const uint ADQ_CALIBRATION_USER_1  = 0x00070001;

        /// <summary>
        /// System constants for the variable TTLVoltageLevel returned in the structure ADQBoardInfo
        /// One of these constants should also passed to the routine adqSetBoardTTLVoltage in the parameter 'Voltage'
        /// </summary>
        public const uint ADQ_TTL_VOLTAGE_LEVEL_3_3_VOLTS = 0x00000001;
        public const uint ADQ_TTL_VOLTAGE_LEVEL_5_0_VOLTS = 0x00000002;

        /// <summary>
        /// System constants for the variable ClockType in the structure ADQGeneralModuleInfo
        /// </summary>
        public const uint ADQ_CLOCK_TYPE_NONE = 0x00080000;
        public const uint ADQ_CLOCK_TYPE_SOFTWARE = 0x00080001;
        public const uint ADQ_CLOCK_TYPE_HARDWARE = 0x00080002;

        /// <summary>
        /// System constants for the variable CounterType in the structure ADQCNTRModuleInfo
        /// </summary>
        public const uint ADQ_CNTR_TYPE_8254 = 0x00090000;              // Counter type 8254
        public const uint ADQ_CNTR_TYPE_COUNT_DOWN_32_BIT = 0x00090001; // Counter type Count Down 32-Bit

        /// <summary>
        /// System constants for the variable PhysicalUnit in the structure ADQAIRange
        /// </summary>
        public const uint ADQ_PHYSICAL_UNIT_VOLT = 0x000A0000;
        public const uint ADQ_PHYSICAL_UNIT_AMPERE = 0x000A0001;
        public const uint ADQ_PHYSICAL_UNIT_OHM = 0x000A0002;


        /// <summary>
        /// System constants for the variable BusType in the structure ADQBoardInfo
        /// </summary>
        public const uint ADQ_BUS_TYPE_PCI = 0x000B0000;
        public const uint ADQ_BUS_TYPE_USB = 0x000B0001;

        /// <summary>
        /// System constants for parameter ReasonForCall in callback functions
        /// </summary>
        public const uint ADQ_CALLED_FOR_START = 0x00000001;
        public const uint ADQ_CALLED_FOR_DATA = 0x00000002;
        public const uint ADQ_CALLED_FOR_STOP = 0x00000004;

        /// <summary>
        /// System constants for using with adqWaitIRQ
        /// </summary>
        public const uint ADQ_TIMEOUT_INFINITE = 0U;

        /// <summary>
        /// System constants for using with continuous functions
        /// </summary>
        public const uint ADQ_NUMBER_UNKNOWN = 0U;

        /// <summary>
        /// System constants for using with adqEnableHWTrigger
        /// </summary>
        public const uint ADQ_HW_START_TRIGGER_TIMEOUT_INFINITE = 0xFFFFFFFFU;

        /// <summary>
        ///  System constants for using with adqPWMEnable
        /// </summary>
        public const uint ADQ_PWM_TIMEOUT_INFINITE = 0xFFFFFFFFU;

        /// <summary>
        ///  System constants for using with adqFINRead
        /// </summary>
        public const uint ADQ_FIN_TIMEOUT_INFINITE = 0xFFFFFFFFU;
    }


    public sealed class AdqErrors
    {
        /// <summary>
        /// Bits 28 to 31 of the error code contain the error type.
        /// </summary>
        public const uint ADQ_ERROR_TYPE_MASK              = 0xF0000000U;
        public const uint ADQ_ERROR_TYPE_FATAL             = 0x80000000U;
        public const uint ADQ_ERROR_TYPE_ERROR             = 0x40000000;
        public const uint ADQ_ERROR_TYPE_WARNING           = 0x20000000;
        public const uint ADQ_ERROR_TYPE_HINT              = 0x10000000;
        public const uint ADQ_ERROR_TYPE_SUCCESS           = 0x00000000;

        /// <summary>
        /// If Bit 25 is set, then internal information is available via adqGetInternalError
        /// and the returned information should be sent to ALLDAQ
        /// </summary>
        public const uint ADQ_ERROR_INTERNAL_INFORMATION   = 0x02000000;

        /// <summary>
        /// If Bit 24 is set, then more information is available via adqGetExtendedError
        /// </summary>
        public const uint ADQ_ERROR_MORE_INFORMATION       = 0x01000000;

        /// <summary>
        /// Bits 16 to 19 of the error code contain the index of the parameter which has caused the error, 1 based.
        /// </summary>
        public const uint ADQ_ERROR_PARAMETER_MASK         = 0x000F0000;
        public const uint ADQ_ERROR_PARAMETER_NONE         = 0x00000000;
        public const uint ADQ_ERROR_PARAMETER_1            = 0x00010000;
        public const uint ADQ_ERROR_PARAMETER_2            = 0x00020000;
        public const uint ADQ_ERROR_PARAMETER_3            = 0x00030000;
        public const uint ADQ_ERROR_PARAMETER_4            = 0x00040000;
        public const uint ADQ_ERROR_PARAMETER_5            = 0x00050000;
        public const uint ADQ_ERROR_PARAMETER_6            = 0x00060000;
        public const uint ADQ_ERROR_PARAMETER_7            = 0x00070000;
        public const uint ADQ_ERROR_PARAMETER_8            = 0x00080000;
        public const uint ADQ_ERROR_PARAMETER_9            = 0x00090000;
        public const uint ADQ_ERROR_PARAMETER_10           = 0x000A0000;
        public const uint ADQ_ERROR_PARAMETER_11           = 0x000B0000;

        /// <summary>
        /// The error codes themselves are in the lower 16 bits.
        /// </summary>
        public const uint ADQ_ERROR_NONE                        = 0x00000000;
        public const uint ADQ_ERROR_INTERNAL                    = 0x00000001;
        public const uint ADQ_ERROR_INVALID_PARAMETER           = 0x00000002;
        public const uint ADQ_ERROR_NO_DEVICES                  = 0x00000003;
        public const uint ADQ_ERROR_ALREADY_OPEN                = 0x00000004;
        public const uint ADQ_ERROR_NOT_OPEN                    = 0x00000005;
        public const uint ADQ_ERROR_INVALID_BOARD_INDEX         = 0x00000006;
        public const uint ADQ_ERROR_CANNOT_OPEN_BOARD           = 0x00000007;
        public const uint ADQ_ERROR_INVALID_MODULE_INDEX        = 0x00000008;
        public const uint ADQ_ERROR_UNKNOWN_SYSTEM_QUERY        = 0x00000009;
        public const uint ADQ_ERROR_BUFFER_TOO_SMALL            = 0x0000000A;
        public const uint ADQ_ERROR_UNSUPPORTED                 = 0x0000000B;
        public const uint ADQ_ERROR_IRQ_ALREADY_ENABLED         = 0x0000000C;
        public const uint ADQ_ERROR_IRQ_NOT_ENABLED             = 0x0000000D;
        public const uint ADQ_ERROR_TIMEOUT                     = 0x0000000E;
        public const uint ADQ_ERROR_MODULE_BUSY                 = 0x0000000F;
        public const uint ADQ_ERROR_KERNEL_BUFFER_OVERFLOW      = 0x00000010;
        public const uint ADQ_ERROR_AI_INVALID_CHANNEL_INDEX    = 0x00000011;
        public const uint ADQ_ERROR_AI_INVALID_RANGE_INDEX      = 0x00000012;
        public const uint ADQ_ERROR_AI_INVALID_FRAME_LENGTH     = 0x00000013;
        public const uint ADQ_ERROR_WRITE_OP_NOT_RUNNING        = 0x00000014;
        public const uint ADQ_ERROR_KERNEL_BUFFER_UNDERFLOW     = 0x00000015;
        public const uint ADQ_ERROR_STRUCTURE_SIZE_OR_ID_FALSE  = 0x00000016;
        public const uint ADQ_ERROR_STRUCTURE_ID_FALSE          = 0x00000017;
        public const uint ADQ_ERROR_NETWORK_COMMUNICATION       = 0x00000018;
        public const uint ADQ_ERROR_NETWORK_TIMEOUT             = 0x00000019;
        public const uint ADQ_ERROR_FREQUENCY_INVALID           = 0x0000001A;
    }
}