﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the AIReadContinuous project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using AIReadContinuous.ALLDAQ;
using AIReadContinuous.Utility;
using ALLDAQ.Boards;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace AIReadContinuous
{
    public partial class FormMain : Form
    {
        private readonly Color _gridLineColor = Color.DarkSlateGray;
        private readonly Color _labelColor = Color.LightSkyBlue;
        private const int _rangeSubItemIndex = 1;
        private const int _idxChartTypeLine = 0;
        private const int _idxChartTypeLineAndMarkers = 1;
        private const int _idxXAxisLabelsTime = 0;
        private const int _bit0Mask = 1;
        private const int _bit1Mask = 2;
        private const string _fmtStringPhysValues = "{0:0.000000}";
        private const string _txtInfinite = "Infinite";
        private const string _txtStartingAcquisition = "Starting acquisition...";
        private const string _txtWaitingForStartTrigger = "Waiting for start trigger...";
        private const string _txtStartTriggerEventOccured = "Start trigger event occurred.";
        private const string _txtAcquisitionStarted = "Acquisition started.";
        private const string _txtErrorOccurred = "Error occurred.";
        private const string _txtReady = "Ready";

        private string _selectedUnitSymbol;
        private ModuleLocator _moduleLocator;
        private readonly List<AnalogInputModule> _aiModules = new List<AnalogInputModule>();
        private AnalogInputModule _selectedAiModule;
        private int _selectedModuleItemIndex = -1;
        private bool _isAppClosing;
        private ListViewHitTestInfo _hitinfoChannelList;
        private readonly ComboBox _coBoxRanges = new ComboBox();
        private string[] _xAxisLabels;
        private string _imgInitialDirectory = string.Empty;

        private delegate void DisplayChartValuesDelegate();
        private DisplayChartValuesDelegate _updateChartDelegate;

        private delegate void UpdatePropertyDelegate(AdqCommon.PropertyType changedProperty);
        private UpdatePropertyDelegate _updatePropertyDelegate;

        public FormMain()
        {
            InitializeComponent();
            AdqCommon.SetDoubleBuffered(lvChannelList);
            InitRangeComboBox();
            coBoxChartType.SelectedIndex = _idxChartTypeLine;
            coBoxXAxisLabels.SelectedIndex = _idxXAxisLabelsTime;
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            DisableComboBoxMouseWheel();
            EnableUI(false);
            InitChart();
            _updateChartDelegate = new DisplayChartValuesDelegate(UpdateChartGraphic);
            _updatePropertyDelegate = new UpdatePropertyDelegate(UpdateProperty);
            try
            {
                _moduleLocator = new ModuleLocator(_aiModules, OnModuleInfoTextChanged);
                _moduleLocator.FindModules();

                FillModuleCombobox();
                SelectFirstComboboxItem(coBoxModules);
                AddInfoTextLine("Number of analog input modules found: " + _aiModules.Count.ToString());
            }
            catch (DllNotFoundException ex)
            {
                AddInfoTextLine("The device driver is not properly installed." + Environment.NewLine + ex.Message);
                tabControlMain.SelectedTab = tpInfo;
            }
        }

        private void FillModuleCombobox()
        {
            coBoxModules.Items.Clear();
            foreach (AnalogInputModule aiModule in _aiModules)
            {
                coBoxModules.Items.Add(aiModule.ModuleName);
            }
        }

        private void DisableComboBoxMouseWheel()
        {
            coBoxModules.MouseWheel += ComboBox_MouseWheel;
        }

        private void ComboBox_MouseWheel(object sender, MouseEventArgs e)
        {
            ((HandledMouseEventArgs)e).Handled = true;
        }

        private void InitChart()
        {
            chart1.BackColor = Color.Black;
            ChartArea ca = chart1.ChartAreas[0];
            ca.BackColor = Color.Black;
        }

        private void UpdateChart(AnalogInputModule aiModule)
        {
            chart1.GetToolTipText -= Chart_GetToolTipText;
            chart1.Series.Clear();
            chart1.Legends.Clear();

            chart1.ChartAreas.Clear();
            var ca = new ChartArea();
            chart1.ChartAreas.Add(ca);
            InitChartTitles(ca, aiModule.ModuleName, aiModule.ChannelSettings[0].Range.UnitName);
            InitChartArea(ca, aiModule.MinResolution);
            InitLegend(chart1, ca, chkBoxShowLegend.Checked);
            foreach (ChannelSetting chanSetting in aiModule.ChannelSettings)
            {
                ChartValueType cvt = GetChartXValueType(coBoxXAxisLabels.SelectedIndex);
                InitSeries(chanSetting.ChannelIndex, chart1, coBoxChartType.SelectedIndex, cvt);
            }
        }

        private void InitChartTitles(ChartArea chartArea, string chartTitle, string yAxisTitle)
        {
            chart1.Titles.SuspendUpdates();
            chart1.Titles.Clear();

            var title = new Title();
            chart1.Titles.Add(title);
            InitChartTitle(title, chartArea, chartTitle);

            title = new Title();
            chart1.Titles.Add(title);
            InitYAxisTitle(title, chartArea, yAxisTitle);

            title = new Title();
            chart1.Titles.Add(title);
            string titleText = GetXAxisTitle();
            InitXAxisTitle(title, chartArea, titleText);

            chart1.Titles.ResumeUpdates();
        }

        private void InitChartTitle(Title title, ChartArea chartArea, string titleText)
        {
            title.DockedToChartArea = chartArea.Name;
            title.Text = titleText;
            title.Alignment = ContentAlignment.TopCenter;
            title.Docking = Docking.Top;
            title.IsDockedInsideChartArea = false;
            title.ForeColor = _labelColor;
            title.Position.Auto = false;
            title.Position.X = 3;
            title.Position.Y = 0;
            title.Position.Width = 94;
            title.Position.Height = 5;
        }

        private void InitYAxisTitle(Title title, ChartArea chartArea, string titleText)
        {
            title.DockedToChartArea = chartArea.Name;
            title.Text = titleText;
            title.Position.Auto = false;
            title.Position.X = 0.5f;
            title.Position.Y = 0;
            title.Position.Width = 5;
            title.Position.Height = 95;
            title.Alignment = ContentAlignment.TopCenter;
            title.Docking = Docking.Left;
            title.IsDockedInsideChartArea = false;
            title.TextOrientation = TextOrientation.Rotated270;
            title.ForeColor = _labelColor;
        }

        private void InitXAxisTitle(Title title, ChartArea chartArea, string titleText)
        {
            title.DockedToChartArea = chartArea.Name;
            title.Text = titleText;
            title.Position.Auto = false;
            title.Position.X = 3;
            title.Position.Y = 95;
            title.Position.Width = 94;
            title.Position.Height = 5;
            title.Alignment = ContentAlignment.BottomCenter;
            title.Docking = Docking.Bottom;
            title.IsDockedInsideChartArea = false;
            title.ForeColor = _labelColor;
        }

        private void InitChartArea(ChartArea chartArea, double resolutionYAxis)
        {
            chartArea.BackColor = Color.Black;

            chartArea.AxisX.MajorGrid.LineColor = _gridLineColor;
            chartArea.AxisX.MajorTickMark.LineColor = _gridLineColor;
            chartArea.AxisX.MinorGrid.LineColor = _gridLineColor;
            chartArea.AxisX.MinorTickMark.LineColor = _gridLineColor;
            chartArea.AxisX.LineColor = _gridLineColor;
            chartArea.AxisX.LineWidth = 1;
            chartArea.AxisX.LabelStyle.ForeColor = _labelColor;
            chartArea.AxisX.ScrollBar.ButtonColor = Color.DarkGray;
            chartArea.AxisX.ScrollBar.BackColor = Color.Gray;
            chartArea.CursorX.IsUserSelectionEnabled = true;

            chartArea.AxisY.MajorGrid.LineColor = _gridLineColor;
            chartArea.AxisY.MajorTickMark.LineColor = _gridLineColor;
            chartArea.AxisY.MinorGrid.LineColor = _gridLineColor;
            chartArea.AxisY.MinorTickMark.LineColor = _gridLineColor;
            chartArea.AxisY.LineColor = _gridLineColor;
            chartArea.AxisY.LineWidth = 1;
            chartArea.AxisY.IsLabelAutoFit = false;
            chartArea.AxisY.LabelStyle.ForeColor = _labelColor;
            chartArea.AxisY.LabelStyle.Format = _fmtStringPhysValues;
            chartArea.AxisY.ScrollBar.ButtonColor = Color.DarkGray;
            chartArea.AxisY.ScrollBar.BackColor = Color.Gray;

            double scrollSize = resolutionYAxis / 4.0;
            chartArea.AxisY.ScaleView.SmallScrollMinSize = scrollSize;
            chartArea.AxisY.ScaleView.SmallScrollSize = scrollSize;
            chartArea.CursorY.Interval = scrollSize;
            chartArea.CursorY.IsUserSelectionEnabled = true;
        }

        private void InitSeries(uint channelIndex, Chart chart, int chartTypeIndex, ChartValueType chartXValueType)
        {
            Series series = new Series
            {
                Name = "Channel " + channelIndex.ToString()
            };

            if (chartTypeIndex == _idxChartTypeLine)
            {
                series.ChartType = SeriesChartType.Line;
            }
            else if (chartTypeIndex == _idxChartTypeLineAndMarkers)
            {
                series.ChartType = SeriesChartType.Line;
                series.MarkerStyle = AdqCommon.MarkerStyles[channelIndex % AdqCommon.MarkerStyles.Length];
            }
            else
            {
                series.ChartType = SeriesChartType.Point;
            }
            series.Color = AdqCommon.ChannelColors[channelIndex % AdqCommon.ChannelColors.Length];
            series.YValueType = ChartValueType.Double;
            series.XValueType = chartXValueType;
            series.IsXValueIndexed = false;
            chart.Series.Add(series);
        }

        private void InitLegend(Chart chart, ChartArea chartArea, bool isEnabled)
        {
            var legend = new Legend("Legend1")
            {
                ForeColor = Color.Gray, 
                BackColor = Color.Transparent,
                DockedToChartArea = chartArea.Name,
                Enabled = isEnabled
            };
            chart.Legends.Add(legend);
        }

        private ChartValueType GetChartXValueType(int selectedIndex)
        {
            return selectedIndex == _idxXAxisLabelsTime ? ChartValueType.String : ChartValueType.UInt32;
        }

        private string GetXAxisTitle()
        {
            return (coBoxXAxisLabels.SelectedIndex == _idxXAxisLabelsTime) ? "Time" : "Value index";
        }

        private void AddInfoTextLine(string text)
        {
            textBoxInfo.AppendText(string.Format("[{0}] {1}" + Environment.NewLine,
                DateTime.Now.ToString("HH:mm:ss.fffffff"), text));
        }

        private void OnModuleInfoTextChanged(object sender, InfoTextChangedEventArgs e)
        {
            if (_isAppClosing) return;

            AddInfoTextLine(e.InfoText);
        }

        private void OnModulePropertyChanged(object sender, ModulePropertyChangedEventArgs e)
        {
            if (_isAppClosing) return;

            if (e.ChangedProperty == AdqCommon.PropertyType.DisplayBuffer)
            {
                BeginInvoke(_updateChartDelegate);
            }
            else
            {
                BeginInvoke(_updatePropertyDelegate, new object[] { e.ChangedProperty });
            }
        }

        private void UpdateProperty(AdqCommon.PropertyType changedProperty)
        {
            if (changedProperty == AdqCommon.PropertyType.UpdateNumberOfFrames)
            {
                UpdateNumberOfFrames();
            }
            else if (changedProperty == AdqCommon.PropertyType.ActualSampleFrequency)
            {
                UpdateActualSampleFrequency();
            }
            else if (changedProperty == AdqCommon.PropertyType.DisplayBufferLength)
            {
                UpdateNumberOfFramesToDisplay();
            }
            else if (changedProperty == AdqCommon.PropertyType.AcquisitionStarting)
            {
                UpdateAcquisitionStarting();
            }
            else if (changedProperty == AdqCommon.PropertyType.AcquisitionStarted)
            {
                UpdateAcquisitionStarted();
            }
            else if (changedProperty == AdqCommon.PropertyType.AcquisitionEnded)
            {
                AcquisitionEnded();
            }
            else if (changedProperty == AdqCommon.PropertyType.WaitingForStartTrigger)
            {
                UpdateWaitingForStartTrigger();
            }
            else if (changedProperty == AdqCommon.PropertyType.StartTriggerEventOccured)
            {
                UpdateStartTriggerEventOccured();
            }
            else if (changedProperty == AdqCommon.PropertyType.AcquisitionErrorOccurred)
            {
                UpdateAcquisitionErrorOccured();
            }
        }

        private void UpdateNumberOfFrames()
        {
            textBoxFramesAcquired.Text = _selectedAiModule.TotalFramesAcquired.ToString("N0");
            textBoxFramesDisplayed.Text = _selectedAiModule.TotalFramesDisplayed.ToString("N0");
            textBoxFramesSkipped.Text = _selectedAiModule.TotalFramesSkipped.ToString("N0");
            progressUpdateTimer.Start();
        }

        private void UpdateActualSampleFrequency()
        {
            textBoxActualFrequency.Text = AdqCommon.FormatFrequency(_selectedAiModule.ActualSampleFrequency);
        }

        private void UpdateNumberOfFramesToDisplay()
        {
            textBoxFramesToDisplay.Text = _selectedAiModule.DisplayBufferNumberOfFrames.ToString();
        }

        private void UpdateAcquisitionStarting()
        {
            AddInfoTextLine(_txtStartingAcquisition);
            lblAcquisitionStatus.BackColor = Color.LimeGreen;
            lblAcquisitionStatus.Text = _txtStartingAcquisition;
        }

        private void UpdateAcquisitionStarted()
        {
            AddInfoTextLine(_txtAcquisitionStarted);
            lblAcquisitionStatus.BackColor = Color.Lime;
            lblAcquisitionStatus.Text = _txtAcquisitionStarted;
        }

        private void UpdateWaitingForStartTrigger()
        {
            AddInfoTextLine(_txtWaitingForStartTrigger);
            lblAcquisitionStatus.BackColor = Color.Gold;
            lblAcquisitionStatus.Text = _txtWaitingForStartTrigger;
        }

        private void UpdateStartTriggerEventOccured()
        {
            AddInfoTextLine(_txtStartTriggerEventOccured + " " + _txtAcquisitionStarted);
            lblAcquisitionStatus.BackColor = Color.LightGreen;
            lblAcquisitionStatus.Text = _txtStartTriggerEventOccured + Environment.NewLine 
                + _txtAcquisitionStarted;
        }

        private void UpdateAcquisitionErrorOccured()
        {
            lblAcquisitionStatus.BackColor = Color.OrangeRed;
            lblAcquisitionStatus.Text = _txtErrorOccurred;
        }

        private void ProgressUpdateTimer_Tick(object sender, EventArgs e)
        {
            if (!_selectedAiModule.RequestUpdateNumberOfFrames)
            {
                progressUpdateTimer.Stop();
                _selectedAiModule.RequestUpdateNumberOfFrames = true;
            }
        }

        private void GenerateAxisLabels(uint numberOfFrames)
        {
            if (_xAxisLabels == null || _xAxisLabels.Length != numberOfFrames)
            {
                _xAxisLabels = new string[numberOfFrames];
            }

            double xValue = 0.0;
            double xValueDelta = GetXValueDelta();
            if (coBoxXAxisLabels.SelectedIndex == _idxXAxisLabelsTime)
            {
                for (int i = 0; i < numberOfFrames; i++)
                {
                    _xAxisLabels[i] = AdqCommon.FormatSampleTime(xValue);
                    xValue += xValueDelta;
                }
            }
            else
            {
                for (int i = 0; i < numberOfFrames; i++)
                {
                    _xAxisLabels[i] = ((int)xValue).ToString();
                    xValue += xValueDelta;
                }
            }
        }

        private double GetXValueDelta()
        {
            return (coBoxXAxisLabels.SelectedIndex == _idxXAxisLabelsTime) ? _selectedAiModule.SampleTime : 1.0;
        }

        private void UpdateChartGraphic()
        {
            chart1.SuspendLayout();
            for (int i = 0; i < chart1.Series.Count; i++)
            {
                RemovePoints(chart1.Series[i]);
            }
            chart1.ResumeLayout();
            DisplayChartValues();
        }

        private void RemovePoints(Series series)
        {
            while (series.Points.Count > 0)
            {
                series.Points.RemoveAt(series.Points.Count - 1);
            }
            series.Points.Clear();
        }

        private void DisplayChartValues()
        {
            uint frameLength = _selectedAiModule.FrameLength;
            uint bufferLength = _selectedAiModule.DisplayBufferLength;
            double[] displayFramesBuffer = _selectedAiModule.DisplayBuffer;
            chart1.SuspendLayout();

            for (uint idx = 0; idx < frameLength; idx++)
            {
                Series series = chart1.Series[(int)idx];
                uint valIdx = idx;
                int pointIdx = 0;
                while (valIdx < bufferLength)
                {
                    double physicalValue = displayFramesBuffer[valIdx];
                    var dp = new DataPoint(pointIdx, physicalValue)
                    {
                        AxisLabel = _xAxisLabels[pointIdx]
                    };

                    series.Points.Add(dp);
                    valIdx += frameLength;
                    pointIdx++;
                }
            }
            chart1.ChartAreas[0].RecalculateAxesScale();
            chart1.ResumeLayout();
            graphicUpdateTimer.Start();
        }

        private void GraphicUpdateTimer_Tick(object sender, EventArgs e)
        {
            graphicUpdateTimer.Stop();
            _selectedAiModule.RequestFramesToDisplay = true;
        }

        private void SelectFirstComboboxItem(ComboBox coBox)
        {
            if (coBox.Items.Count > 0)
            {
                coBox.SelectedIndex = 0;
            }
        }

        private void CoBoxModules_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (coBoxModules.SelectedIndex == _selectedModuleItemIndex) return;

            _selectedModuleItemIndex = coBoxModules.SelectedIndex;
            ClearDisplayedValues();
            chart1.Titles.Clear();

            EnableUI(coBoxModules.SelectedIndex >= 0);
            btnStartStop.Enabled = coBoxModules.SelectedIndex >= 0;

            if (_selectedAiModule != null)
            {
                _selectedAiModule.ModuleInfoTextChanged -= OnModuleInfoTextChanged;
                _selectedAiModule.ModulePropertyChanged -= OnModulePropertyChanged;
                _selectedAiModule.ClearModuleResources();
            }

            if (coBoxModules.SelectedIndex < 0)
            {
                _selectedAiModule = null;
                textBoxSampleFrequency.Text = string.Empty;
                textBoxDisplaySeconds.Text = string.Empty;
                _selectedUnitSymbol = string.Empty;

            }
            else
            {
                _selectedAiModule = _aiModules[coBoxModules.SelectedIndex];
                _selectedAiModule.ModuleInfoTextChanged += OnModuleInfoTextChanged;
                _selectedAiModule.ModulePropertyChanged += OnModulePropertyChanged;
                SetStopAfterFramesText(_selectedAiModule.StopAfterFrames);
                textBoxSampleFrequency.Text = string.Format("{0:0.###}", _selectedAiModule.DesiredSampleFrequency);
                textBoxDisplaySeconds.Text = string.Format("{0:0.###}", _selectedAiModule.DisplaySeconds);
                _selectedUnitSymbol = _selectedAiModule.DefaultRange.UnitSymbol;
            }

            textBoxActualFrequency.Text = string.Empty;
            textBoxFramesToDisplay.Text = string.Empty;

            FillChannelsAndRanges(_selectedAiModule);
            EnableHardwareTriggerSettings(_selectedAiModule);
        }

        private void FillChannelsAndRanges(AnalogInputModule aiModule)
        {
            FillRanges(aiModule);
            FillChannels(aiModule);
            UpdateRangesToolstripMenuItems(aiModule);
        }

        private void FillChannels(AnalogInputModule aiModule)
        {
            lvChannelList.Items.Clear();
            if (aiModule == null) return;

            for (uint chIdx = 0; chIdx < aiModule.NumberOfChannels; chIdx++)
            {
                ListViewItem lvItem = lvChannelList.Items.Add("Channel " + chIdx.ToString());
                lvItem.SubItems.Add(aiModule.DefaultRange.RangeText);
                lvItem.Tag = aiModule.DefaultRange;
                lvItem.BackColor = Color.Black;
                lvItem.ForeColor = AdqCommon.ChannelColors[chIdx % AdqCommon.ChannelColors.Length];
            }
            CheckFirstChannels(2);
        }

        private void CheckFirstChannels(int count)
        {
            int idx = 0;
            while (idx < lvChannelList.Items.Count)
            {
                lvChannelList.Items[idx].Checked = idx < count;
                idx++;
            }
        }

        private void FillRanges(AnalogInputModule aiModule)
        {
            _coBoxRanges.Items.Clear();
            if (aiModule == null) return;

            foreach (AnalogRange range in aiModule.Ranges)
            {
                _coBoxRanges.Items.Add(new RangeComboBoxItem(range));
            }
            SelectFirstComboboxItem(_coBoxRanges);
        }

        private void UpdateRangesToolstripMenuItems(AnalogInputModule aiModule)
        {
            tsMnuItemSetRange.DropDownItems.Clear();

            if (aiModule != null && aiModule.NumberOfRanges > 1)
            {
                foreach (AnalogRange range in aiModule.Ranges)
                {
                    var item = new ToolStripMenuItem(range.RangeText, null, TsMnuItemSetRange_Click)
                    {
                        Tag = range
                    };
                    tsMnuItemSetRange.DropDownItems.Add(item);
                }
            }
            else
            {
                var item = new ToolStripMenuItem("No selection available")
                {
                    Enabled = false
                };
                tsMnuItemSetRange.DropDownItems.Add(item);
            }
        }

        private void TsMnuItemSetRange_Click(object sender, EventArgs e)
        {
            if (sender is ToolStripMenuItem)
            {
                AnalogRange range = (AnalogRange)((ToolStripMenuItem)sender).Tag;
                SetChannelsToRange(range);
            }
        }

        private void SetChannelsToRange(AnalogRange range)
        {
            foreach (ListViewItem lvItem in lvChannelList.Items)
            {
                lvItem.Tag = range;
                lvItem.SubItems[_rangeSubItemIndex].Text = range.RangeText;
            }
        }

        private void LvChannelList_MouseDown(object sender, MouseEventArgs e)
        {
            _hitinfoChannelList = lvChannelList.HitTest(e.X, e.Y);
        }

        private void LvChannelList_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button != MouseButtons.Left || _selectedAiModule == null || _selectedAiModule.ModuleIsBusy || _selectedAiModule.Ranges.Count <= 1 ||
            _hitinfoChannelList == null || _hitinfoChannelList.SubItem == null) return;

            ListViewHitTestInfo hitInfo = lvChannelList.HitTest(e.X, e.Y);
            if (_hitinfoChannelList.Item == hitInfo.Item && _hitinfoChannelList.Item.SubItems[_rangeSubItemIndex] == hitInfo.SubItem)
            {
                AnalogRange range = (AnalogRange)_hitinfoChannelList.Item.Tag;
                _coBoxRanges.Bounds = hitInfo.SubItem.Bounds;
                _coBoxRanges.SelectedIndex = (int)range.RangeIndex;
                _coBoxRanges.Visible = true;
                _coBoxRanges.BringToFront();
                _coBoxRanges.Focus();
            }
        }

        private void InitRangeComboBox()
        {
            _coBoxRanges.DropDownStyle = ComboBoxStyle.DropDownList;
            _coBoxRanges.Parent = lvChannelList;
            _coBoxRanges.Visible = false;
            _coBoxRanges.DropDownClosed += CoBoxRanges_DropDownClosed;
            _coBoxRanges.Leave += CoBoxRanges_Leave;
            _coBoxRanges.KeyPress += CoBoxRanges_KeyPress;
        }

        private void CoBoxRanges_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Return)
            {
                UpdateListViewItemChannelList();
            }
            else if (e.KeyChar == (char)Keys.Escape)
            {
                AnalogRange range = (AnalogRange)_hitinfoChannelList.Item.Tag;
                _coBoxRanges.SelectedIndex = (int)range.RangeIndex;
                _coBoxRanges.Visible = false;
            }
        }

        private void CoBoxRanges_DropDownClosed(object sender, EventArgs e)
        {
            UpdateListViewItemChannelList();
        }

        private void CoBoxRanges_Leave(object sender, EventArgs e)
        {
            UpdateListViewItemChannelList();
        }

        private void UpdateListViewItemChannelList()
        {
            if (_hitinfoChannelList == null || _hitinfoChannelList.SubItem == null || _coBoxRanges.SelectedItem == null) return;

            RangeComboBoxItem rangeItem = (RangeComboBoxItem)_coBoxRanges.SelectedItem;
            _hitinfoChannelList.Item.Tag = rangeItem.Range;
            _hitinfoChannelList.Item.SubItems[_rangeSubItemIndex].Text = rangeItem.ToString();

            _coBoxRanges.Visible = false;
            lvChannelList.Focus();
        }

        private void ContextMnuStripChanList_Opening(object sender, CancelEventArgs e)
        {
            if (_coBoxRanges.Visible)
            {
                _coBoxRanges.Visible = false;
            }
        }

        private void EnableUI(bool enabled)
        {
            coBoxModules.Enabled = enabled;
            textBoxSampleFrequency.Enabled = enabled;
            textBoxDisplaySeconds.Enabled = enabled;
            coBoxStopAfterFrames.Enabled = enabled;
            coBoxChartType.Enabled = enabled;
            coBoxXAxisLabels.Enabled = enabled;
            btnReset.Enabled = enabled;
            tsMnuItemSelectAll.Enabled = enabled;
            tsMnuItemSelectNone.Enabled = enabled;
            tsMnuItemInvertSelection.Enabled = enabled;
            btnClear.Enabled = enabled;
        }

        private void TextBoxSampleFrequency_TextChanged(object sender, EventArgs e)
        {
            if (_selectedAiModule == null) return;

            _selectedAiModule.DesiredSampleFrequency = ParseDoubleValue(textBoxSampleFrequency.Text);
        }

        private void TextBoxDisplaySeconds_TextChanged(object sender, EventArgs e)
        {
            if (_selectedAiModule == null) return;

            _selectedAiModule.DisplaySeconds = ParseDoubleValue(textBoxDisplaySeconds.Text);
        }

        private double ParseDoubleValue(string valueText)
        {
            bool isValid = double.TryParse(valueText, out double value);
            if (!isValid || value <= 0) return 0.0;

            return value;
        }

        private void SetStopAfterFramesText(ulong stopAfterFrames)
        {
            coBoxStopAfterFrames.Text = (stopAfterFrames == AdqConstants.ADQ_NUMBER_UNKNOWN) ? _txtInfinite : stopAfterFrames.ToString();
        }

        private void CoBoxStopAfterFrames_Leave(object sender, EventArgs e)
        {
            if (_selectedAiModule == null) return;

            _selectedAiModule.StopAfterFrames = ParseStopAfterFrames(coBoxStopAfterFrames.Text);
        }

        private ulong ParseStopAfterFrames(string stopAfterFramesText)
        {
            if (stopAfterFramesText == _txtInfinite)
            {
                return AdqConstants.ADQ_NUMBER_UNKNOWN;
            }

            bool isValid = ulong.TryParse(stopAfterFramesText, out ulong numFrames);
            if (!isValid)
            {
                coBoxStopAfterFrames.Text = _txtInfinite;
                return AdqConstants.ADQ_NUMBER_UNKNOWN;
            }
            return numFrames;
        }

        private void ChkBoxShowLegend_CheckedChanged(object sender, EventArgs e)
        {
            if (chart1.Legends.Count > 0)
            {
                chart1.Legends[0].Enabled = chkBoxShowLegend.Checked;
            }
        }

        private void LvChannelList_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            if (_selectedAiModule.ModuleIsBusy)
            {
                e.NewValue = e.CurrentValue;
            }
        }

        private void LvChannelList_ItemChecked(object sender, ItemCheckedEventArgs e)
        {
            _selectedAiModule.FrameLength = (uint)lvChannelList.CheckedItems.Count;
            textBoxNumberOfSelectedChannels.Text = _selectedAiModule.FrameLength.ToString();
        }

        private void EnableHardwareTriggerSettings(AnalogInputModule aiModule)
        {
            textBoxTriggerDescription.Text = aiModule != null ? aiModule.TriggerDescription : string.Empty;
            bool triggerAvailable = aiModule != null && aiModule.HardwareTrigger != null;

            chkBoxTriggerStartRisingEdge0.Enabled = triggerAvailable && (aiModule.HardwareTrigger.TriggerStartSource & _bit0Mask) != 0;
            chkBoxTriggerStartRisingEdge1.Enabled = triggerAvailable && (aiModule.HardwareTrigger.TriggerStartSource & _bit1Mask) != 0;

            chkBoxTriggerStartFallingEdge0.Enabled = chkBoxTriggerStartRisingEdge0.Enabled;
            chkBoxTriggerStartFallingEdge1.Enabled = chkBoxTriggerStartRisingEdge1.Enabled;

            chkBoxTriggerStopRisingEdge0.Enabled = triggerAvailable && (aiModule.HardwareTrigger.TriggerStopSource & _bit0Mask) != 0;
            chkBoxTriggerStopRisingEdge1.Enabled = triggerAvailable && (aiModule.HardwareTrigger.TriggerStopSource & _bit1Mask) != 0;

            chkBoxTriggerStopFallingEdge0.Enabled = chkBoxTriggerStopRisingEdge0.Enabled;
            chkBoxTriggerStopFallingEdge1.Enabled = chkBoxTriggerStopRisingEdge1.Enabled;

            chkBoxTriggerStartRisingEdge0.Checked = triggerAvailable && (aiModule.HardwareTrigger.TriggerStartRisingEdges & _bit0Mask) != 0;
            chkBoxTriggerStartRisingEdge1.Checked = triggerAvailable && (aiModule.HardwareTrigger.TriggerStartRisingEdges & _bit1Mask) != 0;

            chkBoxTriggerStartFallingEdge0.Checked = triggerAvailable && (aiModule.HardwareTrigger.TriggerStartFallingEdges & _bit0Mask) != 0;
            chkBoxTriggerStartFallingEdge1.Checked = triggerAvailable && (aiModule.HardwareTrigger.TriggerStartFallingEdges & _bit1Mask) != 0;

            chkBoxTriggerStopRisingEdge0.Checked = triggerAvailable && (aiModule.HardwareTrigger.TriggerStopRisingEdges & _bit0Mask) != 0;
            chkBoxTriggerStopRisingEdge1.Checked = triggerAvailable && (aiModule.HardwareTrigger.TriggerStopRisingEdges & _bit1Mask) != 0;

            chkBoxTriggerStopFallingEdge0.Checked = triggerAvailable && (aiModule.HardwareTrigger.TriggerStopFallingEdges & _bit0Mask) != 0;
            chkBoxTriggerStopFallingEdge1.Checked = triggerAvailable && (aiModule.HardwareTrigger.TriggerStopFallingEdges & _bit1Mask) != 0;

            chkBoxTriggerTimeout.Enabled = triggerAvailable && CheckStartTriggerEnabledCondition();
            chkBoxTriggerTimeout.Checked = triggerAvailable && aiModule.HardwareTrigger.UseTriggerTimeout;

            textBoxTriggerTimeout.Enabled = chkBoxTriggerTimeout.Checked;
            textBoxTriggerTimeout.Text = triggerAvailable ? aiModule.HardwareTrigger.TriggerTimeoutMilliseconds.ToString() : string.Empty;

            chkBoxChannelListPerTrigger.Enabled = triggerAvailable && CheckChanListPerTriggerCondition();
            chkBoxChannelListPerTrigger.Checked = triggerAvailable && aiModule.HardwareTrigger.ChannelListPerTrigger;

            chkBoxAutomaticRearm.Enabled = triggerAvailable && CheckRearmConditions();
            chkBoxAutomaticRearm.Checked = triggerAvailable && aiModule.HardwareTrigger.TriggerAutomaticRearm;
        }

        private void ChkBoxTriggerTimeout_CheckedChanged(object sender, EventArgs e)
        {
            textBoxTriggerTimeout.Enabled = chkBoxTriggerTimeout.Checked;
        }

        private void ChkBoxTriggerStartEdge_CheckedChanged(object sender, EventArgs e)
        {
            chkBoxChannelListPerTrigger.Enabled = CheckChanListPerTriggerCondition();
            chkBoxAutomaticRearm.Enabled = CheckRearmConditions();
            chkBoxTriggerTimeout.Enabled = CheckStartTriggerEnabledCondition();
        }

        private void ChkBoxTriggerStopEdge_CheckedChanged(object sender, EventArgs e)
        {
            chkBoxAutomaticRearm.Enabled = CheckRearmConditions();
        }

        private bool CheckChanListPerTriggerCondition()
        {
            return _selectedAiModule.HardwareTrigger.SupportsChannelListPerTrigger
                && (chkBoxTriggerStartRisingEdge0.Checked
                || chkBoxTriggerStartRisingEdge1.Checked
                || chkBoxTriggerStartFallingEdge0.Checked
                || chkBoxTriggerStartFallingEdge1.Checked);
        }

        private bool CheckRearmConditions()
        {
            return CheckStartTriggerEnabledCondition()
                && (chkBoxTriggerStopRisingEdge0.Checked
                || chkBoxTriggerStopRisingEdge1.Checked
                || chkBoxTriggerStopFallingEdge0.Checked
                || chkBoxTriggerStopFallingEdge1.Checked);
        }

        private bool CheckStartTriggerEnabledCondition()
        {
            return (chkBoxTriggerStartRisingEdge0.Checked
                || chkBoxTriggerStartRisingEdge1.Checked
                || chkBoxTriggerStartFallingEdge0.Checked
                || chkBoxTriggerStartFallingEdge1.Checked);
        }

        private void BtnReset_Click(object sender, EventArgs e)
        {
            _selectedAiModule.Reset();
        }

        private void BtnStartStop_Click(object sender, EventArgs e)
        {
            if (_selectedAiModule == null) return;

            if (_selectedAiModule.ModuleIsBusy)
            {
                _selectedAiModule.CancelAcquisition();
                return;
            }

            ClearDisplayedValues();
            EnableUI(false);
            btnStartStop.Text = "Stop";
            try
            {
                UpdateChannelSettings();
                UpdateModuleTriggerSettings();
                _selectedAiModule.InitAcquisition();
                GenerateAxisLabels(_selectedAiModule.DisplayBufferNumberOfFrames);
                UpdateChart(_selectedAiModule);
                _selectedAiModule.RequestFramesToDisplay = true;
                _selectedAiModule.RequestUpdateNumberOfFrames = true;
                _selectedAiModule.StartAcquisition();
            }
            catch (Exception ex)
            {
                ResetUiStatus();
                textBoxInfo.AppendText(ex.Message + Environment.NewLine);
                tabControlMain.SelectedTab = tpInfo;
            }
        }

        private void ResetUiStatus()
        {
            EnableUI(true);
            btnStartStop.Text = "Start";
            lblAcquisitionStatus.BackColor = Color.White;
            lblAcquisitionStatus.Text = _txtReady;
        }

        private void AcquisitionEnded()
        {
            progressUpdateTimer.Stop();
            if (_isAppClosing) return;

            if (!_selectedAiModule.ModuleIsBusy)
            {
                ResetUiStatus();
                EnableChartTooltipFunction();
            }
        }

        private void EnableChartTooltipFunction()
        {
            if (chart1.Series.Count > 0)
            {
                chart1.GetToolTipText += Chart_GetToolTipText;
            }
        }

        private void Chart_GetToolTipText(object sender, ToolTipEventArgs e)
        {
            if (e.HitTestResult.ChartElementType == ChartElementType.DataPoint)
            {
                var series = e.HitTestResult.Series;
                if (series != null)
                {
                    var dp = series.Points[e.HitTestResult.PointIndex];
                    e.Text = FormatTooltipText(dp);
                }
            }
            else if (e.HitTestResult.ChartElementType == ChartElementType.Axis
                || e.HitTestResult.ChartElementType == ChartElementType.TickMarks)
            {
                if (sender is Chart)
                {
                    HitTestResult result = ((Chart)sender).HitTest(e.X, e.Y, ChartElementType.DataPoint);
                    if (result.ChartElementType == ChartElementType.DataPoint)
                    {
                        var dp = result.Series.Points[result.PointIndex];
                        e.Text = FormatTooltipText(dp);
                    }
                }
            }
        }

        private string FormatTooltipText(DataPoint dp)
        {
            return string.Format("X={0}, Y={1}",
                        dp.XValue,
                        AdqCommon.FormatPhysicalValue(_fmtStringPhysValues, dp.YValues[0]) + _selectedUnitSymbol);
        }

        private void UpdateChannelSettings()
        {
            _selectedAiModule.ChannelSettings.Clear();
            foreach (ListViewItem item in lvChannelList.CheckedItems)
            {
                AnalogRange range = (AnalogRange)item.Tag;
                _selectedAiModule.AddChannelSetting((uint)item.Index, range);
            }
            _selectedAiModule.UpdateChannelList();
        }

        private void UpdateModuleTriggerSettings()
        {
            if (_selectedAiModule.HardwareTrigger == null) return;

            uint timeout = 0;
            bool useTimeout = chkBoxTriggerTimeout.Enabled && chkBoxTriggerTimeout.Checked;

            if (useTimeout)
            {
                try
                {
                    timeout = uint.Parse(textBoxTriggerTimeout.Text);
                }
                catch (Exception ex)
                {
                    throw new ApplicationException("Error trigger Time-out value: " 
                        + textBoxTriggerTimeout.Text + Environment.NewLine +  ex.Message);
                }
            }
            _selectedAiModule.HardwareTrigger.TriggerTimeoutMilliseconds = timeout;
            _selectedAiModule.HardwareTrigger.UseTriggerTimeout = useTimeout;

            _selectedAiModule.HardwareTrigger.ChannelListPerTrigger = chkBoxChannelListPerTrigger.Enabled && chkBoxChannelListPerTrigger.Checked;
            _selectedAiModule.HardwareTrigger.TriggerAutomaticRearm = chkBoxAutomaticRearm.Enabled && chkBoxAutomaticRearm.Checked;

            uint startTriggerRisingEdge = 0;
            uint startTriggerFallingEdge = 0;
            if (_selectedAiModule.HardwareTrigger.TriggerStartSource != 0)
            {
                if ((_selectedAiModule.HardwareTrigger.TriggerStartSource & _bit0Mask) != 0)
                {
                    if (chkBoxTriggerStartRisingEdge0.Checked)
                    {
                        startTriggerRisingEdge |= _bit0Mask;
                    }

                    if (chkBoxTriggerStartFallingEdge0.Checked)
                    {
                        startTriggerFallingEdge |= _bit0Mask;
                    }
                }

                if ((_selectedAiModule.HardwareTrigger.TriggerStartSource & _bit1Mask) != 0)
                {
                    if (chkBoxTriggerStartRisingEdge1.Checked)
                    {
                        startTriggerRisingEdge |= _bit1Mask;
                    }

                    if (chkBoxTriggerStartFallingEdge1.Checked)
                    {
                        startTriggerFallingEdge |= _bit1Mask;
                    }
                }
            }

            _selectedAiModule.HardwareTrigger.TriggerStartRisingEdges = startTriggerRisingEdge;
            _selectedAiModule.HardwareTrigger.TriggerStartFallingEdges = startTriggerFallingEdge;

            uint stopTriggerRisingEdge = 0;
            uint stopTriggerFallingEdge = 0;
            if (_selectedAiModule.HardwareTrigger.TriggerStopSource != 0)
            {
                if ((_selectedAiModule.HardwareTrigger.TriggerStopSource & _bit0Mask) != 0)
                {
                    if (chkBoxTriggerStopRisingEdge0.Checked)
                    {
                        stopTriggerRisingEdge |= _bit0Mask;
                    }

                    if (chkBoxTriggerStopFallingEdge0.Checked)
                    {
                        stopTriggerFallingEdge |= _bit0Mask;
                    }
                }

                if ((_selectedAiModule.HardwareTrigger.TriggerStopSource & _bit1Mask) != 0)
                {
                    if (chkBoxTriggerStopRisingEdge1.Checked)
                    {
                        stopTriggerRisingEdge |= _bit1Mask;
                    }

                    if (chkBoxTriggerStopFallingEdge1.Checked)
                    {
                        stopTriggerFallingEdge |= _bit1Mask;
                    }
                }
            }

            _selectedAiModule.HardwareTrigger.TriggerStopRisingEdges = stopTriggerRisingEdge;
            _selectedAiModule.HardwareTrigger.TriggerStopFallingEdges = stopTriggerFallingEdge;
        }

        private void FormMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_selectedAiModule != null && _selectedAiModule.ModuleIsBusy)
            {
                DialogResult result = MessageBox.Show("The acquisition is still running."
                                         + Environment.NewLine
                                         + "Do you want to close the program?",
                                         "Module busy",
                                         MessageBoxButtons.YesNo,
                                         MessageBoxIcon.Warning);
                if (result == DialogResult.Yes)
                {
                    _isAppClosing = true;
                    _selectedAiModule.RequestFramesToDisplay = false;
                    graphicUpdateTimer.Stop();
                    progressUpdateTimer.Stop();
                    _selectedAiModule.ModuleInfoTextChanged -= OnModuleInfoTextChanged;
                    _selectedAiModule.ModulePropertyChanged -= OnModulePropertyChanged;
                    _selectedAiModule.StopAcquisition();
                    e.Cancel = false;
                }
                else
                {
                    e.Cancel = true;
                }
                return;
            }
            _isAppClosing = true;
        }

        private void BtnClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void FormMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            CloseAdqDriver();
        }

        private void CloseAdqDriver()
        {
            if (_moduleLocator != null)
            {
                _moduleLocator.CloseAdqDriver();
            }
        }

        private void ClearDisplayedValues()
        {
            chart1.Series.Clear();
            textBoxInfo.Clear();
            textBoxFramesAcquired.Text = string.Empty;
            textBoxFramesSkipped.Text = string.Empty;
            textBoxFramesDisplayed.Text = string.Empty;
        }

        private void BtnClear_Click(object sender, EventArgs e)
        {
            ClearDisplayedValues();
        }

        private void TextBoxInfo_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Modifiers == Keys.Control && e.KeyCode == Keys.A)
            {
                textBoxInfo.SelectionStart = 0;
                textBoxInfo.SelectionLength = textBoxInfo.Text.Length;
            }
        }

        private void TsMnuItemSelectAll_Click(object sender, EventArgs e)
        {
            CheckListViewItems(true);
        }

        private void TsMnuItemInvertSelection_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < lvChannelList.Items.Count; i++)
            {
                lvChannelList.Items[i].Checked = !lvChannelList.Items[i].Checked;
            }
        }

        private void TsMnuItemSelectNone_Click(object sender, EventArgs e)
        {
            CheckListViewItems(false);
        }

        private void CheckListViewItems(bool check)
        {
            for (int i = 0; i < lvChannelList.Items.Count; i++)
            {
                lvChannelList.Items[i].Checked = check;
            }
        }

        private void BtnInfo_Click(object sender, EventArgs e)
        {
            var aboutBox = new AboutBox();
            aboutBox.ShowDialog();
        }

        private string GetImageFileName(AnalogInputModule aiModule, string fileExt)
        {
            string s = string.Empty;
            if (aiModule != null)
            {
                s = string.Format("{0}, Board {1}, Module {2}, ",
                    aiModule.ModelName,
                    aiModule.BoardIndex,
                    aiModule.ModuleIndex);
            }
            return s + DateTime.Now.ToString("yyyy-MM-dd HH.mm.ss") + fileExt;
        }

        private ChartImageFormat GetImageFormat(int filterIndex)
        {
            ChartImageFormat imgFormat;
            switch (filterIndex)
            {
                case 1:
                    imgFormat = ChartImageFormat.Png;
                    break;
                case 2:
                    imgFormat = ChartImageFormat.Gif;
                    break;
                case 3:
                    imgFormat = ChartImageFormat.Bmp;
                    break;
                case 4:
                    imgFormat = ChartImageFormat.Tiff;
                    break;
                case 5:
                    imgFormat = ChartImageFormat.Jpeg;
                    break;
                default:
                    imgFormat = ChartImageFormat.Png;
                    break;
            }
            return imgFormat;
        }

        private void TsMenuItemSaveGraphic_Click(object sender, EventArgs e)
        {
            saveFileDialog1.FileName = GetImageFileName(_selectedAiModule, ".png");
            saveFileDialog1.InitialDirectory = _imgInitialDirectory;
            saveFileDialog1.Filter = "PNG files (*.png)|*.png|Gif files (*.gif)|*.gif|BMP files (*.bmp)|*.bmp|Tiff files (*.tif)|*.tif|Jpeg files (*.jpg)|*.jpg";
            DialogResult result = saveFileDialog1.ShowDialog();

            if (result == DialogResult.OK)
            {
                _imgInitialDirectory = Path.GetDirectoryName(saveFileDialog1.FileName);
                ChartImageFormat imgFormat = GetImageFormat(saveFileDialog1.FilterIndex);
                chart1.SaveImage(saveFileDialog1.FileName, imgFormat);
            }
        }

        private void TsMnuItemCopyToClipboard_Click(object sender, EventArgs e)
        {
            using (MemoryStream ms = new MemoryStream())
            {
                chart1.SaveImage(ms, ChartImageFormat.Png);
                var bitmap = new Bitmap(ms);
                Clipboard.SetImage(bitmap);
            }
        }
    }
}
