﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the AIReadInterval project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using AIReadInterval.Utility;
using ALLDAQ_DLL.Devices.Boards;
using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Threading.Tasks;

namespace AIReadInterval.ALLDAQ
{
    public class AnalogInputModule
    {
        public uint AdqError
        {
            get; set;
        }
        public AdqDriver.ADQAIChannel[] _channelList;
        public event EventHandler<InfoTextChangedEventArgs> ModuleInfoTextChanged;

        public AnalogInputModule(
            uint boardIndex,
            uint moduleIndex,
            ref AdqDriver.ADQBoardInfo boardInfo,
            ref AdqDriver.ADQGeneralModuleInfo generalModuleInfo,
            ref AdqDriver.ADQAIModuleInfo aiModuleInfo)
        {
            NumberOfFrames = AdqCommon.DefaultNumberOfFrames;
            BoardIndex = boardIndex;
            ModuleIndex = moduleIndex;
            ModelName = AdqCommon.GetModelName(boardInfo.BoardName);
            SerialNumber = boardInfo.SerialNumber;
            NumberOfChannels = aiModuleInfo.NumberOfChannels;
            NumberOfRanges = aiModuleInfo.NumberOfRanges;
            MaxfrequencyHz = generalModuleInfo.ClockBaseFrequencyHz / generalModuleInfo.ClockMinimumDivider;
            DesiredSampleFrequency = Math.Min(100000.0, MaxfrequencyHz);

            _channelList = new AdqDriver.ADQAIChannel[NumberOfChannels];
            BuildModuleName();
            AddRanges(ref aiModuleInfo);
            DefaultRange = Ranges[0];
        }

        private void AddRanges(ref AdqDriver.ADQAIModuleInfo aiModuleInfo)
        {
            for (uint rangeIdx = 0; rangeIdx < aiModuleInfo.NumberOfRanges; rangeIdx++)
            {
                AdqDriver.ADQAIRange aiRange = aiModuleInfo.Ranges[rangeIdx];

                var physicalRange = new AnalogRange(
                    rangeIdx,
                    aiRange.PhysicalUnit,
                    aiRange.DigitalMin,
                    aiRange.DigitalMax,
                    aiRange.PhysicalMin,
                    aiRange.PhysicalMax);
                Ranges.Add(physicalRange);
            }
        }

        public uint Stop()
        {
            return AdqDriver.adqStop(BoardIndex, ModuleIndex);
        }

        public string ModelName
        {
            get;
        }
        public uint SerialNumber
        {
            get;
        }
        public uint BoardIndex
        {
            get;
        }
        public uint ModuleIndex
        {
            get;
        }
        public string ModuleName
        {
            get; private set;
        }
        public uint NumberOfChannels
        {
            get;
        }
        public uint NumberOfRanges
        {
            get;
        }
        public double MaxfrequencyHz
        {
            get;
        }
        public List<ChannelSetting> ChannelSettings { get; } = new List<ChannelSetting>();

        private void BuildModuleName()
        {
            ModuleName = string.Format("{0}, Board {1}, Module {2}, {3} {4}, {5} {6}, max. {7}",
                ModelName,
                BoardIndex,
                ModuleIndex,
                NumberOfChannels,
                NumberOfChannels == 1 ? "Channel" : "Channels",
                NumberOfRanges,
                NumberOfRanges == 1 ? "Range" : "Ranges",
                AdqCommon.FormatFrequency(MaxfrequencyHz));
        }

        public void ConfigureModule()
        {
            if (NumberOfFrames == 0)
            {
                throw new ApplicationException("Please enter a valid number of frames.");
            }

            if (FrameLength == 0)
            {
                throw new ApplicationException("Please select channels in the channel list.");
            }

            if (DesiredSampleFrequency <= 0)
            {
                throw new ApplicationException("Please enter a valid sample frequency.");
            }

            SetChannelListAndFrequency();
            SampleTime = 1.0 / _actualSampleFrequency;
            AdqError = AdqErrors.ADQ_ERROR_NONE;
            uint numberOfTotalValues = NumberOfFrames * FrameLength;
            if (Frames == null || Frames.Length != numberOfTotalValues)
            {
                Frames = new uint[numberOfTotalValues];
                PhysicalFrames = new double[numberOfTotalValues];
            }

            _numberOfFramesRead = 0;
        }

        public void UpdateChannelList()
        {
            for (int i = 0; i < _channelList.Length; i++)
            {
                _channelList[i].AcquireChannel = false;
                _channelList[i].RangeIndex = 0;
            }

            MinResolution = 1.0;

            foreach (ChannelSetting chSetting in ChannelSettings)
            {
                _channelList[chSetting.ChannelIndex].AcquireChannel = true;
                _channelList[chSetting.ChannelIndex].RangeIndex = chSetting.Range.RangeIndex;
                if (MinResolution > chSetting.Range.PhysicalResolution)
                {
                    MinResolution = chSetting.Range.PhysicalResolution;
                }
            }
            FrameLength = (uint)ChannelSettings.Count;
        }

        private void SetChannelListAndFrequency()
        {
            uint adqError = AdqDriver.adqAISetChannelListAndFrequency(BoardIndex, ModuleIndex,
                 _channelList, NumberOfChannels, DesiredSampleFrequency, out _actualSampleFrequency);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqAISetChannelListAndFrequency", adqError);
                throw new ApplicationException("An error occured on calling adqAISetChannelListAndFrequency.");
            }
        }

        public uint NumberOfFrames
        {
            get; set;
        }
        public uint FrameLength
        {
            get; set;
        }

        private uint _numberOfFramesRead;
        public uint NumberOfFramesRead
        {
            get
            {
                return _numberOfFramesRead;
            }
        }

        public uint TotalNumberOfValuesRead
        {
            get; private set;
        }
        public uint[] Frames
        {
            get; private set;
        }
        public DateTime DateTimeFirstFrame
        {
            get; private set;
        }
        public double[] PhysicalFrames
        {
            get; private set;
        }
        public double MinResolution
        {
            get; private set;
        }

        double _actualSampleFrequency;
        public double ActualSampleFrequency
        {
            get
            {
                return _actualSampleFrequency;
            }
        }

        public double SampleTime
        {
            get; private set;
        }
        public double DesiredSampleFrequency
        {
            get; set;
        }
        public List<AnalogRange> Ranges { get; } = new List<AnalogRange>();
        public AnalogRange DefaultRange
        {
            get;
        }

        public double GetSecondsToWait()
        {
            if (_actualSampleFrequency == 0)
            {
                return 0.0;
            }

            return NumberOfFrames / _actualSampleFrequency;
        }

        public void ReadValuesFromModule()
        {
            TotalNumberOfValuesRead = 0;
            GCHandle gch = GCHandle.Alloc(Frames, GCHandleType.Pinned);
            DateTimeFirstFrame = DateTime.Now;
            AdqError = AdqDriver.adqAIReadBuffer(BoardIndex, ModuleIndex,
                FrameLength, NumberOfFrames, Frames, out _numberOfFramesRead);
            TotalNumberOfValuesRead = FrameLength * _numberOfFramesRead;
            gch.Free();
            CalculatePhysicalFrames();
        }

        private void CalculatePhysicalFrames()
        {
            ChannelSetting channelSetting;
            AnalogRange range;
            uint channelIndex;

            Parallel.For(0, FrameLength, idx =>
            {
                channelSetting = ChannelSettings[(int)idx];
                range = channelSetting.Range;
                channelIndex = channelSetting.ChannelIndex;

                uint valIdx = (uint)idx;
                while (valIdx < TotalNumberOfValuesRead)
                {
                    PhysicalFrames[valIdx] = range.DigitalToPhysical(Frames[valIdx]);
                    valIdx += FrameLength;
                }
            });
        }

        public void ClearResources()
        {
            _numberOfFramesRead = 0;
            TotalNumberOfValuesRead = 0;
            Frames = null;
            PhysicalFrames = null;
        }

        public uint Reset()
        {
            uint adqError = AdqDriver.adqResetModule(BoardIndex, ModuleIndex);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqResetModule", adqError);
            }
            else
            {
                RaiseInfoTextChanged(string.Format("Reset board {0} module {1}", BoardIndex, ModuleIndex));
            }
            return adqError;
        }

        private void SetFunctionErrorInfo(string functionName, uint adqError)
        {
            RaiseInfoTextChanged(string.Format("API-function {0} returned Error number: 0x{1:X}", functionName, adqError));
        }

        protected virtual void RaiseInfoTextChanged(string infoText)
        {
            if (ModuleInfoTextChanged != null)
            {
                var args = new InfoTextChangedEventArgs
                {
                    InfoText = infoText
                };

                ModuleInfoTextChanged(this, args);
            }
        }
    }
}
