﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the AIReadInterval project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using AIReadInterval.Utility;
using ALLDAQ_DLL.Devices.Boards;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace AIReadInterval.ALLDAQ
{
    public class ModuleLocator
    {
        private uint _adqOpenStatusError = AdqCommon.NotOpen;
        private readonly ICollection<AnalogInputModule> _aiModules;
        public event EventHandler<InfoTextChangedEventArgs> OnInfoTextChanged;

        public ModuleLocator(
            ICollection<AnalogInputModule> aiModules,
            EventHandler<InfoTextChangedEventArgs> infoTextChangedHandler)
        {
            _aiModules = aiModules;
            OnInfoTextChanged += infoTextChangedHandler;
        }

        private uint OpenAdqDriver()
        {
            uint adqError = AdqDriver.adqOpen();
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqOpen", adqError);
            }
            return adqError;
        }

        public void CloseAdqDriver()
        {
            if (_adqOpenStatusError == AdqErrors.ADQ_ERROR_NONE)
            {
                AdqDriver.adqClose();
                _adqOpenStatusError = AdqCommon.NotOpen;
            }
        }

        private uint GetNumberOfBoards()
        {
            uint adqError = AdqDriver.adqGetNumberOfBoards(out uint numBoards);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetNumberOfBoards", adqError);
                return 0;
            }
            return numBoards;
        }

        public void FindModules()
        {
            _adqOpenStatusError = OpenAdqDriver();
            if (_adqOpenStatusError == AdqErrors.ADQ_ERROR_NONE)
            {
                uint numberOfBoards = GetNumberOfBoards();
                for (uint boardIdx = 0; boardIdx < numberOfBoards; boardIdx++)
                {
                    uint adqError = AdqDriver.adqGetNumberOfModules(boardIdx, out uint numberOfModules);
                    if (adqError != AdqErrors.ADQ_ERROR_NONE)
                    {
                        SetFunctionErrorInfo("adqGetNumberOfModules", adqError);
                        continue;
                    }

                    adqError = GetModuleTypes(boardIdx, numberOfModules);
                    if (adqError != AdqErrors.ADQ_ERROR_NONE)
                    {
                        continue;
                    }
                }
            }
        }

        private uint GetModuleTypes(uint boardIndex, uint numberOfModules)
        {
            uint adqError = AdqErrors.ADQ_ERROR_NONE;

            for (uint moduleIdx = 0; moduleIdx < numberOfModules; moduleIdx++)
            {
                adqError = AdqDriver.adqGetModuleType(boardIndex, moduleIdx, out uint moduleType);
                if (adqError != AdqErrors.ADQ_ERROR_NONE)
                {
                    SetFunctionErrorInfo("adqGetModuleType", adqError);
                    continue;
                }

                if (moduleType == AdqConstants.ADQ_MODULE_TYPE_AI)
                {
                    adqError = GetAiModuleInfo(boardIndex, moduleIdx);
                    if (adqError != AdqErrors.ADQ_ERROR_NONE)
                    {
                        continue;
                    }
                }
            }
            return adqError;
        }


        private uint GetAiModuleInfo(uint boardIndex, uint moduleIndex)
        {
            uint adqError;
            var adqGeneralModuleInfo = new AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID);
            var adqAiModuleInfo = new AdqDriver.ADQAIModuleInfo(AdqDriver.ADQAIModuleInfo_ID);

            IntPtr moduleInfo = Marshal.AllocHGlobal((int)adqAiModuleInfo.Size);
            Marshal.StructureToPtr(adqAiModuleInfo, moduleInfo, false);

            adqError = AdqDriver.adqGetModuleInfo(boardIndex,
                                                  moduleIndex,
                                                  ref adqGeneralModuleInfo,
                                                  adqAiModuleInfo.Size,
                                                  moduleInfo,
                                                  out uint infoSize);
            adqAiModuleInfo = (AdqDriver.ADQAIModuleInfo)Marshal.PtrToStructure(moduleInfo, typeof(AdqDriver.ADQAIModuleInfo));
            Marshal.FreeHGlobal(moduleInfo);

            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetModuleInfo", adqError);
                return adqError;
            }

            var boardInfo = new AdqDriver.ADQBoardInfo(AdqDriver.ADQBoardInfo_ID);
            adqError = GetBoardInfo(boardIndex, ref boardInfo);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                return adqError;
            }

            Debug.Assert(adqAiModuleInfo.Size == infoSize,
                string.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                boardIndex, moduleIndex, adqAiModuleInfo.Size, infoSize));

            if (adqAiModuleInfo.RangeInfoPerChannel)
            {
                RaiseInfoTextChanged(string.Format("Board {0}, {1}, Module {2} has individual ranges."
                    + Environment.NewLine
                    + "Please update to a newer version of this program.",
                    boardIndex,
                    boardInfo.BoardName,
                    moduleIndex));
                return adqError;
            }

            if (IsClockControlledModule(adqGeneralModuleInfo.ClockType))
            {
                Debug.Assert(adqGeneralModuleInfo.ClockMinimumDivider > 0,
                    string.Format("Board {0}: The ClockMinimumDivider of the module {1} is zero", boardIndex, moduleIndex));
                var aiModule = new AnalogInputModule(boardIndex, moduleIndex, ref boardInfo, ref adqGeneralModuleInfo, ref adqAiModuleInfo);
                _aiModules.Add(aiModule);
            }

            return adqError;
        }

        private bool IsClockControlledModule(uint clockType)
        {
            return clockType == AdqConstants.ADQ_CLOCK_TYPE_HARDWARE || clockType == AdqConstants.ADQ_CLOCK_TYPE_SOFTWARE;
        }

        private uint GetBoardInfo(uint boardIndex, ref AdqDriver.ADQBoardInfo boardInfo)
        {
            uint adqError = AdqDriver.adqGetBoardInfo(boardIndex, ref boardInfo);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetBoardInfo", adqError);
            }
            return adqError;
        }

        private void SetFunctionErrorInfo(string functionName, uint adqError)
        {
            RaiseInfoTextChanged(string.Format("API-function {0} returned Error number: 0x{1:X}", functionName, adqError));
        }

        protected virtual void RaiseInfoTextChanged(string infoText)
        {
            if (OnInfoTextChanged != null)
            {
                var args = new InfoTextChangedEventArgs
                {
                    InfoText = infoText
                };
                OnInfoTextChanged(this, args);
            }
        }
    }
}
