﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the AISingle project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using AISingle.Utility;
using ALLDAQ.Boards;
using System;
using System.Collections.Generic;

namespace AISingle.ALLDAQ
{
    public class AnalogInputModule
    {
        private const uint _notInitialized = uint.MaxValue;
        private readonly List<AnalogRange> _ranges = new List<AnalogRange>();
        private uint _selectedRangeIndex = _notInitialized;
        private readonly uint _numberOfRanges;

        public event EventHandler<InfoTextChangedEventArgs> ModuleInfoTextChanged;

        public AnalogInputModule(string boardName, uint boardIndex, uint moduleIndex,
            ref AdqDriver.ADQGeneralModuleInfo generalInfo, ref AdqDriver.ADQAIModuleInfo aiModuleInfo)
        {
            Factor = 1.0;
            BoardIndex = boardIndex;
            ModuleIndex = moduleIndex;
            NumberOfChannels = aiModuleInfo.NumberOfChannels;
            Description = generalInfo.ModuleDescription;
            _numberOfRanges = aiModuleInfo.NumberOfRanges;
            AddRanges(ref aiModuleInfo);
            BuildModuleName(boardName);
        }

        private void AddRanges(ref AdqDriver.ADQAIModuleInfo adqAiModuleInfo)
        {
            for (uint i = 0; i < adqAiModuleInfo.NumberOfRanges; i++)
            {
                AdqDriver.ADQAIRange range = adqAiModuleInfo.Ranges[i];

                var physRange = new AnalogRange(
                    range.PhysicalUnit,
                    range.DigitalMin,
                    range.DigitalMax,
                    range.PhysicalMin,
                    range.PhysicalMax);
                _ranges.Add(physRange);
            }
        }

        public uint BoardIndex { get; }
        public uint ModuleIndex { get; }
        public uint NumberOfChannels { get; }
        public string Description { get; }
        public string ModuleName { get; private set; }

        public AnalogRange GetRange(int index)
        {
            return _ranges[index];
        }
        public AnalogRange SelectedRange { get; private set; }
        public uint SelectedChannelIndex { get; private set; } = _notInitialized;
        public double Factor { get; set; }
        public double Offset { get; set; }

        public uint SetChannelAndRange(uint channelIndex, uint rangeIndex)
        {
            uint adqError = AdqErrors.ADQ_ERROR_NONE;
            if (IsChannelChanged(channelIndex) || IsRangeChanged(rangeIndex))
            {
                adqError = AdqDriver.adqAISetChannelAndRange(BoardIndex, ModuleIndex, channelIndex, rangeIndex);
                if (adqError == AdqErrors.ADQ_ERROR_NONE)
                {
                    SetCurrentChannel(channelIndex);
                    SetCurrentRange(rangeIndex);
                    InfoText = string.Format("Board {0}, module {1}, channel {2}, range {3} configured",
                        BoardIndex,
                        ModuleIndex,
                        channelIndex,
                        SelectedRange.RangeText);
                }
                else
                {
                    SetFunctionErrorInfo("adqAISetChannelAndRange", adqError);
                }
            }
            return adqError;
        }

        public uint ReadSingle(out uint digitalValue)
        {
            uint adqError = AdqDriver.adqReadSingle(BoardIndex, ModuleIndex, out digitalValue);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqReadSingle", adqError);
            }
            return adqError;
        }

        public uint Reset()
        {
            uint adqError = AdqDriver.adqResetModule(BoardIndex, ModuleIndex);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqResetModule", adqError);
            }
            else
            {
                InfoText = string.Format("Reset board {0} module {1}", BoardIndex, ModuleIndex);
            }
            _selectedRangeIndex = _notInitialized;
            SelectedChannelIndex = _notInitialized;
            return adqError;
        }

        private bool IsRangeChanged(uint rangeIndex)
        {
            return _selectedRangeIndex == _notInitialized || _selectedRangeIndex != rangeIndex;
        }

        private void SetCurrentRange(uint rangeIndex)
        {
            SelectedRange = rangeIndex < _ranges.Count ? _ranges[(int)rangeIndex] : null;
            _selectedRangeIndex = rangeIndex;
        }

        private bool IsChannelChanged(uint channelIndex)
        {
            return SelectedChannelIndex == _notInitialized || SelectedChannelIndex != channelIndex;
        }

        private void SetCurrentChannel(uint channelIndex)
        {
            SelectedChannelIndex = channelIndex;
        }

        public int NumberOfRanges
        {
            get { return _ranges.Count; }
        }

        private string _infoText = string.Empty;
        public string InfoText
        {
            get { return _infoText; }
            set
            {
                if (!string.IsNullOrEmpty(value))
                {
                    _infoText = value;
                    var args = new InfoTextChangedEventArgs
                    {
                        InfoText = _infoText
                    };

                    OnModuleInfoTextChanged(args);
                }
            }
        }

        private void SetFunctionErrorInfo(string functionName, uint adqError)
        {
            InfoText = string.Format("API-function {0} returned Error number: 0x{1:X}", functionName, adqError);
        }

        protected virtual void OnModuleInfoTextChanged(InfoTextChangedEventArgs e)
        {
            ModuleInfoTextChanged?.Invoke(this, e);
        }

        private void BuildModuleName(string boardName)
        {
            ModuleName = string.Format("{0}, Board {1}, Module {2}, {3} {4}, {5} {6}",
                AdqCommon.GetModelName(boardName),
                BoardIndex,
                ModuleIndex,
                NumberOfChannels,
                NumberOfChannels == 1 ? "Channel" : "Channels",
                _numberOfRanges,
                _numberOfRanges == 1 ? "Range" : "Ranges");
        }
    }
}
