﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the AISingle project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Windows.Forms;
using AISingle.ALLDAQ;
using AISingle.Utility;
using ALLDAQ.Boards;

namespace AISingle
{
    public partial class FormMain : Form
    {
        private bool _factorIsValid;
        private bool _offsetIsValid;
        private ModuleLocator _moduleLocator;
        private readonly List<AnalogInputModule> _aiModules = new List<AnalogInputModule>();
        private AnalogInputModule _selectedModule;

        public FormMain()
        {
            InitializeComponent();
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            EnableUI(false);
            try
            {
                _moduleLocator = new ModuleLocator(_aiModules, AdqConstants.ADQ_MODULE_TYPE_AI, OnModuleInfoTextChanged);
                _moduleLocator.FindModules();

                FillModuleCombobox();
                SelectFirstComboboxItem(coBoxModules);
                AddInfoTextLine("Number of analog input modules found: " + _aiModules.Count.ToString());
            }
            catch (DllNotFoundException ex)
            {
                AddInfoTextLine("The ALLDAQ device driver is not properly installed." + Environment.NewLine + ex.Message);
            }
        }

        private void FillModuleCombobox()
        {
            coBoxModules.Items.Clear();
            foreach (AnalogInputModule aiModule in _aiModules)
            {
                coBoxModules.Items.Add(aiModule.ModuleName);
            }
        }

        private void OnModuleInfoTextChanged(object sender, InfoTextChangedEventArgs e)
        {
            AddInfoTextLine(e.InfoText);
        }

        private void CoBoxModules_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (_selectedModule != null)
            {
                _selectedModule.ModuleInfoTextChanged -= OnModuleInfoTextChanged;
            }

            EnableUI(coBoxModules.SelectedIndex >= 0);
            if (coBoxModules.SelectedIndex < 0)
            {
                _selectedModule = null;
            }
            else
            {
                _selectedModule = _aiModules[coBoxModules.SelectedIndex];
                _selectedModule.ModuleInfoTextChanged += OnModuleInfoTextChanged;
            }
            UpdateModuleProperties(_selectedModule);
            FillChannels(_selectedModule);
            FillRanges(_selectedModule);
        }
        
        private void UpdateModuleProperties(AnalogInputModule aiModule)
        {
            if (aiModule == null)
            {
                textBoxDescription.Clear();
                textBoxFactor.Clear();
                textBoxOffset.Clear();
                return;
            }

            textBoxDescription.Text = aiModule.Description;
            textBoxFactor.Text = aiModule.Factor.ToString();
            textBoxOffset.Text = aiModule.Offset.ToString();
            _factorIsValid = true;
            _offsetIsValid = true;
        }

        private void FillChannels(AnalogInputModule aiModule)
        {
            coBoxChannels.Items.Clear();
            if (aiModule != null)
            {
                for (int i = 0; i < aiModule.NumberOfChannels; i++)
                {
                    coBoxChannels.Items.Add(i.ToString());
                }
                SelectFirstComboboxItem(coBoxChannels);
            }
       }

        private void FillRanges(AnalogInputModule aiModule)
        {
            coBoxRanges.Items.Clear();
            if (aiModule != null)
            {
                for (int i = 0; i < aiModule.NumberOfRanges; i++)
                {
                    var physicalRange = aiModule.GetRange(i);
                    coBoxRanges.Items.Add(physicalRange.RangeText);
                }
                SelectFirstComboboxItem(coBoxRanges);
            }
        }

        private void TextBoxFactor_Leave(object sender, EventArgs e)
        {
            if (_selectedModule == null) return;
            _factorIsValid = double.TryParse(textBoxFactor.Text, out double value);
            if (!_factorIsValid)
            {
                AddInfoTextLine("The entered value for Factor is not a valid number: \"" + textBoxFactor.Text + "\"");
                return;
            }
            _selectedModule.Factor = value;
        }

        private void TextBoxOffset_Leave(object sender, EventArgs e)
        {
            if (_selectedModule == null) return;
            _offsetIsValid = double.TryParse(textBoxOffset.Text, out double value);
            if (!_offsetIsValid)
            {
                AddInfoTextLine("The entered value for Offset is not a valid number: \"" + textBoxOffset.Text + "\"");
                return;
            }
            _selectedModule.Offset = value;
        }

        private void BtnReadValue_Click(object sender, EventArgs e)
        {
            if (_selectedModule == null) return;
            if (!_factorIsValid || !_offsetIsValid)
            {
                AddInfoTextLine("Please correct the entered Factor and Offset.");
                return;
            }

            ReadAnalogValue(_selectedModule, (uint)coBoxChannels.SelectedIndex, (uint)coBoxRanges.SelectedIndex);
        }

        private void ReadAnalogValue(AnalogInputModule aiModule, uint channelIndex, uint rangeIndex)
        {
            uint adqError = aiModule.SetChannelAndRange(channelIndex, rangeIndex);
            if (adqError != AdqErrors.ADQ_ERROR_NONE) return;

            ReadSingleValue(aiModule);
        }

        private void ReadSingleValue(AnalogInputModule aiModule)
        {
            uint adqError = aiModule.ReadSingle(out uint digitalValue);
            if (adqError == AdqErrors.ADQ_ERROR_NONE)
            {
                DisplayValue(aiModule, digitalValue);
            }
        }

        private void DisplayValue(AnalogInputModule aiModule, uint digitalValue)
        {
            double physicalValue = aiModule.SelectedRange.DigitalToPhysical(digitalValue);
            string s = string.Format("[{0}] Board {1}, Module {2}, Channel {3}", 
                DateTime.Now.ToString("HH:mm:ss.fff"),
                aiModule.BoardIndex, 
                aiModule.ModuleIndex, 
                aiModule.SelectedChannelIndex)
                + string.Format(" --> digital: {0}, physical: {1:0.000000000}{2}",
                    digitalValue,
                    physicalValue,
                    aiModule.SelectedRange.UnitSymbol);

            if (aiModule.Factor != 1.0 || aiModule.Offset != 0.0)
            {
                physicalValue = physicalValue * aiModule.Factor + aiModule.Offset;
                s += string.Format("; Factor: {0}, Offset: {1} --> {2:0.000000000}{3}", 
                    aiModule.Factor, aiModule.Offset, physicalValue, aiModule.SelectedRange.UnitSymbol);
            }
            s += Environment.NewLine;
            textBoxInfo.AppendText(s);
        }

        private void AddInfoTextLine(string text)
        {
            textBoxInfo.AppendText(string.Format("[{0}] {1}" + Environment.NewLine,
                    DateTime.Now.ToString("HH:mm:ss.fff"), text));
        }

        private void SelectFirstComboboxItem(ComboBox coBox)
        {
            if (coBox.Items.Count > 0)
            {
                coBox.SelectedIndex = 0;
            }
        }

        private void BtnReset_Click(object sender, EventArgs e)
        {
            _selectedModule.Reset();
        }

        private void CloseAdqDriver()
        {
            if (_moduleLocator != null)
            {
                _moduleLocator.CloseAdqDriver();
            }
        }

        private void EnableUI(bool enabled)
        {
            btnReadValue.Enabled = enabled;
            btnReset.Enabled = enabled;
            coBoxChannels.Enabled = enabled;
            coBoxRanges.Enabled = enabled;
        }

        private void FormMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            CloseAdqDriver();
        }

        private void BtnClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void BtnClear_Click(object sender, EventArgs e)
        {
            ClearInfoText();
        }

        private void ClearInfoText()
        {
            textBoxInfo.Clear();
        }

        private void BtnInfo_Click(object sender, EventArgs e)
        {
            var aboutBox = new AboutBox();
            aboutBox.ShowDialog();
        }
    }
}
