﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the AOSingle project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using AOSingle.Utility;
using System;
using System.Text;

namespace AOSingle.Models
{
    public class AoRange
    {
        private readonly double _physicalResolution;
        private readonly double _digitalOffset;

        public AoRange(uint rangeIndex, uint digitalMin, uint digitalMax, uint physicalUnit, double physicalMin, double physicalMax)
        {
            RangeIndex = rangeIndex;
            DigitalMin = digitalMin;
            DigitalMax = digitalMax;
            PhysicalUnit = physicalUnit;
            PhysicalMin = physicalMin;
            PhysicalMax = physicalMax;
            UnitSymbol = AdqCommon.GetPhysicalUnitSymbol(physicalUnit);
            SetRangeText(UnitSymbol);
            _physicalResolution = (PhysicalMax - PhysicalMin) / (DigitalMax - DigitalMin);
            _digitalOffset = DigitalMin - PhysicalMin / _physicalResolution;
            CurrentDigitalValue = PhysicalToDigital(0);
        }

        public uint RangeIndex { get; }
        public uint DigitalMin { get; }
        public uint DigitalMax { get; }
        public uint PhysicalUnit { get; }
        public double PhysicalMin { get; }
        public double PhysicalMax { get; }
        public string UnitSymbol { get; }
        public string RangeText { get; private set; }
        public uint CurrentDigitalValue { get; set; }

        private void SetRangeText(string unitSymbol)
        {
            StringBuilder rangeTxt = new StringBuilder();

            rangeTxt.Append(AdqCommon.FormatPhysicalValue(PhysicalMin));
            rangeTxt.Append(unitSymbol);
            rangeTxt.Append("...");
            rangeTxt.Append(AdqCommon.FormatPhysicalValue(PhysicalMax));
            rangeTxt.Append(unitSymbol);

            RangeText = rangeTxt.ToString();
        }

        public uint PhysicalToDigital(double physicalValue)
        {
            return (uint)Math.Round(physicalValue / _physicalResolution + _digitalOffset);
        }

        internal double DigitalToPhysical(uint digitalValue)
        {
            return _physicalResolution * digitalValue + PhysicalMin;
        }

        internal string ValidatePhysicalValue(double physicalValue)
        {
            string msg = string.Empty;
            if (physicalValue < PhysicalMin || physicalValue > PhysicalMax)
            {
                msg = "The physical value must be within the range limits.";
            }
            return msg;
        }

        internal string ValidateDigitalValue(uint digital)
        {
            string msg = string.Empty;
            if (digital < DigitalMin || digital > DigitalMax)
            {
                msg = "The digital value must be within the range limits.";
            }
            return msg;
        }
    }
}