﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the AOSingle project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using AOSingle.Models;
using AOSingle.Utility;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows;

namespace AOSingle.ViewModels
{
    public class MainViewModel : INotifyPropertyChanged, IDataErrorInfo
    {
        private readonly ModuleLocator _moduleLocator;
        private readonly Dictionary<string, string> _errorInfo = new Dictionary<string, string>();

        public RelayCommand ResetModuleCommand { get; private set; }
        public RelayCommand WriteValueCommand { get; private set; }
        public RelayCommand ClearInfoTextCommand { get; private set; }
        public RelayCommand CloseCommand { get; private set; }

        public event PropertyChangedEventHandler PropertyChanged;

        public MainViewModel()
        {
            InitCommands();
            _moduleLocator = new ModuleLocator(MainViewModel_PropertyChanged);
            _moduleLocator.InitModuleLocator();
        }

        private void InitCommands()
        {
            ResetModuleCommand = new RelayCommand(ResetModuleExecute, CanResetModuleExecute);
            WriteValueCommand = new RelayCommand(WriteValueExecute, CanWriteValueExecute);
            ClearInfoTextCommand = new RelayCommand(ClearInfoTextExecute);
            CloseCommand = new RelayCommand(CloseApplicationExecute);
        }

        private bool CanWriteValueExecute(object obj)
        {
            return _selectedAoRange != null && _errorInfo.Count == 0;
        }

        private void WriteValueExecute(object obj)
        {
            bool success = _selectedModule.WriteSingleValue(_digitalValue);
            if (success)
            {
                _selectedAoRange.CurrentDigitalValue = _digitalValue;
                AppendInfoText(string.Format("Write to board {0}, module {1}, value digital: {2}, physical: {3}{4}",
                    _selectedModule.BoardIndex,
                    _selectedModule.ModuleIndex,
                    _digitalValue,
                    _physicalValueText,
                    UnitSymbol));
            }
        }

        private bool CanResetModuleExecute(object obj)
        {
            return _selectedModule != null;
        }

        private void ResetModuleExecute(object obj)
        {
            bool success = _selectedModule.ResetModule();
            if (success)
            {
                PhysicalToWrite = 0.0;
                _selectedAoRange.CurrentDigitalValue = _digitalValue;
                AppendInfoText(string.Format("Reset board {0}, module {1}", _selectedModule.BoardIndex, _selectedModule.ModuleIndex));
            }
        }

        public List<AoModule> AoModules
        {
            get { return _moduleLocator.AoModules; }
        }

        private AoModule _selectedModule;
        public AoModule SelectedModule
        {
            get { return _selectedModule; }
            set
            {
                if (_selectedModule == value) return;

                if (_selectedModule != null)
                {
                    _selectedModule.PropertyChanged -= MainViewModel_PropertyChanged;
                }
                _selectedModule = value;
                if (_selectedModule != null)
                {
                    _selectedModule.PropertyChanged += MainViewModel_PropertyChanged;
                    AoRanges = _selectedModule.AoRanges;
                    if (AoRanges.Count == 1)
                    {
                        SelectedAoRange = AoRanges[0];
                    }
                }
                else
                {
                    AoRanges = null;
                    SelectedAoRange = null;
                }
                NotifyPropertyChanged(nameof(SelectedModule));
                NotifyPropertyChanged(nameof(AoRanges));
            }
        }

        public List<AoRange> AoRanges { get; private set; }

        private AoRange _selectedAoRange;
        public AoRange SelectedAoRange
        {
            get { return _selectedAoRange; }
            set
            {
                if (_selectedAoRange != value)
                {
                    _selectedAoRange = value;
                    UnitSymbol = _selectedAoRange?.UnitSymbol;
                    DigitalValueText = _selectedAoRange.CurrentDigitalValue.ToString();
                    NotifyPropertyChanged(nameof(SelectedAoRange));
                    NotifyPropertyChanged(nameof(IsEnabled));
                    NotifyPropertyChanged(nameof(UnitSymbol));
                }
            }
        }

        public string UnitSymbol { get; private set; }

        public bool IsEnabled
        {
            get { return _selectedAoRange != null; }
        }

        private double _physicalToWrite;
        public double PhysicalToWrite
        {
            get { return _physicalToWrite; }
            set
            {
                if (_physicalToWrite != value)
                {
                    _physicalToWrite = value;
                    _digitalValue = _selectedAoRange.PhysicalToDigital(_physicalToWrite);
                    _digitalValueText = _digitalValue.ToString();
                    _physicalValueText = AdqCommon.FormatDoubleValue(_physicalToWrite);

                    _errorInfo.Remove(nameof(PhysicalValueText));
                    _errorInfo.Remove(nameof(DigitalValueText));
                    NotifyPropertyChanged(nameof(PhysicalValueText));
                    NotifyPropertyChanged(nameof(PhysicalToWrite));
                    NotifyPropertyChanged(nameof(DigitalValueText));
                    WriteValueCommand.RaiseCanExecuteChanged();
                }
            }
        }

        private string _physicalValueText = "0";
        public string PhysicalValueText
        {
            get { return _physicalValueText; }
            set
            {
                if (_physicalValueText != value)
                {
                    _physicalValueText = value;
                    string msg = ValidateDoubleValue(_physicalValueText, out double physicalValue);
                    if (string.IsNullOrEmpty(msg))
                    {
                        msg = _selectedAoRange.ValidatePhysicalValue(physicalValue);
                    }

                    if (string.IsNullOrEmpty(msg))
                    {
                        uint digital = _selectedAoRange.PhysicalToDigital(physicalValue);
                        _physicalToWrite = _selectedAoRange.DigitalToPhysical(digital);
                        _physicalValueText = AdqCommon.FormatDoubleValue(_physicalToWrite);
                        _digitalValue = digital;
                        _digitalValueText = digital.ToString();
                        _errorInfo.Remove(nameof(PhysicalValueText));
                        _errorInfo.Remove(nameof(DigitalValueText));
                        NotifyPropertyChanged(nameof(PhysicalToWrite));
                        NotifyPropertyChanged(nameof(DigitalValueText));
                    }
                    else
                    {
                        _errorInfo.AddOrUpdate(nameof(PhysicalValueText), msg);
                    }
                    NotifyPropertyChanged(nameof(PhysicalValueText));
                    WriteValueCommand.RaiseCanExecuteChanged();
                }
            }
        }

        private uint _digitalValue;
        private string _digitalValueText = string.Empty;
        public string DigitalValueText
        {
            get { return _digitalValueText; }
            set
            {
                if (_digitalValueText != value)
                {
                    _digitalValueText = value;
                    string msg = ValidateDigitalValue(_digitalValueText, out uint digital);
                    if (string.IsNullOrEmpty(msg))
                    {
                        msg = _selectedAoRange.ValidateDigitalValue(digital);
                    }

                    if (string.IsNullOrEmpty(msg))
                    {
                        _digitalValue = digital;
                        _physicalToWrite = _selectedAoRange.DigitalToPhysical(digital);
                        _physicalValueText = AdqCommon.FormatDoubleValue(_physicalToWrite);
                        _errorInfo.Remove(nameof(DigitalValueText));
                        _errorInfo.Remove(nameof(PhysicalValueText));
                        NotifyPropertyChanged(nameof(PhysicalValueText));
                        NotifyPropertyChanged(nameof(PhysicalToWrite));
                    }
                    else
                    {
                        _errorInfo.AddOrUpdate(nameof(DigitalValueText), msg);
                    }

                    NotifyPropertyChanged(nameof(DigitalValueText));
                    WriteValueCommand.RaiseCanExecuteChanged();
                }
            }
        }

        public string InfoText { get; private set; } = string.Empty;

        public string Error => "MainViewModel.Error";

        public string this[string propertyName]
        {
            get
            {
                return _errorInfo.ContainsKey(propertyName) ? _errorInfo[propertyName] : null;
            }
        }

        private string ValidateDoubleValue(string textValue, out double doubleValue)
        {
            doubleValue = 0.0;
            string msg = null;
            if (string.IsNullOrEmpty(textValue))
            {
                msg = "The value cannot be empty";
            }
            else if (!double.TryParse(textValue, out doubleValue))
            {
                msg = "The entered text '" + textValue + "' is not a valid number";
            }
            return msg;
        }

        private string ValidateDigitalValue(string textValue, out uint digital)
        {
            digital = 0;
            string msg = null;
            if (string.IsNullOrEmpty(textValue))
            {
                msg = "The value cannot be empty";
            }
            else if (!uint.TryParse(textValue, out digital))
            {
                msg = "The entered text '" + textValue + "' is not a valid number";
            }
            return msg;
        }

        private void AppendInfoText(string text)
        {
            if (InfoText != string.Empty)
            {
                InfoText += Environment.NewLine;
            }
            InfoText += "[" + DateTime.Now.ToLongTimeString() + "] " + text;
            NotifyPropertyChanged(nameof(InfoText));
        }

        private void ClearInfoTextExecute(object obj)
        {
            InfoText = string.Empty;
            NotifyPropertyChanged(nameof(InfoText));
        }

        private void CloseApplicationExecute(object obj)
        {
            Application.Current.MainWindow.Close();
        }

        void NotifyPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        void MainViewModel_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(ModuleLocator.InfoText))
            {
                AppendInfoText(_moduleLocator.InfoText);
            }
        }
    }
}
