﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the DIReadContinuous project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using CommonSettings.Utility;
using DIReadContinuous.ALLDAQ;
using DIReadContinuous.Utility;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace DIReadContinuous
{
    public partial class FormMain : Form
    {
        private const int _idxChartTypeLine = 0;
        private const int _idxChartTypeLineAndMarkers = 1;
        private const int _idxChartTypePoint = 2;
        private const int _idxXAxisLabelsTime = 0;

        private const string _txtInfinite = "Infinite";
        private const string _txtStartingAcquisition = "Starting acquisition...";
        private const string _txtAcquisitionStarted = "Acquisition started.";
        private const string _txtErrorOccurred = "Error occurred.";
        private const string _txtReady = "Ready";
        private const string _txtInvalidInput = "Invalid input";

        private ModuleLocator _moduleLocator;
        private readonly List<DigitalInputModule> _modules = new List<DigitalInputModule>();
        private DigitalInputModule _selectedModule;
        private bool _isAppClosing;
        private int _selectedModuleItemIndex = -1;
        private string _imgInitialDirectory = string.Empty;
        private bool _isSampleFrequencyValid;
        private bool _isNumberOfValuesValid;

        private delegate void UpdatePropertyDelegate(PropertyType changedProperty);
        private UpdatePropertyDelegate _updatePropertyDelegate;
        private delegate void DisplayChartValuesDelegate();
        private DisplayChartValuesDelegate _updateChartDelegate;

        public FormMain()
        {
            InitializeComponent();
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            EnableUI(false);
            _updatePropertyDelegate = new UpdatePropertyDelegate(UpdateProperty);
            _updateChartDelegate = new DisplayChartValuesDelegate(UpdateChartGraphic);
            InitChart();
            try
            {
                _moduleLocator = new ModuleLocator(_modules, OnModuleInfoTextChanged);
                _moduleLocator.FindModules();
                FillModuleCombobox();
                SelectFirstComboboxItem(coBoxModules);
                SelectFirstComboboxItem(coBoxChartXAxisLabels);
                SelectFirstComboboxItem(coBoxChartSeriesType);
                textBoxInfo.AppendText(string.Format("Number of clock controlled digital input modules found: {0}" + Environment.NewLine, _modules.Count));
            }
            catch (DllNotFoundException ex)
            {
                textBoxInfo.AppendText("The device driver is not properly installed." + Environment.NewLine);
                textBoxInfo.AppendText(ex.Message + Environment.NewLine);
                tabControlMain.SelectedTab = tabPageInfo;
            }
        }

        private void FillModuleCombobox()
        {
            coBoxModules.Items.Clear();
            foreach (DigitalInputModule module in _modules)
            {
                coBoxModules.Items.Add(module.ModuleName);
            }
        }

        private void SelectFirstComboboxItem(ComboBox coBox)
        {
            if (coBox.Items.Count > 0)
            {
                coBox.SelectedIndex = 0;
            }
        }

        private void CoBoxModules_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (coBoxModules.SelectedIndex == _selectedModuleItemIndex) return;

            _selectedModuleItemIndex = coBoxModules.SelectedIndex;
            ClearDisplayedValues();
            chart.Titles.Clear();
            EnableUI(coBoxModules.SelectedIndex >= 0);
            btnStart.Enabled = coBoxModules.SelectedIndex >= 0;
            if(_selectedModule != null)
            {
                _selectedModule.InfoTextChanged -= OnModuleInfoTextChanged;
                _selectedModule.ModulePropertyChanged -= OnModulePropertyChanged;
                _selectedModule.ClearModuleResources();
            }

            if(coBoxModules.SelectedIndex < 0)
            {
                _selectedModule = null;
                textBoxSampleFrequency.Text = string.Empty;
                textBoxValuesToDisplay.Text = string.Empty;
            }
            else
            {
                _selectedModule = _modules[coBoxModules.SelectedIndex];
                _selectedModule.InfoTextChanged += OnModuleInfoTextChanged;
                _selectedModule.ModulePropertyChanged += OnModulePropertyChanged;
                SetStopAfterValuesText(_selectedModule.StopAfterValues);
                textBoxSampleFrequency.Text = string.Format("{0:0.###}", _selectedModule.DesiredFrequency);
                textBoxValuesToDisplay.Text = _selectedModule.DisplayNumberOfValues.ToString();
            }
            textBoxActualSampleFrequency.Text = string.Empty;
            textBoxValuesAcquired.Text = string.Empty;
            textBoxValuesDisplayed.Text = string.Empty;
            textBoxValuesSkipped.Text = string.Empty;
            FillBits(_selectedModule);
        }

        private void SetStopAfterValuesText(ulong stopAfterValues)
        {
            coBoxStopAfterValues.Text = (stopAfterValues == AdqConstants.ADQ_NUMBER_UNKNOWN) ? _txtInfinite : stopAfterValues.ToString();
        }

        private void ListView1_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            if (_selectedModule.ModuleIsBusy)
            {
                e.NewValue = e.CurrentValue;
                return;
            }

            int sCount = listView1.CheckedItems.Count;
            if (e.NewValue == CheckState.Checked) { ++sCount; }
            if (e.NewValue == CheckState.Unchecked) { --sCount; }
            textBoxSelectedBits.Text = sCount.ToString();
            _selectedModule.BitsSelected = sCount;
        }

        private void FillBits(DigitalInputModule module)
        {
            listView1.Items.Clear();
            if(module == null)
            {
                var lvItem = new ListViewItem
                {
                    Checked = false,
                    Text = "No selection available"
                };
                listView1.Enabled = false;
                listView1.Items.Add(lvItem);
                return;
            }

            listView1.Enabled = true;
            for (uint index = 0; index < module.PortWidthBits; index++)
            {
                var lvItem = new ListViewItem
                {
                    Checked = false,
                    Text = string.Format("Bit {0}", index),
                    ForeColor = AdqCommon.BitColors[index % AdqCommon.BitColors.Length]
                };
                listView1.Items.Add(lvItem);            
            }

            int idx = 0;
            while (idx < listView1.Items.Count && idx < 2)
            {
                listView1.Items[idx].Checked = true;
                idx++;
            }
        }

        private void UpdateBits()
        {
            if(_selectedModule != null)
            {
                textBoxSelectedBits.Text = string.Format("{0}", listView1.CheckedItems.Count);
                _selectedModule.BitsSelected = listView1.CheckedItems.Count;
            }
        }

        private void OnModuleInfoTextChanged(object sender, InfoTextChangedEventArgs e)
        {
            if (_isAppClosing) return;
            AddInfoTextLine(e.InfoText);
        }

        private void AddInfoTextLine(string text)
        {
            textBoxInfo.AppendText(string.Format("[{0}] {1}" + Environment.NewLine,
                DateTime.Now.ToString("HH:mm:ss.fffffff"), text));
        }

        private void OnModulePropertyChanged(object sender, ModulePropertyChangedEventArgs e)
        {
            if (_isAppClosing) return;

            if (e.ChangedProperty == PropertyType.DisplayBuffer)
            {
                BeginInvoke(_updateChartDelegate);
            }
            else
            {
                BeginInvoke(_updatePropertyDelegate, new object[] { e.ChangedProperty });
            }
        }

        private void UpdateProperty(PropertyType changedProperty)
        {
            if (changedProperty == PropertyType.UpdateNumberOfValues)
            {
                UpdateNumberOfValues();
            }
            else if (changedProperty == PropertyType.ActualSampleFrequency)
            {
                UpdateActualSampleFrequencyAndDisplayWidth();
            }
            else if (changedProperty == PropertyType.UpdateValuesToDisplay)
            {
                UpdateValuesToDisplay();
            }
            else if (changedProperty == PropertyType.AcquisitionStarting)
            {
                UpdateAcquisitionStarting();
            }
            else if (changedProperty == PropertyType.AcquisitionStarted)
            {
                UpdateAcquisitionStarted();
            }
            else if (changedProperty == PropertyType.AcquisitionEnded)
            {
                AcquisitionEnded();
            }
            else if (changedProperty == PropertyType.AcquisitionErrorOccurred)
            {
                UpdateAcquisitionErrorOccured();
            }
        }

        private void UpdateActualSampleFrequencyAndDisplayWidth()
        {
            textBoxActualSampleFrequency.Text = AdqCommon.FormatFrequency(_selectedModule.ActualSampleFrequency);
            textBoxMinDisplayWidth.Text = AdqCommon.FormatSampleTime(_selectedModule.ActualSampleTime * _selectedModule.DisplayNumberOfValues);
        }

        private void UpdateNumberOfValues()
        {
            textBoxValuesAcquired.Text = _selectedModule.TotalValuesAcquired.ToString("N0");
            textBoxValuesDisplayed.Text = _selectedModule.TotalValuesDisplayed.ToString("N0");
            textBoxValuesSkipped.Text = _selectedModule.TotalValuesSkipped.ToString("N0");
            progressUpdateTimer.Start();
        }

        private void UpdateValuesToDisplay()
        {
            textBoxValuesToDisplay.Text = _selectedModule.DisplayNumberOfValues.ToString();
        }

        private void UpdateAcquisitionStarting()
        {
            AddInfoTextLine(_txtStartingAcquisition);
            lblAcquisitionStatus.BackColor = Color.LimeGreen;
            lblAcquisitionStatus.Text = _txtStartingAcquisition;
        }

        private void UpdateAcquisitionStarted()
        {
            AddInfoTextLine(_txtAcquisitionStarted);
            lblAcquisitionStatus.BackColor = Color.Lime;
            lblAcquisitionStatus.Text = _txtAcquisitionStarted;
        }

        private void UpdateAcquisitionErrorOccured()
        {
            lblAcquisitionStatus.BackColor = Color.OrangeRed;
            lblAcquisitionStatus.Text = _txtErrorOccurred;
        }

        private void AcquisitionEnded()
        {
            progressUpdateTimer.Stop();
            if (_isAppClosing) return;

            if (!_selectedModule.ModuleIsBusy)
            {
                ResetUiStatus();
            }
        }

        private void TextBoxSampleFrequency_TextChanged(object sender, EventArgs e)
        {
            if (_selectedModule == null) return;

            _isSampleFrequencyValid = double.TryParse(textBoxSampleFrequency.Text, out double sampleFrequency);
            if (!_isSampleFrequencyValid || sampleFrequency <= 0.0 || sampleFrequency > _selectedModule.MaxFrequency)
            {
                _isSampleFrequencyValid = false;
                _selectedModule.DesiredFrequency = 0.0;
                return;
            }
            _selectedModule.DesiredFrequency = sampleFrequency;
            UpdateMinDisplayWidth();
        }

        private void TextBoxValuesToDisplay_TextChanged(object sender, EventArgs e)
        {
            if (_selectedModule == null) return;

            _isNumberOfValuesValid = uint.TryParse(textBoxValuesToDisplay.Text, out uint numberOfValuesToDisplay);
            if (!_isNumberOfValuesValid || numberOfValuesToDisplay == 0)
            {
                _isNumberOfValuesValid = false;
                _selectedModule.DisplayNumberOfValues = 0;
                return;
            }
            _selectedModule.DisplayNumberOfValues = numberOfValuesToDisplay;
            UpdateMinDisplayWidth();
        }

        private void UpdateMinDisplayWidth()
        {
            if (!_isSampleFrequencyValid || !_isNumberOfValuesValid)
            {
                textBoxMinDisplayWidth.Text = _txtInvalidInput;
                return;
            }

            double sampleTime = 1.0 / _selectedModule.DesiredFrequency * _selectedModule.DisplayNumberOfValues;
            textBoxMinDisplayWidth.Text = AdqCommon.FormatSampleTime(sampleTime);
        }

        private void CoBoxStopAfterValues_Leave(object sender, EventArgs e)
        {
            if (_selectedModule == null) return;
            _selectedModule.StopAfterValues = ParseStopAfterValues(coBoxStopAfterValues.Text);
        }

        private ulong ParseStopAfterValues(string stopAfterValuesText)
        {
            if (stopAfterValuesText == _txtInfinite)
            {
                return AdqConstants.ADQ_NUMBER_UNKNOWN;
            }
            bool isValid = ulong.TryParse(stopAfterValuesText, out ulong numValues);
            if (!isValid || numValues <= 0)
            {
                coBoxStopAfterValues.Text = _txtInfinite;
                return AdqConstants.ADQ_NUMBER_UNKNOWN;
            }
            return numValues;
        }

        private void EnableUI(bool enabled)
        {
            coBoxModules.Enabled = enabled;
            textBoxSampleFrequency.Enabled = enabled;
            textBoxValuesToDisplay.Enabled = enabled;
            coBoxStopAfterValues.Enabled = enabled;
            coBoxChartSeriesType.Enabled = enabled;
            coBoxChartXAxisLabels.Enabled = enabled;
            btnReset.Enabled = enabled;
            btnClear.Enabled = enabled;
        }

        private void ResetUiStatus()
        {
            EnableUI(true);
            btnStart.Text = "Start";
            lblAcquisitionStatus.BackColor = Color.White;
            lblAcquisitionStatus.Text = _txtReady;
        }

        private void BtnReset_Click(object sender, EventArgs e)
        {
            if (_selectedModule != null)
            {
                _selectedModule.Reset();
            }
        }

        private void BtnClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void BtnClear_Click(object sender, EventArgs e)
        {
            ClearDisplayedValues();
        }

        private void ClearDisplayedValues()
        {
            chart.Series.Clear();
            textBoxInfo.Clear();
            textBoxValuesAcquired.Text = string.Empty;
            textBoxValuesSkipped.Text = string.Empty;
            textBoxValuesDisplayed.Text = string.Empty;
        }

        private void FormMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (_moduleLocator != null)
            {
                _moduleLocator.CloseAdqDriver();
            }
        }

        private void FormMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_selectedModule != null && _selectedModule.ModuleIsBusy)
            {
                DialogResult result = MessageBox.Show("The acquisition is still running."
                                         + Environment.NewLine
                                         + "Do you want to close the program?",
                                         "Module busy",
                                         MessageBoxButtons.YesNo,
                                         MessageBoxIcon.Warning);
                if (result == DialogResult.Yes)
                {
                    _isAppClosing = true;
                    _selectedModule.RequestValuesToDisplay = false;
                    progressUpdateTimer.Stop();
                    graphicUpdateTimer.Stop();
                    _selectedModule.InfoTextChanged -= OnModuleInfoTextChanged;
                    _selectedModule.ModulePropertyChanged -= OnModulePropertyChanged;
                    _selectedModule.StopAcquisition();
                    e.Cancel = false;
                }
                else
                {
                    e.Cancel = true;
                }
                return;
            }
            _isAppClosing = true;
        }

        private void BtnStartStop_Click(object sender, EventArgs e)
        {
            if (_selectedModule == null) return;
            if (_selectedModule.ModuleIsBusy)
            {
                _selectedModule.CancelAcquisition();
                UpdateBits();
                return;
            }

            ClearDisplayedValues();
            EnableUI(false);
            btnStart.Text = "Stop";
            try
            {
                _selectedModule.InitAcquisition();
                UpdateChart();
                _selectedModule.RequestValuesToDisplay = true;
                _selectedModule.RequestUpdateNumberOfValues = true;
                _selectedModule.StartAcquisition();
            }
            catch (Exception ex)
            {
                ResetUiStatus();
                AddInfoTextLine(ex.Message);
                tabControlMain.SelectedTab = tabPageInfo;
            }
        }

        private void BtnInfo_Click(object sender, EventArgs e)
        {
            var aboutBox = new AboutBox();
            aboutBox.ShowDialog();
        }

        private void ProgressUpdateTimer_Tick(object sender, EventArgs e)
        {
            if (!_selectedModule.RequestUpdateNumberOfValues)
            {
                progressUpdateTimer.Stop();
                _selectedModule.RequestUpdateNumberOfValues = true;
            }
        }

        private void GraphicUpdateTimer_Tick(object sender, EventArgs e)
        {
            graphicUpdateTimer.Stop();
            _selectedModule.RequestValuesToDisplay = true;
        }

        private void InitChart()
        {
            chart.BackColor = AdqCommon.DefaultBackgroundColor;
            chart.Update();
        }

        private void UpdateChart()
        {
            chart.Legends.Clear();
            chart.Series.Clear();
            chart.ChartAreas.Clear();
            InitChartXAxisTitle();
            int position = 0;
            foreach(ListViewItem lvItem in listView1.Items)
            {
                if (lvItem.Checked)
                {
                    AddGraph(lvItem, listView1.CheckedItems.Count, position);
                    position++;
                }
            }
        }

        private void AddGraph(ListViewItem lvItem, int numberOfChartAreas, int position)
        {
            var legend = new Legend
            {
                Name = "legend" + lvItem.Index.ToString(),
                ForeColor = lvItem.ForeColor,
                IsDockedInsideChartArea = true,
                Docking = Docking.Top,
                BackColor = Color.Transparent
            };
            chart.Legends.Add(legend);

            var chartarea = new ChartArea
            {
                Name = "chartarea" + lvItem.Index.ToString(),
                BackColor = AdqCommon.DefaultBackgroundColor
            };
            chartarea.AxisY.Maximum = 1.05;
            chartarea.AxisY.Interval = 1;
            chartarea.CursorX.IsUserSelectionEnabled = true;
            chartarea.CursorY.IsUserSelectionEnabled = true;
            chartarea.AxisX.MajorGrid.LineColor = AdqCommon.DefaultAxisColor;
            chartarea.AxisX.MajorTickMark.LineColor = AdqCommon.DefaultAxisColor;
            chartarea.AxisX.MinorGrid.LineColor = AdqCommon.DefaultAxisColor;
            chartarea.AxisX.MinorTickMark.LineColor = AdqCommon.DefaultAxisColor;
            chartarea.AxisX.LineColor = Color.Transparent;
            chartarea.AxisX.LineWidth = 1;
            chartarea.AxisX.LabelStyle.ForeColor = AdqCommon.DefaultLabelColor;
            chartarea.AxisX.TitleForeColor = AdqCommon.DefaultLabelColor;
            chartarea.AxisX.ScrollBar.ButtonColor = AdqCommon.DefaultScrollBarColor;
            chartarea.AxisX.ScrollBar.BackColor = AdqCommon.DefaultScrollBarColor;
            chartarea.AxisY.MajorGrid.LineColor = AdqCommon.DefaultAxisColor;
            chartarea.AxisY.MajorTickMark.LineColor = AdqCommon.DefaultAxisColor;
            chartarea.AxisY.MinorGrid.LineColor = AdqCommon.DefaultAxisColor;
            chartarea.AxisY.MinorTickMark.LineColor = AdqCommon.DefaultAxisColor;
            chartarea.AxisY.LineColor = AdqCommon.DefaultAxisColor;
            chartarea.AxisY.LineWidth = 1;
            chartarea.AxisY.LabelStyle.ForeColor = AdqCommon.DefaultLabelColor;
            chartarea.AxisY.TitleForeColor = AdqCommon.DefaultLabelColor;
            chartarea.AxisY.ScrollBar.ButtonColor = AdqCommon.DefaultScrollBarColor;
            chartarea.AxisY.ScrollBar.BackColor = AdqCommon.DefaultScrollBarColor;
            if (radioButtonVertical.Checked)
            {
                chartarea.Position.Auto = false;
                chartarea.Position.X = 3;
                chartarea.Position.Y = 100.0f / numberOfChartAreas * position;
                chartarea.Position.Height = 100.0f / numberOfChartAreas;
                chartarea.Position.Width = 95;
            }
            else if (radioButtonAuto.Checked)
            {
                chartarea.Position.Auto = true;
            }
            chart.ChartAreas.Add(chartarea);
            legend.DockedToChartArea = chartarea.Name;

            var series = new Series
            {
                Name = "series" + lvItem.Index.ToString(),
                Tag = lvItem.Index,
                YValueType = ChartValueType.Int32,
                XValueType = ChartValueType.String,
                IsXValueIndexed = false,
                Legend = legend.Name,
                LegendText = lvItem.Text,
                Color = AdqCommon.BitColors[lvItem.Index % AdqCommon.BitColors.Length]
            };
            switch (coBoxChartSeriesType.SelectedIndex)
            {
                case _idxChartTypeLine:
                    series.ChartType = SeriesChartType.StepLine;
                    break;
                case _idxChartTypeLineAndMarkers:
                    series.ChartType = SeriesChartType.StepLine;
                    series.MarkerStyle = MarkerStyle.Circle;
                    break;
                case _idxChartTypePoint:
                    series.ChartType = SeriesChartType.Point;
                    series.MarkerStyle = MarkerStyle.Circle;
                    break;
                default:
                    series.ChartType = SeriesChartType.StepLine;
                    series.MarkerStyle = MarkerStyle.Circle;
                    break;
            }
            for (uint i = 0; i < _selectedModule.DisplayBufferNumberOfValues; i++)
            {
                series.Points.AddY(0);
                series.Points[series.Points.Count - 1].AxisLabel = GetXAsixLabel(0);
            }
            series.ChartArea = chartarea.Name;
            chart.Series.Add(series);
        }

        private void InitChartXAxisTitle()
        {
            chart.Titles.Clear();
            var xAxisTitle = new Title(coBoxChartXAxisLabels.Text);
            xAxisTitle.Position.Auto = false;
            xAxisTitle.Position.X = 3;
            xAxisTitle.Position.Y = 95;
            xAxisTitle.Position.Width = 94;
            xAxisTitle.Position.Height = 5;
            xAxisTitle.Alignment = ContentAlignment.BottomCenter;
            xAxisTitle.Docking = Docking.Bottom;
            xAxisTitle.IsDockedInsideChartArea = false;
            xAxisTitle.ForeColor = AdqCommon.DefaultLabelColor;
            chart.Titles.Add(xAxisTitle);
        }

        private string GetXAsixLabel(double xValue)
        {
            if (coBoxChartXAxisLabels.SelectedIndex == _idxXAxisLabelsTime)
            {
                return AdqCommon.FormatSampleTime(xValue);
            }
            return ((int)xValue).ToString();
        }

        private double GetXValueDelta()
        {
            return (coBoxChartXAxisLabels.SelectedIndex == _idxXAxisLabelsTime) ? _selectedModule.ActualSampleTime : 1.0;
        }

        private double GetXValueBufferStart(uint numberOfValuesReturned)
        {
            return (coBoxChartXAxisLabels.SelectedIndex == _idxXAxisLabelsTime) ?
                (_selectedModule.TotalValuesAcquired - numberOfValuesReturned) * _selectedModule.ActualSampleTime :
                _selectedModule.TotalValuesAcquired - numberOfValuesReturned;
        }

        private void UpdateChartGraphic()
        {
            DisplayChartValues();
            graphicUpdateTimer.Start();
        }

        private void DisplayChartValues()
        {
            uint[] displayValuesBuffer = _selectedModule.NewValuesDisplayBuffer;
            uint numberOfValuesReturned = _selectedModule.NumberOfValidValues;
            double xValueBufferStart = GetXValueBufferStart(numberOfValuesReturned);
            double xValueDelta = GetXValueDelta();
            chart.SuspendLayout();
            foreach (Series series in chart.Series)
            {
                double xValue = xValueBufferStart;
                for (uint i = 0; i < numberOfValuesReturned; i++)
                {
                    series.Points.RemoveAt(0);
                    series.Points.AddY(GetBitValue((int)series.Tag, displayValuesBuffer[i]));
                    series.Points[series.Points.Count - 1].AxisLabel = GetXAsixLabel(xValue);
                    xValue += xValueDelta;
                }
            }
            chart.ResumeLayout();
        }

        private int GetBitValue(int bitNumber, uint value)
        {
            return 1 & ((int)value >> bitNumber);
        }

        private string GetImageFileName(DigitalInputModule module, string fileExt)
        {
            string s = string.Empty;
            if (module != null)
            {
                s = string.Format("{0}, Board {1}, Module {2}, ",
                    module.ModelName,
                    module.BoardIndex,
                    module.ModuleIndex);
            }
            return s + DateTime.Now.ToString("yyyy-MM-dd HH.mm.ss") + fileExt;
        }

        private ChartImageFormat GetImageFormat(int filterIndex)
        {
            ChartImageFormat imgFormat;
            switch (filterIndex)
            {
                case 1:
                    imgFormat = ChartImageFormat.Png;
                    break;
                case 2:
                    imgFormat = ChartImageFormat.Gif;
                    break;
                case 3:
                    imgFormat = ChartImageFormat.Bmp;
                    break;
                case 4:
                    imgFormat = ChartImageFormat.Tiff;
                    break;
                case 5:
                    imgFormat = ChartImageFormat.Jpeg;
                    break;
                default:
                    imgFormat = ChartImageFormat.Png;
                    break;
            }
            return imgFormat;
        }

        private void TsMenuItemSaveGraphic_Click(object sender, EventArgs e)
        {
            saveFileDialog1.FileName = GetImageFileName(_selectedModule, ".png");
            saveFileDialog1.InitialDirectory = _imgInitialDirectory;
            saveFileDialog1.Filter = "PNG files (*.png)|*.png|Gif files (*.gif)|*.gif|BMP files (*.bmp)|*.bmp|Tiff files (*.tif)|*.tif|Jpeg files (*.jpg)|*.jpg";
            DialogResult result = saveFileDialog1.ShowDialog();
            if (result == DialogResult.OK)
            {
                _imgInitialDirectory = Path.GetDirectoryName(saveFileDialog1.FileName);
                ChartImageFormat imgFormat = GetImageFormat(saveFileDialog1.FilterIndex);
                chart.SaveImage(saveFileDialog1.FileName, imgFormat);
            }
        }

        private void TsMenuItemCopyToClipboard_Click(object sender, EventArgs e)
        {
            using (MemoryStream ms = new MemoryStream())
            {
                chart.SaveImage(ms, ChartImageFormat.Png);
                var bitmap = new Bitmap(ms);
                Clipboard.SetImage(bitmap);
            }
        }

        private void ToolStripMenuItemSelectAll_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < listView1.Items.Count; i++)
            {
                listView1.Items[i].Checked = true;
            }
        }

        private void ToolStripMenuItemInvertSelection_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < listView1.Items.Count; i++)
            {
                listView1.Items[i].Checked = !listView1.Items[i].Checked;
            }
        }

        private void ToolStripMenuItemSelectNone_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < listView1.Items.Count; i++)
            {
                listView1.Items[i].Checked = false;
            }
        }
    }
}