﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the FINRead project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using FINRead.Utility;
using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace FINRead.Models
{
    public class ModuleLocator : IDisposable, INotifyPropertyChanged
    {
        private bool _disposed;
        private uint _adqOpenError = AdqCommon.NotOpen;

        public event PropertyChangedEventHandler PropertyChanged;

        public ModuleLocator(PropertyChangedEventHandler eventHandler)
        {
            PropertyChanged += eventHandler;
        }

        public void InitModuleLocator()
        {
            try
            {
                _adqOpenError = OpenAdqDriver();
                if (_adqOpenError != AdqErrors.ADQ_ERROR_NONE) return;

                FindFinModules();
                if (FinModules.Count == 0)
                {
                    InfoText = "No FIN modules found.";
                }
            }
            catch (DllNotFoundException ex)
            {
                InfoText = "The device driver is not properly installed." + Environment.NewLine
                             + ex.Message;
            }
        }

        private uint OpenAdqDriver()
        {
            uint adqError = AdqDriver.adqOpen();
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqOpen", adqError);
            }
            return adqError;
        }

        private void FindFinModules()
        {
            uint numberOfBoards = GetNumberOfBoards();
            for (uint boardIdx = 0; boardIdx < numberOfBoards; boardIdx++)
            {
                uint adqError = AdqDriver.adqGetNumberOfModules(boardIdx, out uint numberOfModules);
                if (adqError != AdqErrors.ADQ_ERROR_NONE)
                {
                    SetFunctionErrorInfo("adqGetNumberOfModules", adqError);
                    continue;
                }
                GetFinModuleTypes(boardIdx, numberOfModules);
            }
        }

        private uint GetFinModuleTypes(uint boardIndex, uint numberOfModules)
        {
            uint adqError = AdqErrors.ADQ_ERROR_NONE;
            for (uint moduleIdx = 0; moduleIdx < numberOfModules; moduleIdx++)
            {
                adqError = AdqDriver.adqGetModuleType(boardIndex, moduleIdx, out uint currentModuleType);
                if (adqError != AdqErrors.ADQ_ERROR_NONE)
                {
                    SetFunctionErrorInfo("adqGetModuleType", adqError);
                    continue;
                }

                if (currentModuleType == AdqConstants.ADQ_MODULE_TYPE_FIN)
                {
                    string boardName = GetBoardName(boardIndex);
                    var finModule = new FinModule(boardName, boardIndex, moduleIdx);
                    FinModules.Add(finModule);
                }
            }

            return adqError;
        }

        public List<FinModule> FinModules { get; } = new List<FinModule>();

        private uint GetNumberOfBoards()
        {
            uint adqError = AdqDriver.adqGetNumberOfBoards(out uint numBoards);
            if (_adqOpenError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetNumberOfBoards", adqError);
                return 0;
            }
            return numBoards;
        }

        private string GetBoardName(uint boardIndex)
        {
            string s = string.Empty;
            var boardInfo = new AdqDriver.ADQBoardInfo(AdqDriver.ADQBoardInfo_ID);

            uint adqError = AdqDriver.adqGetBoardInfo(boardIndex, ref boardInfo);
            if (adqError == AdqErrors.ADQ_ERROR_NONE)
            {
                s = boardInfo.BoardName;
            }
            return s;
        }

        private void CloseAdqDriver()
        {
            if (_adqOpenError == AdqErrors.ADQ_ERROR_NONE)
            {
                AdqDriver.adqClose();
                _adqOpenError = AdqCommon.NotOpen;
            }
        }

        private void SetFunctionErrorInfo(string functionName, uint adqError)
        {
            InfoText = string.Format("API-function {0} returned Error number: 0x{1:X}", functionName, adqError);
        }

        private string _infoText;
        public string InfoText
        {
            get { return _infoText; }
            private set
            {
                _infoText = value;
                NotifyPropertyChanged(nameof(InfoText));
            }
        }

        void NotifyPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        ~ModuleLocator()
        {
            Dispose(false);
        }

        public void Dispose()
        {
            Dispose(true);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (!_disposed)
            {
                _disposed = true;
                CloseAdqDriver();
            }
        }
    }
}
