﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the FINRead project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using FINRead.Models;
using FINRead.Utility;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows;

namespace FINRead.ViewModels
{
    public class MainViewModel : INotifyPropertyChanged, IDataErrorInfo
    {
        private readonly Dictionary<string, string> _errorInfo = new Dictionary<string, string>();
        private ModuleLocator _moduleLocator;

        public event PropertyChangedEventHandler PropertyChanged;

        public RelayCommand ResetModuleCommand { get; private set; }
        public RelayCommand FinReadCommand { get; private set; }
        public RelayCommand ClearInfoTextCommand { get; private set; }
        public RelayCommand CloseCommand { get; private set; }

        public MainViewModel()
        {
            InitCommands();
        }

        private void InitCommands()
        {
            ResetModuleCommand = new RelayCommand(ResetModuleExecute, CanResetModuleExecute);
            FinReadCommand = new RelayCommand(FinReadExecute, CanFinReadExecute);
            ClearInfoTextCommand = new RelayCommand(ClearInfoTextExecute);
            CloseCommand = new RelayCommand(CloseApplicationExecute);
        }

        internal void Init()
        {
            _moduleLocator = new ModuleLocator(MainViewModel_PropertyChanged);
            _moduleLocator.InitModuleLocator();
            if (FinModules.Count > 0)
            {
                SelectedModule = FinModules[0];
            }
        }

        public List<FinModule> FinModules
        {
            get { return _moduleLocator.FinModules; }
        }

        private FinModule _selectedModule;
        public FinModule SelectedModule
        {
            get { return _selectedModule; }
            set
            {
                if (_selectedModule == value) return;
                if (_selectedModule != null)
                {
                    _selectedModule.PropertyChanged -= MainViewModel_PropertyChanged;
                }
                _selectedModule = value;

                if (_selectedModule != null)
                {
                    _selectedModule.PropertyChanged += MainViewModel_PropertyChanged;
                    FinTimeoutText = _selectedModule.TimeoutMilliseconds.ToString();
                }

                NotifyPropertyChanged(nameof(SelectedModule));
                ResetModuleCommand.RaiseCanExecuteChanged();
                FinReadCommand.RaiseCanExecuteChanged();
            }
        }

        private string _finTimeoutText = string.Empty;
        public string FinTimeoutText
        {
            get { return _finTimeoutText; }
            set
            {
                if (_selectedModule == null) return;
                if (_finTimeoutText != value)
                {
                    _finTimeoutText = value;
                    string msg = ValidateTimeoutValue(_finTimeoutText, out uint timeoutValue);
                    if (!string.IsNullOrEmpty(msg))
                    {
                        _errorInfo.AddOrUpdate(nameof(FinTimeoutText), msg);
                    }
                    else
                    {
                        _selectedModule.TimeoutMilliseconds = timeoutValue;
                        _errorInfo.Remove(nameof(FinTimeoutText));
                    }

                    NotifyPropertyChanged(nameof(FinTimeoutText));
                    FinReadCommand.RaiseCanExecuteChanged();
                }
            }
        }

        private string ValidateTimeoutValue(string textValue, out uint timeout)
        {
            timeout = 0;
            string msg = string.Empty;
            if (string.IsNullOrEmpty(textValue))
            {
                msg = "The value cannot be empty";
            }
            else
            {
                if (!uint.TryParse(textValue, out timeout))
                {
                    msg = "The entered text \"" + textValue + "\" is not a valid number";
                }
            }
            return msg;
        }

        private bool CanFinReadExecute(object obj)
        {
            return _selectedModule != null && !_selectedModule.IsModuleBusy && _errorInfo.Count == 0;
        }

        private async void FinReadExecute(object obj)
        {
            AppendInfoText(string.Format("Starting FIN Read operation, time-out {0} milliseconds...", _selectedModule.TimeoutMilliseconds));
            FinReadData finData = await _selectedModule.FinReadAsync();
            UpdateFinReadMeasuredValues(finData);
        }

        private void UpdateFinReadMeasuredValues(FinReadData finData)
        {
            if (finData == null) return;

            if (finData.HasError)
            {
                AppendInfoText(finData.ErrorText);
                return;
            }

            string s = "FIN Read measured values:"
                + Environment.NewLine
                + string.Format("- Frequency: {0:0.########} Hz", finData.Frequency);

            if (finData.Frequency > 0)
            {
                double period = 1 / finData.Frequency;
                s += Environment.NewLine;
                s += string.Format("- Period: {0:0.#########} sec", period);
                s += Environment.NewLine;
                s += string.Format("- High-time: {0:0.#########} sec", finData.HighTime);
                s += Environment.NewLine;
                s += string.Format("- Low-time: {0:0.#########} sec", period - finData.HighTime);
                s += Environment.NewLine;
                s += string.Format("- Duty cycle: {0:0.#########} %", (finData.HighTime / period) * 100.0);
            }
            AppendInfoText(s);
        }

        private bool CanResetModuleExecute(object obj)
        {
            return _selectedModule != null;
        }

        private void ResetModuleExecute(object obj)
        {
            bool succeeded = _selectedModule.ResetModule();
            if (succeeded)
            {
                AppendInfoText(string.Format("Reset board {0}, module {1}", _selectedModule.BoardIndex, _selectedModule.ModuleIndex));
            }
        }

        public bool CanSelectFinModule
        {
            get { return (_selectedModule == null && FinModules.Count > 0) || (_selectedModule != null && !_selectedModule.IsModuleBusy); }
        }

        private void ClearInfoTextExecute(object obj)
        {
            InfoText = string.Empty;
            NotifyPropertyChanged(nameof(InfoText));
        }

        private void AppendInfoText(string text)
        {
            if (InfoText != string.Empty)
            {
                InfoText += Environment.NewLine;
            }
            InfoText += DateTime.Now.ToString("[HH:mm:ss.fff] ") + text;
            NotifyPropertyChanged(nameof(InfoText));
        }

        public string InfoText { get; private set; } = string.Empty;

        public string Error => "MainViewModel.Error";

        public string this[string propertyName]
        {
            get
            {
                return _errorInfo.ContainsKey(propertyName) ? _errorInfo[propertyName] : null;
            }
        }

        void NotifyPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        void MainViewModel_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(_moduleLocator.InfoText))
            {
                AppendInfoText(_moduleLocator.InfoText);
            }
            else if (e.PropertyName == nameof(_selectedModule.ModuleInfoText))
            {
                AppendInfoText(_selectedModule.ModuleInfoText);
            }
            else if (e.PropertyName == nameof(_selectedModule.IsModuleBusy))
            {
                FinReadCommand.RaiseCanExecuteChanged();
                NotifyPropertyChanged(nameof(CanSelectFinModule));
            }
        }

        private void CloseApplicationExecute(object obj)
        {
            Application.Current.MainWindow.Close();
        }

        internal void OnWindowClosing(object sender, CancelEventArgs e)
        {
            if (_selectedModule != null && _selectedModule.IsModuleBusy)
            {
                _selectedModule.PropertyChanged -= MainViewModel_PropertyChanged;
                _selectedModule.ResetModule();
            }
        }
    }
}
