﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the I2CReadWrite project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ_DLL.Devices.Boards;
using I2CReadWrite.Utility;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace I2CReadWrite.Models
{
    public class ModuleLocator : IDisposable, INotifyPropertyChanged
    {
        private bool _disposed;
        private uint _adqOpenError = AdqCommon.NotOpen;

        public event PropertyChangedEventHandler PropertyChanged;

        public ModuleLocator(PropertyChangedEventHandler eventHandler)
        {
            PropertyChanged += eventHandler;
        }

        public void InitModuleLocator()
        {
            try
            {
                _adqOpenError = OpenAdqDriver();
                if (_adqOpenError != AdqErrors.ADQ_ERROR_NONE)
                {
                    return;
                }

                FindI2CModules();
                if (I2cModules.Count == 0)
                {
                    InfoText = "No I2C modules found.";
                }
            }
            catch (DllNotFoundException ex)
            {
                InfoText = "The device driver is not properly installed." + Environment.NewLine
                             + ex.Message;
            }
        }

        private uint OpenAdqDriver()
        {
            uint adqError = AdqDriver.adqOpen();
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqOpen", adqError);
            }
            return adqError;
        }

        private void FindI2CModules()
        {
            uint numberOfBoards = GetNumberOfBoards();
            for (uint boardIdx = 0; boardIdx < numberOfBoards; boardIdx++)
            {
                uint adqError = AdqDriver.adqGetNumberOfModules(boardIdx, out uint numberOfModules);
                if (adqError != AdqErrors.ADQ_ERROR_NONE)
                {
                    SetFunctionErrorInfo("adqGetNumberOfModules", adqError);
                    continue;
                }
                GetModuleTypes(boardIdx, numberOfModules);
            }
        }

        private uint GetModuleTypes(uint boardIndex, uint numberOfModules)
        {
            uint adqError = AdqErrors.ADQ_ERROR_NONE;
            for (uint moduleIdx = 0; moduleIdx < numberOfModules; moduleIdx++)
            {
                adqError = AdqDriver.adqGetModuleType(boardIndex, moduleIdx, out uint currentModuleType);
                if (adqError != AdqErrors.ADQ_ERROR_NONE)
                {
                    SetFunctionErrorInfo("adqGetModuleType", adqError);
                    continue;
                }

                if (currentModuleType == AdqConstants.ADQ_MODULE_TYPE_I2C)
                {
                    GetI2CModuleInfo(boardIndex, moduleIdx);
                }
            }

            return adqError;
        }

        private uint GetI2CModuleInfo(uint boardIndex, uint moduleIndex)
        {
            var adqGeneralModuleInfo = new AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID);
            var adqI2cModuleInfo = new AdqDriver.ADQI2CModuleInfo(AdqDriver.ADQI2CModuleInfo_ID);

            IntPtr moduleInfo = Marshal.AllocHGlobal((int)adqI2cModuleInfo.Size);
            Marshal.StructureToPtr(adqI2cModuleInfo, moduleInfo, false);

            uint adqError = AdqDriver.adqGetModuleInfo(boardIndex, moduleIndex, ref adqGeneralModuleInfo, adqI2cModuleInfo.Size, moduleInfo, out uint infoSize);
            adqI2cModuleInfo = (AdqDriver.ADQI2CModuleInfo)Marshal.PtrToStructure(moduleInfo, typeof(AdqDriver.ADQI2CModuleInfo));
            Marshal.FreeHGlobal(moduleInfo);

            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetModuleInfo", adqError);
                return adqError;
            }

            Debug.Assert(adqI2cModuleInfo.Size == infoSize,
                string.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                boardIndex, moduleIndex, adqI2cModuleInfo.Size, infoSize));

            string boardName = GetBoardName(boardIndex);
            var i2cModule = new I2cModule(boardName, boardIndex, moduleIndex, ref adqGeneralModuleInfo, ref adqI2cModuleInfo);
            I2cModules.Add(i2cModule);

            return adqError;
        }

        public List<I2cModule> I2cModules { get; } = new List<I2cModule>();

        private uint GetNumberOfBoards()
        {
            uint adqError = AdqDriver.adqGetNumberOfBoards(out uint numBoards);
            if (_adqOpenError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetNumberOfBoards", adqError);
                return 0;
            }
            return numBoards;
        }

        private string GetBoardName(uint boardIndex)
        {
            string s = "";
            var boardInfo = new AdqDriver.ADQBoardInfo(AdqDriver.ADQBoardInfo_ID);

            uint adqError = AdqDriver.adqGetBoardInfo(boardIndex, ref boardInfo);
            if (adqError == AdqErrors.ADQ_ERROR_NONE)
            {
                s = boardInfo.BoardName;
            }
            return s;
        }

        private void CloseAdqDriver()
        {
            if (_adqOpenError == AdqErrors.ADQ_ERROR_NONE)
            {
                AdqDriver.adqClose();
                _adqOpenError = AdqCommon.NotOpen;
            }
        }

        private void SetFunctionErrorInfo(string functionName, uint adqError)
        {
            InfoText = string.Format("API-function {0} returned Error number: 0x{1:X}", functionName, adqError);
        }

        private string _infoText;
        public string InfoText
        {
            get
            {
                return _infoText;
            }
            private set
            {
                _infoText = value;
                NotifyPropertyChanged(nameof(InfoText));
            }
        }

        void NotifyPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        ~ModuleLocator()
        {
            Dispose(false);
        }

        public void Dispose()
        {
            Dispose(true);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (!_disposed)
            {
                _disposed = true;
                CloseAdqDriver();
            }
        }
    }
}
