﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the I2CReadWrite project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using I2CReadWrite.Models;
using I2CReadWrite.Utility;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Globalization;
using System.Windows;

namespace I2CReadWrite.ViewModels
{
    public class MainViewModel : INotifyPropertyChanged
    {
        private ModuleLocator _moduleLocator;

        public event PropertyChangedEventHandler PropertyChanged;

        public RelayCommand ResetModuleCommand
        {
            get; private set;
        }
        public RelayCommand SetI2cSlaveAddressCommand
        {
            get; private set;
        }
        public RelayCommand I2cReadCommand
        {
            get; private set;
        }
        public RelayCommand I2cWriteCommand
        {
            get; private set;
        }
        public RelayCommand I2cWriteThenReadCommand
        {
            get; private set;
        }
        public RelayCommand ClearInfoTextCommand
        {
            get; private set;
        }
        public RelayCommand CloseCommand
        {
            get; private set;
        }

        public MainViewModel()
        {
            InitCommands();
        }

        private void InitCommands()
        {
            ResetModuleCommand = new RelayCommand(ResetModuleExecute, CanResetModuleExecute);
            SetI2cSlaveAddressCommand = new RelayCommand(SetI2cSlaveAddressExecute, CanSetI2cSlaveAddressExecute);
            I2cReadCommand = new RelayCommand(I2cReadBytesExecute, CanI2cReadBytesExecute);
            I2cWriteCommand = new RelayCommand(I2cWriteBytesExecute, CanI2cWriteBytesExecute);
            I2cWriteThenReadCommand = new RelayCommand(I2cWriteThenReadExecute, CanI2cWriteThenReadExecute);
            ClearInfoTextCommand = new RelayCommand(ClearInfoTextExecute);
            CloseCommand = new RelayCommand(CloseApplicationExecute);
        }

        public void Init()
        {
            _moduleLocator = new ModuleLocator(MainViewModel_PropertyChanged);
            _moduleLocator.InitModuleLocator();
            if (IsI2cModuleAvailable)
            {
                SelectedModule = I2cModules[0];
            }
        }

        public List<I2cModule> I2cModules
        {
            get
            {
                return _moduleLocator.I2cModules;
            }
        }

        private I2cModule _selectedModule;
        public I2cModule SelectedModule
        {
            get
            {
                return _selectedModule;
            }
            set
            {
                if (_selectedModule == value)
                {
                    return;
                }

                if (_selectedModule != null)
                {
                    _selectedModule.PropertyChanged -= MainViewModel_PropertyChanged;
                }
                _selectedModule = value;

                if (_selectedModule != null)
                {
                    _selectedModule.PropertyChanged += MainViewModel_PropertyChanged;
                }

                ResetModuleCommand.RaiseCanExecuteChanged();
                SetI2cSlaveAddressCommand.RaiseCanExecuteChanged();
                I2cReadCommand.RaiseCanExecuteChanged();
                I2cWriteCommand.RaiseCanExecuteChanged();
                I2cWriteThenReadCommand.RaiseCanExecuteChanged();
                NotifyPropertyChanged(nameof(SelectedModule));
                NotifyPropertyChanged(nameof(ModuleDescription));
            }
        }

        private bool CanResetModuleExecute(object obj)
        {
            return _selectedModule != null;
        }

        private void ResetModuleExecute(object obj)
        {
            bool succeeded = _selectedModule.ResetModule();
            if (succeeded)
            {
                AppendInfoText(string.Format("Reset board {0}, module {1}", _selectedModule.BoardIndex, _selectedModule.ModuleIndex));
            }
        }

        private bool CanSetI2cSlaveAddressExecute(object obj)
        {
            return CanUseI2cModule;
        }

        private void SetI2cSlaveAddressExecute(object obj)
        {
            try
            {
                uint i2cAddressValue = SetNewSlaveAddressValue();
                bool succeeded = _selectedModule.SetSlaveAddress(i2cAddressValue);
                if (succeeded)
                {
                    AppendInfoText(string.Format("Board {0}, module {1}, I2C slave address set to 0x{2:X}",
                        _selectedModule.BoardIndex, _selectedModule.ModuleIndex, i2cAddressValue));
                }
            }
            catch (Exception ex)
            {
                AppendInfoText("Error setting slave address:" + Environment.NewLine + ex.Message);
            }
        }

        private uint SetNewSlaveAddressValue()
        {
            uint newI2cAddressValue = uint.Parse(_newI2cAddress, NumberStyles.AllowHexSpecifier);
            string i2cAddress = newI2cAddressValue.ToString("X");
            if (!_i2cAddresses.Contains(i2cAddress))
            {
                _i2cAddresses.Add(i2cAddress);
            }
            SelectedI2cAddress = i2cAddress;
            return newI2cAddressValue;
        }

        private readonly ObservableCollection<string> _i2cAddresses = new ObservableCollection<string>();
        public IEnumerable<string> I2cAddresses
        {
            get
            {
                return _i2cAddresses;
            }
        }

        private string _selectedI2cAddress;
        public string SelectedI2cAddress
        {
            get
            {
                return _selectedI2cAddress;
            }
            set
            {
                if (_selectedI2cAddress != value)
                {
                    _selectedI2cAddress = value;
                    NotifyPropertyChanged(nameof(SelectedI2cAddress));
                }
            }
        }

        private string _newI2cAddress = string.Empty;
        public string NewI2cAddress
        {
            get
            {
                return _selectedI2cAddress;
            }
            set
            {
                if (!string.IsNullOrEmpty(value))
                {
                    _newI2cAddress = value;
                }
            }
        }

        private bool CanI2cReadBytesExecute(object obj)
        {
            return _selectedModule != null;
        }

        private void I2cReadBytesExecute(object obj)
        {
            bool succeeded = _selectedModule.Read((uint)(_i2cReadNumBytesSelectedIndex + 1));
            if (succeeded)
            {
                DisplayBytes("Read", _selectedModule.BytesRead);
            }
        }

        private void DisplayBytes(string readOrWriteOperation, byte[] bytes)
        {
            string s = GetOperationDescription(readOrWriteOperation, bytes.Length);
            foreach (byte oneByte in bytes)
            {
                s += string.Format("{0:X2} ", oneByte);
            }
            AppendInfoText(s.TrimEnd());
        }

        private static string GetOperationDescription(string operation, int len)
        {
            return operation + " " + len.ToString() + (len == 1 ? " byte:" : " bytes:") + " hex ";
        }

        private int _i2cReadNumBytesSelectedIndex;
        public int I2cReadNumBytesSelectedIndex
        {
            get
            {
                return _i2cReadNumBytesSelectedIndex;
            }
            set
            {
                if (_i2cReadNumBytesSelectedIndex != value)
                {
                    _i2cReadNumBytesSelectedIndex = value;
                    NotifyPropertyChanged(nameof(I2cReadNumBytesSelectedIndex));
                }
            }
        }

        private readonly ObservableCollection<string> _bytesToWrite = new ObservableCollection<string>();
        public IEnumerable<string> BytesToWrite
        {
            get
            {
                return _bytesToWrite;
            }
        }

        private string _selectedBytesToWriteText;
        public string SelectedBytesToWrite
        {
            get
            {
                return _selectedBytesToWriteText;
            }
            set
            {
                if (_selectedBytesToWriteText != value)
                {
                    _selectedBytesToWriteText = value;
                    NotifyPropertyChanged(nameof(SelectedBytesToWrite));
                }
            }
        }

        private string _newBytesToWrite = string.Empty;
        public string NewBytesToWrite
        {
            get
            {
                return _selectedBytesToWriteText;
            }
            set
            {
                _newBytesToWrite = value;
            }
        }

        private bool CanI2cWriteBytesExecute(object obj)
        {
            return _selectedModule != null;
        }

        private void I2cWriteBytesExecute(object obj)
        {
            string bytesToWrite = _newBytesToWrite.Trim();

            if (string.IsNullOrEmpty(bytesToWrite))
            {
                AppendInfoText("Please enter bytes to write.");
                return;
            }

            byte[] bytes = GetByteArrayToWrite(bytesToWrite);
            if (bytes != null)
            {
                WriteBytesToModule(bytesToWrite, bytes);
            }
        }

        private void WriteBytesToModule(string bytesToWrite, byte[] bytes)
        {
            bool succeeded = _selectedModule.Write(bytes);
            if (succeeded)
            {
                UpdateBytesToWriteProperties(bytesToWrite);
                DisplayBytes("Wrote", bytes);
            }
        }

        private byte[] GetByteArrayToWrite(string bytesToWrite)
        {
            string[] strBytes = bytesToWrite.Trim().Split((char[])null, StringSplitOptions.RemoveEmptyEntries);

            int len = strBytes.Length;
            if (len == 0 || len > AdqCommon.MaxBytesReadWrite)
            {
                AppendInfoText(string.Format("Input error: {0} elements found. Only 1...{1} bytes can be written.", len, AdqCommon.MaxBytesReadWrite));
                return null;
            }

            var bytes = new byte[len];

            for (int i = 0; i < len; i++)
            {
                bool succeeded = byte.TryParse(strBytes[i], NumberStyles.AllowHexSpecifier, CultureInfo.InvariantCulture, out bytes[i]);
                if (!succeeded)
                {
                    AppendInfoText(string.Format("Error parsing element '{0}' as byte.", strBytes[i]));
                    return null;
                }
            }
            return bytes;
        }

        private void UpdateBytesToWriteProperties(string bytesToWrite)
        {
            if (!_bytesToWrite.Contains(bytesToWrite))
            {
                _bytesToWrite.Add(bytesToWrite);
            }
            SelectedBytesToWrite = bytesToWrite;
        }

        private readonly ObservableCollection<string> _bytesToWriteBeforeRead = new ObservableCollection<string>();
        public IEnumerable<string> BytesToWriteBeforeRead
        {
            get
            {
                return _bytesToWriteBeforeRead;
            }
        }

        private bool CanI2cWriteThenReadExecute(object obj)
        {
            return _selectedModule != null;
        }

        private void I2cWriteThenReadExecute(object obj)
        {
            string bytesToWrite = _newBytesToWriteBeforeRead.Trim();

            if (string.IsNullOrEmpty(bytesToWrite))
            {
                AppendInfoText("Please enter bytes to write.");
                return;
            }

            byte[] bytes = GetByteArrayToWrite(bytesToWrite);
            if (bytes != null)
            {
                bool succeeded = _selectedModule.WriteThenRead(bytes, (uint)(_i2cNumBytesToReadAfterWriteSelectedIndex + 1));
                if (succeeded)
                {
                    UpdateBytesToWriteBeforeReadProperties(bytesToWrite);
                    DisplayBytes("Wrote", bytes);
                    DisplayBytes("Then read", _selectedModule.BytesRead);
                }
            }
        }

        private void UpdateBytesToWriteBeforeReadProperties(string bytesToWrite)
        {
            if (!_bytesToWriteBeforeRead.Contains(bytesToWrite))
            {
                _bytesToWriteBeforeRead.Add(bytesToWrite);
            }
            SelectedBytesToWriteBeforeRead = bytesToWrite;
        }

        private string _selectedBytesToWriteBeforeReadText;
        public string SelectedBytesToWriteBeforeRead
        {
            get
            {
                return _selectedBytesToWriteBeforeReadText;
            }
            set
            {
                if (_selectedBytesToWriteBeforeReadText != value)
                {
                    _selectedBytesToWriteBeforeReadText = value;
                    NotifyPropertyChanged(nameof(SelectedBytesToWriteBeforeRead));
                }
            }
        }

        private string _newBytesToWriteBeforeRead = string.Empty;
        public string NewBytesToWriteBeforeRead
        {
            get
            {
                return _selectedBytesToWriteBeforeReadText;
            }
            set
            {
                _newBytesToWriteBeforeRead = value;
            }
        }

        private int _i2cNumBytesToReadAfterWriteSelectedIndex;
        public int I2cNumBytesToReadAfterWriteSelectedIndex
        {
            get
            {
                return _i2cNumBytesToReadAfterWriteSelectedIndex;
            }
            set
            {
                if (_i2cNumBytesToReadAfterWriteSelectedIndex != value)
                {
                    _i2cNumBytesToReadAfterWriteSelectedIndex = value;
                    NotifyPropertyChanged(nameof(I2cNumBytesToReadAfterWriteSelectedIndex));
                }
            }
        }

        public bool IsI2cModuleAvailable
        {
            get
            {
                return _moduleLocator.I2cModules.Count > 0;
            }
        }

        public bool CanUseI2cModule
        {
            get
            {
                return _selectedModule != null;
            }
        }

        public string ModuleDescription
        {
            get
            {
                return _selectedModule != null ? _selectedModule.Description : string.Empty;
            }
        }

        private void ClearInfoTextExecute(object obj)
        {
            InfoText = string.Empty;
            NotifyPropertyChanged(nameof(InfoText));
        }

        private void AppendInfoText(string text)
        {
            if (InfoText != string.Empty)
            {
                InfoText += Environment.NewLine;
            }
            InfoText += "[" + DateTime.Now.ToLongTimeString() + "] " + text;
            NotifyPropertyChanged(nameof(InfoText));
        }

        public string InfoText { get; private set; } = string.Empty;

        void NotifyPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        void MainViewModel_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(InfoText))
            {
                AppendInfoText(_moduleLocator.InfoText);
            }
            else if (e.PropertyName == nameof(_selectedModule.ModuleInfoText))
            {
                AppendInfoText(_selectedModule.ModuleInfoText);
            }
        }

        private void CloseApplicationExecute(object obj)
        {
            Application.Current.MainWindow.Close();
        }
    }
}
