﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the Interrupt project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using Interrupt.Utility;
using System;
using System.ComponentModel;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;

namespace Interrupt.Models
{
    public class IrqModule
    {
        private const string _fmtDateTime = "[HH:mm:ss.fff] ";
        private readonly uint _portMaxValue;
        private readonly AdqDriver.adqIrqCallback _irqCallbackFunction = new AdqDriver.adqIrqCallback(AdqIrqCallbackFunction);
        private GCHandle _objectHandle = new GCHandle();
        private BackgroundWorker _worker;
        private AdqDriver.ADQIRQEnable _irqEnable = new AdqDriver.ADQIRQEnable(AdqDriver.ADQIRQEnable_ID);

        public event PropertyChangedEventHandler PropertyChanged;
        public event EventHandler<BitChangeSettingsChangedEventArgs> BitChangeSettingsChanged;
        public event EventHandler<PortBitPatternCompareSettingsEventArgs> PortBitPatternCompareSettingsChanged;
        public event EventHandler<BitChangeStatusChangedEventArgs> BitChangeStatusChanged;
        public event EventHandler<CommonIrqStatusChangedEventArgs> CommonIrqStatusChanged;
        public event EventHandler<PortCompareStatusChangedEventArgs> PortCompareStatusChanged;
        public event EventHandler<PortStatusChangedEventArgs> PortStatusChanged;
        public event EventHandler<ExternalStatusChangedEventArgs> ExternalStatusChanged;

        public IrqModule(string boardName, uint boardIndex, uint moduleIndex, 
            ref AdqDriver.ADQGeneralModuleInfo generalModuleInfo, 
            ref AdqDriver.ADQIRQInfo adqIrqInfo, uint portWidthBits)
        {
            BoardIndex = boardIndex;
            ModuleIndex = moduleIndex;
            ModuleType = generalModuleInfo.ModuleType;
            IrqDescription = adqIrqInfo.IRQDescription;
            SupportsBitChange = adqIrqInfo.SupportsBitChange;
            SupportsBitPatternCompare = adqIrqInfo.SupportsBitPatternCompare;

            SupportedPortStatusErrors = adqIrqInfo.SupportsPortStatusError;
            SupportedFaultRecovery = adqIrqInfo.SupportsFaultRecovery;
            SupportedExternalIrqs = adqIrqInfo.SupportsExternal;
            PortWidthBits = portWidthBits;
            UseIrqCallbackFunction = true;

            _portMaxValue = GetMaxPortValue(PortWidthBits);
            TimeoutMilliseconds = AdqCommon.DefaultIrqWaitTimeoutMilliseconds;
            ModuleName = GetModuleName(boardName);
        }

        private uint GetMaxPortValue(uint portWidthBits)
        {
            return (portWidthBits < 32U) ? (uint)((1 << (int)portWidthBits) - 1) : uint.MaxValue;
        }

        public uint BoardIndex { get; }
        public uint ModuleIndex { get; }
        public uint ModuleType { get; }
        public string ModuleName { get; }
        public string IrqDescription { get; }
        public uint PortWidthBits { get; }
        public bool SupportsBitChange { get; }
        public bool SupportsBitPatternCompare { get; }
        public uint SupportedPortStatusErrors { get; }
        public uint SupportedFaultRecovery { get; }
        public uint SupportedExternalIrqs { get; }
        public bool UseIrqCallbackFunction { get; set; }
        public uint TimeoutMilliseconds { get; set; }

        public bool BitChangeIsEnabled
        {
            get
            {
                return _irqEnable.PortBitChangeRisingEdge != 0 || _irqEnable.PortBitChangeFallingEdge != 0;
            }
        }

        public bool BitPatternCompareIsEnabled
        {
            get
            {
                return _irqEnable.PortBitPatternCompare;
            }
        }

        public uint PortBitChangeRisingEdge
        {
            get { return _irqEnable.PortBitChangeRisingEdge; }
            set
            {
                if (_irqEnable.PortBitChangeRisingEdge != value)
                {
                    bool wasEnabled = BitChangeIsEnabled;
                    _irqEnable.PortBitChangeRisingEdge = value;
                    NotifyBitChangeMasksChanged(wasEnabled);
                }
            }
        }

        public uint PortBitChangeFallingEdge
        {
            get { return _irqEnable.PortBitChangeFallingEdge; }
            set
            {
                if (_irqEnable.PortBitChangeFallingEdge != value)
                {
                    bool wasEnabled = BitChangeIsEnabled;
                    _irqEnable.PortBitChangeFallingEdge = value;
                    NotifyBitChangeMasksChanged(wasEnabled);
                }
            }
        }

        private void NotifyBitChangeMasksChanged(bool wasEnabled)
        {
            if (wasEnabled != BitChangeIsEnabled)
            {
                BitChangeSettingsChanged?.Invoke(this, new BitChangeSettingsChangedEventArgs());
            }
        }

        public bool PortBitPatternCompare
        {
            get { return _irqEnable.PortBitPatternCompare; }
            set
            {
                if (_irqEnable.PortBitPatternCompare != value)
                {
                    _irqEnable.PortBitPatternCompare = value;
                    PortBitPatternCompareSettingsChanged?.Invoke(this, new PortBitPatternCompareSettingsEventArgs());
                }
            }
        }

        public uint PortBitPatternValue
        {
            get { return _irqEnable.PortBitPatternValue; }
            set { _irqEnable.PortBitPatternValue = value; }
        }

        public uint EnabledPortStatusErrors
        {
            get { return _irqEnable.PortStatusError; }
            set { _irqEnable.PortStatusError = value; }
        }

        public uint EnabledFaultRecovery
        {
            get { return _irqEnable.FaultRecovery; }
            set { _irqEnable.FaultRecovery = value; }
        }

        public uint ExternalRisingEdge
        {
            get { return _irqEnable.ExternalRisingEdge; }
            set { _irqEnable.ExternalRisingEdge = value; }
        }

        public uint ExternalFallingEdge
        {
            get { return _irqEnable.ExternalFallingEdge; }
            set { _irqEnable.ExternalFallingEdge = value; }
        }

        public bool ResetIRQCount
        {
            get { return _irqEnable.ResetIRQCount; }
            set { _irqEnable.ResetIRQCount = value; }
        }

        private bool _isModuleIdle = true;
        public bool IsModuleIdle
        {
            get { return _isModuleIdle; }
            private set
            {
                _isModuleIdle = value;
                NotifyPropertyChanged(nameof(IsModuleIdle));
            }
        }

        private static uint AdqIrqCallbackFunction(uint boardIndex, uint moduleIndex, ref AdqDriver.ADQIRQStatus irqStatus, uint error, IntPtr userContext)
        {
            if (error == AdqErrors.ADQ_ERROR_NONE && userContext != IntPtr.Zero)
            {
                GCHandle objHandle = GCHandle.FromIntPtr(userContext);
                IrqModule irqModule = (IrqModule)objHandle.Target;

                irqModule.UpdateIrqStatus(boardIndex, moduleIndex, ref irqStatus, DateTime.Now);
            }
            //just return the passed error code if the driver should stop calling this function again in case of error != AdqErrors.ADQ_ERROR_NONE
            return error;
        }

        public void DisableInterrupt()
        {
            uint adqError = AdqDriver.adqDisableIRQ(BoardIndex, ModuleIndex);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetErrorText("adqDisableIRQ", adqError);
            }
            else
            {
                ModuleInfoText = "Disabled IRQ on " + ModuleName;
            }
            IsModuleIdle = true;
        }

        public void EnableInterrupt()
        {
            if (!ConfigureModule()) return;

            ModuleInfoText = GetIrqEnableInfoText();

            AdqDriver.adqDisableIRQ(BoardIndex, ModuleIndex);
            AdqDriver.adqIrqCallback irqCallbackFunction = SetupCallbackFunction();

            uint adqError = AdqDriver.adqEnableIRQ(BoardIndex,
                                                   ModuleIndex,
                                                   ref _irqEnable,
                                                   irqCallbackFunction,
                                                   AdqDriver.ADQIRQStatus_ID,
                                                   (IntPtr)_objectHandle);

            if (adqError == AdqErrors.ADQ_ERROR_NONE)
            {
                IsModuleIdle = false;
                ModuleInfoText = "IRQ enabled on " + ModuleName + ", selected option: "
                    + (UseIrqCallbackFunction ? "IRQ-Callback" : "IRQ-Wait");
            }
            else
            {
                SetErrorText("adqEnableIRQ", adqError);
                return;
            }

            if (!UseIrqCallbackFunction)
            {
                StartBackgroundWorker();
            }
        }

        private bool ConfigureModule()
        {
            if (!CheckIrqConfiguration()) return false;
            if (!SetDioDirectionToInput()) return false;

            return true;
        }

        private bool CheckIrqConfiguration()
        {
            if ((_irqEnable.PortBitChangeRisingEdge == 0) 
                && (_irqEnable.PortBitChangeFallingEdge == 0)
                && (!_irqEnable.PortBitPatternCompare)
                && (_irqEnable.PortStatusError == 0)
                && (_irqEnable.FaultRecovery == 0)
                && (_irqEnable.ExternalRisingEdge == 0)
                && (_irqEnable.ExternalFallingEdge == 0))
            {
                ModuleErrorText = "Please select the interrupts that should be enabled.";
                return false;
            }

            if (_irqEnable.PortBitPatternCompare && (_irqEnable.PortBitChangeRisingEdge != 0 || _irqEnable.PortBitChangeFallingEdge != 0))
            {
                ModuleErrorText = "Bit pattern compare and Bit-change cannot be enabled at the same time.";
                return false;
            }
            return true;
        }

        private bool SetDioDirectionToInput()
        {
            if (ModuleType == AdqConstants.ADQ_MODULE_TYPE_DIO)
            {
                uint adqError = AdqDriver.adqDIOSetDirection(BoardIndex, ModuleIndex, AdqConstants.ADQ_DIO_INPUT);
                if (adqError != AdqErrors.ADQ_ERROR_NONE)
                {
                    SetErrorText("adqDIOSetDirection", adqError);
                    return false;
                }
            }
            return true;
        }

        private string GetIrqEnableInfoText()
        {
            var sb = new StringBuilder();
            sb.AppendFormat("Board {0}, Module {1} interrupt enable settings:" + Environment.NewLine, BoardIndex, ModuleIndex);
            sb.AppendLine("\tPortBitChangeRisingEdge = 0x" + _irqEnable.PortBitChangeRisingEdge.ToString("X"));
            sb.AppendLine("\tPortBitChangeFallingEdge = 0x" + _irqEnable.PortBitChangeFallingEdge.ToString("X"));
            sb.AppendLine("\tPortBitPatternCompare = " + _irqEnable.PortBitPatternCompare.ToString());
            sb.AppendLine("\tPortBitPatternValue = 0x" + _irqEnable.PortBitPatternValue.ToString("X"));
            sb.AppendLine("\tExternalRisingEdge = 0x" + _irqEnable.ExternalRisingEdge.ToString("X"));
            sb.AppendLine("\tExternalFallingEdge = 0x" + _irqEnable.ExternalFallingEdge.ToString("X"));
            sb.AppendLine("\tPortStatusError = 0x" + _irqEnable.PortStatusError.ToString("X"));
            sb.AppendLine("\tFaultRecovery = 0x" + _irqEnable.FaultRecovery.ToString("X"));
            sb.Append("\tResetIRQCount = " + _irqEnable.ResetIRQCount.ToString());
            return sb.ToString();
        }

        private AdqDriver.adqIrqCallback SetupCallbackFunction()
        {
            AdqDriver.adqIrqCallback irqCallbackFunction = null;

            if (UseIrqCallbackFunction)
            {
                irqCallbackFunction = _irqCallbackFunction;
                if (!_objectHandle.IsAllocated)
                {
                    _objectHandle = GCHandle.Alloc(this);
                }
            }
            else
            {
                if (_objectHandle.IsAllocated)
                {
                    _objectHandle.Free();
                }
            }
            return irqCallbackFunction;
        }

        private void StartBackgroundWorker()
        {
            ModuleInfoText = string.Format("Starting IRQ-Wait thread, time-out {0} milliseconds", TimeoutMilliseconds);

            if (_worker == null)
            {
                _worker = new BackgroundWorker();
                _worker.DoWork += Worker_DoWork;
                _worker.RunWorkerCompleted += Worker_RunWorkerCompleted;
            }
            _worker.RunWorkerAsync();
        }

        private void Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            var irqStatus = new AdqDriver.ADQIRQStatus(AdqDriver.ADQIRQStatus_ID);

            uint adqError;

            while (true)
            {
                adqError = AdqDriver.adqWaitIRQ(BoardIndex,
                                            ModuleIndex,
                                            ref irqStatus,
                                            TimeoutMilliseconds);
                if (adqError == AdqErrors.ADQ_ERROR_NONE)
                {
                    UpdateIrqStatus(BoardIndex, ModuleIndex, ref irqStatus, DateTime.Now);
                }
                else
                {
                    SetErrorText("adqWaitIRQ", adqError);
                    if (adqError == AdqErrors.ADQ_ERROR_TIMEOUT)
                    {
                        ModuleInfoText = string.Format("Error 0x{0:X}: Time-out reached after {1} milliseconds", adqError, TimeoutMilliseconds);
                    }
                    break;
                }
            }
        }

        private void Worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            DisableInterrupt();
            ModuleInfoText = "IRQ-Wait thread ended";
        }

        private void UpdateIrqStatus(uint boardIndex, uint moduleIndex, ref AdqDriver.ADQIRQStatus irqStatus, DateTime timeIrqDetected)
        {
            if (irqStatus.PortBitChangeRisingEdge != 0 || irqStatus.PortBitChangeFallingEdge != 0)
            {
                var args = new BitChangeStatusChangedEventArgs()
                {
                    StatusRisingEdge = irqStatus.PortBitChangeRisingEdge,
                    StatusFallingEdge = irqStatus.PortBitChangeFallingEdge
                };
                OnBitChangeStatusChanged(args);
            }

            if (irqStatus.PortBitPatternCompare)
            {
                var compareArgs = new PortCompareStatusChangedEventArgs()
                {
                    StatusCompare = irqStatus.PortBitPatternCompare,
                    TimeCompareMatched = timeIrqDetected
                };
                OnPortCompareStatusChanged(compareArgs);
            }

            if (irqStatus.PortStatusError != 0 || irqStatus.FaultRecovery != 0)
            {
                var portStatusArgs = new PortStatusChangedEventArgs()
                {
                    StatusPortErrorValue = irqStatus.PortStatusError,
                    StatusFaultRecoveryValue = irqStatus.FaultRecovery
                };
                OnPortStatusChanged(portStatusArgs);
            }

            if (irqStatus.ExternalFallingEdge != 0 || irqStatus.ExternalRisingEdge != 0)
            {
                var externalArgs = new ExternalStatusChangedEventArgs()
                {
                    ExternalStatusRisingEdge = irqStatus.ExternalRisingEdge,
                    ExternalStatusFallingEdge = irqStatus.ExternalFallingEdge
                };
                OnExternalStatusChanged(externalArgs);
            }

            var commonArgs = new CommonIrqStatusChangedEventArgs()
            {
                IrqCount = irqStatus.IRQCount,
                TriggerValue = irqStatus.TriggerValue,
                InfoText = GetIrqStatusInfoText(boardIndex, moduleIndex, ref irqStatus, timeIrqDetected)
            };
            OnCommonIrqStatusChanged(commonArgs);
        }

        private string GetIrqStatusInfoText(uint boardIndex, uint moduleIndex, ref AdqDriver.ADQIRQStatus irqStatus, DateTime timeIrqDetected)
        {
            var sb = new StringBuilder();
            sb.Append(timeIrqDetected.ToString(_fmtDateTime));
            sb.AppendFormat("Board {0}, Module {1} interrupt status:" + Environment.NewLine, boardIndex, moduleIndex);

            if (irqStatus.PortBitChangeRisingEdge != 0)
            {
                sb.AppendLine("\tPortBitChangeRisingEdge = 0x" + irqStatus.PortBitChangeRisingEdge.ToString("X"));
            }

            if (irqStatus.PortBitChangeFallingEdge != 0)
            {
                sb.AppendLine("\tPortBitChangeFallingEdge = 0x" + irqStatus.PortBitChangeFallingEdge.ToString("X"));
            }

            if (irqStatus.PortBitPatternCompare)
            {
                sb.AppendLine("\tPortBitPatternCompare = " + irqStatus.PortBitPatternCompare.ToString());
            }

            if (irqStatus.PortStatusError != 0)
            {
                sb.AppendLine("\tPortStatusError = 0x" + irqStatus.PortStatusError.ToString("X"));
            }

            if (irqStatus.FaultRecovery != 0)
            {
                sb.AppendLine("\tFaultRecovery = 0x" + irqStatus.FaultRecovery.ToString("X"));
            }

            if (irqStatus.ExternalRisingEdge != 0)
            {
                sb.AppendLine("\tExternalRisingEdge = 0x" + irqStatus.ExternalRisingEdge.ToString("X"));
            }

            if (irqStatus.ExternalFallingEdge != 0)
            {
                sb.AppendLine("\tExternalFallingEdge = 0x" + irqStatus.ExternalFallingEdge.ToString("X"));
            }

            sb.AppendLine("\tIRQ Count = " + irqStatus.IRQCount.ToString());
            sb.Append("\tTrigger value = 0x" + irqStatus.TriggerValue.ToString("X"));
            return sb.ToString();
        }

        private void OnBitChangeStatusChanged(BitChangeStatusChangedEventArgs e)
        {
            BitChangeStatusChanged?.Invoke(this, e);
        }

        private void OnPortCompareStatusChanged(PortCompareStatusChangedEventArgs e)
        {
            PortCompareStatusChanged?.Invoke(this, e);
        }

        private void OnPortStatusChanged(PortStatusChangedEventArgs e)
        {
            PortStatusChanged?.Invoke(this, e);
        }

        private void OnExternalStatusChanged(ExternalStatusChangedEventArgs e)
        {
            ExternalStatusChanged?.Invoke(this, e);
        }

        private void OnCommonIrqStatusChanged(CommonIrqStatusChangedEventArgs e)
        {
            CommonIrqStatusChanged?.Invoke(this, e);
        }

        private void SetErrorText(string functionName, uint adqError)
        {
            ModuleErrorText = string.Format("Board {0}, Module {1}: API-function {2} returned error number: 0x{3:X}",
                BoardIndex,
                ModuleIndex,
                functionName,
                adqError);
        }

        string _errorText = string.Empty;
        public string ModuleErrorText
        {
            get { return _errorText; }
            set
            {
                if (string.IsNullOrEmpty(value))
                {
                    _errorText = string.Empty;
                    return;
                }

                _errorText = DateTime.Now.ToString(_fmtDateTime) + value;
                NotifyPropertyChanged(nameof(ModuleErrorText));
            }
        }

        string _infoText = string.Empty;
        public string ModuleInfoText
        {
            get { return _infoText; }
            set
            {
                if (string.IsNullOrEmpty(value))
                {
                    _infoText = string.Empty;
                    return;
                }

                _infoText = DateTime.Now.ToString(_fmtDateTime) + value;
                NotifyPropertyChanged(nameof(ModuleInfoText));
            }
        }

        void NotifyPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        private string GetModuleName(string boardName)
        {
            string s = string.Empty;
            if (ModuleType == AdqConstants.ADQ_MODULE_TYPE_DIO ||
                ModuleType == AdqConstants.ADQ_MODULE_TYPE_DI ||
                ModuleType == AdqConstants.ADQ_MODULE_TYPE_DO)
            {
                s = string.Format(", {0} Bit", PortWidthBits);
            }
            else if (ModuleType == AdqConstants.ADQ_MODULE_TYPE_EXTINT)
            {
                s = string.Format(", 0x{0:X}", SupportedExternalIrqs);
            }
            else if (ModuleType == AdqConstants.ADQ_MODULE_TYPE_AI)
            {
                if (SupportedPortStatusErrors != 0 || SupportedFaultRecovery != 0)
                {
                    s = ", Port status";
                }
            }

            return string.Format("{0}, Board {1}, Module {2}, Type {3}{4}",
                AdqCommon.GetModelName(boardName),
                BoardIndex,
                ModuleIndex,
                AdqCommon.GetModuleTypeName(ModuleType),
                s);
        }

        public bool ResetModule()
        {
            uint adqError = AdqDriver.adqResetModule(BoardIndex, ModuleIndex);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetErrorText("adqResetModule", adqError);
                return false;
            }
            return true;
        }

        public string ValidatePortMaxValue(uint value)
        {
            string msg = string.Empty;
            if (value > _portMaxValue)
            {
                msg = string.Format("The value cannot be greater than 0x{0:X}", _portMaxValue);
            }
            return msg;
        }

        public string ValidateExtIrqValue(uint value)
        {
            return ValidateEnabledBits(SupportedExternalIrqs, value);
        }

        public string ValidateEnabledPortStatusErrorValue(uint value)
        {
            return ValidateEnabledBits(SupportedPortStatusErrors, value);
        }

        public string ValidateEnabledFaultRecoveryValue(uint value)
        {
            return ValidateEnabledBits(SupportedFaultRecovery, value);
        }

        private string ValidateEnabledBits(uint supportedValue, uint enteredValue)
        {
            string msg = string.Empty;
            if ((~supportedValue & enteredValue) != 0)
            {
                msg = string.Format("The entered value 0x{0:X} is out of the range of supported bits", enteredValue);
            }
            return msg;
        }
    }
}
