﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the Interrupt project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using Interrupt.Models;
using Interrupt.Utility;
using System.ComponentModel;

namespace Interrupt.ViewModels
{
    public class BitPatternCompareViewModel : ErrorInfoViewModelBase
    {
        private IrqModule _irqModule;

        public void UpdateIrqModule(IrqModule irqModule)
        {
            if (_supportsIrqType)
            {
                _irqModule.PortCompareStatusChanged -= PortCompareStatusChanged;
                _irqModule.BitChangeSettingsChanged -= BitChangeSettingsChanged;
                _irqModule.PropertyChanged -= BitPatternCompareViewModel_PropertyChanged;
            }

            _irqModule = irqModule;
            _supportsIrqType = irqModule != null && irqModule.SupportsBitPatternCompare;
            _enableCompare = irqModule != null && irqModule.PortBitPatternCompare;

            if (_supportsIrqType)
            {
                UpdatePortCompareValueText(irqModule.PortBitPatternValue);
                _irqModule.PortCompareStatusChanged += PortCompareStatusChanged;
                _irqModule.BitChangeSettingsChanged += BitChangeSettingsChanged;
                _irqModule.PropertyChanged += BitPatternCompareViewModel_PropertyChanged;
            }
            else
            {
                UpdatePortCompareValueText(0);
            }
            ClearStatusIndicators();
            NotifyPropertyChanged(nameof(CanUseBitPatternCompare));
            NotifyPropertyChanged(nameof(EnableCompare));
            NotifyPropertyChanged(nameof(SupportsIrqType));
        }

        private void BitPatternCompareViewModel_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(IrqModule.IsModuleIdle))
            {
                UpdateModuleIdleStatus();
            }
        }

        private void UpdateModuleIdleStatus()
        {
            NotifyPropertyChanged(nameof(CanUseBitPatternCompare));
        }

        private bool _supportsIrqType;
        public bool SupportsIrqType
        {
            get { return _supportsIrqType; }
        }

        public bool CanUseBitPatternCompare
        {
            get { return _supportsIrqType && _irqModule != null && _irqModule.IsModuleIdle && !_irqModule.BitChangeIsEnabled; }
        }

        private void BitChangeSettingsChanged(object sender, BitChangeSettingsChangedEventArgs e)
        {
            NotifyPropertyChanged(nameof(CanUseBitPatternCompare));
        }

        public void ClearStatusIndicators()
        {
            StatusCompareValueMatched = false;
            LastTimeCompareMatched = string.Empty;
            NotifyPropertyChanged(nameof(StatusCompareValueMatched));
            NotifyPropertyChanged(nameof(LastTimeCompareMatched));
        }

        private void UpdatePortCompareValueText(uint compareValue)
        {
            _portCompareValueText = AdqCommon.FormatHexValue(compareValue);
            _errorInfo.Remove(nameof(PortCompareValueText));
            NotifyPropertyChanged(nameof(PortCompareValueText));
            NotifyPropertyChanged(nameof(IsInputValid));
        }

        private bool _enableCompare;
        public bool EnableCompare
        {
            get { return _enableCompare; }
            set
            {
                if (_enableCompare != value)
                {
                    _enableCompare = value;
                    if (_irqModule != null)
                    {
                        _irqModule.PortBitPatternCompare = value;
                    }
                    NotifyPropertyChanged(nameof(EnableCompare));
                }
            }
        }

        private string _portCompareValueText = "0";
        public string PortCompareValueText
        {
            get { return _portCompareValueText; }
            set
            {
                if (_portCompareValueText != value)
                {
                    _portCompareValueText = value;

                    string msg = InputValidation.ValidateHexValue(_portCompareValueText, out uint compareValue);

                    if (!string.IsNullOrEmpty(msg))
                    {
                        _errorInfo.AddOrUpdate(nameof(PortCompareValueText), msg);
                    }
                    else
                    {
                        msg = _irqModule.ValidatePortMaxValue(compareValue);
                        if (!string.IsNullOrEmpty(msg))
                        {
                            _errorInfo.AddOrUpdate(nameof(PortCompareValueText), msg);
                        }
                        else
                        {
                            _errorInfo.Remove(nameof(PortCompareValueText));
                            _irqModule.PortBitPatternValue = compareValue;
                            _portCompareValueText = AdqCommon.FormatHexValue(compareValue);
                        }

                        NotifyPropertyChanged(nameof(PortCompareValueText));
                        NotifyPropertyChanged(nameof(IsInputValid));
                    }
                }
            }
        }

        private void PortCompareStatusChanged(object sender, PortCompareStatusChangedEventArgs e)
        {
            StatusCompareValueMatched = e.StatusCompare;
            LastTimeCompareMatched = e.TimeCompareMatched.ToString("HH:mm:ss.fff");
            NotifyPropertyChanged(nameof(StatusCompareValueMatched));
            NotifyPropertyChanged(nameof(LastTimeCompareMatched));
        }

        public bool StatusCompareValueMatched { get; private set; }
        public string LastTimeCompareMatched { get; private set; }
    }
}
