﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the Interrupt project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using Interrupt.Models;
using Interrupt.Utility;
using System.Collections.ObjectModel;
using System.ComponentModel;

namespace Interrupt.ViewModels
{
    public class ExternalIrqViewModel : ErrorInfoViewModelBase
    {
        private IrqModule _irqModule;

        public void UpdateIrqModule(IrqModule irqModule)
        {
            if (SupportsIrqType)
            {
                _irqModule.ExternalStatusChanged -= ExternalStatusChanged;
                _irqModule.PropertyChanged -= ExternalIrqViewModel_PropertyChanged;
            }

            _irqModule = irqModule;
            SupportsIrqType = irqModule != null && irqModule.SupportedExternalIrqs != 0;

            if (SupportsIrqType)
            {
                UpdateBitMaskViewModels();
                UpdateRisingEdgesText(irqModule.ExternalRisingEdge);
                UpdateRisingBitMaskEdges(irqModule.ExternalRisingEdge);
                UpdateFallingEdgesText(irqModule.ExternalFallingEdge);
                UpdateFallingBitMaskEdges(irqModule.ExternalFallingEdge);
                _irqModule.ExternalStatusChanged += ExternalStatusChanged;
                _irqModule.PropertyChanged += ExternalIrqViewModel_PropertyChanged;
            }
            else
            {
                BitMaskEdgeItems.Clear();
                BitEdgeIndicatorItems.Clear();
                UpdateRisingEdgesText(0);
                UpdateFallingEdgesText(0);
            }
            ClearStatusIndicators();
            NotifyPropertyChanged(nameof(SupportsIrqType));
            UpdateModuleIdleStatus();
        }

        public bool SupportsIrqType { get; private set; }

        public bool CanEditIrqSettings
        {
            get { return SupportsIrqType && _irqModule != null && _irqModule.IsModuleIdle; }
        }


        private void UpdateBitMaskViewModels()
        {
            BitMaskEdgeItems.Clear();
            BitEdgeIndicatorItems.Clear();

            int bitIdx = 0;
            while (bitIdx < 32 && ((_irqModule.SupportedExternalIrqs >> bitIdx) & 1) != 0)
            {
                var bmItem = new BitMaskEdgeItemViewModel(bitIdx);
                bmItem.PropertyChanged += BmItem_PropertyChanged;
                BitMaskEdgeItems.Add(bmItem);

                var indItem = new BitEdgeIndicatorItemViewModel(bitIdx);
                BitEdgeIndicatorItems.Add(indItem);
                bitIdx++;
            }
        }

        private void BmItem_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(BitMaskEdgeItemViewModel.BitRisingEdge))
            {
                UpdateRisingEdgeValue();
            }
            else if (e.PropertyName == nameof(BitMaskEdgeItemViewModel.BitFallingEdge))
            {
                UpdateFallingEdgeValue();
            }
        }

        private void UpdateRisingEdgeValue()
        {
            uint edgesValue = 0;
            uint bit = 1;
            foreach (BitMaskEdgeItemViewModel bitMaskEdgeItem in BitMaskEdgeItems)
            {
                if (bitMaskEdgeItem.BitRisingEdge)
                {
                    edgesValue |= bit;
                }
                bit <<= 1;
            }
            _irqModule.ExternalRisingEdge = edgesValue;
            UpdateRisingEdgesText(edgesValue);
        }

        private void UpdateRisingEdgesText(uint edgesValue)
        {
            _valueRisingEdgesText = AdqCommon.FormatHexValue(edgesValue);
            _errorInfo.Remove(nameof(ValueRisingEdgesText));
            NotifyPropertyChanged(nameof(ValueRisingEdgesText));
            NotifyPropertyChanged(nameof(IsInputValid));
        }

        private void UpdateFallingEdgeValue()
        {
            uint edgesValue = 0;
            uint bit = 1;
            foreach (BitMaskEdgeItemViewModel bitMaskEdgeItem in BitMaskEdgeItems)
            {
                if (bitMaskEdgeItem.BitFallingEdge)
                {
                    edgesValue |= bit;
                }
                bit <<= 1;
            }
            _irqModule.ExternalFallingEdge = edgesValue;
            UpdateFallingEdgesText(edgesValue);
        }

        private void UpdateFallingEdgesText(uint edgesValue)
        {
            _valueFallingEdgesText = AdqCommon.FormatHexValue(edgesValue);
            _errorInfo.Remove(nameof(ValueFallingEdgesText));
            NotifyPropertyChanged(nameof(ValueFallingEdgesText));
            NotifyPropertyChanged(nameof(IsInputValid));
        }

        public ObservableCollection<BitMaskEdgeItemViewModel> BitMaskEdgeItems { get; } 
            = new ObservableCollection<BitMaskEdgeItemViewModel>();
        public ObservableCollection<BitEdgeIndicatorItemViewModel> BitEdgeIndicatorItems { get; } 
            = new ObservableCollection<BitEdgeIndicatorItemViewModel>();

        public void ClearStatusIndicators()
        {
            UpdateStatusRisingEdges(0);
            UpdateStatusFallingEdges(0);
        }

        private string _valueRisingEdgesText = "0";
        public string ValueRisingEdgesText
        {
            get { return _valueRisingEdgesText; }
            set
            {
                if (_valueRisingEdgesText == value) return;

                _valueRisingEdgesText = value;

                string msg = InputValidation.ValidateHexValue(_valueRisingEdgesText, out uint edgesValue);
                if (!string.IsNullOrEmpty(msg))
                {
                    _errorInfo.AddOrUpdate(nameof(ValueRisingEdgesText), msg);
                }
                else
                {
                    msg = _irqModule.ValidateExtIrqValue(edgesValue);
                    if (!string.IsNullOrEmpty(msg))
                    {
                        _errorInfo.AddOrUpdate(nameof(ValueRisingEdgesText), msg);
                    }
                    else
                    {
                        _errorInfo.Remove(nameof(ValueRisingEdgesText));
                        _irqModule.ExternalRisingEdge = edgesValue;
                        _valueRisingEdgesText = AdqCommon.FormatHexValue(edgesValue);
                        UpdateRisingBitMaskEdges(edgesValue);
                    }
                }

                NotifyPropertyChanged(nameof(ValueRisingEdgesText));
                NotifyPropertyChanged(nameof(IsInputValid));
            }
        }

        private string _valueFallingEdgesText = "0";
        public string ValueFallingEdgesText
        {
            get { return _valueFallingEdgesText; }
            set
            {
                if (_valueFallingEdgesText == value) return;

                _valueFallingEdgesText = value;

                string msg = InputValidation.ValidateHexValue(value, out uint edgesValue);

                if (!string.IsNullOrEmpty(msg))
                {
                    _errorInfo.AddOrUpdate(nameof(ValueFallingEdgesText), msg);
                }
                else
                {
                    msg = _irqModule.ValidateExtIrqValue(edgesValue);
                    if (!string.IsNullOrEmpty(msg))
                    {
                        _errorInfo.AddOrUpdate(nameof(ValueFallingEdgesText), msg);
                    }
                    else
                    {
                        _errorInfo.Remove(nameof(ValueFallingEdgesText));
                        _irqModule.ExternalFallingEdge = edgesValue;
                        _valueFallingEdgesText = AdqCommon.FormatHexValue(edgesValue);
                        UpdateFallingBitMaskEdges(edgesValue);
                    }
                }

                NotifyPropertyChanged(nameof(ValueFallingEdgesText));
                NotifyPropertyChanged(nameof(IsInputValid));
            }
        }

        private void UpdateRisingBitMaskEdges(uint edgesValue)
        {
            int bit = 1;
            foreach (BitMaskEdgeItemViewModel bitMaskEdgeItem in BitMaskEdgeItems)
            {
                bitMaskEdgeItem.BitRisingEdge = (bit & edgesValue) != 0;
                bit <<= 1;
            }
        }

        private void UpdateFallingBitMaskEdges(uint edgesValue)
        {
            int bit = 1;
            foreach (BitMaskEdgeItemViewModel bitMaskEdgeItem in BitMaskEdgeItems)
            {
                bitMaskEdgeItem.BitFallingEdge = (bit & edgesValue) != 0;
                bit <<= 1;
            }
        }


        private void ExternalStatusChanged(object sender, ExternalStatusChangedEventArgs e)
        {
            UpdateStatusRisingEdges(e.ExternalStatusRisingEdge);
            UpdateStatusFallingEdges(e.ExternalStatusFallingEdge);
        }

        private void UpdateStatusRisingEdges(uint edgesValue)
        {
            if (StatusExtIrqRisingEdges != edgesValue)
            {
                StatusExtIrqRisingEdges = edgesValue;
                NotifyPropertyChanged(nameof(StatusExtIrqRisingEdges));
                UpdateRisingBitIndicatorsEdges(edgesValue);
            }
        }

        private void UpdateStatusFallingEdges(uint edgesValue)
        {
            if (StatusExtIrqFallingEdges != edgesValue)
            {
                StatusExtIrqFallingEdges = edgesValue;
                NotifyPropertyChanged(nameof(StatusExtIrqFallingEdges));
                UpdateFallingBitIndicatorsEdges(edgesValue);
            }
        }

        public uint StatusExtIrqRisingEdges { get; private set; }
        public uint StatusExtIrqFallingEdges { get; private set; }

        private void UpdateRisingBitIndicatorsEdges(uint edgesValue)
        {
            int bit = 1;
            foreach (BitEdgeIndicatorItemViewModel bitIndicatorItem in BitEdgeIndicatorItems)
            {
                bitIndicatorItem.RisingEdge = (bit & edgesValue) != 0;
                bit <<= 1;
            }
        }

        private void UpdateFallingBitIndicatorsEdges(uint edgesValue)
        {
            int bit = 1;
            foreach (BitEdgeIndicatorItemViewModel bitIndicatorItem in BitEdgeIndicatorItems)
            {
                bitIndicatorItem.FallingEdge = (bit & edgesValue) != 0;
                bit <<= 1;
            }
        }

        private void ExternalIrqViewModel_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(IrqModule.IsModuleIdle))
            {
                UpdateModuleIdleStatus();
            }
        }

        private void UpdateModuleIdleStatus()
        {
            NotifyPropertyChanged(nameof(CanEditIrqSettings));
        }
    }
}
