﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the Interrupt project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using Interrupt.Models;
using Interrupt.Utility;
using System.Collections.ObjectModel;
using System.ComponentModel;

namespace Interrupt.ViewModels
{
    public class PortStatusViewModel : ErrorInfoViewModelBase
    {
        private IrqModule _irqModule;

        public void UpdateIrqModule(IrqModule irqModule)
        {
            if (_supportsPortStatusError || _supportsFaultRecovery)
            {
                _irqModule.PortStatusChanged -= PortStatusChanged;
                _irqModule.PropertyChanged -= PortStatusViewModel_PropertyChanged;
            }

            _irqModule = irqModule;
            _supportsPortStatusError = irqModule != null && irqModule.SupportedPortStatusErrors != 0; 
            _supportsFaultRecovery = irqModule != null && irqModule.SupportedFaultRecovery != 0;

            InitPortStatusItemViewModels();
            if (_supportsPortStatusError || _supportsFaultRecovery)
            {
                UpdateEnabledPortStatusErrorsText(irqModule.EnabledPortStatusErrors);
                UpdatePortStatusErrorBitItems(irqModule.EnabledPortStatusErrors);
                UpdateEnabledFaultRecoveryText(irqModule.EnabledFaultRecovery);
                UpdateFaultRecoveryBitItems(irqModule.EnabledFaultRecovery);
                _irqModule.PortStatusChanged += PortStatusChanged;
                _irqModule.PropertyChanged += PortStatusViewModel_PropertyChanged;

            }
            else
            {
                EnabledPortStatusBits.Clear();
                PortStatusIndicatorBits.Clear();
                UpdateEnabledPortStatusErrorsText(0);
                UpdateEnabledFaultRecoveryText(0);
            }
            ClearStatusIndicators();
            NotifyPropertyChanged(nameof(SupportsIrqType));
            UpdateModuleIdleStatus();
        }

        private void PortStatusViewModel_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(IrqModule.IsModuleIdle))
            {
                UpdateModuleIdleStatus();
            }
        }

        private void UpdateModuleIdleStatus()
        {
            NotifyPropertyChanged(nameof(CanEditIrqSettings));
            NotifyPropertyChanged(nameof(CanEditPortStatusErrorSettings));
            NotifyPropertyChanged(nameof(CanEditFaultRecoverySettings));
        }

        public bool SupportsIrqType
        {
            get { return _supportsPortStatusError || _supportsFaultRecovery; }
        }

        private bool _supportsPortStatusError;
        public bool CanEditPortStatusErrorSettings
        {
            get { return _supportsPortStatusError && _irqModule.IsModuleIdle; }
        }

        private bool _supportsFaultRecovery;
        public bool CanEditFaultRecoverySettings
        {
            get { return _supportsFaultRecovery && _irqModule.IsModuleIdle; }
        }

        public bool CanEditIrqSettings
        {
            get { return SupportsIrqType && _irqModule.IsModuleIdle; }
        }

        private void InitPortStatusItemViewModels()
        {
            EnabledPortStatusBits.Clear();
            PortStatusIndicatorBits.Clear();

            uint portStatusBits = _irqModule.SupportedPortStatusErrors | _irqModule.SupportedFaultRecovery;
            int bitNumber = 0;
            while (bitNumber < 32 && portStatusBits != 0)
            {
                if ((portStatusBits & 1) != 0)
                {
                    bool statusErrorSupported = IsBitSet(_irqModule.SupportedPortStatusErrors, bitNumber);
                    bool faultRecoverySupported = IsBitSet(_irqModule.SupportedFaultRecovery, bitNumber);

                    var bitItem = new PortStatusItemViewModel(bitNumber, statusErrorSupported, faultRecoverySupported);
                    bitItem.PropertyChanged += BitItem_PropertyChanged;
                    EnabledPortStatusBits.Add(bitItem);

                    bitItem = new PortStatusItemViewModel(bitNumber, statusErrorSupported, faultRecoverySupported);
                    PortStatusIndicatorBits.Add(bitItem);
                }

                portStatusBits >>= 1;
                bitNumber++;
            }
        }

        private bool IsBitSet(uint value, int bitNumber)
        {
            return (bitNumber < 32) ? ((value >> bitNumber) & 1) != 0 : false;
        }

        private void BitItem_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(PortStatusItemViewModel.BitStatusError))
            {
                UpdatePortStatusErrorsText();
            }
            else if (e.PropertyName == nameof(PortStatusItemViewModel.BitFaultRecovery))
            {
                UpdateFaultRecoveryText();
            }
        }

        private void UpdatePortStatusErrorsText()
        {
            uint statusValue = 0;
            uint bit = 1;
            foreach (PortStatusItemViewModel enabledPortStatusBit in EnabledPortStatusBits)
            {
                if (enabledPortStatusBit.BitStatusError)
                {
                    statusValue += enabledPortStatusBit.BinaryValue;
                }
                bit <<= 1;
            }
            _irqModule.EnabledPortStatusErrors = statusValue;
            UpdateEnabledPortStatusErrorsText(statusValue);
        }

        private void UpdateEnabledPortStatusErrorsText(uint statusValue)
        {
            _enabledPortStatusErrorsText = AdqCommon.FormatHexValue(statusValue);
            _errorInfo.Remove(nameof(EnabledPortStatusErrorsText));
            NotifyPropertyChanged(nameof(EnabledPortStatusErrorsText));
            NotifyPropertyChanged(nameof(IsInputValid));
        }

        private void UpdateFaultRecoveryText()
        {
            uint faultRecoveryValue = 0;
            uint bit = 1;
            foreach (PortStatusItemViewModel enabledPortStatusBit in EnabledPortStatusBits)
            {
                if (enabledPortStatusBit.BitFaultRecovery)
                {
                    faultRecoveryValue += enabledPortStatusBit.BinaryValue;
                }
                bit <<= 1;
            }
            _irqModule.EnabledFaultRecovery = faultRecoveryValue;
            UpdateEnabledFaultRecoveryText(faultRecoveryValue);
        }

        private void UpdateEnabledFaultRecoveryText(uint faultRecoveryValue)
        {
            _enabledFaultRecoveryText = AdqCommon.FormatHexValue(faultRecoveryValue);
            _errorInfo.Remove(nameof(EnabledFaultRecoveryText));
            NotifyPropertyChanged(nameof(EnabledFaultRecoveryText));
            NotifyPropertyChanged(nameof(IsInputValid));
        }

        public ObservableCollection<PortStatusItemViewModel> EnabledPortStatusBits { get; } 
            = new ObservableCollection<PortStatusItemViewModel>();
        public ObservableCollection<PortStatusItemViewModel> PortStatusIndicatorBits { get; } 
            = new ObservableCollection<PortStatusItemViewModel>();

        public void ClearStatusIndicators()
        {
            UpdatePortStatusErrorIndicators(0);
            UpdateFaultRecoveryIndicators(0);
        }

        private string _enabledPortStatusErrorsText = "0";
        public string EnabledPortStatusErrorsText
        {
            get { return _enabledPortStatusErrorsText; }
            set
            {
                if (_enabledPortStatusErrorsText != value)
                {
                    _enabledPortStatusErrorsText = value;

                    string msg = InputValidation.ValidateHexValue(_enabledPortStatusErrorsText, out uint statusErrors);

                    if (!string.IsNullOrEmpty(msg))
                    {
                        _errorInfo.AddOrUpdate(nameof(EnabledPortStatusErrorsText), msg);
                    }
                    else
                    {
                        msg = _irqModule.ValidateEnabledPortStatusErrorValue(statusErrors);
                        if (!string.IsNullOrEmpty(msg))
                        {
                            _errorInfo.AddOrUpdate(nameof(EnabledPortStatusErrorsText), msg);
                        }
                        else
                        {
                            _errorInfo.Remove(nameof(EnabledPortStatusErrorsText));
                            _irqModule.EnabledPortStatusErrors = statusErrors;
                            _enabledPortStatusErrorsText = AdqCommon.FormatHexValue(statusErrors);
                            UpdatePortStatusErrorBitItems(statusErrors);
                        }
                    }

                    NotifyPropertyChanged(nameof(EnabledPortStatusErrorsText));
                    NotifyPropertyChanged(nameof(IsInputValid));
                }
            }
        }


        private string _enabledFaultRecoveryText = "0";
        public string EnabledFaultRecoveryText
        {
            get { return _enabledFaultRecoveryText; }
            set
            {
                if (_enabledFaultRecoveryText != value)
                {
                    _enabledFaultRecoveryText = value;

                    string msg = InputValidation.ValidateHexValue(_enabledFaultRecoveryText, out uint faultRecovery);

                    if (!string.IsNullOrEmpty(msg))
                    {
                        _errorInfo.AddOrUpdate(nameof(EnabledFaultRecoveryText), msg);
                    }
                    else
                    {
                        msg = _irqModule.ValidateEnabledFaultRecoveryValue(faultRecovery);
                        if (!string.IsNullOrEmpty(msg))
                        {
                            _errorInfo.AddOrUpdate(nameof(EnabledFaultRecoveryText), msg);
                        }
                        else
                        {
                            _errorInfo.Remove(nameof(EnabledFaultRecoveryText));
                            _irqModule.EnabledFaultRecovery = faultRecovery;
                            _enabledFaultRecoveryText = AdqCommon.FormatHexValue(faultRecovery);
                            UpdateFaultRecoveryBitItems(faultRecovery);
                        }
                    }

                    NotifyPropertyChanged(nameof(EnabledFaultRecoveryText));
                    NotifyPropertyChanged(nameof(IsInputValid));
                }
            }
        }

        private void UpdatePortStatusErrorBitItems(uint statusError)
        {
            foreach (PortStatusItemViewModel enabledPortStatusBit in EnabledPortStatusBits)
            {
                enabledPortStatusBit.BitStatusError = ((statusError >> enabledPortStatusBit.BitNumber) & 1) != 0;
            }
        }

        private void UpdateFaultRecoveryBitItems(uint faultRecovery)
        {
            foreach (PortStatusItemViewModel enabledPortStatusBit in EnabledPortStatusBits)
            {
                enabledPortStatusBit.BitFaultRecovery = ((faultRecovery >> enabledPortStatusBit.BitNumber) & 1) != 0;
            }
        }

        private void PortStatusChanged(object sender, PortStatusChangedEventArgs e)
        {
            UpdatePortStatusErrorIndicators(e.StatusPortErrorValue);
            UpdateFaultRecoveryIndicators(e.StatusFaultRecoveryValue);
        }

        private void UpdatePortStatusErrorIndicators(uint portStatusValue)
        {
            if (_portStatusErrorIndicatorValue != portStatusValue)
            {
                _portStatusErrorIndicatorValue = portStatusValue;
                NotifyPropertyChanged(nameof(PortStatusErrorIndicatorValue));
                UpdatePortStatusErrorIndicatorBits(portStatusValue);
            }
        }

        private void UpdateFaultRecoveryIndicators(uint faultRecoveryValue)
        {
            if (_faultRecoveryIndicatorValue != faultRecoveryValue)
            {
                _faultRecoveryIndicatorValue = faultRecoveryValue;
                NotifyPropertyChanged(nameof(FaultRecoveryIndicatorValue));
                UpdateFaultRecoveryIndicatorBits(faultRecoveryValue);
            }
        }

        private uint _portStatusErrorIndicatorValue;
        public uint PortStatusErrorIndicatorValue
        {
            get { return _portStatusErrorIndicatorValue; }
        }

        private uint _faultRecoveryIndicatorValue;
        public uint FaultRecoveryIndicatorValue
        {
            get { return _faultRecoveryIndicatorValue; }
        }

        private void UpdatePortStatusErrorIndicatorBits(uint portStatusValue)
        {
            foreach (PortStatusItemViewModel portStatusIndicatorBit in PortStatusIndicatorBits)
            {
                portStatusIndicatorBit.BitStatusError = ((portStatusValue >> portStatusIndicatorBit.BitNumber) & 1) != 0;
            }
        }

        private void UpdateFaultRecoveryIndicatorBits(uint faultRecoveryValue)
        {
            foreach (PortStatusItemViewModel portStatusIndicatorBit in PortStatusIndicatorBits)
            {
                portStatusIndicatorBit.BitFaultRecovery = ((faultRecoveryValue >> portStatusIndicatorBit.BitNumber) & 1) != 0;
            }
        }
    }
}
