﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferAO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;
using WriteBufferAO.Utility;

namespace WriteBufferAO.Models
{
    class BoardLocator : IDisposable, INotifyPropertyChanged
    {
        private bool _disposed = false;
        private uint _adqOpenError = AdqCommon.NotOpen;

        public event PropertyChangedEventHandler PropertyChanged;

        public BoardLocator(PropertyChangedEventHandler handler)
        {
            PropertyChanged += handler;
        }

        public void InitBoardLocator()
        {
            try
            {
                _adqOpenError = OpenAdqDriver();
                if (_adqOpenError != AdqErrors.ADQ_ERROR_NONE) return;

                AdqAoBoards = new List<AdqAoBoard>();
                FindAdqBoards(AdqConstants.ADQ_MODULE_TYPE_AO);
            }
            catch (DllNotFoundException ex)
            {
                InfoText = "The device driver is not properly installed." + Environment.NewLine
                           + ex.Message;
            }
        }

        private void FindAdqBoards(uint moduleType)
        {
            uint numberOfBoards = GetNumberOfBoards();
            for (uint boardIndex = 0; boardIndex < numberOfBoards; boardIndex++)
            {
                uint adqError = AdqDriver.adqGetNumberOfModules(boardIndex, out uint numberOfModules);
                if (adqError != AdqErrors.ADQ_ERROR_NONE)
                {
                    SetFunctionErrorInfo("adqGetNumberOfModules", adqError);
                    continue;
                }

                adqError = GetModules(boardIndex, numberOfModules, moduleType);
                if (adqError != AdqErrors.ADQ_ERROR_NONE)
                {
                    continue;
                }
            }
        }

        private uint GetNumberOfBoards()
        {
            uint adqError = AdqDriver.adqGetNumberOfBoards(out uint numBoards);
            if (_adqOpenError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetNumberOfBoards", adqError);
                return 0;
            }
            return numBoards;
        }

        private uint GetModules(uint boardIndex, uint numberOfModules, uint moduleType)
        {
            uint adqError = AdqErrors.ADQ_ERROR_NONE;
            AdqAoBoard aoBoard = null;
            for (uint moduleIdx = 0; moduleIdx < numberOfModules; moduleIdx++)
            {
                adqError = AdqDriver.adqGetModuleType(boardIndex, moduleIdx, out uint currentModuleType);
                if (adqError != AdqErrors.ADQ_ERROR_NONE)
                {
                    SetFunctionErrorInfo("adqGetModuleType", adqError);
                    continue;
                }

                if (moduleType == currentModuleType)
                {
                    if (aoBoard == null)
                    {
                        aoBoard = GetBoardInfo(boardIndex);
                        if (aoBoard == null)  break;
                        AdqAoBoards.Add(aoBoard);
                    }

                    adqError = GetModuleInfo(boardIndex, moduleIdx, aoBoard);
                    if (adqError != AdqErrors.ADQ_ERROR_NONE)
                    {
                        continue;
                    }
                }
            }
            return adqError;
        }

        private uint GetModuleInfo(uint boardIndex, uint moduleIndex, AdqAoBoard aoBoard)
        {
            var adqGeneralModuleInfo = new AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID);
            var adqAoModuleInfo = new AdqDriver.ADQAOModuleInfo(AdqDriver.ADQAOModuleInfo_ID);

            IntPtr moduleInfo = Marshal.AllocHGlobal((int)adqAoModuleInfo.Size);
            Marshal.StructureToPtr(adqAoModuleInfo, moduleInfo, false);

            uint adqError = AdqDriver.adqGetModuleInfo(boardIndex, moduleIndex, ref adqGeneralModuleInfo, 
                adqAoModuleInfo.Size, moduleInfo, out uint infoSize);
            adqAoModuleInfo = (AdqDriver.ADQAOModuleInfo)Marshal.PtrToStructure(moduleInfo, typeof(AdqDriver.ADQAOModuleInfo));
            Marshal.FreeHGlobal(moduleInfo);

            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqGetModuleInfo", adqError);
                return adqError;
            }

            Debug.Assert(adqAoModuleInfo.Size == infoSize, 
                string.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}", 
                    boardIndex, moduleIndex, adqAoModuleInfo.Size, infoSize));

            if (IsClockControlledModule(adqGeneralModuleInfo.ClockType))
            {
                Debug.Assert(adqGeneralModuleInfo.ClockMinimumDivider > 0, 
                    string.Format("Board {0}: The ClockMinimumDivider of the module {1} is zero", boardIndex, moduleIndex));

                var aoModule = new AoModule(boardIndex, moduleIndex, ref adqGeneralModuleInfo, ref adqAoModuleInfo);
                aoBoard.AddModule(aoModule);
            }
            return adqError;
        }

        private bool IsClockControlledModule(uint clockType)
        {
            return clockType == AdqConstants.ADQ_CLOCK_TYPE_HARDWARE || clockType == AdqConstants.ADQ_CLOCK_TYPE_SOFTWARE;
        }

        private AdqAoBoard GetBoardInfo(uint boardIndex)
        {
            AdqAoBoard aoBoard = null;
            var boardInfo = new AdqDriver.ADQBoardInfo(AdqDriver.ADQBoardInfo_ID);

            uint adqError = AdqDriver.adqGetBoardInfo(boardIndex, ref boardInfo);
            if (adqError == AdqErrors.ADQ_ERROR_NONE)
            {
                aoBoard = new AdqAoBoard(boardIndex, ref boardInfo);
            }
            else
            {
                SetFunctionErrorInfo("adqGetBoardInfo", adqError);
            }
            return aoBoard;
        }

        private uint OpenAdqDriver()
        {
            uint adqError = AdqDriver.adqOpen();
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqOpen", adqError);
            }
            return adqError;
        }

        private void CloseAdqDriver()
        {
            if (_adqOpenError == AdqErrors.ADQ_ERROR_NONE)
            {
                AdqDriver.adqClose();
                _adqOpenError = AdqCommon.NotOpen;
            }
        }

        public List<AdqAoBoard> AdqAoBoards { get; private set; }

        private void SetFunctionErrorInfo(string functionName, uint adqError)
        {
            InfoText = string.Format("[{0}] API-function {1} returned Error number: 0x{2:X}", 
                DateTime.Now.ToString(AdqCommon.TimeFmt), 
                functionName, 
                adqError);
        }

        private string _infoText;
        public string InfoText
        {
            get { return _infoText; }
            private set
            {
                _infoText = value;
                NotifyPropertyChanged(nameof(InfoText));
            }
        }

        void NotifyPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        ~BoardLocator()
        {
            Dispose(false);
        }

        public void Dispose()
        {
            Dispose(true);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (!_disposed)
            {
                _disposed = true;
                CloseAdqDriver();
            }
        }
    }
}
