﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferAO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using System;
using System.Collections.Generic;
using System.Windows.Media;

namespace WriteBufferAO.Utility
{
    static class AdqCommon
    {
        public static readonly double TwoPI = (2 * Math.PI);
        public static readonly string TimeFmt = "HH:mm:ss.fff";
        public static readonly uint NotOpen = AdqErrors.ADQ_ERROR_TYPE_FATAL | AdqErrors.ADQ_ERROR_NOT_OPEN;

        public static readonly uint MessengerContextInfo = 1;
        public static readonly uint MessengerContextStatus = 2;

        public static Brush[] BackgroundColors = {
            Brushes.Honeydew,
            Brushes.LemonChiffon,
            Brushes.Lavender,
            Brushes.PapayaWhip,
            Brushes.PowderBlue,
            Brushes.Aquamarine,
            Brushes.BlanchedAlmond,
            Brushes.Azure,
            Brushes.MediumTurquoise,
            Brushes.BurlyWood,
            Brushes.LightPink,
            Brushes.Thistle,
            Brushes.PaleGoldenrod,
            Brushes.LightBlue,
            Brushes.LightGreen,
            Brushes.Bisque
        };

        public static string GetPhysicalUnitSymbol(uint physicalUnit)
        {
            return GetDictionaryValue(_physicalUnitSymbols, physicalUnit);
        }

        private static string GetDictionaryValue(IDictionary<uint, string> dictionary, uint key)
        {
            bool found = dictionary.TryGetValue(key, out string stringValue);
            if (!found)
            {
                stringValue = string.Format("0x{0:X8}", key);
            }
            return stringValue;
        }

        private static readonly Dictionary<uint, string> _physicalUnitSymbols = new Dictionary<uint, string>
        {
            {AdqConstants.ADQ_PHYSICAL_UNIT_VOLT, "V"},
            {AdqConstants.ADQ_PHYSICAL_UNIT_AMPERE, "A"},
            {AdqConstants.ADQ_PHYSICAL_UNIT_OHM, "Ω"}
        };


        public static string GetModelName(string boardName)
        {
            int posADQ = boardName.IndexOf("ADQ");
            if (posADQ < 0) posADQ = 0;

            int posEnd = boardName.IndexOf(" ", posADQ);
            if (posEnd < 0)
            {
                posEnd = boardName.Length;
            }

            return boardName.Substring(posADQ, posEnd - posADQ);
        }

        public static string FormatPhysicalValue(double physicalValue)
        {
            string physicalTxt;
            double tmpVal = Math.Abs(physicalValue);
            if (tmpVal == 0)
            {
                physicalTxt = "0";
            }
            else if (tmpVal < 0.001)
            {
                physicalTxt = string.Format("{0:0.######}µ", physicalValue * 1000000);
            }
            else if (tmpVal < 1)
            {
                physicalTxt = string.Format("{0:0.######}m", physicalValue * 1000);
            }
            else
            {
                physicalTxt = string.Format("{0:0.######}", physicalValue);
            }
            return physicalTxt;
        }

        public static string FormatFrequency(double frequency)
        {
            string s;
            if (frequency >= 1000000)
            {
                s = string.Format("{0:0.######}MHz", frequency / 1000000);
            }
            else if (frequency >= 1000)
            {
                s = string.Format("{0:0.######}kHz", frequency / 1000);
            }
            else
            {
                s = string.Format("{0:0.########}Hz", frequency);
            }
            return s;
        }

        public static string FormatTimeSpan(double seconds)
        {
            string s = string.Empty;
            try
            {
                TimeSpan ts = TimeSpan.FromSeconds(seconds);

                s = string.Concat(ts.Days >= 1 ? string.Concat(ts.Days, " d ") : "",
                                     ts.Hours >= 1 ? string.Concat(ts.Hours, " h ") : "",
                                     ts.Minutes >= 1 ? string.Concat(ts.Minutes, " min ") : "",
                                     ts.Seconds >= 1 ? string.Concat(ts.Seconds, " sec ") : "",
                                     ts.Milliseconds >= 1 ? string.Concat(ts.Milliseconds, " ms") : "");
                if (s == string.Empty && seconds > 0)
                {
                    s = "< 1 ms";
                }
            }
            catch (OverflowException)
            {
                ;
            }
            return s;
        }
    }
}

