﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferAO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Threading.Tasks;
using WriteBufferAO.Messaging;
using WriteBufferAO.Utility;

namespace WriteBufferAO.ViewModels
{
    public class ArbitrarySignalViewModel : INotifyPropertyChanged, IDataErrorInfo
    {
        private const char _defaultValueSeparator = ';';

        private readonly Dictionary<string, string> _errorInfo;
        private readonly IDialogService _dialogService;

        public event PropertyChangedEventHandler PropertyChanged;

        public RelayCommand BrowseCsvFileCommand { get; private set; }
        public RelayCommand ReadCsvFileCommand { get; private set; }

        public ArbitrarySignalViewModel(IDialogService dialogService)
        {
            _dialogService = dialogService;
            _errorInfo = new Dictionary<string, string>();
            _csvColumnHeaders = new ObservableCollection<CsvColumnHeader>();
            InitCommands();
        }

        private void InitCommands()
        {
            BrowseCsvFileCommand = new RelayCommand(BrowseCsvFileExecute, CanBrowseCsvFileExecute);
            ReadCsvFileCommand = new RelayCommand(ReadCsvFileExecute, CanReadCsvFileExecute);
        }

        private string _csvFileName;
        public string CsvFileName
        {
            get { return _csvFileName; }
            set
            {
                if (_csvFileName != value)
                {
                    _csvFileName = value;
                    RaisePropertyChanged(nameof(CsvFileName));
                    ReadCsvFileCommand.RaiseCanExecuteChanged();
                }
            }
        }

        private bool CanBrowseCsvFileExecute(object obj)
        {
            return true;
        }

        private void BrowseCsvFileExecute(object obj)
        {
            string csvFileName = _dialogService.BrowseFileDialog("", "CSV Files (*.csv)|*.csv|All Files (*.*)|*.*");
            if (string.IsNullOrEmpty(csvFileName)) return;

            CsvFileName = csvFileName;
            ReadCsvHeadLine(csvFileName);
        }

        private void ReadCsvHeadLine(string fileName)
        {
            InitArbitraryValues();
            NumValuesReadFromFile = _arbitraryValues.Count;

            _csvColumnHeaders.Clear();
            string[] columnHeaders;
            using (var reader = new StreamReader(File.OpenRead(fileName)))
            {
                var line = reader.ReadLine();
                columnHeaders = line.Split(_valueSeparator);
                for (int i = 0; i < columnHeaders.Length; i++)
                {
                    _csvColumnHeaders.Add(new CsvColumnHeader(columnHeaders[i], i));
                }
            }

            if (_csvColumnHeaders.Count > 0)
            {
                SelectedCsvColumnHeader = _csvColumnHeaders[0];
            }
        }

        private char _valueSeparator = _defaultValueSeparator;
        private string _valueSeparatorText = _defaultValueSeparator.ToString();
        public string ValueSeparatorText
        {
            get { return _valueSeparatorText; }
            set
            {
                if (_valueSeparatorText != value)
                {
                    _valueSeparatorText = value;

                    string msg = ValidateValueSeparator(_valueSeparatorText, out _valueSeparator);
                    UpdateErrorInfo(nameof(ValueSeparatorText), msg);
                    RaisePropertyChanged(nameof(ValueSeparatorText));
                }
            }
        }

        private CsvColumnHeader _selectedCsvColumnHeader;
        public CsvColumnHeader SelectedCsvColumnHeader
        {
            get { return _selectedCsvColumnHeader; }
            set
            {
                if (_selectedCsvColumnHeader != value)
                {
                    _selectedCsvColumnHeader = value;
                    RaisePropertyChanged(nameof(SelectedCsvColumnHeader));
                }
            }
        }

        private ObservableCollection<CsvColumnHeader> _csvColumnHeaders;
        public ObservableCollection<CsvColumnHeader> CsvColumnsHeaders
        {
            get { return _csvColumnHeaders; }
            set
            {
                if (_csvColumnHeaders != value)
                {
                    _csvColumnHeaders = value;
                }
            }
        }

        private int _valuesStartLine = 1;
        private string _valuesStartLineText = "1";
        public string ValuesStartLineText
        {
            get { return _valuesStartLineText; }
            set
            {
                if (_valuesStartLineText != value)
                {
                    _valuesStartLineText = value;
                    string msg = ValidateValueStartLine(_valuesStartLineText, out _valuesStartLine);

                    UpdateErrorInfo(nameof(ValuesStartLineText), msg);
                    RaisePropertyChanged(nameof(ValuesStartLineText));
                    RaisePropertyChanged(nameof(IsGeneratorInputValid));
                }
            }
        }

        private List<double> _arbitraryValues;
        public List<double> ArbitraryValues
        {
            get { return _arbitraryValues; }
            set
            {
                if (_arbitraryValues != value)
                {
                    _arbitraryValues = value;
                    RaisePropertyChanged(nameof(ArbitraryValues));
                }
            }
        }

        private bool _isCsvReadingIdle = true;
        public bool IsCsvReadingIdle
        {
            get { return _isCsvReadingIdle; }
            set
            {
                if (_isCsvReadingIdle != value)
                {
                    _isCsvReadingIdle = value;
                    RaisePropertyChanged(nameof(IsCsvReadingIdle));
                    ReadCsvFileCommand.RaiseCanExecuteChanged();
                }
            }
        }

        private bool CanReadCsvFileExecute(object obj)
        {
            return !string.IsNullOrEmpty(_csvFileName) || _isCsvReadingIdle;
        }

        private async void ReadCsvFileExecute(object obj)
        {
            try
            {
                InitArbitraryValues();
                NumValuesReadFromFile = _arbitraryValues.Count;

                IsCsvReadingIdle = false;
                var progressIndicator = new Progress<int>(UpdateValuesRead);
                int x = await ReadCsvFileAsync(progressIndicator);
            }
            finally
            {
                IsCsvReadingIdle = true;
                NumValuesReadFromFile = _arbitraryValues.Count;
            }
        }

        private void InitArbitraryValues()
        {
            if (_arbitraryValues == null)
            {
                _arbitraryValues = new List<double>();
            }
            else
            {
                _arbitraryValues.Clear();
            }
        }

        private void UpdateValuesRead(int numberOfValues)
        {
            NumValuesReadFromFile = numberOfValues;
        }

        private int _numValuesReadFromFile = 0;
        public int NumValuesReadFromFile
        {
            get { return _numValuesReadFromFile; }
            set
            {
                if (_numValuesReadFromFile != value)
                {
                    _numValuesReadFromFile = value;
                    RaisePropertyChanged(nameof(NumValuesReadFromFile));
                }
            }
        }

        private void UpdateErrorInfo(string propertyName, string msg)
        {
            if (!string.IsNullOrEmpty(msg))
            {
                _errorInfo.AddOrUpdate(propertyName, msg);
            }
            else
            {
                _errorInfo.Remove(propertyName);
            }
        }

        public string Error
        {
            get { return "ArbitrarySignalViewModel.Error"; }
        }

        public string this[string propertyName]
        {
            get
            {
                return _errorInfo.ContainsKey(propertyName) ? _errorInfo[propertyName] : null;
            }
        }

        private string ValidateValueSeparator(string textValue, out char valueSeparator)
        {
            string msg = string.Empty;
            string tmp = textValue.Trim();

            valueSeparator = '\0';
            if (string.IsNullOrEmpty(tmp))
            {
                msg = "The value cannot be empty";
            }
            else 
            {
                if (tmp.Length == 1)
                {
                    valueSeparator = tmp[0];
                }
                else if (tmp.Length == 2)
                {
                    if (tmp == "\\t")
                    {
                        valueSeparator = '\t';
                    }
                    else
                    {
                        msg = "For 2 characters only \\t is supported";
                    }
                }
                else
                {
                    msg = "The entered text '" + textValue + "' is not a valid separator";
                }
            }
            return msg;
        }

        private string ValidateValueStartLine(string textValue, out int lineNumber)
        {
            lineNumber = 0;
            string msg = string.Empty;
            if (string.IsNullOrEmpty(textValue))
            {
                msg = "The value cannot be empty";
            }
            else
            {
                if (!int.TryParse(textValue, out lineNumber))
                {
                    msg = "The entered value \"" + textValue + "\" is not a valid number";
                }
                else if (lineNumber < 0)
                {
                    msg = "The start line number must not be negative";
                }
            }
            return msg;
        }

        public bool IsGeneratorInputValid
        {
            get
            {
                return (_errorInfo.Count == 0) 
                    && (!string.IsNullOrEmpty(_csvFileName))
                    && (_arbitraryValues != null) 
                    && (_arbitraryValues.Count > 0);
            }
        }

        async Task<int> ReadCsvFileAsync(IProgress<int> progress)
        {
            return await Task.Run(() => ReadCsvFile(progress));
        }

        private int ReadCsvFile(IProgress<int> progress)
        {
            int currentLine = 0;
            int convertedValues = 0;
            using (var reader = new StreamReader(File.OpenRead(_csvFileName)))
            {
                string line;
                int columnCount = _csvColumnHeaders.Count;
                int columnIndex = _selectedCsvColumnHeader.ColumnIndex;
                Messenger.Default.Send(new InfoMessage(string.Format("Start reading CSV file: column: {0}", _selectedCsvColumnHeader.ColumnName)));
                while ((line = reader.ReadLine()) != null)
                {
                    if (currentLine >= _valuesStartLine)
                    {
                        var values = line.Split(_valueSeparator);
                        if (values.Length == columnCount)
                        {
                            bool isConverted = double.TryParse(values[columnIndex], out double value);
                            if (!isConverted)
                            {
                                Messenger.Default.Send(new InfoMessage(string.Format("Reading CSV file line number {0}: cannot convert value: {1}", currentLine, values[columnIndex])));
                                break;
                            }
                            _arbitraryValues.Add(value);
                            convertedValues++;
                            if (convertedValues == 100000)
                            {
                                progress.Report(_arbitraryValues.Count);                                
                                convertedValues = 0;
                            }
                        }
                        else
                        {
                            Messenger.Default.Send(new InfoMessage(string.Format("Reading CSV file: skipping line number: {0}", currentLine)));
                        }
                    }
                    currentLine++;
                }
            }
            return _arbitraryValues.Count;
        }

        void RaisePropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
