﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferAO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows;
using WriteBufferAO.Messaging;
using WriteBufferAO.Models;
using WriteBufferAO.Utility;

namespace WriteBufferAO.ViewModels
{
    public class MainViewModel : INotifyPropertyChanged
    {
        private readonly BoardLocator _boardLocator;
        private readonly DialogService _dialogService;
        private readonly List<AoModuleViewModel> _runningModules;

        public RelayCommand CloseCommand { get; set; }

        public event PropertyChangedEventHandler PropertyChanged;

        public MainViewModel()
        {
            _isBoardIdle = true;
            _dialogService = new DialogService();
            _runningModules = new List<AoModuleViewModel>();
            InitCommands();
            InitMessages();
            _boardLocator = new BoardLocator(MainViewModel_PropertyChanged);
            _boardLocator.InitBoardLocator();
            if (_boardLocator.AdqAoBoards.Count == 1)
            {
                SelectedBoard = _boardLocator.AdqAoBoards[0];
            }
        }

        private void InitCommands()
        {
            CloseCommand = new RelayCommand(CloseApplication, CanCloseApplication);
        }

        private void InitMessages()
        {
            Messenger.Default.Register<InfoMessage>(this, OnInfoMessageReceived, AdqCommon.MessengerContextInfo);
            Messenger.Default.Register<ModuleStatusMessage>(this, OnModuleStatusMessageReceived, AdqCommon.MessengerContextStatus);
        }

        private void OnInfoMessageReceived(InfoMessage obj)
        {
            AppendInfoText(obj.InfoText);
        }

        private void OnModuleStatusMessageReceived(ModuleStatusMessage obj)
        {
            if (obj.IsIdle)
            {
                _runningModules.Remove(obj.ModuleViewModel);
                if (_runningModules.Count == 0)
                {
                    IsBoardIdle = true;
                }
            }
            else
            {
                if (_runningModules.Count == 0)
                {
                    IsBoardIdle = false;
                }
                _runningModules.Add(obj.ModuleViewModel);
            }
        }

        private void StopRunningModules()
        {
            foreach (var aoModuleVm in AoModuleViewModels)
            {
                aoModuleVm.StopModule();
            }
        }

        private bool _isBoardIdle;
        public bool IsBoardIdle
        {
            get { return _isBoardIdle; }
            set
            {
                if (_isBoardIdle != value)
                {
                    _isBoardIdle = value;
                    NotifyPropertyChanged(nameof(IsBoardIdle));
                }
            }
        }

        public List<AdqAoBoard> AdqAoBoards
        {
            get { return _boardLocator.AdqAoBoards; }
        }

        AdqAoBoard _selectedBoard;
        public AdqAoBoard SelectedBoard
        {
            get { return _selectedBoard; }
            set
            {
                if (_selectedBoard != value)
                {
                    AoModuleViewModels.Clear();
                    _selectedBoard = value;
                    if (_selectedBoard != null)
                    {
                        InitAoModuleViewModels(_selectedBoard);
                    }
                    NotifyPropertyChanged(nameof(SelectedBoard));
                }
            }
        }

        private void InitAoModuleViewModels(AdqAoBoard aoBoard)
        {
            int idx = 0;
            foreach (AoModule aoModule in aoBoard.AoModules)
            {
                var background = AdqCommon.BackgroundColors[idx % AdqCommon.BackgroundColors.Length];
                var vm = new AoModuleViewModel(aoModule, background, _dialogService);
                AoModuleViewModels.Add(vm);
                idx++;
            }
        }

        public ObservableCollection<AoModuleViewModel> AoModuleViewModels { get; } 
            = new ObservableCollection<AoModuleViewModel>();
        public string InfoText { get; private set; } = string.Empty;

        private void AppendInfoText(string text)
        {
            if (InfoText != string.Empty)
            {
                InfoText += Environment.NewLine;
            }
            InfoText += text;
            NotifyPropertyChanged(nameof(InfoText));
        }

        void NotifyPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        private void CloseApplication(object obj)
        {
            StopRunningModules();
            Application.Current.MainWindow.Close();
        }

        private bool CanCloseApplication(object obj)
        {
            return true;
        }

        void MainViewModel_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(AoModule.DesiredOutputFrequency))
            {
                NotifyPropertyChanged(nameof(AoModule.DesiredOutputFrequency));
            }
            else if (e.PropertyName == nameof(AoModule.ActualOutputFrequency))
            {
                NotifyPropertyChanged(nameof(AoModule.ActualOutputFrequency));
            }
            else if (e.PropertyName == nameof(BoardLocator.InfoText))
            {
                AppendInfoText(_boardLocator.InfoText);
            }
        }
    }
}
