﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferAO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using System.Collections.Generic;
using System.ComponentModel;
using WriteBufferAO.Models;
using WriteBufferAO.Utility;

namespace WriteBufferAO.ViewModels
{
    public class SignalViewModel : INotifyPropertyChanged, IDataErrorInfo
    {
        private readonly string _fmtStr = "{0:0.###}";
        private readonly Dictionary<string, string> _errorInfo = new Dictionary<string, string>();

        public RelayCommand SetFrequencyEnd { get; private set; }
        public RelayCommand SetAmplitudeMinEnd { get; private set; }
        public RelayCommand SetAmplitudeMaxEnd { get; private set; }

        public event PropertyChangedEventHandler PropertyChanged;

        private readonly SignalViewModelSettings _signalViewModelSettings;
        private readonly SignalSettings _signalSettings;
        public SignalViewModel(SignalViewModelSettings signalViewModelSettings, SignalSettings signalSettings)
        {
            _signalViewModelSettings = signalViewModelSettings;
            _signalSettings = signalSettings;

            _isEnabled = _signalViewModelSettings.IsEnabled;
            _phaseShift = _signalSettings.PhaseShift;
            _phaseAngleText = string.Format(_fmtStr, _phaseShift * 360.0);
            _signalBeginFrequency = _signalSettings.SignalBeginFrequency;
            _signalBeginFrequencyText = string.Format(_fmtStr, _signalBeginFrequency);
            _signalEndFrequency = _signalSettings.SignalEndFrequency;
            _signalEndFrequencyText = string.Format(_fmtStr, _signalEndFrequency);
            _signalBeginAmplitudeMin = _signalSettings.SignalBeginPhysicalMin;
            _signalBeginAmplitudeMinText = string.Format(_fmtStr, _signalBeginAmplitudeMin);
            _signalBeginAmplitudeMax = _signalSettings.SignalBeginPhysicalMax;
            _signalBeginAmplitudeMaxText = string.Format(_fmtStr, _signalBeginAmplitudeMax); 
            _signalEndAmplitudeMin = _signalSettings.SignalEndPhysicalMin;
            _signalEndAmplitudeMinText = string.Format(_fmtStr, _signalEndAmplitudeMin); 
            _signalEndAmplitudeMax = _signalSettings.SignalEndPhysicalMax;
            _signalEndAmplitudeMaxText = string.Format(_fmtStr, _signalEndAmplitudeMax);
            InitSignalGenerators();
            InitCommands();
        }

        private void InitCommands()
        {
            SetFrequencyEnd = new RelayCommand(SetFrequencyEndExecute, CanSetFrequencyEndExecute);
            SetAmplitudeMinEnd = new RelayCommand(SetAmplitudeMinEndExecute, CanSetAmplitudeMinEndExecute);
            SetAmplitudeMaxEnd = new RelayCommand(SetAmplitudeMaxEndExecute, CanSetAmplitudeMaxEndExecute);
        }

        private bool CanSetFrequencyEndExecute(object obj)
        {
            return _isEnabled;
        }

        private void SetFrequencyEndExecute(object obj)
        {
            SignalEndFrequencyText = _signalBeginFrequencyText;
        }

        private bool CanSetAmplitudeMinEndExecute(object obj)
        {
            return _isEnabled;
        }

        private void SetAmplitudeMinEndExecute(object obj)
        {
            SignalEndAmplitudeMinText = _signalBeginAmplitudeMinText;
        }

        private bool CanSetAmplitudeMaxEndExecute(object obj)
        {
            return _isEnabled;
        }

        private void SetAmplitudeMaxEndExecute(object obj)
        {
            SignalEndAmplitudeMaxText = _signalBeginAmplitudeMaxText;
        }

        private void InitSignalGenerators()
        {
            SignalGenerators = new List<ISignalGenerator>
            {
                new SineWaveGenerator(),
                new SquareWaveGenerator(),
                new TriangleWaveGenerator(),
                new SawtoothFallingWaveGenerator(),
                new SawtoothRisingWaveGenerator(),
                new SincWaveGenerator(),
                new SineCutWaveGenerator(),
                new SineModSquareWaveGenerator(),
                new ChargeDischargeWaveGenerator(),
                new MexicanHatWaveGenerator(),
                new MorletWaveletGenerator(),
                new ShannonWaveletGenerator(),
                new PoissonWaveletGenerator(),
                new RandomRectangleWaveGenerator(),
                new NonReturnToZeroWaveGenerator(),
                new WhiteNoiseSignalGenerator(),
                new GaussianNoiseSignalGenerator()
            };

            _selectedSignalGenerator = SignalGenerators[0];
        }

        public List<ISignalGenerator> SignalGenerators { get; private set; }

        ISignalGenerator _selectedSignalGenerator;
        public ISignalGenerator SelectedSignalGenerator
        {
            get { return _selectedSignalGenerator; }
            set
            {
                if (_selectedSignalGenerator != value)
                {
                    _selectedSignalGenerator = value;
                    RaisePropertyChanged(nameof(SelectedSignalGenerator));
                }
            }
        }

        internal void ApplySignalGenerator(WaveForm waveForm)
        {
            if (_isEnabled)
            {
                var signalSettings = new SignalSettings
                {
                    PhaseShift = _phaseShift,
                    SignalBeginFrequency = _signalBeginFrequency,
                    SignalBeginPhysicalMin = _signalBeginAmplitudeMin,
                    SignalBeginPhysicalMax = _signalBeginAmplitudeMax,
                    SignalEndFrequency = _signalEndFrequency,
                    SignalEndPhysicalMin = _signalEndAmplitudeMin,
                    SignalEndPhysicalMax = _signalEndAmplitudeMax,
                    RangePhysicalMin = _signalSettings.RangePhysicalMin,
                    RangePhysicalMax = _signalSettings.RangePhysicalMax
                };

                waveForm.ApplySignalGenerator(_selectedSignalGenerator, signalSettings);
            }
        }

        private bool _isEnabled;
        public bool IsEnabled
        {
            get { return _isEnabled; }
            set
            {
                if (_isEnabled != value)
                {
                    _isEnabled = value;
                    RaisePropertyChanged(nameof(IsEnabled));
                }
            }
        }

        public string SignalName
        {
            get { return _signalViewModelSettings.SignalName; }
        }

        private double _phaseShift;
        private string _phaseAngleText;
        public string PhaseAngleText
        {
            get { return _phaseAngleText; }
            set
            {
                if (_phaseAngleText != value)
                {
                    _phaseAngleText = value;
                    string msg = InputValidation.ValidateValueIsWithinRange(_phaseAngleText, 0.0, 360.0, out double phaseAngle);
                    if (string.IsNullOrEmpty(msg))
                    {
                        _phaseShift = phaseAngle / 360.0;
                    }
                    UpdateErrorInfo(nameof(PhaseAngleText), msg);
                    RaisePropertyChanged(nameof(PhaseAngleText));
                    RaisePropertyChanged(nameof(IsGeneratorInputValid));
                }
            }
        }

        private double _signalBeginFrequency;
        private string _signalBeginFrequencyText;
        public string SignalBeginFrequencyText
        {
            get { return _signalBeginFrequencyText; }
            set
            {
                if (_signalBeginFrequencyText != value)
                {
                    _signalBeginFrequencyText = value;

                    string msg = InputValidation.ValidateValueIsPositive(_signalBeginFrequencyText, out _signalBeginFrequency);
                    UpdateErrorInfo(nameof(SignalBeginFrequencyText), msg);
                    RaisePropertyChanged(nameof(SignalBeginFrequencyText));
                    RaisePropertyChanged(nameof(IsGeneratorInputValid));
                }
            }
        }

        private double _signalEndFrequency;
        private string _signalEndFrequencyText;
        public string SignalEndFrequencyText
        {
            get { return _signalEndFrequencyText; }
            set
            {
                if (_signalEndFrequencyText != value)
                {
                    _signalEndFrequencyText = value;

                    string msg = InputValidation.ValidateValueIsPositive(_signalEndFrequencyText, out _signalEndFrequency);
                    UpdateErrorInfo(nameof(SignalEndFrequencyText), msg);
                    RaisePropertyChanged(nameof(SignalEndFrequencyText));
                    RaisePropertyChanged(nameof(IsGeneratorInputValid));
                }
            }
        }

        private double _signalBeginAmplitudeMin;
        private string _signalBeginAmplitudeMinText;
        public string SignalBeginAmplitudeMinText
        {
            get { return _signalBeginAmplitudeMinText; }
            set
            {
                if (_signalBeginAmplitudeMinText != value)
                {
                    _signalBeginAmplitudeMinText = value;
                    string msg = InputValidation.ValidateValueIsWithinRange(_signalBeginAmplitudeMinText, 
                        _signalSettings.RangePhysicalMin, _signalSettings.RangePhysicalMax, out _signalBeginAmplitudeMin);
                    UpdateErrorInfo(nameof(SignalBeginAmplitudeMinText), msg);
                    RaisePropertyChanged(nameof(SignalBeginAmplitudeMinText));
                    RaisePropertyChanged(nameof(IsGeneratorInputValid));
                }
            }
        }

        private double _signalBeginAmplitudeMax;
        private string _signalBeginAmplitudeMaxText;
        public string SignalBeginAmplitudeMaxText
        {
            get { return _signalBeginAmplitudeMaxText; }
            set
            {
                if (_signalBeginAmplitudeMaxText != value)
                {
                    _signalBeginAmplitudeMaxText = value;

                    string msg = InputValidation.ValidateValueIsWithinRange(_signalBeginAmplitudeMaxText, 
                        _signalSettings.RangePhysicalMin, _signalSettings.RangePhysicalMax, out _signalBeginAmplitudeMax);
                    UpdateErrorInfo(nameof(SignalBeginAmplitudeMaxText), msg);
                    RaisePropertyChanged(nameof(SignalBeginAmplitudeMaxText));
                    RaisePropertyChanged(nameof(IsGeneratorInputValid));
                }
            }
        }

        private double _signalEndAmplitudeMin;
        private string _signalEndAmplitudeMinText;
        public string SignalEndAmplitudeMinText
        {
            get { return _signalEndAmplitudeMinText; }
            set
            {
                if (_signalEndAmplitudeMinText != value)
                {
                    _signalEndAmplitudeMinText = value;

                    string msg = InputValidation.ValidateValueIsWithinRange(_signalEndAmplitudeMinText, 
                        _signalSettings.RangePhysicalMin, _signalSettings.RangePhysicalMax, out _signalEndAmplitudeMin);
                    UpdateErrorInfo(nameof(SignalEndAmplitudeMinText), msg);
                    RaisePropertyChanged(nameof(SignalEndAmplitudeMinText));
                    RaisePropertyChanged(nameof(IsGeneratorInputValid));
                }
            }
        }

        private double _signalEndAmplitudeMax;
        private string _signalEndAmplitudeMaxText;
        public string SignalEndAmplitudeMaxText
        {
            get { return _signalEndAmplitudeMaxText; }
            set
            {
                if (_signalEndAmplitudeMaxText != value)
                {
                    _signalEndAmplitudeMaxText = value;

                    string msg = InputValidation.ValidateValueIsWithinRange(_signalEndAmplitudeMaxText, 
                        _signalSettings.RangePhysicalMin, _signalSettings.RangePhysicalMax, out _signalEndAmplitudeMax);
                    UpdateErrorInfo(nameof(SignalEndAmplitudeMaxText), msg);
                    RaisePropertyChanged(nameof(SignalEndAmplitudeMaxText));
                    RaisePropertyChanged(nameof(IsGeneratorInputValid));
                }
            }
        }

        private void UpdateErrorInfo(string propertyName, string msg)
        {
            if (string.IsNullOrEmpty(msg))
            {
                _errorInfo.Remove(propertyName);
            }
            else
            {
                _errorInfo.AddOrUpdate(propertyName, msg);
            }
        }

        public bool IsGeneratorInputValid
        {
            get
            {
                return _errorInfo.Count == 0;
            }
        }

        public string Error
        {
            get { return "SignalViewModel.Error"; }
        }

        public string this[string propertyName]
        {
            get
            {
                return _errorInfo.ContainsKey(propertyName) ? _errorInfo[propertyName] : null;
            }
        }

        void RaisePropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
