﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the AIReadContinuous project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.ComponentModel
Imports System.Runtime.InteropServices
Imports AIReadContinuous.ALLDAQ.Boards

Public Class AnalogInputModule
    Private Const _notificationsPerSecond As Integer = 10

    Private ReadOnly _maxFrequencyHz As Double
    Private _channelList As AdqDriver.ADQAIChannel()
    Private _worker As BackgroundWorker
    Private _notifyAfterFrames As UInteger
    Private _adqError As UInteger
    Private _errorFunctionName As String

    Private _frames As UInteger()
    Private _physicalFrames As Double()
    Private _physicalFrames1 As Double()
    Private _physicalFrames2 As Double()
    Private _physicalIndex As UInteger

    Public Event ModuleInfoTextChanged As EventHandler(Of InfoTextChangedEventArgs)
    Public Event ModulePropertyChanged As EventHandler(Of ModulePropertyChangedEventArgs)

    Public Sub New(boardName As String, boardIndex As UInteger, moduleIndex As UInteger,
                   ByRef generalInfo As AdqDriver.ADQGeneralModuleInfo, ByRef aiModuleInfo As AdqDriver.ADQAIModuleInfo)
        ModelName = AdqCommon.GetModelName(boardName)
        Me.BoardIndex = boardIndex
        Me.ModuleIndex = moduleIndex
        NumberOfChannels = aiModuleInfo.NumberOfChannels
        NumberOfRanges = aiModuleInfo.NumberOfRanges
        _maxFrequencyHz = generalInfo.ClockBaseFrequencyHz / generalInfo.ClockMinimumDivider
        DesiredSampleFrequency = Math.Round(_maxFrequencyHz * AdqCommon.DefaultDesiredSampleFrequencyFactor)

        BuildModuleName()
        DisplaySeconds = AdqCommon.DefaultDisplayNumberOfFrames / DesiredSampleFrequency
        AddRanges(aiModuleInfo)
        DefaultRange = Ranges(0)
        GetTriggerInfo(boardIndex, moduleIndex)
    End Sub

    Private Sub AddRanges(ByRef adqAiModuleInfo As AdqDriver.ADQAIModuleInfo)
        Dim rangeIdx As Integer = 0
        While rangeIdx < adqAiModuleInfo.NumberOfRanges
            Dim range As AdqDriver.ADQAIRange = adqAiModuleInfo.Ranges(rangeIdx)

            Dim physRange = New AnalogRange(
                CUInt(rangeIdx),
                range.PhysicalUnit,
                range.DigitalMin,
                range.DigitalMax,
                range.PhysicalMin,
                range.PhysicalMax)
            Ranges.Add(physRange)
            rangeIdx += 1
        End While
    End Sub

    Public ReadOnly Property ModelName As String
    Public ReadOnly Property BoardIndex As UInteger
    ReadOnly Property ModuleIndex As UInteger

    Private _moduleName As String
    Public ReadOnly Property ModuleName As String
        Get
            Return _moduleName
        End Get
    End Property

    ReadOnly Property NumberOfChannels As UInteger
    ReadOnly Property NumberOfRanges As UInteger
    Public ReadOnly Property ChannelSettings As List(Of ChannelSetting) = New List(Of ChannelSetting)

    Private _hardwareTrigger As HardwareTrigger
    Public ReadOnly Property HardwareTrigger As HardwareTrigger
        Get
            Return _hardwareTrigger
        End Get
    End Property

    Public ReadOnly Property TriggerDescription As String
        Get
            If _hardwareTrigger IsNot Nothing Then
                Return _hardwareTrigger.TriggerDescription
            End If
            Return "No hardware trigger supported"
        End Get
    End Property

    Private Sub BuildModuleName()
        _moduleName = String.Format("{0}, Board {1}, Module {2}, {3} {4}, {5} {6}, max. {7}",
            ModelName,
            BoardIndex,
            ModuleIndex,
            NumberOfChannels,
            IIf(NumberOfChannels = 1, "Channel", "Channels"),
            NumberOfRanges,
            IIf(NumberOfRanges = 1, "Range", "Ranges"),
            AdqCommon.FormatFrequency(_maxFrequencyHz))
    End Sub

    Private Function GetTriggerInfo(boardIndex As UInteger, moduleIndex As UInteger) As UInteger
        Dim hwTriggerInfo = New AdqDriver.ADQHWTriggerInfo(AdqDriver.ADQHWTriggerInfo_ID)
        Dim adqError As UInteger = AdqDriver.adqGetHWTriggerInfo(boardIndex, moduleIndex, hwTriggerInfo)

        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetErrorInfo("adqGetHWTriggerInfo", adqError)
        Else
            If hwTriggerInfo.HWTriggerStartSource <> 0 OrElse hwTriggerInfo.HWTriggerStopSource <> 0 Then
                _hardwareTrigger = New HardwareTrigger(Me.BoardIndex, Me.ModuleIndex, hwTriggerInfo)
            End If
        End If

        Return adqError
    End Function

    Public Sub AddChannelSetting(channelIndex As UInteger, range As AnalogRange)
        ChannelSettings.Add(New ChannelSetting(channelIndex, range))
    End Sub

    Public Sub UpdateChannelList()
        If _channelList Is Nothing Then
            'In Visual Basic, the size of an array is declared with the upper bound of the array
            'That's why the channel list must be allocated with NumberOfChannels - 1
            _channelList = New AdqDriver.ADQAIChannel(CInt(NumberOfChannels - 1)) {}
        End If

        For i = 0 To _channelList.Length - 1
            _channelList(i).acquireChannel = False
            _channelList(i).rangeIndex = 0
        Next

        _minResolution = Double.MaxValue
        For Each chSetting As ChannelSetting In ChannelSettings
            _channelList(CInt(chSetting.ChannelIndex)).acquireChannel = True
            _channelList(CInt(chSetting.ChannelIndex)).rangeIndex = chSetting.Range.RangeIndex
            If _minResolution > chSetting.Range.PhysicalResolution Then
                _minResolution = chSetting.Range.PhysicalResolution
            End If
        Next
        _frameLength = CUInt(ChannelSettings.Count)
    End Sub

    Public Sub InitAcquisition()
        If _moduleIsBusy Then Return
        ResetModule()
        ConfigureModule()
        _moduleIsBusy = True
        If _worker Is Nothing Then
            _worker = New BackgroundWorker With
                {
                    .WorkerReportsProgress = True,
                    .WorkerSupportsCancellation = True
                }
            AddHandler _worker.DoWork, AddressOf Worker_DoWork
            AddHandler _worker.ProgressChanged, AddressOf Worker_ProgressChanged
            AddHandler _worker.RunWorkerCompleted, AddressOf Worker_RunWorkerCompleted
        End If
    End Sub

    Private Sub ConfigureModule()
        CheckCommonSettings()
        SetChannelListAndFrequencySampleTime()

        If DisplaySeconds < _sampleTime Then
            Throw New ApplicationException(String.Format("The display time ({0} sec) should be larger than the sample time {1} sec.", DisplaySeconds, _sampleTime))
        End If

        Dim displayNumberOfFrames As UInteger = CUInt(DisplaySeconds * _actualSampleFrequency)
        _notifyAfterFrames = CUInt(Math.Min(displayNumberOfFrames, _actualSampleFrequency / _notificationsPerSecond))

        If StopAfterFrames > 0 AndAlso StopAfterFrames < _notifyAfterFrames Then
            _notifyAfterFrames = CUInt(StopAfterFrames)
        End If

        Dim numberOfValues As UInteger = _notifyAfterFrames * _frameLength

        If _frames Is Nothing OrElse _frames.Length <> numberOfValues Then
            _frames = New UInteger(CInt(numberOfValues - 1)) {}
        End If

        If StopAfterFrames > 0 AndAlso StopAfterFrames < displayNumberOfFrames Then
            displayNumberOfFrames = CUInt(StopAfterFrames)
        End If

        _displayBufferNumberOfFrames = displayNumberOfFrames

        Dim numberOfPhysicalValues As Integer = CInt(displayNumberOfFrames * _frameLength)

        If _physicalFrames1 Is Nothing OrElse _physicalFrames1.Length <> numberOfPhysicalValues Then
            _physicalFrames1 = New Double(numberOfPhysicalValues - 1) {}
            _physicalFrames2 = New Double(numberOfPhysicalValues - 1) {}
        End If
        _displayBufferLength = CUInt(numberOfPhysicalValues)
        RaisePropertyChanged(AdqCommon.PropertyType.DisplayBufferLength)
    End Sub

    Private Sub CheckCommonSettings()
        If DesiredSampleFrequency = 0 Then
            Throw New ApplicationException("Please enter a valid sample frequency.")
        End If

        If DisplaySeconds = 0 Then
            Throw New ApplicationException("Please enter a valid display time.")
        End If

        If _frameLength = 0 Then
            Throw New ApplicationException("Please select channels in the channel list.")
        End If
    End Sub

    Private Function SetChannelListAndFrequencySampleTime() As Boolean
        Dim sampleFrequency As Double
        Dim adqError As UInteger = AdqDriver.adqAISetChannelListAndFrequency(BoardIndex, ModuleIndex,
                 _channelList, NumberOfChannels, DesiredSampleFrequency, sampleFrequency)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            UpdateActualSampleFrequencyAndSampleTime(sampleFrequency)
        Else
            Throw New ApplicationException(String.Format("Function adqAISetChannelListAndFrequency returned Error number: 0x{0:X}", adqError))
        End If
        Return adqError = AdqErrors.ADQ_ERROR_NONE
    End Function

    Private Function IsUsingStartTrigger() As Boolean
        Return _hardwareTrigger IsNot Nothing AndAlso _hardwareTrigger.StartTriggerEnabled
    End Function

    Private Sub EnableHardwareTrigger()
        If _hardwareTrigger IsNot Nothing Then
            _hardwareTrigger.EnableHardwareTrigger()
        End If
    End Sub

    Public Sub StartAcquisition()
        UpdateModuleInfo("Acquisition initialized")
        _worker.RunWorkerAsync()
    End Sub

    Private Sub Worker_DoWork(sender As Object, e As DoWorkEventArgs)
        Dim worker As BackgroundWorker = CType(sender, BackgroundWorker)

        _errorFunctionName = String.Empty
        _totalFramesAcquired = 0
        _totalFramesDisplayed = 0
        _totalFramesSkipped = 0
        worker.ReportProgress(AdqCommon.PropertyType.AcquisitionStarting)

        _physicalFrames = _physicalFrames1
        _physicalIndex = 0
        Dim numberOfFramesReturned As UInteger
        Dim gchFrames As GCHandle = GCHandle.Alloc(_frames, GCHandleType.Pinned)

        If IsUsingStartTrigger() Then
            worker.ReportProgress(AdqCommon.PropertyType.WaitingForStartTrigger)
        End If
        EnableHardwareTrigger()

        _adqError = AdqDriver.adqAIReadContinuousStart(BoardIndex,
                                     ModuleIndex,
                                     _notifyAfterFrames,
                                     StopAfterFrames,
                                     Nothing,
                                     IntPtr.Zero)

        If _adqError <> AdqErrors.ADQ_ERROR_NONE Then
            gchFrames.Free()
            _errorFunctionName = "adqAIReadContinuousStart"
            Return
        End If

        If IsUsingStartTrigger() Then
            worker.ReportProgress(AdqCommon.PropertyType.StartTriggerEventOccured)
        Else
            worker.ReportProgress(AdqCommon.PropertyType.AcquisitionStarted)
        End If


        While True
            If worker.CancellationPending Then
                e.Cancel = True
                Exit While
            End If

            _adqError = AdqDriver.adqAIReadContinuous(BoardIndex, ModuleIndex, _frameLength, _notifyAfterFrames, _frames, True, numberOfFramesReturned)
            _totalFramesAcquired += numberOfFramesReturned
            If _adqError <> AdqErrors.ADQ_ERROR_NONE Then
                If StopAfterFrames > 0 AndAlso StopAfterFrames = _totalFramesAcquired Then
                    _adqError = AdqErrors.ADQ_ERROR_NONE
                    Exit While
                End If

                If numberOfFramesReturned > 0 Then
                    ProcessAcquiredFrames(worker, numberOfFramesReturned)
                End If

                _errorFunctionName = "adqAIReadContinuous"
                Exit While
            End If

            If numberOfFramesReturned > 0 Then
                ProcessAcquiredFrames(worker, numberOfFramesReturned)
            End If
        End While
        gchFrames.Free()
        AdqDriver.adqStop(BoardIndex, ModuleIndex)
    End Sub

    Private Sub ProcessAcquiredFrames(worker As BackgroundWorker, numberOfFramesReturned As UInteger)
        CalculatePhysicalFrames(numberOfFramesReturned)

        If RequestUpdateNumberOfFrames Then
            RequestUpdateNumberOfFrames = False
            worker.ReportProgress(AdqCommon.PropertyType.UpdateNumberOfFrames)
        End If
    End Sub

    Private Sub CalculatePhysicalFrames(numberOfFramesReturned As UInteger)
        Dim numberOfValuesReturned As UInteger = numberOfFramesReturned * _frameLength
        Dim digitalIndex As UInteger = 0

        While digitalIndex < numberOfValuesReturned
            Dim processed As UInteger = ProcessFrames(numberOfValuesReturned, _physicalIndex, digitalIndex)
            digitalIndex += processed
            _physicalIndex += processed
            If _physicalIndex >= _displayBufferLength Then
                SwitchDisplayFrameBuffer()
                _physicalIndex = 0
            End If
        End While
    End Sub

    Private Function ProcessFrames(numberOfValuesReturned As UInteger, physicalIndex As UInteger, digitalIndex As UInteger) As UInteger
        Dim numProcessed As UInteger = 0
        Dim valIdx As Integer
        Dim physValIndex As Integer
        Dim idx As Integer = 0
        While idx < _frameLength
            Dim chanSetting As ChannelSetting = ChannelSettings(idx)
            Dim Range As AnalogRange = chanSetting.Range

            physValIndex = CInt(physicalIndex + idx)
            valIdx = CInt(digitalIndex + idx)
            While valIdx < numberOfValuesReturned AndAlso physValIndex < _displayBufferLength
                _physicalFrames(physValIndex) = Range.DigitalToPhysical(_frames(valIdx))
                valIdx += CInt(_frameLength)
                physValIndex += CInt(_frameLength)
                numProcessed += 1UI
            End While

            idx += 1
        End While
        Return numProcessed
    End Function

    Private Sub SwitchDisplayFrameBuffer()
        If RequestFramesToDisplay Then
            RequestFramesToDisplay = False
            If ReferenceEquals(_physicalFrames, _physicalFrames1) Then
                _displayBuffer = _physicalFrames1
                _physicalFrames = _physicalFrames2
            Else
                _displayBuffer = _physicalFrames2
                _physicalFrames = _physicalFrames1
            End If
            _totalFramesDisplayed += _displayBufferNumberOfFrames
            RaisePropertyChanged(AdqCommon.PropertyType.DisplayBuffer)
        Else
            _totalFramesSkipped += _displayBufferNumberOfFrames
        End If
    End Sub

    Private Sub Worker_ProgressChanged(sender As Object, e As ProgressChangedEventArgs)
        RaisePropertyChanged(CType(e.ProgressPercentage, AdqCommon.PropertyType))
    End Sub

    Private Sub Worker_RunWorkerCompleted(sender As Object, e As RunWorkerCompletedEventArgs)
        _moduleIsBusy = False

        RaisePropertyChanged(AdqCommon.PropertyType.AcquisitionEnded)
        RaisePropertyChanged(AdqCommon.PropertyType.UpdateNumberOfFrames)

        If _adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetErrorInfo(_errorFunctionName, _adqError)
            RaisePropertyChanged(AdqCommon.PropertyType.AcquisitionErrorOccurred)
        End If
        UpdateModuleInfo("Acquisition ended.")
    End Sub

    Public Sub CancelAcquisition()
        _worker.CancelAsync()
    End Sub

    Public Function StopAcquisition() As UInteger
        CancelAcquisition()

        Dim adqError As UInteger = AdqDriver.adqStop(BoardIndex, ModuleIndex)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetErrorInfo("adqStop", adqError)
        End If
        Return adqError
    End Function

    Public Function ResetModule() As UInteger
        Dim adqError As UInteger = AdqDriver.adqResetModule(BoardIndex, ModuleIndex)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            UpdateModuleInfo(String.Format("Reset board {0} module {1}", BoardIndex, ModuleIndex))
        Else
            SetErrorInfo("adqResetModule", adqError)
        End If
        Return adqError
    End Function

    Private Sub SetErrorInfo(functionName As String, adqError As UInteger)
        UpdateModuleInfo(String.Format("Function {0} returned Error number: 0x{1:X}", functionName, adqError))
    End Sub

    Private Sub UpdateModuleInfo(info As String)
        If ModuleInfoTextChangedEvent IsNot Nothing Then
            Dim args = New InfoTextChangedEventArgs With
                {
                    .InfoText = info
                }
            RaiseEvent ModuleInfoTextChanged(Me, args)
        End If
    End Sub

    Private _moduleIsBusy As Boolean
    ReadOnly Property ModuleIsBusy As Boolean
        Get
            Return _moduleIsBusy
        End Get
    End Property

    Public Property RequestFramesToDisplay As Boolean

    Public Property RequestUpdateNumberOfFrames As Boolean

    Private _totalFramesAcquired As ULong
    ReadOnly Property TotalFramesAcquired As ULong
        Get
            Return _totalFramesAcquired
        End Get
    End Property

    Private _totalFramesDisplayed As ULong
    ReadOnly Property TotalFramesDisplayed As ULong
        Get
            Return _totalFramesDisplayed
        End Get
    End Property

    Private _totalFramesSkipped As ULong
    ReadOnly Property TotalFramesSkipped As ULong
        Get
            Return _totalFramesSkipped
        End Get
    End Property

    Public Property StopAfterFrames As ULong

    Public Property DisplaySeconds As Double

    Private _frameLength As UInteger
    Public Property FrameLength As UInteger
        Get
            Return _frameLength
        End Get
        Set(value As UInteger)
            _frameLength = value
        End Set
    End Property

    Private _displayBufferNumberOfFrames As UInteger
    ReadOnly Property DisplayBufferNumberOfFrames As UInteger
        Get
            Return _displayBufferNumberOfFrames
        End Get
    End Property

    Private _displayBufferLength As UInteger
    ReadOnly Property DisplayBufferLength As UInteger
        Get
            Return _displayBufferLength
        End Get
    End Property

    Private _displayBuffer As Double()
    ReadOnly Property DisplayBuffer As Double()
        Get
            Return _displayBuffer
        End Get
    End Property

    Private _minResolution As Double
    ReadOnly Property MinResolution As Double
        Get
            Return _minResolution
        End Get
    End Property

    Private Sub RaisePropertyChanged(propType As AdqCommon.PropertyType)
        If ModulePropertyChangedEvent IsNot Nothing Then
            Dim args = New ModulePropertyChangedEventArgs With
                {
                    .ChangedProperty = propType
                }
            RaiseEvent ModulePropertyChanged(Me, args)
        End If
    End Sub

    Private Sub UpdateActualSampleFrequencyAndSampleTime(sampleFrequency As Double)
        _actualSampleFrequency = sampleFrequency
        _sampleTime = 1.0 / sampleFrequency

        RaisePropertyChanged(AdqCommon.PropertyType.ActualSampleFrequency)
    End Sub

    Private _actualSampleFrequency As Double
    ReadOnly Property ActualSampleFrequency As Double
        Get
            Return _actualSampleFrequency
        End Get
    End Property

    Private _sampleTime As Double
    ReadOnly Property SampleTime As Double
        Get
            Return _sampleTime
        End Get
    End Property

    Public Property DesiredSampleFrequency As Double

    Public Sub ClearModuleResources()
        _worker = Nothing
        _frames = Nothing
        _physicalFrames1 = Nothing
        _physicalFrames2 = Nothing
    End Sub

    ReadOnly Property DefaultRange As AnalogRange

    ReadOnly Property Ranges As List(Of AnalogRange) = New List(Of AnalogRange)
End Class
