﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the AIReadContinuous project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.Globalization
Imports System.Reflection
Imports System.Windows.Forms.DataVisualization.Charting
Imports AIReadContinuous.ALLDAQ.Boards

Public Class AdqCommon
    Private Const _oneMicrosecond As Double = 0.000001
    Private Const _oneMillisecond As Double = 0.001
    Private Const _oneSecond As Double = 1.0
    Private Const _oneMinute As Double = 60
    Private Const _oneHour As Double = 3600

    Public Shared ReadOnly NotOpen As UInteger = AdqErrors.ADQ_ERROR_TYPE_FATAL Or AdqErrors.ADQ_ERROR_NOT_OPEN
    Public Shared ReadOnly DefaultNumberOfFrames As UInteger = 50
    Public Shared ReadOnly DefaultDesiredSampleFrequencyFactor As Double = 0.5
    Public Shared ReadOnly DefaultDisplayNumberOfFrames As UInteger = 1000
    Public Shared ReadOnly DefaultTriggerTimeout As UInteger = 5000

    Public Enum PropertyType As Integer
        ActualSampleFrequency
        DisplayBuffer
        DisplayBufferLength
        UpdateNumberOfFrames
        AcquisitionStarting
        WaitingForStartTrigger
        StartTriggerEventOccured
        AcquisitionStarted
        AcquisitionEnded
        AcquisitionErrorOccurred
    End Enum

    Public Shared ChannelColors As Color() = {
            Color.SkyBlue,
            Color.LimeGreen,
            Color.MediumOrchid,
            Color.LightCoral,
            Color.SteelBlue,
            Color.YellowGreen,
            Color.Turquoise,
            Color.HotPink,
            Color.Khaki,
            Color.Tan,
            Color.DarkSeaGreen,
            Color.CornflowerBlue,
            Color.Plum,
            Color.CadetBlue,
            Color.PeachPuff,
            Color.LightSalmon}

    Public Shared MarkerStyles As MarkerStyle() = {MarkerStyle.Circle,
                                                   MarkerStyle.Cross,
                                                   MarkerStyle.Diamond,
                                                   MarkerStyle.Square,
                                                   MarkerStyle.Star10,
                                                   MarkerStyle.Star4,
                                                   MarkerStyle.Star5,
                                                   MarkerStyle.Star6,
                                                   MarkerStyle.Triangle}

    Public Shared Function GetPhysicalUnitName(physicalUnit As UInteger) As String
        Return GetDictionaryValue(_physicalUnitNames, physicalUnit)
    End Function

    Public Shared Function GetPhysicalUnitSymbol(physicalUnit As UInteger) As String
        Return GetDictionaryValue(_physicalUnitSymbols, physicalUnit)
    End Function

    Private Shared Function GetDictionaryValue(dictionary As IDictionary(Of UInteger, String), key As UInteger) As String
        Dim stringValue As String = String.Empty
        Dim found As Boolean = dictionary.TryGetValue(key, stringValue)
        If Not found Then
            stringValue = String.Format("&H{0:X8}", key)
        End If
        Return stringValue
    End Function

    Private Shared ReadOnly _physicalUnitNames As Dictionary(Of UInteger, String) =
            New Dictionary(Of UInteger, String) From
            {
                {AdqConstants.ADQ_PHYSICAL_UNIT_VOLT, "Volt"},
                {AdqConstants.ADQ_PHYSICAL_UNIT_AMPERE, "Ampere"},
                {AdqConstants.ADQ_PHYSICAL_UNIT_OHM, "Ohm"}
            }

    Private Shared ReadOnly _physicalUnitSymbols As Dictionary(Of UInteger, String) =
            New Dictionary(Of UInteger, String) From
            {
                {AdqConstants.ADQ_PHYSICAL_UNIT_VOLT, "V"},
                {AdqConstants.ADQ_PHYSICAL_UNIT_AMPERE, "A"},
                {AdqConstants.ADQ_PHYSICAL_UNIT_OHM, "Ω"}
            }

    Public Shared Function GetModelName(boardName As String) As String
        Dim posADQ As Integer = boardName.IndexOf("ADQ")
        If posADQ < 0 Then
            posADQ = 0
        End If

        Dim posEnd As Integer = boardName.IndexOf(" ", posADQ)
        If posEnd < 0 Then
            posEnd = boardName.Length
        End If

        Return boardName.Substring(posADQ, posEnd - posADQ)
    End Function

    Public Shared Function GetTriggerDescription(ByRef triggerInfo As AdqDriver.ADQHWTriggerInfo) As String
        Dim description As String

        If triggerInfo.HWTriggerStartSource = 0 AndAlso triggerInfo.HWTriggerStopSource = 0 Then
            description = "No hardware trigger supported."
        Else
            If triggerInfo.HWTriggerStartSource <> 0 Then
                description = "Start trigger lines: " & GetTriggerLines(triggerInfo.HWTriggerStartSource)
            Else
                description = "No start trigger available."
            End If
            description &= Environment.NewLine

            If triggerInfo.HWTriggerStopSource <> 0 Then
                description &= "Stop trigger lines: " & GetTriggerLines(triggerInfo.HWTriggerStopSource)
            Else
                description &= "No stop trigger available."
            End If
            description &= Environment.NewLine

            If triggerInfo.SupportsChannelListPerTrigger Then
                description &= "Supports channel list per trigger."
            Else
                description &= "Channel list per trigger not supported."
            End If
            description &= Environment.NewLine
        End If

        Return description
    End Function

    Private Shared Function GetTriggerLines(triggerSource As UInteger) As String
        Dim lines As String = String.Empty
        Dim bitMask As Integer = 1
        For i = 0 To 31
            If (triggerSource And bitMask) <> 0 Then
                If Not String.IsNullOrEmpty(lines) Then
                    lines &= ", "
                End If
                lines &= i.ToString()
            End If
            bitMask <<= 1
        Next
        Return lines
    End Function

    Public Shared Function FormatFrequency(frequency As Double) As String
        Dim s As String
        If frequency >= 1000000 Then
            s = String.Format("{0:0.######}MHz", frequency / 1000000)
        ElseIf frequency >= 1000 Then
            s = String.Format("{0:0.######}kHz", frequency / 1000)
        Else
            s = String.Format("{0:0.########}Hz", frequency)
        End If

        Return s
    End Function

    Public Shared Function FormatSampleTime(seconds As Double) As String
        If seconds = 0.0 Then Return "0"

        Dim s As String
        If seconds < _oneMicrosecond Then
            s = String.Format("{0:0##}ns", seconds * 1000000000)
        ElseIf seconds < _oneMillisecond Then
            s = String.Format(CultureInfo.InvariantCulture, "{0:0.#######}µs", seconds * 1000000)
        ElseIf seconds < _oneSecond Then
            s = String.Format(CultureInfo.InvariantCulture, "{0:0.#######}ms", seconds * 1000)
        ElseIf seconds < _oneMinute Then
            s = String.Format(CultureInfo.InvariantCulture, "{0:0.0000000}s", seconds)
        ElseIf seconds < _oneHour Then
            Dim minutes As Integer = CInt(seconds / 60.0)
            Dim sec As Double = seconds Mod 60.0
            s = String.Format(CultureInfo.InvariantCulture, "{0:00}:{1:00.0000000}m", minutes, sec)
        Else
            Dim hours As Integer = CInt(seconds / _oneHour)
            Dim minutes As Integer = CInt(seconds Mod _oneHour)
            Dim sec As Double = seconds Mod _oneMinute
            s = String.Format(CultureInfo.InvariantCulture, "{0:00}:{1:00}:{2:00.0000000}h", hours, minutes, sec)
        End If
        Return s
    End Function

    Public Shared Function FormatPhysicalValue(physicalValue As Double) As String
        Return FormatPhysicalValue("{0:0.##}", physicalValue)
    End Function

    Public Shared Function FormatPhysicalValue(formatString As String, physicalValue As Double) As String
        Dim physicalTxt As String
        Dim tmpVal As Double = Math.Abs(physicalValue)
        If tmpVal = 0 Then
            physicalTxt = "0"
        ElseIf tmpVal < 0.000001 Then
            physicalTxt = String.Format(formatString, physicalValue * 1000000000.0) & "n"
        ElseIf tmpVal < 0.001 Then
            physicalTxt = String.Format(formatString, physicalValue * 1000000) & "µ"
        ElseIf tmpVal < 1.0 Then
            physicalTxt = String.Format(formatString, physicalValue * 1000) & "m"
        Else
            physicalTxt = String.Format(formatString, physicalValue)
        End If
        Return physicalTxt
    End Function

    Public Shared Sub SetDoubleBuffered(ctrl As Control)
        ' http://www.csharp-examples.net/set-doublebuffered/
        ' set instance non-public property with name "DoubleBuffered" to true
        Dim t As Type = GetType(Control)
        t.InvokeMember("DoubleBuffered",
                BindingFlags.SetProperty Or BindingFlags.Instance Or BindingFlags.NonPublic,
                Nothing, ctrl, New Object() {True})
    End Sub
End Class
