﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the AOSingle project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.ComponentModel

Public Class MainViewModel
    Implements INotifyPropertyChanged, IDataErrorInfo

    Private ReadOnly _moduleLocator As ModuleLocator
    Private ReadOnly _errorInfo As Dictionary(Of String, String) = New Dictionary(Of String, String)

    Public Property ResetModuleCommand As RelayCommand
    Public Property WriteValueCommand As RelayCommand
    Public Property ClearInfoTextCommand As RelayCommand
    Public Property CloseCommand As RelayCommand

    Private Event PropertyChanged As PropertyChangedEventHandler _
        Implements INotifyPropertyChanged.PropertyChanged

    Public Sub New()
        InitCommands()
        _moduleLocator = New ModuleLocator(AddressOf MainViewModel_PropertyChanged)
        _moduleLocator.InitModuleLocator()
    End Sub

    Private Sub InitCommands()
        ResetModuleCommand = New RelayCommand(AddressOf ResetModuleExecute, AddressOf CanResetModuleExecute)
        WriteValueCommand = New RelayCommand(AddressOf WriteValueExecute, AddressOf CanWriteValueExecute)
        ClearInfoTextCommand = New RelayCommand(AddressOf ClearInfoTextExecute)
        CloseCommand = New RelayCommand(AddressOf CloseApplicationExecute)
    End Sub

    Private Function CanWriteValueExecute(obj As Object) As Boolean
        Return _selectedAoRange IsNot Nothing AndAlso _errorInfo.Count = 0
    End Function

    Private Sub WriteValueExecute(obj As Object)
        Dim success As Boolean = _selectedModule.WriteSingleValue(_digitalValue)
        If success Then
            _selectedAoRange.CurrentDigitalValue = _digitalValue
            AppendInfoText(String.Format("Write to board {0}, module {1}, value digital: {2}, physical: {3}{4}",
                    _selectedModule.BoardIndex,
                    _selectedModule.ModuleIndex,
                    _digitalValue,
                    _physicalValueText,
                    _unitSymbol))
        End If
    End Sub

    Private Function CanResetModuleExecute(obj As Object) As Boolean
        Return _selectedModule IsNot Nothing
    End Function

    Private Sub ResetModuleExecute(obj As Object)
        Dim success As Boolean = _selectedModule.ResetModule()
        If success Then
            PhysicalToWrite = 0.0
            _selectedAoRange.CurrentDigitalValue = _digitalValue
            AppendInfoText(String.Format("Reset board {0}, module {1}", _selectedModule.BoardIndex, _selectedModule.ModuleIndex))
        End If
    End Sub

    Public ReadOnly Property AoModules As List(Of AoModule)
        Get
            Return _moduleLocator.AoModules
        End Get
    End Property

    Private _selectedModule As AoModule
    Public Property SelectedModule As AoModule
        Get
            Return _selectedModule
        End Get
        Set
            If _selectedModule Is Value Then
                Return
            End If

            If _selectedModule IsNot Nothing Then
                RemoveHandler _selectedModule.PropertyChanged, AddressOf MainViewModel_PropertyChanged
            End If
            _selectedModule = Value
            If _selectedModule IsNot Nothing Then
                AddHandler _selectedModule.PropertyChanged, AddressOf MainViewModel_PropertyChanged
                _aoRanges = _selectedModule.AoRanges
                If _aoRanges.Count = 1 Then
                    SelectedAoRange = _aoRanges(0)
                End If
            Else
                _aoRanges = Nothing
                SelectedAoRange = Nothing
            End If
            NotifyPropertyChanged(NameOf(SelectedModule))
            NotifyPropertyChanged(NameOf(AoRanges))
        End Set
    End Property

    Private _aoRanges As List(Of AoRange)
    Public ReadOnly Property AoRanges As List(Of AoRange)
        Get
            Return _aoRanges
        End Get
    End Property

    Private _selectedAoRange As AoRange
    Public Property SelectedAoRange As AoRange
        Get
            Return _selectedAoRange
        End Get
        Set
            If _selectedAoRange IsNot Value Then
                _selectedAoRange = Value
                _unitSymbol = _selectedAoRange?.UnitSymbol
                DigitalValueText = _selectedAoRange.CurrentDigitalValue.ToString()
                NotifyPropertyChanged(NameOf(SelectedAoRange))
                NotifyPropertyChanged(NameOf(IsEnabled))
                NotifyPropertyChanged(NameOf(UnitSymbol))
            End If
        End Set
    End Property

    Private _unitSymbol As String
    Public ReadOnly Property UnitSymbol As String
        Get
            Return _unitSymbol
        End Get
    End Property

    Public ReadOnly Property IsEnabled As Boolean
        Get
            Return _selectedAoRange IsNot Nothing
        End Get
    End Property

    Private _physicalToWrite As Double
    Public Property PhysicalToWrite As Double
        Get
            Return _physicalToWrite
        End Get
        Set
            If _physicalToWrite <> Value Then
                _physicalToWrite = Value
                _digitalValue = _selectedAoRange.PhysicalToDigital(_physicalToWrite)
                _digitalValueText = _digitalValue.ToString()
                _physicalValueText = AdqCommon.FormatDoubleValue(_physicalToWrite)

                _errorInfo.Remove(NameOf(PhysicalValueText))
                _errorInfo.Remove(NameOf(DigitalValueText))
                NotifyPropertyChanged(NameOf(PhysicalValueText))
                NotifyPropertyChanged(NameOf(PhysicalToWrite))
                NotifyPropertyChanged(NameOf(DigitalValueText))
                WriteValueCommand.RaiseCanExecuteChanged()
            End If
        End Set
    End Property

    Private _physicalValueText As String = "0"
    Public Property PhysicalValueText As String
        Get
            Return _physicalValueText
        End Get
        Set
            If _physicalValueText <> Value Then
                _physicalValueText = Value
                Dim physicalValue As Double
                Dim msg As String = ValidateDoubleValue(_physicalValueText, physicalValue)
                If String.IsNullOrEmpty(msg) Then
                    msg = _selectedAoRange.ValidatePhysicalValue(physicalValue)
                End If

                If String.IsNullOrEmpty(msg) Then
                    Dim digital As UInteger = _selectedAoRange.PhysicalToDigital(physicalValue)
                    _physicalToWrite = _selectedAoRange.DigitalToPhysical(digital)
                    _physicalValueText = AdqCommon.FormatDoubleValue(_physicalToWrite)
                    _digitalValue = digital
                    _digitalValueText = digital.ToString()
                    _errorInfo.Remove(NameOf(PhysicalValueText))
                    _errorInfo.Remove(NameOf(DigitalValueText))
                    NotifyPropertyChanged(NameOf(PhysicalToWrite))
                    NotifyPropertyChanged(NameOf(DigitalValueText))
                Else
                    _errorInfo.AddOrUpdate(NameOf(PhysicalValueText), msg)
                End If
                NotifyPropertyChanged(NameOf(PhysicalValueText))
                WriteValueCommand.RaiseCanExecuteChanged()
            End If
        End Set
    End Property

    Private _digitalValue As UInteger
    Private _digitalValueText As String = String.Empty
    Public Property DigitalValueText As String
        Get
            Return _digitalValueText
        End Get
        Set
            If _digitalValueText <> Value Then
                _digitalValueText = Value
                Dim digital As UInteger
                Dim msg As String = ValidateDigitalValue(_digitalValueText, digital)
                If String.IsNullOrEmpty(msg) Then
                    msg = _selectedAoRange.ValidateDigitalValue(digital)
                End If

                If String.IsNullOrEmpty(msg) Then
                    _digitalValue = digital
                    _physicalToWrite = _selectedAoRange.DigitalToPhysical(digital)
                    _physicalValueText = AdqCommon.FormatDoubleValue(_physicalToWrite)
                    _errorInfo.Remove(NameOf(DigitalValueText))
                    _errorInfo.Remove(NameOf(PhysicalValueText))
                    NotifyPropertyChanged(NameOf(PhysicalValueText))
                    NotifyPropertyChanged(NameOf(PhysicalToWrite))
                Else
                    _errorInfo.AddOrUpdate(NameOf(DigitalValueText), msg)
                End If
                NotifyPropertyChanged(NameOf(DigitalValueText))
                WriteValueCommand.RaiseCanExecuteChanged()
            End If
        End Set
    End Property

    Private _infoText As String = String.Empty
    Public ReadOnly Property InfoText As String
        Get
            Return _infoText
        End Get
    End Property

    Default Public ReadOnly Property Item(propertyName As String) As String Implements IDataErrorInfo.Item
        Get
            Dim value As String = String.Empty
            If _errorInfo.TryGetValue(propertyName, value) Then
                Return value
            End If
            Return Nothing
        End Get
    End Property

    Private Function ValidateDoubleValue(textValue As String, ByRef doubleValue As Double) As String
        doubleValue = 0.0
        Dim msg As String = Nothing
        If String.IsNullOrEmpty(textValue) Then
            msg = "The value cannot be empty"
        ElseIf Not Double.TryParse(textValue, doubleValue) Then
            msg = "The entered text """ & textValue & """ is not a valid number"
        End If
        Return msg
    End Function

    Private Function ValidateDigitalValue(textValue As String, ByRef digital As UInteger) As String
        digital = 0
        Dim msg As String = Nothing
        If String.IsNullOrEmpty(textValue) Then
            msg = "The value cannot be empty"
        ElseIf Not UInteger.TryParse(textValue, digital) Then
            msg = "The entered text """ & textValue & """ is not a valid number"
        End If
        Return msg
    End Function

    Public ReadOnly Property [Error] As String Implements IDataErrorInfo.Error
        Get
            Return "MainViewModel.Error"
        End Get
    End Property

    Private Sub AppendInfoText(text As String)
        If _infoText <> String.Empty Then
            _infoText &= Environment.NewLine
        End If
        _infoText &= "[" & Date.Now.ToLongTimeString() & "] " & text
        NotifyPropertyChanged(NameOf(InfoText))
    End Sub

    Private Sub ClearInfoTextExecute(obj As Object)
        _infoText = String.Empty
        NotifyPropertyChanged(NameOf(InfoText))
    End Sub

    Private Sub CloseApplicationExecute(obj As Object)
        System.Windows.Application.Current.MainWindow.Close()
    End Sub

    Public Sub NotifyPropertyChanged(ByVal propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub

    Private Sub MainViewModel_PropertyChanged(sender As Object, e As PropertyChangedEventArgs)
        If e.PropertyName = NameOf(_selectedModule.ModuleInfoText) Then
            AppendInfoText(_selectedModule.ModuleInfoText)
        End If
    End Sub
End Class
