﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file is part of the CommonSettings project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports CommonSettings.ALLDAQ.Boards

Public Class FormMain

    Private _moduleLocator As ModuleLocator
    Private ReadOnly _aiModules As List(Of AnalogModule) = New List(Of AnalogModule)
    Private _selectedAnalogModule As AnalogModule
    Private ReadOnly _dioBoards As List(Of Board) = New List(Of Board)
    Private _selectedDioBoard As Board
    Private ReadOnly _diModules As List(Of DigitalInputModule) = New List(Of DigitalInputModule)
    Private _selectedDiModule As DigitalInputModule

    Private Sub FormMain_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        Try
            _moduleLocator = New ModuleLocator(_aiModules, _diModules, _dioBoards, AddressOf OnInfoTextChanged)
            _moduleLocator.InitModuleLocator()
            FillAiModuleCombobox()
            FillDioBoardCombobox()
            FillDiModuleCombobox()
            AddInfoTextLine(String.Format("Number of analog input and output modules found: {0}", _aiModules.Count))
            AddInfoTextLine(String.Format("Number of digital boards with TTL-voltage switching found: {0}", _dioBoards.Count))
            AddInfoTextLine(String.Format("Number of digital input modules with digital filters found: {0}", _diModules.Count))
            SelectFirstComboboxItem(CoBoxAiModules)
            SelectFirstComboboxItem(CoBoxDioBoard)
            SelectFirstComboboxItem(CoBoxDiModules)
        Catch ex As DllNotFoundException
            textBoxInfo.AppendText("The device driver is not properly installed." & Environment.NewLine)
            textBoxInfo.AppendText(ex.Message & Environment.NewLine)
        End Try
    End Sub

    Private Sub FillAiModuleCombobox()
        CoBoxAiModules.Items.Clear()
        For Each aiModule As AnalogModule In _aiModules
            CoBoxAiModules.Items.Add(aiModule.ModuleName)
        Next
    End Sub

    Private Sub FillDioBoardCombobox()
        CoBoxDioBoard.Items.Clear()
        For Each dioBoard As Board In _dioBoards
            CoBoxDioBoard.Items.Add(dioBoard.BoardName)
        Next
    End Sub

    Private Sub FillDiModuleCombobox()
        CoBoxDiModules.Items.Clear()
        For Each diModule As DigitalInputModule In _diModules
            CoBoxDiModules.Items.Add(diModule.ModuleName)
        Next
    End Sub

    Private Sub SelectFirstComboboxItem(coBox As ComboBox)
        If coBox.Items.Count > 0 Then
            coBox.SelectedIndex = 0
        End If
    End Sub

    Private Sub CoBoxAiModules_SelectedIndexChanged(sender As Object, e As EventArgs) Handles CoBoxAiModules.SelectedIndexChanged
        If _selectedAnalogModule IsNot Nothing Then
            RemoveHandler _selectedAnalogModule.InfoTextChanged, AddressOf OnInfoTextChanged
        End If
        If CoBoxAiModules.SelectedIndex < 0 Then
            _selectedAnalogModule = Nothing
        Else
            Dim oldSelectedModule As AnalogModule = _selectedAnalogModule
            _selectedAnalogModule = _aiModules(CoBoxAiModules.SelectedIndex)
            AddHandler _selectedAnalogModule.InfoTextChanged, AddressOf OnInfoTextChanged
            If oldSelectedModule Is _selectedAnalogModule Then
                Return
            End If
            GetAnalogCalibration()
        End If
    End Sub

    Private Sub CoBoxDioBoards_SelectedIndexChanged(sender As Object, e As EventArgs) Handles CoBoxDioBoard.SelectedIndexChanged
        If _selectedDioBoard IsNot Nothing Then
            RemoveHandler _selectedDioBoard.InfoTextChanged, AddressOf OnInfoTextChanged
        End If
        If CoBoxDioBoard.SelectedIndex < 0 Then
            _selectedDioBoard = Nothing
        Else
            Dim oldSelectedBoard As Board = _selectedDioBoard
            _selectedDioBoard = _dioBoards(CoBoxDioBoard.SelectedIndex)
            AddHandler _selectedDioBoard.InfoTextChanged, AddressOf OnInfoTextChanged
            If oldSelectedBoard Is _selectedDioBoard Then
                Return
            End If
            FillVoltageLevelsComboBox()
            GetDigitalTTLVoltage()
        End If
    End Sub

    Private Sub FillVoltageLevelsComboBox()
        CoBoxVoltageLevels.Items.Clear()
        If _selectedDioBoard Is Nothing Then
            Return
        End If

        For Each voltageLevel As KeyValuePair(Of UInteger, String) In _selectedDioBoard.VoltageLevels
            CoBoxVoltageLevels.Items.Add(voltageLevel.Value)
        Next
        SelectFirstComboboxItem(CoBoxVoltageLevels)
    End Sub

    Private Sub CoBoxDiModules_SelectedIndexChanged(sender As Object, e As EventArgs) Handles CoBoxDiModules.SelectedIndexChanged
        If _selectedDiModule IsNot Nothing Then
            RemoveHandler _selectedDiModule.InfoTextChanged, AddressOf OnInfoTextChanged
        End If
        If CoBoxDiModules.SelectedIndex < 0 Then
            _selectedDiModule = Nothing
        Else
            Dim oldSelectedDiModule As DigitalInputModule = _selectedDiModule
            _selectedDiModule = _diModules(CoBoxDiModules.SelectedIndex)
            AddHandler _selectedDiModule.InfoTextChanged, AddressOf OnInfoTextChanged
            If oldSelectedDiModule Is _selectedDiModule Then
                Return
            End If
            FillFilterCombobox()
            GetDigitalFilter()
        End If
    End Sub

    Private Sub FillFilterCombobox()
        CoBoxFilters.Items.Clear()
        If _selectedDiModule Is Nothing Then
            Return
        End If

        For Each filterTime As ULong In _selectedDiModule.FilterTimes
            CoBoxFilters.Items.Add(FormatFilterTime(filterTime))
        Next
        SelectFirstComboboxItem(CoBoxFilters)
    End Sub

    Private Sub BtnSet_Calibration_Click(sender As Object, e As EventArgs) Handles BtnSet_Calibration.Click
        If _selectedAnalogModule Is Nothing Then
            Return
        End If
        If RbtnFactory.Checked Then
            SetAnalogCalibration(AdqConstants.ADQ_CALIBRATION_FACTORY)
        ElseIf RbtnUser.Checked Then
            SetAnalogCalibration(AdqConstants.ADQ_CALIBRATION_USER_1)
        End If
    End Sub

    Private Sub BtnGet_Calibration_Click(sender As Object, e As EventArgs) Handles BtnGet_Calibration.Click
        If _selectedAnalogModule Is Nothing Then
            Return
        End If
        GetAnalogCalibration()
    End Sub

    Private Sub GetAnalogCalibration()
        Dim calibrationID As UInteger
        Dim succeeded As Boolean = _selectedAnalogModule.GetSelectedCalibration(calibrationID)
        If succeeded Then
            AddInfoTextLine(String.Format("Board {0}, Module {1}, Get Calibration: {2}",
                                    _selectedAnalogModule.BoardIndex,
                                    _selectedAnalogModule.ModuleIndex,
                                    AdqCommon.GetAdjustmentText(calibrationID)))
        End If
    End Sub

    Private Sub SetAnalogCalibration(calibrationID As UInteger)
        Dim succeeded As Boolean = _selectedAnalogModule.SetSelectedCalibration(calibrationID)
        If succeeded Then
            AddInfoTextLine(String.Format("Board {0}, Module {1}, Set Calibration: {2}",
                                    _selectedAnalogModule.BoardIndex,
                                    _selectedAnalogModule.ModuleIndex,
                                    AdqCommon.GetAdjustmentText(calibrationID)))
        End If
    End Sub

    Private Sub BtnGetTTLVoltage_Click(sender As Object, s As EventArgs) Handles btnGetTTLVoltage.Click
        If _selectedDioBoard Is Nothing Then Return

        GetDigitalTTLVoltage()
    End Sub

    Private Sub BtnSetTTLVoltage_Click(sender As Object, e As EventArgs) Handles btnSetTTLVoltage.Click
        If _selectedDioBoard Is Nothing Then Return

        SetDigitalVoltage(_selectedDioBoard.VoltageLevels(CoBoxVoltageLevels.SelectedIndex))
    End Sub

    Private Sub SetDigitalVoltage(voltageLevel As KeyValuePair(Of UInteger, String))
        Dim succeeded As Boolean = _selectedDioBoard.SetTTLVoltage(voltageLevel.Key)
        If succeeded Then
            AddInfoTextLine(String.Format("Board {0}, Set TTL-Voltage: {1}",
                                    _selectedDioBoard.BoardIndex, voltageLevel.Value))
        End If
    End Sub

    Private Sub GetDigitalTTLVoltage()
        Dim voltage As UInteger
        Dim succeeded As Boolean = _selectedDioBoard.GetTTLVoltage(voltage)
        If succeeded Then
            AddInfoTextLine(String.Format("Board {0}, Get TTL-Voltage: {1}",
                                    _selectedDioBoard.BoardIndex, AdqCommon.GetVoltageText(voltage)))
        End If
    End Sub

    Private Sub BtnGetFilter_Click(sender As Object, e As EventArgs) Handles btnGetFilter.Click
        If _selectedDiModule Is Nothing Then Return

        GetDigitalFilter()
    End Sub

    Private Sub BtnSetFilter_Click(sender As Object, e As EventArgs) Handles btnSetFilter.Click
        If _selectedDiModule Is Nothing Then Return

        SetDigitalFilter(_selectedDiModule.FilterTimes(CoBoxFilters.SelectedIndex))
    End Sub

    Private Sub GetDigitalFilter()
        Dim filterPicoseconds As ULong
        Dim succeeded As Boolean = _selectedDiModule.GetDigitalFilter(filterPicoseconds)
        If succeeded Then
            AddInfoTextLine(String.Format("Board {0}, Module {1}, Get Filter: {2}",
                                    _selectedDiModule.BoardIndex,
                                    _selectedDiModule.ModuleIndex,
                                    FormatFilterTime(filterPicoseconds)))
        End If
    End Sub

    Private Sub SetDigitalFilter(filterPicoseconds As ULong)
        Dim succeeded As Boolean = _selectedDiModule.SetDigitalFilter(filterPicoseconds)
        If succeeded Then
            AddInfoTextLine(String.Format("Board {0}, Module {1}, Set Filter: {2}",
                                    _selectedDiModule.BoardIndex,
                                    _selectedDiModule.ModuleIndex,
                                    FormatFilterTime(filterPicoseconds)))
        End If
    End Sub

    Private Function FormatFilterTime(filterPicoseconds As ULong) As String
        Return AdqCommon.FormatDoubleValue(filterPicoseconds / 1000000000000.0, "s")
    End Function

    Private Sub AddInfoTextLine(text As String)
        textBoxInfo.AppendText(String.Format("[{0}] {1}" & Environment.NewLine,
                    Date.Now.ToString("HH:mm:ss.fff"), text))
    End Sub

    Private Sub OnInfoTextChanged(sender As Object, e As InfoTextChangedEventArgs)
        AddInfoTextLine(e.InfoText)
    End Sub

    Private Sub FormMain_FormClosing(sender As Object, e As FormClosingEventArgs) Handles MyBase.FormClosing
        CloseAdqDriver()
    End Sub

    Private Sub BtnClose_Click(sender As Object, e As EventArgs) Handles BtnClose.Click
        Close()
    End Sub

    Private Sub CloseAdqDriver()
        If _moduleLocator IsNot Nothing Then
            _moduleLocator.CloseAdqDriver()
        End If
    End Sub

    Private Sub BtnInfo_Click(sender As Object, e As EventArgs) Handles BtnInfo.Click
        AboutBox.ShowDialog()
    End Sub

    Private Sub BtnClear_Click(sender As Object, e As EventArgs) Handles BtnClear.Click
        textBoxInfo.Clear()
    End Sub

    Private Sub BtnResetAnalogModule_Click(sender As Object, e As EventArgs) Handles BtnResetAnalogModule.Click
        If _selectedAnalogModule IsNot Nothing Then
            Dim succeeded As Boolean = _selectedAnalogModule.Reset()
            If succeeded Then
                AddInfoTextLine(String.Format("Reset board {0}, module {1}",
                        _selectedAnalogModule.BoardIndex, _selectedAnalogModule.ModuleIndex))
            End If
        End If
    End Sub

    Private Sub BtnResetDioBoard_Click(sender As Object, e As EventArgs) Handles BtnResetDioBoard.Click
        If _selectedDioBoard IsNot Nothing Then
            Dim succeeded As Boolean = _selectedDioBoard.Reset()
            If succeeded Then
                AddInfoTextLine(String.Format("Reset board {0}", _selectedDioBoard.BoardIndex))
            End If
        End If
    End Sub

    Private Sub BtnResetDiModule_Click(sender As Object, e As EventArgs) Handles BtnResetDiModule.Click
        If _selectedDiModule IsNot Nothing Then
            Dim succeeded As Boolean = _selectedDiModule.Reset()
            If succeeded Then
                AddInfoTextLine(String.Format("Reset board {0}, module {1}",
                        _selectedDiModule.BoardIndex, _selectedDiModule.ModuleIndex))
            End If
        End If
    End Sub
End Class
