﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the DIReadContinuous project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports DIReadContinuous.ALLDAQ.Boards
Imports System.ComponentModel
Imports System.Runtime.InteropServices

Namespace DIReadContinuous.ALLDAQ
    Class DigitalInputModule
        Private Const _notificationsPerSecond As Integer = 10
        Private _errorFunctionName As String
        Private _adqError As UInteger
        Private ReadOnly _moduleType As UInteger

        Public Event ModuleInfoTextChanged As EventHandler(Of InfoTextChangedEventArgs)
        Public Event ModulePropertyChanged As EventHandler(Of ModulePropertyChangedEventArgs)

        Private _worker As BackgroundWorker
        Private _bufferValues As UInteger()
        Private _bufferValues1 As UInteger()
        Private _bufferValues2 As UInteger()

        Sub New(ByVal boardName As String, ByVal boardIndex As UInteger, ByVal moduleIndex As UInteger,
                ByRef generalModuleInfo As AdqDriver.ADQGeneralModuleInfo, ByVal portWidthBits As UInteger)
            ModelName = AdqCommon.GetModelName(boardName)
            Me.BoardIndex = boardIndex
            Me.ModuleIndex = moduleIndex
            _moduleType = generalModuleInfo.ModuleType
            Me.PortWidthBits = portWidthBits
            MaxFrequency = generalModuleInfo.ClockBaseFrequencyHz / generalModuleInfo.ClockMinimumDivider
            DesiredFrequency = Math.Min(MaxFrequency, AdqCommon.DefaultDesiredSampleFrequency)
            DisplayNumberOfValues = AdqCommon.DefaultDisplayNumberOfValues
            BuildModuleName()
        End Sub


        Public ReadOnly Property ModelName As String
        Public ReadOnly Property BoardIndex As UInteger
        Public ReadOnly Property ModuleIndex As UInteger

        Private _moduleName As String
        Public ReadOnly Property ModuleName As String
            Get
                Return _moduleName
            End Get
        End Property

        Public ReadOnly Property PortWidthBits As UInteger

        Private _notifyAfterValues As UInteger
        Public ReadOnly Property NotifyAfterValues As UInteger
            Get
                Return _notifyAfterValues
            End Get
        End Property
        Public Property StopAfterValues As ULong

        Private _moduleIsBusy As Boolean
        Public ReadOnly Property ModuleIsBusy As Boolean
            Get
                Return _moduleIsBusy
            End Get
        End Property

        Public Property DesiredFrequency As Double
        Public ReadOnly Property MaxFrequency As Double

        Private _actualSampleFrequency As Double
        Public ReadOnly Property ActualSampleFrequency As Double
            Get
                Return _actualSampleFrequency
            End Get
        End Property

        Public Property DisplayNumberOfValues As UInteger
        Public Property RequestValuesToDisplay As Boolean
        Public Property RequestUpdateNumberOfValues As Boolean

        Private _displayBufferNumberOfValues As UInteger
        Public ReadOnly Property DisplayBufferNumberOfValues As UInteger
            Get
                Return _displayBufferNumberOfValues
            End Get
        End Property

        Public Property BitsSelected As Integer

        Private _sampleTime As Double
        Public ReadOnly Property SampleTime As Double
            Get
                Return _sampleTime
            End Get
        End Property

        Private _totalValuesAcquired As ULong
        Public ReadOnly Property TotalValuesAcquired As ULong
            Get
                Return _totalValuesAcquired
            End Get
        End Property

        Private _totalValuesDisplayed As ULong
        Public ReadOnly Property TotalValuesDisplayed As ULong
            Get
                Return _totalValuesDisplayed
            End Get
        End Property

        Private _totalValuesSkipped As ULong
        Public ReadOnly Property TotalValuesSkipped As ULong
            Get
                Return _totalValuesSkipped
            End Get
        End Property

        Private _newValuesDisplayBuffer As UInteger()
        Public ReadOnly Property NewValuesDisplayBuffer As UInteger()
            Get
                Return _newValuesDisplayBuffer
            End Get
        End Property

        Private _numberOfValidValues As UInteger
        Public ReadOnly Property NumberOfValidValues As UInteger
            Get
                Return _numberOfValidValues
            End Get
        End Property

        Protected Sub BuildModuleName()
            _moduleName = String.Format("{0}, Board {1}, Module {2}, Type {3}, {4} Bit, max. {5}",
                                        ModelName,
                                        BoardIndex,
                                        ModuleIndex,
                                        AdqCommon.GetModuleTypeName(_moduleType),
                                        PortWidthBits,
                                        AdqCommon.FormatFrequency(MaxFrequency))
        End Sub

        Protected Sub SetErrorInfo(ByVal functionName As String, ByVal adqError As UInteger)
            UpdateModuleInfo(String.Format("API-function {0} returned Error number: &H{1:X}", functionName, adqError))
        End Sub

        Private Sub UpdateModuleInfo(ByVal info As String)
            If ModuleInfoTextChangedEvent Is Nothing Then Return

            Dim args = New InfoTextChangedEventArgs With {
                    .InfoText = info
                }
            RaiseEvent ModuleInfoTextChanged(Me, args)
        End Sub

        Private Sub RaisePropertyChanged(ByVal propertyType As AdqCommon.PropertyType)
            If ModulePropertyChangedEvent Is Nothing Then Return

            Dim args = New ModulePropertyChangedEventArgs With {
                    .ChangedProperty = propertyType
                }
            RaiseEvent ModulePropertyChanged(Me, args)
        End Sub

        Private Sub ConfigureModule()
            CheckCommonSettings()
            SetDirection()
            SetFrequency()
            _notifyAfterValues = Math.Min(DisplayNumberOfValues, CUInt(_actualSampleFrequency / _notificationsPerSecond))

            If _notifyAfterValues < AdqCommon.DefaultMinNotifyAfterValues Then
                _notifyAfterValues = AdqCommon.DefaultMinNotifyAfterValues
            End If

            If StopAfterValues > 0 AndAlso StopAfterValues < _notifyAfterValues Then
                _notifyAfterValues = CUInt(StopAfterValues)
            End If

            If StopAfterValues > 0 AndAlso StopAfterValues < DisplayNumberOfValues Then
                DisplayNumberOfValues = CUInt(StopAfterValues)
            End If

            Dim numberOfValues As UInteger = _notifyAfterValues

            If _bufferValues1 Is Nothing OrElse _bufferValues1.Length <> numberOfValues Then
                _bufferValues1 = New UInteger(CInt(numberOfValues - 1)) {}
                _bufferValues2 = New UInteger(CInt(numberOfValues - 1)) {}
            End If

            _displayBufferNumberOfValues = DisplayNumberOfValues
            RaisePropertyChanged(AdqCommon.PropertyType.UpdateValuesToDisplay)
        End Sub

        Private Sub CheckCommonSettings()
            If DesiredFrequency = 0 Then
                Throw New ApplicationException("Please enter a valid sample frequency.")
            End If

            If DisplayNumberOfValues = 0 Then
                Throw New ApplicationException("Please enter a valid number of values to display.")
            End If

            If BitsSelected = 0 Then
                Throw New ApplicationException("Please select bits in the bit list.")
            End If
        End Sub

        Protected Sub SetFrequency()
            Dim actualFrequency As Double = Nothing
            Dim adqError As UInteger = AdqDriver.adqSetFrequency(BoardIndex, ModuleIndex, DesiredFrequency, actualFrequency)

            If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                Throw New ApplicationException(String.Format("API-function adqSetFrequency returned Error number: 0x{0:X}", adqError))
            Else
                _actualSampleFrequency = actualFrequency
                _sampleTime = 1.0 / actualFrequency
                RaisePropertyChanged(AdqCommon.PropertyType.ActualSampleFrequency)
            End If
        End Sub

        Public Sub InitAcquisition()
            If _moduleIsBusy Then Return
            Reset()
            ConfigureModule()
            _moduleIsBusy = True

            If _worker Is Nothing Then
                _worker = New BackgroundWorker With {
                    .WorkerReportsProgress = True,
                    .WorkerSupportsCancellation = True
                }
                AddHandler _worker.DoWork, AddressOf Worker_DoWork
                AddHandler _worker.ProgressChanged, AddressOf Worker_ProgressChanged
                AddHandler _worker.RunWorkerCompleted, AddressOf Worker_RunWorkerCompleted
            End If
        End Sub

        Public Sub StartAcquisition()
            UpdateModuleInfo("Acquisition initialized")
            _worker.RunWorkerAsync()
        End Sub

        Public Sub CancelAcquisition()
            _worker.CancelAsync()
        End Sub

        Public Function StopAcquisition() As UInteger
            CancelAcquisition()
            Dim adqError As UInteger = AdqDriver.adqStop(BoardIndex, ModuleIndex)

            If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                SetErrorInfo("adqStop", adqError)
            End If

            Return adqError
        End Function

        Private Sub Worker_DoWork(ByVal sender As Object, ByVal e As DoWorkEventArgs)
            Dim worker As BackgroundWorker = TryCast(sender, BackgroundWorker)
            _errorFunctionName = String.Empty
            _totalValuesAcquired = 0
            _totalValuesDisplayed = 0
            _totalValuesSkipped = 0
            worker.ReportProgress(AdqCommon.PropertyType.AcquisitionStarting)
            _bufferValues = _bufferValues1
            Dim gchValues1 As GCHandle = GCHandle.Alloc(_bufferValues1, GCHandleType.Pinned)
            Dim gchValues2 As GCHandle = GCHandle.Alloc(_bufferValues2, GCHandleType.Pinned)
            _adqError = AdqDriver.adqReadContinuousStart(BoardIndex, ModuleIndex, _notifyAfterValues, StopAfterValues, Nothing, IntPtr.Zero)

            If _adqError <> AdqErrors.ADQ_ERROR_NONE Then
                _errorFunctionName = "adqReadContinuousStart"
                gchValues1.Free()
                gchValues2.Free()
                Return
            End If

            worker.ReportProgress(AdqCommon.PropertyType.AcquisitionStarted)
            Dim numberOfValuesReturned As UInteger = Nothing

            While True
                If worker.CancellationPending Then
                    e.Cancel = True
                    Exit While
                End If

                _adqError = AdqDriver.adqReadContinuous(BoardIndex, ModuleIndex, _notifyAfterValues, _bufferValues, True, numberOfValuesReturned)
                _totalValuesAcquired += numberOfValuesReturned
                If _adqError <> AdqErrors.ADQ_ERROR_NONE Then

                    If StopAfterValues > 0 AndAlso StopAfterValues = _totalValuesAcquired Then
                        _adqError = AdqErrors.ADQ_ERROR_NONE
                        Exit While
                    End If

                    If numberOfValuesReturned > 0 Then
                        ProcessAcquiredValues(worker, numberOfValuesReturned)
                    End If

                    _errorFunctionName = "adqReadContinuous"
                    Exit While
                End If

                If numberOfValuesReturned > 0 Then
                    ProcessAcquiredValues(worker, numberOfValuesReturned)
                End If
            End While
            gchValues1.Free()
            gchValues2.Free()
            AdqDriver.adqStop(BoardIndex, ModuleIndex)
        End Sub

        Private Sub Worker_ProgressChanged(ByVal sender As Object, ByVal e As ProgressChangedEventArgs)
            RaisePropertyChanged(CType(e.ProgressPercentage, AdqCommon.PropertyType))
        End Sub

        Private Sub Worker_RunWorkerCompleted(ByVal sender As Object, ByVal e As RunWorkerCompletedEventArgs)
            _moduleIsBusy = False
            RaisePropertyChanged(AdqCommon.PropertyType.AcquisitionEnded)
            RaisePropertyChanged(AdqCommon.PropertyType.UpdateNumberOfValues)

            If _adqError <> AdqErrors.ADQ_ERROR_NONE Then
                SetErrorInfo(_errorFunctionName, _adqError)
                RaisePropertyChanged(AdqCommon.PropertyType.AcquisitionErrorOccurred)
            End If

            UpdateModuleInfo("Acquisition ended.")
        End Sub

        Private Sub ProcessAcquiredValues(ByVal worker As BackgroundWorker, ByVal numberOfValuesReturned As UInteger)
            SwitchDisplayValueBuffer(numberOfValuesReturned)

            If RequestUpdateNumberOfValues Then
                RequestUpdateNumberOfValues = False
                worker.ReportProgress(AdqCommon.PropertyType.UpdateNumberOfValues)
            End If
        End Sub

        Private Sub SwitchDisplayValueBuffer(ByVal numberOfValuesReturned As UInteger)
            _numberOfValidValues = numberOfValuesReturned

            If RequestValuesToDisplay Then
                RequestValuesToDisplay = False

                If ReferenceEquals(_bufferValues, _bufferValues1) Then
                    _newValuesDisplayBuffer = _bufferValues1
                    _bufferValues = _bufferValues2
                ElseIf ReferenceEquals(_bufferValues, _bufferValues2) Then
                    _newValuesDisplayBuffer = _bufferValues2
                    _bufferValues = _bufferValues1
                End If

                _totalValuesDisplayed += numberOfValuesReturned
                RaisePropertyChanged(AdqCommon.PropertyType.DisplayBuffer)
            Else
                _totalValuesSkipped += numberOfValuesReturned
            End If
        End Sub

        Public Sub ClearModuleResources()
            _worker = Nothing
            _bufferValues = Nothing
            _bufferValues1 = Nothing
            _bufferValues2 = Nothing
        End Sub

        Public Function Reset() As UInteger
            Dim adqError As UInteger = AdqDriver.adqResetModule(BoardIndex, ModuleIndex)

            If adqError = AdqErrors.ADQ_ERROR_NONE Then
                UpdateModuleInfo(String.Format("Reset board {0} module {1}", BoardIndex, ModuleIndex))
            Else
                SetErrorInfo("adqResetModule", adqError)
            End If
            Return adqError
        End Function

        Private Sub SetDirection()
            If _moduleType = AdqConstants.ADQ_MODULE_TYPE_DIO Then
                Dim adqError As UInteger = AdqDriver.adqDIOSetDirection(BoardIndex, ModuleIndex, AdqConstants.ADQ_DIO_INPUT)

                If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                    Throw New ApplicationException(String.Format("API-function adqSetDirection returned Error number: &H{0:X}", adqError))
                End If
            End If
        End Sub
    End Class
End Namespace
