﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the DISingle project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.Runtime.InteropServices
Imports DISingle.ALLDAQ.Boards

Public Class ModuleLocator
    Private _adqOpenError As UInteger = AdqCommon.NotOpen
    Private ReadOnly _diModules As ICollection(Of DigitalModule)
    Private ReadOnly _moduleTypes As List(Of UInteger)

    Public Event OnInfoTextChanged As EventHandler(Of InfoTextChangedEventArgs)

    Public Sub New(diModules As ICollection(Of DigitalModule), moduleTypes As List(Of UInteger),
                   ByRef infoTextChangedHandler As EventHandler(Of InfoTextChangedEventArgs))
        _diModules = diModules
        _moduleTypes = moduleTypes
        AddHandler OnInfoTextChanged, infoTextChangedHandler
    End Sub

    Public Sub FindAdqModules()
        _adqOpenError = OpenAdqDriver()
        If _adqOpenError = AdqErrors.ADQ_ERROR_NONE Then
            Dim numberOfBoards As UInteger = GetNumberOfBoards()
            Dim boardIdx As UInteger = 0
            While boardIdx < numberOfBoards
                Dim numberOfModules As UInteger
                Dim adqError As UInteger = AdqDriver.adqGetNumberOfModules(boardIdx, numberOfModules)
                If (adqError <> AdqErrors.ADQ_ERROR_NONE) Then
                    SetFunctionErrorInfo("adqGetNumberOfModules", adqError)
                    boardIdx += 1UI
                    Continue While
                End If

                GetModuleTypes(boardIdx, numberOfModules)
                boardIdx += 1UI
            End While
        End If
    End Sub

    Private Function OpenAdqDriver() As UInteger
        Dim adqError As UInteger = AdqDriver.adqOpen()
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqOpen", adqError)
        End If
        Return adqError
    End Function

    Private Function GetModuleTypes(boardIndex As UInteger, numberOfModules As UInteger) As UInteger
        Dim adqError As UInteger = AdqErrors.ADQ_ERROR_NONE
        Dim moduleIdx As UInteger = 0
        While moduleIdx < numberOfModules
            Dim moduleType As UInteger
            adqError = AdqDriver.adqGetModuleType(boardIndex, moduleIdx, moduleType)
            If (adqError <> AdqErrors.ADQ_ERROR_NONE) Then
                SetFunctionErrorInfo("adqGetModuleType", adqError)
                moduleIdx += 1UI
                Continue While
            End If

            If _moduleTypes.Contains(moduleType) Then
                If moduleType = AdqConstants.ADQ_MODULE_TYPE_DI Then
                    GetDiModuleInfo(boardIndex, moduleIdx)
                ElseIf moduleType = AdqConstants.ADQ_MODULE_TYPE_DIO Then
                    GetDioModuleInfo(boardIndex, moduleIdx)
                End If
            End If
            moduleIdx += 1UI
        End While
        Return adqError
    End Function

    Private Function GetDiModuleInfo(boardIndex As UInteger, moduleIndex As UInteger) As UInteger
        Dim adqError As UInteger
        Dim adqGeneralModuleInfo As AdqDriver.ADQGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqDiModuleInfo As AdqDriver.ADQDIModuleInfo = New AdqDriver.ADQDIModuleInfo(AdqDriver.ADQDIModuleInfo_ID)

        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqDiModuleInfo.Size))
        Marshal.StructureToPtr(adqDiModuleInfo, moduleInfo, False)

        Dim infoSize As UInteger
        adqError = AdqDriver.adqGetModuleInfo(
            boardIndex,
            moduleIndex,
            adqGeneralModuleInfo,
            adqDiModuleInfo.Size,
            moduleInfo,
            infoSize)
        adqDiModuleInfo = CType(Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQDIModuleInfo)), AdqDriver.ADQDIModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetModuleInfo", adqError)
            Return adqError
        End If

        Debug.Assert(adqDiModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqDiModuleInfo.Size, infoSize))

        Dim boardName As String = GetBoardName(boardIndex)
        Dim adqModule = New DigitalModule(boardName, boardIndex, moduleIndex, adqGeneralModuleInfo, adqDiModuleInfo.PortWidthBits)
        _diModules.Add(adqModule)
        Return adqError
    End Function

    Private Function GetDioModuleInfo(boardIndex As UInteger, moduleIndex As UInteger) As UInteger
        Dim adqError As UInteger
        Dim adqGeneralModuleInfo As AdqDriver.ADQGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqDioModuleInfo As AdqDriver.ADQDIOModuleInfo = New AdqDriver.ADQDIOModuleInfo(AdqDriver.ADQDIOModuleInfo_ID)

        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqDioModuleInfo.Size))
        Marshal.StructureToPtr(adqDioModuleInfo, moduleInfo, False)

        Dim infoSize As UInteger
        adqError = AdqDriver.adqGetModuleInfo(
            boardIndex,
            moduleIndex,
            adqGeneralModuleInfo,
            adqDioModuleInfo.Size,
            moduleInfo,
            infoSize)
        adqDioModuleInfo = CType(Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQDIOModuleInfo)), AdqDriver.ADQDIOModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If (adqError <> AdqErrors.ADQ_ERROR_NONE) Then
            SetFunctionErrorInfo("adqGetModuleInfo", adqError)
            Return adqError
        End If

        Debug.Assert(adqDioModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqDioModuleInfo.Size, infoSize))

        Dim boardName As String = GetBoardName(boardIndex)
        Dim adqModule = New DigitalModule(boardName, boardIndex, moduleIndex, adqGeneralModuleInfo, adqDioModuleInfo.PortWidthBits)
        _diModules.Add(adqModule)
        Return adqError
    End Function

    Private Function GetBoardName(boardIndex As UInteger) As String
        Dim s As String = String.Empty
        Dim boardInfo = New AdqDriver.ADQBoardInfo(AdqDriver.ADQBoardInfo_ID)

        Dim adqError As UInteger = AdqDriver.adqGetBoardInfo(boardIndex, boardInfo)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            s = boardInfo.BoardName
        End If
        Return s
    End Function

    Public Sub CloseAdqDriver()
        If _adqOpenError = AdqErrors.ADQ_ERROR_NONE Then
            AdqDriver.adqClose()
            _adqOpenError = AdqCommon.NotOpen
        End If
    End Sub

    Private Function GetNumberOfBoards() As UInteger
        Dim numBoards As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetNumberOfBoards(numBoards)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetNumberOfBoards", adqError)
            Return 0
        End If
        Return numBoards
    End Function

    Private Sub SetFunctionErrorInfo(functionName As String, adqError As UInteger)
        UpdateInfoText(String.Format("API-function {0} returned Error number: &H{1:X}",
                functionName,
                adqError))
    End Sub

    Private Sub UpdateInfoText(text As String)
        Dim args = New InfoTextChangedEventArgs With {
            .InfoText = text
        }
        RaiseOnInfoTextChanged(args)
    End Sub

    Protected Sub RaiseOnInfoTextChanged(e As InfoTextChangedEventArgs)
        RaiseEvent OnInfoTextChanged(Me, e)
    End Sub
End Class
