﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the DOSingle project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports DOSingle.ALLDAQ.Boards

Public Class DigitalModule
    Private ReadOnly _moduleType As UInteger

    Public Sub New(boardName As String, boardIndex As UInteger, moduleIndex As UInteger,
                   ByRef generalInfo As AdqDriver.ADQGeneralModuleInfo, portWidthBits As UInteger)
        Me.BoardIndex = boardIndex
        Me.ModuleIndex = moduleIndex
        _moduleType = generalInfo.ModuleType
        Description = generalInfo.ModuleDescription
        Me.PortWidthBits = portWidthBits
        MaxPortValue = GetMaxPortValue(portWidthBits)
        ModuleName = GetModuleName(boardName)
        InitDefaultOutputValues(portWidthBits)
    End Sub

    Public Event OnModuleInfoTextChanged As EventHandler(Of InfoTextChangedEventArgs)
    Public ReadOnly Property BoardIndex As UInteger
    Public ReadOnly Property ModuleIndex As UInteger
    Public ReadOnly Property Description As String
    Public ReadOnly Property ModuleName As String
    Public ReadOnly Property PortWidthBits As UInteger
    Public ReadOnly Property MaxPortValue As UInteger
    Public Property OutputValue1 As UInteger
    Public Property OutputValue2 As UInteger
    Public Property OutputValue3 As UInteger

    Private Sub InitDefaultOutputValues(widthBits As UInteger)
        Dim bitIdx As Integer = 0
        While bitIdx < widthBits
            OutputValue1 = OutputValue1 Or CUInt(1 << bitIdx)
            bitIdx += 2
        End While

        bitIdx = 1
        While bitIdx < widthBits
            OutputValue2 = OutputValue2 Or CUInt(1 << bitIdx)
            bitIdx += 2
        End While
    End Sub

    Public Function WriteSingle(ByVal value As UInteger) As Boolean
        Dim success As Boolean = SetDioDirectionToOutput()
        If Not success Then
            Return False
        End If

        Dim adqError As UInteger = AdqDriver.adqWriteSingle(BoardIndex, ModuleIndex, value)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqWriteSingle", adqError)
        End If
        Return adqError = AdqErrors.ADQ_ERROR_NONE
    End Function

    Private Function SetDioDirectionToOutput() As Boolean
        Dim adqError As UInteger = AdqErrors.ADQ_ERROR_NONE
        If _moduleType = AdqConstants.ADQ_MODULE_TYPE_DIO Then
            adqError = AdqDriver.adqDIOSetDirection(BoardIndex, ModuleIndex, AdqConstants.ADQ_DIO_OUTPUT)
            If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                SetFunctionErrorInfo("adqDIOSetDirection", adqError)
            End If
        End If
        Return adqError = AdqErrors.ADQ_ERROR_NONE
    End Function

    Public Function ResetModule() As Boolean
        Dim adqError As UInteger = AdqDriver.adqResetModule(BoardIndex, ModuleIndex)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqResetModule", adqError)
        End If
        Return adqError = AdqErrors.ADQ_ERROR_NONE
    End Function

    Private Function GetMaxPortValue(widthBits As UInteger) As UInteger
        If widthBits < 32 Then
            Return (1UI << CInt(widthBits)) - 1UI
        End If
        Return UInteger.MaxValue
    End Function

    Private Function GetModuleName(boardName As String) As String
        Return String.Format("{0}, Board {1}, Module {2}, Type {3}, {4} Bit",
                AdqCommon.GetModelName(boardName),
                BoardIndex,
                ModuleIndex,
                AdqCommon.GetModuleTypeName(_moduleType),
                PortWidthBits)
    End Function

    Private Sub SetFunctionErrorInfo(functionName As String, adqError As UInteger)
        UpdateInfoText(String.Format("API-function {0} returned Error number: &H{1:X}",
                functionName,
                adqError))
    End Sub

    Private Sub UpdateInfoText(text As String)
        Dim args = New InfoTextChangedEventArgs With {
            .InfoText = text
        }
        RaiseOnModuleInfoTextChanged(args)
    End Sub

    Protected Sub RaiseOnModuleInfoTextChanged(e As InfoTextChangedEventArgs)
        RaiseEvent OnModuleInfoTextChanged(Me, e)
    End Sub
End Class
