﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the Interrupt project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.ComponentModel
Imports System.Runtime.InteropServices
Imports System.Text
Imports Interrupt.ALLDAQ.Boards

Public Class IrqModule
    Implements INotifyPropertyChanged

    Private Const _fmtDateTime As String = "[HH:mm:ss.fff] "
    Private ReadOnly _portMaxValue As UInteger
    Private ReadOnly _irqCallbackFunction As AdqDriver.adqIrqCallback = New AdqDriver.adqIrqCallback(AddressOf AdqIrqCallbackFunction)
    Private _objectHandle As GCHandle = New GCHandle()
    Private _worker As BackgroundWorker
    Private _irqEnable As AdqDriver.ADQIRQEnable = New AdqDriver.ADQIRQEnable(AdqDriver.ADQIRQEnable_ID)

    Public Event PropertyChanged As PropertyChangedEventHandler _
        Implements INotifyPropertyChanged.PropertyChanged
    Public Event BitChangeSettingsChanged As EventHandler(Of BitChangeSettingsChangedEventArgs)
    Public Event PortBitPatternCompareSettingsChanged As EventHandler(Of PortBitPatternCompareSettingsEventArgs)
    Public Event BitChangeStatusChanged As EventHandler(Of BitChangeStatusChangedEventArgs)
    Public Event CommonIrqStatusChanged As EventHandler(Of CommonIrqStatusChangedEventArgs)
    Public Event PortCompareStatusChanged As EventHandler(Of PortCompareStatusChangedEventArgs)
    Public Event PortStatusChanged As EventHandler(Of PortStatusChangedEventArgs)
    Public Event ExternalStatusChanged As EventHandler(Of ExternalStatusChangedEventArgs)

    Public Sub New(boardName As String, boardIndex As UInteger, moduleIndex As UInteger,
                   ByRef moduleInfo As AdqDriver.ADQGeneralModuleInfo,
                   ByRef adqIrqInfo As AdqDriver.ADQIRQInfo, portWidthBits As UInteger)
        Me.BoardIndex = boardIndex
        Me.ModuleIndex = moduleIndex
        ModuleType = moduleInfo.ModuleType
        IrqDescription = adqIrqInfo.IRQDescription
        SupportsBitChange = adqIrqInfo.SupportsBitChange
        SupportsBitPatternCompare = adqIrqInfo.SupportsBitPatternCompare
        SupportedPortStatusErrors = adqIrqInfo.SupportsPortStatusError
        SupportedFaultRecovery = adqIrqInfo.SupportsFaultRecovery
        SupportedExternalIrqs = adqIrqInfo.SupportsExternal
        Me.PortWidthBits = portWidthBits
        UseIrqCallbackFunction = True
        _portMaxValue = GetMaxPortValue(_PortWidthBits)
        TimeoutMilliseconds = AdqCommon.DefaultIrqWaitTimeoutMilliseconds
        ModuleName = GetModuleName(boardName)
    End Sub

    Private Function GetMaxPortValue(widthBits As UInteger) As UInteger
        If widthBits < 32 Then
            Return (1UI << CInt(widthBits)) - 1UI
        End If
        Return UInteger.MaxValue
    End Function

    Public ReadOnly Property BoardIndex As UInteger
    Public ReadOnly Property ModuleIndex As UInteger
    Public ReadOnly Property ModuleType As UInteger
    Public ReadOnly Property ModuleName As String
    Public ReadOnly Property IrqDescription As String
    Public ReadOnly Property SupportsBitChange As Boolean
    Public ReadOnly Property SupportsBitPatternCompare As Boolean
    Public ReadOnly Property SupportedPortStatusErrors As UInteger
    Public ReadOnly Property SupportedFaultRecovery As UInteger
    Public ReadOnly Property SupportedExternalIrqs As UInteger
    Public ReadOnly Property PortWidthBits As UInteger

    Public Property UseIrqCallbackFunction As Boolean
    Public Property TimeoutMilliseconds As UInteger

    Public ReadOnly Property BitChangeIsEnabled As Boolean
        Get
            Return _irqEnable.PortBitChangeRisingEdge <> 0 OrElse _irqEnable.PortBitChangeFallingEdge <> 0
        End Get
    End Property

    Public ReadOnly Property BitPatternCompareIsEnabled As Boolean
        Get
            Return _irqEnable.PortBitPatternCompare
        End Get
    End Property

    Public Property PortBitChangeRisingEdge As UInteger
        Get
            Return _irqEnable.PortBitChangeRisingEdge
        End Get
        Set
            If _irqEnable.PortBitChangeRisingEdge <> Value Then
                Dim wasEnabled As Boolean = BitChangeIsEnabled
                _irqEnable.PortBitChangeRisingEdge = Value
                NotifyBitChangeMasksChanged(wasEnabled)
            End If
        End Set
    End Property

    Public Property PortBitChangeFallingEdge As UInteger
        Get
            Return _irqEnable.PortBitChangeFallingEdge
        End Get
        Set
            If _irqEnable.PortBitChangeFallingEdge <> Value Then
                Dim wasEnabled As Boolean = BitChangeIsEnabled
                _irqEnable.PortBitChangeFallingEdge = Value
                NotifyBitChangeMasksChanged(wasEnabled)
            End If
        End Set
    End Property

    Private Sub NotifyBitChangeMasksChanged(wasEnabled As Boolean)
        If wasEnabled <> BitChangeIsEnabled Then
            RaiseEvent BitChangeSettingsChanged(Me, New BitChangeSettingsChangedEventArgs())
        End If
    End Sub

    Public Property PortBitPatternCompare As Boolean
        Get
            Return _irqEnable.PortBitPatternCompare
        End Get
        Set
            If _irqEnable.PortBitPatternCompare <> Value Then
                _irqEnable.PortBitPatternCompare = Value
                RaiseEvent PortBitPatternCompareSettingsChanged(Me, New PortBitPatternCompareSettingsEventArgs())
            End If
        End Set
    End Property

    Public Property PortBitPatternValue As UInteger
        Get
            Return _irqEnable.PortBitPatternValue
        End Get
        Set(value As UInteger)
            _irqEnable.PortBitPatternValue = value
        End Set
    End Property

    Public Property EnabledPortStatusErrors As UInteger
        Get
            Return _irqEnable.PortStatusError
        End Get
        Set(value As UInteger)
            _irqEnable.PortStatusError = value
        End Set
    End Property

    Public Property EnabledFaultRecovery As UInteger
        Get
            Return _irqEnable.FaultRecovery
        End Get
        Set(value As UInteger)
            _irqEnable.FaultRecovery = value
        End Set
    End Property

    Public Property ExternalRisingEdge As UInteger
        Get
            Return _irqEnable.ExternalRisingEdge
        End Get
        Set(value As UInteger)
            _irqEnable.ExternalRisingEdge = value
        End Set
    End Property

    Public Property ExternalFallingEdge As UInteger
        Get
            Return _irqEnable.ExternalFallingEdge
        End Get
        Set(value As UInteger)
            _irqEnable.ExternalFallingEdge = value
        End Set
    End Property

    Public Property ResetIRQCount As Boolean
        Get
            Return _irqEnable.ResetIRQCount
        End Get
        Set(value As Boolean)
            _irqEnable.ResetIRQCount = value
        End Set
    End Property

    Private _isModuleIdle As Boolean = True
    Public Property IsModuleIdle As Boolean
        Get
            Return _isModuleIdle
        End Get
        Private Set
            _isModuleIdle = Value
            NotifyPropertyChanged(NameOf(IsModuleIdle))
        End Set
    End Property

    Private Shared Function AdqIrqCallbackFunction(boardIndex As UInteger,
                                                 moduleIndex As UInteger,
                                                 ByRef irqStatus As AdqDriver.ADQIRQStatus,
                                                 adqError As UInteger,
                                                 userContext As IntPtr) As UInteger
        If adqError = AdqErrors.ADQ_ERROR_NONE AndAlso userContext <> IntPtr.Zero Then
            Dim objHandle As GCHandle = GCHandle.FromIntPtr(userContext)
            Dim irqModule As IrqModule = CType(objHandle.Target, IrqModule)
            irqModule.UpdateIrqStatus(boardIndex, moduleIndex, irqStatus, DateTime.Now)
        End If
        'just return the passed error code if the driver should stop calling this function again in case of adqError <> AdqErrors.ADQ_ERROR_NONE
        Return adqError
    End Function

    Public Sub DisableInterrupt()
        Dim adqError As UInteger = AdqDriver.adqDisableIRQ(_BoardIndex, _ModuleIndex)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetErrorText("adqDisableIRQ", adqError)
        Else
            ModuleInfoText = "Disabled IRQ on " & _ModuleName
        End If
        IsModuleIdle = True
    End Sub

    Public Sub EnableInterrupt()
        If Not ConfigureModule() Then Return

        ModuleInfoText = GetIrqEnableInfoText()
        AdqDriver.adqDisableIRQ(_BoardIndex, _ModuleIndex)
        Dim irqCallbackFunction As AdqDriver.adqIrqCallback = SetupCallbackFunction()
        Dim adqError As UInteger = AdqDriver.adqEnableIRQ(BoardIndex,
                                                   ModuleIndex,
                                                    _irqEnable,
                                                   irqCallbackFunction,
                                                   AdqDriver.ADQIRQStatus_ID,
                                                   CType(_objectHandle, IntPtr))

        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            IsModuleIdle = False
            ModuleInfoText = "IRQ enabled on " & _ModuleName & ", selected option: " _
                & If(_UseIrqCallbackFunction, "IRQ-Callback", "IRQ-Wait")
        Else
            SetErrorText("adqEnableIRQ", adqError)
            Return
        End If

        If Not _UseIrqCallbackFunction Then
            StartBackgroundWorker()
        End If
    End Sub

    Private Function ConfigureModule() As Boolean
        If (Not CheckIrqConfiguration()) Then Return False
        If (Not SetDioDirectionToInput()) Then Return False
        Return True
    End Function

    Private Function CheckIrqConfiguration() As Boolean
        If (_irqEnable.PortBitChangeRisingEdge = 0) _
                AndAlso (_irqEnable.PortBitChangeFallingEdge = 0) _
                AndAlso (Not _irqEnable.PortBitPatternCompare) _
                AndAlso (_irqEnable.PortStatusError = 0) _
                AndAlso (_irqEnable.FaultRecovery = 0) _
                AndAlso (_irqEnable.ExternalRisingEdge = 0) _
                AndAlso (_irqEnable.ExternalFallingEdge = 0) Then
            ModuleErrorText = "Please select the interrupts that should be enabled."
            Return False
        End If

        If _irqEnable.PortBitPatternCompare AndAlso
            (_irqEnable.PortBitChangeRisingEdge <> 0 OrElse _irqEnable.PortBitChangeFallingEdge <> 0) Then
            ModuleErrorText = "Bit pattern compare and Bit-change cannot be enabled at the same time."
            Return False
        End If
        Return True
    End Function

    Private Function SetDioDirectionToInput() As Boolean
        If _ModuleType = AdqConstants.ADQ_MODULE_TYPE_DIO Then
            Dim adqError As UInteger = AdqDriver.adqDIOSetDirection(_BoardIndex, _ModuleIndex, AdqConstants.ADQ_DIO_INPUT)
            If (adqError <> AdqErrors.ADQ_ERROR_NONE) Then
                SetErrorText("adqDIOSetDirection", adqError)
                Return False
            End If
        End If
        Return True
    End Function

    Private Function GetIrqEnableInfoText() As String
        Dim sb As StringBuilder = New StringBuilder()
        sb.AppendFormat("Board {0}, Module {1} interrupt enable settings:" & Environment.NewLine, _BoardIndex, _ModuleIndex)
        sb.AppendLine(vbTab & "PortBitChangeRisingEdge = &H" & _irqEnable.PortBitChangeRisingEdge.ToString("X"))
        sb.AppendLine(vbTab & "PortBitChangeFallingEdge = &H" & _irqEnable.PortBitChangeFallingEdge.ToString("X"))
        sb.AppendLine(vbTab & "PortBitPatternCompare = " & _irqEnable.PortBitPatternCompare.ToString())
        sb.AppendLine(vbTab & "PortBitPatternValue = &H" & _irqEnable.PortBitPatternValue.ToString("X"))
        sb.AppendLine(vbTab & "ExternalRisingEdge = &H" & _irqEnable.ExternalRisingEdge.ToString("X"))
        sb.AppendLine(vbTab & "ExternalFallingEdge = &H" & _irqEnable.ExternalFallingEdge.ToString("X"))
        sb.AppendLine(vbTab & "PortStatusError = &H" & _irqEnable.PortStatusError.ToString("X"))
        sb.AppendLine(vbTab & "FaultRecovery = &H" & _irqEnable.FaultRecovery.ToString("X"))
        sb.Append(vbTab & "ResetIRQCount = " & _irqEnable.ResetIRQCount.ToString())
        Return sb.ToString()
    End Function

    Private Function SetupCallbackFunction() As AdqDriver.adqIrqCallback
        Dim irqCallbackFunction As AdqDriver.adqIrqCallback = Nothing

        If _UseIrqCallbackFunction Then
            irqCallbackFunction = _irqCallbackFunction
            If Not _objectHandle.IsAllocated Then
                _objectHandle = GCHandle.Alloc(Me)
            End If
        Else
            If _objectHandle.IsAllocated Then
                _objectHandle.Free()
            End If
        End If
        Return irqCallbackFunction
    End Function

    Private Sub StartBackgroundWorker()
        ModuleInfoText = String.Format("Starting IRQ-Wait thread, time-out {0} milliseconds", TimeoutMilliseconds)

        If _worker Is Nothing Then
            _worker = New BackgroundWorker()
            AddHandler _worker.DoWork, AddressOf Worker_DoWork
            AddHandler _worker.RunWorkerCompleted, AddressOf Worker_RunWorkerCompleted
        End If
        _worker.RunWorkerAsync()
    End Sub

    Private Sub Worker_DoWork(sender As Object, e As DoWorkEventArgs)
        Dim irqStatus As AdqDriver.ADQIRQStatus = New AdqDriver.ADQIRQStatus(AdqDriver.ADQIRQStatus_ID)

        Dim adqError As UInteger

        While True
            adqError = AdqDriver.adqWaitIRQ(BoardIndex,
                                            ModuleIndex,
                                             irqStatus,
                                            TimeoutMilliseconds)
            If adqError = AdqErrors.ADQ_ERROR_NONE Then
                UpdateIrqStatus(BoardIndex, ModuleIndex, irqStatus, DateTime.Now)
            Else
                SetErrorText("adqWaitIRQ", adqError)
                If (adqError = AdqErrors.ADQ_ERROR_TIMEOUT) Then
                    ModuleInfoText = String.Format("Error &H{0:X}: Time-out reached after {1} milliseconds", adqError, TimeoutMilliseconds)
                End If
                Exit While
            End If
        End While
    End Sub

    Private Sub Worker_RunWorkerCompleted(sender As Object, e As RunWorkerCompletedEventArgs)
        DisableInterrupt()
        ModuleInfoText = "IRQ-Wait thread ended"
    End Sub

    Private Sub UpdateIrqStatus(boardIndex As UInteger, moduleIndex As UInteger,
                                ByRef irqStatus As AdqDriver.ADQIRQStatus, timeIrqDetected As DateTime)
        If irqStatus.PortBitChangeRisingEdge <> 0 OrElse irqStatus.PortBitChangeFallingEdge <> 0 Then
            Dim args As BitChangeStatusChangedEventArgs = New BitChangeStatusChangedEventArgs With {
                .StatusRisingEdge = irqStatus.PortBitChangeRisingEdge,
                .StatusFallingEdge = irqStatus.PortBitChangeFallingEdge
            }
            OnBitChangeStatusChanged(args)
        End If

        If irqStatus.PortBitPatternCompare Then
            Dim compareArgs As PortCompareStatusChangedEventArgs = New PortCompareStatusChangedEventArgs With {
                .StatusCompare = irqStatus.PortBitPatternCompare,
                .TimeCompareMatched = timeIrqDetected
            }
            OnPortCompareStatusChanged(compareArgs)
        End If

        If irqStatus.PortStatusError <> 0 OrElse irqStatus.FaultRecovery <> 0 Then
            Dim portStatusArgs As PortStatusChangedEventArgs = New PortStatusChangedEventArgs With {
                .StatusPortErrorValue = irqStatus.PortStatusError,
                .StatusFaultRecoveryValue = irqStatus.FaultRecovery
            }
            OnPortStatusChanged(portStatusArgs)
        End If

        If irqStatus.ExternalFallingEdge <> 0 OrElse irqStatus.ExternalRisingEdge <> 0 Then
            Dim externalArgs As ExternalStatusChangedEventArgs = New ExternalStatusChangedEventArgs With {
                .ExternalStatusRisingEdge = irqStatus.ExternalRisingEdge,
                .ExternalStatusFallingEdge = irqStatus.ExternalFallingEdge
            }
            OnExternalStatusChanged(externalArgs)
        End If

        Dim commonArgs As CommonIrqStatusChangedEventArgs = New CommonIrqStatusChangedEventArgs With {
                .StatusIrqCount = irqStatus.IRQCount,
                .StatusTriggerValue = irqStatus.TriggerValue,
                .InfoText = GetIrqStatusInfoText(boardIndex, moduleIndex, irqStatus, timeIrqDetected)
        }
        OnCommonIrqStatusChanged(commonArgs)
    End Sub

    Private Function GetIrqStatusInfoText(boardIndex As UInteger, moduleIndex As UInteger,
                                          ByRef irqStatus As AdqDriver.ADQIRQStatus, timeIrqDetected As Date) As String
        Dim sb As StringBuilder = New StringBuilder()
        sb.Append(timeIrqDetected.ToString(_fmtDateTime))
        sb.AppendFormat("Board {0}, Module {1} interrupt status:" & Environment.NewLine, boardIndex, moduleIndex)

        If irqStatus.PortBitChangeRisingEdge <> 0 Then
            sb.AppendLine(vbTab & "PortBitChangeRisingEdge = &H" & irqStatus.PortBitChangeRisingEdge.ToString("X"))
        End If

        If irqStatus.PortBitChangeFallingEdge <> 0 Then
            sb.AppendLine(vbTab & "PortBitChangeFallingEdge = &H" & irqStatus.PortBitChangeFallingEdge.ToString("X"))
        End If

        If irqStatus.PortBitPatternCompare Then
            sb.AppendLine(vbTab & "PortBitPatternCompare = " & irqStatus.PortBitPatternCompare.ToString())
        End If

        If irqStatus.PortStatusError <> 0 Then
            sb.AppendLine(vbTab & "PortStatusError = &H" & irqStatus.PortStatusError.ToString("X"))
        End If

        If irqStatus.FaultRecovery <> 0 Then
            sb.AppendLine(vbTab & "FaultRecovery = &H" & irqStatus.FaultRecovery.ToString("X"))
        End If

        If irqStatus.ExternalRisingEdge <> 0 Then
            sb.AppendLine(vbTab & "ExternalRisingEdge = &H" & irqStatus.ExternalRisingEdge.ToString("X"))
        End If

        If irqStatus.ExternalFallingEdge <> 0 Then
            sb.AppendLine(vbTab & "ExternalFallingEdge = &H" & irqStatus.ExternalFallingEdge.ToString("X"))
        End If

        sb.AppendLine(vbTab & "IRQ Count = " & irqStatus.IRQCount.ToString())
        sb.Append(vbTab & "Trigger value = &H" & irqStatus.TriggerValue.ToString("X"))
        Return sb.ToString()
    End Function

    Private Sub OnBitChangeStatusChanged(e As BitChangeStatusChangedEventArgs)
        RaiseEvent BitChangeStatusChanged(Me, e)
    End Sub

    Private Sub OnPortCompareStatusChanged(e As PortCompareStatusChangedEventArgs)
        RaiseEvent PortCompareStatusChanged(Me, e)
    End Sub

    Private Sub OnPortStatusChanged(e As PortStatusChangedEventArgs)
        RaiseEvent PortStatusChanged(Me, e)
    End Sub

    Private Sub OnExternalStatusChanged(e As ExternalStatusChangedEventArgs)
        RaiseEvent ExternalStatusChanged(Me, e)
    End Sub

    Private Sub OnCommonIrqStatusChanged(e As CommonIrqStatusChangedEventArgs)
        RaiseEvent CommonIrqStatusChanged(Me, e)
    End Sub

    Private Sub SetErrorText(functionName As String, adqError As UInteger)
        ModuleErrorText = String.Format("Board {0}, Module {1}: API-function {2} returned error number: &H{3:X}",
                _BoardIndex,
                _ModuleIndex,
                functionName,
                adqError)
    End Sub

    Private _errorText As String = String.Empty
    Public Property ModuleErrorText As String
        Get
            Return _errorText
        End Get
        Set
            If String.IsNullOrEmpty(Value) Then
                _errorText = String.Empty
                Return
            End If

            _errorText = DateTime.Now.ToString(_fmtDateTime) & Value
            NotifyPropertyChanged(NameOf(ModuleErrorText))
        End Set
    End Property

    Private _infoText As String = String.Empty
    Public Property ModuleInfoText As String
        Get
            Return _infoText
        End Get
        Set

            If String.IsNullOrEmpty(Value) Then
                _infoText = String.Empty
                Return
            End If

            _infoText = Date.Now.ToString(_fmtDateTime) & Value
            NotifyPropertyChanged(NameOf(ModuleInfoText))
        End Set
    End Property

    Private Sub NotifyPropertyChanged(propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub

    Private Function GetModuleName(boardName As String) As String
        Dim s As String = String.Empty
        If _ModuleType = AdqConstants.ADQ_MODULE_TYPE_DIO OrElse
                _ModuleType = AdqConstants.ADQ_MODULE_TYPE_DI OrElse
                _ModuleType = AdqConstants.ADQ_MODULE_TYPE_DO Then
            s = String.Format(", {0} Bit", _PortWidthBits)
        ElseIf _ModuleType = AdqConstants.ADQ_MODULE_TYPE_EXTINT Then
            s = String.Format(", &H{0:X}", _SupportedExternalIrqs)
        ElseIf _ModuleType = AdqConstants.ADQ_MODULE_TYPE_AI Then
            If _SupportedPortStatusErrors <> 0 OrElse _SupportedFaultRecovery <> 0 Then
                s = ", Port status"
            End If
        End If

        Return String.Format("{0}, Board {1}, Module {2}, Type {3}{4}",
                AdqCommon.GetModelName(boardName),
                _BoardIndex,
                _ModuleIndex,
                AdqCommon.GetModuleTypeName(_ModuleType),
                s)
    End Function

    Public Function ResetModule() As Boolean
        Dim adqError As UInteger = AdqDriver.adqResetModule(_BoardIndex, _ModuleIndex)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetErrorText("adqResetModule", adqError)
            Return False
        End If
        Return True
    End Function

    Public Function ValidatePortMaxValue(value As UInteger) As String
        Dim msg As String = String.Empty
        If value > _portMaxValue Then
            msg = String.Format("The value cannot be greater than &H{0:X}", _portMaxValue)
        End If
        Return msg
    End Function

    Public Function ValidateExtIrqValue(value As UInteger) As String
        Return ValidateEnabledBits(_SupportedExternalIrqs, value)
    End Function

    Public Function ValidateEnabledPortStatusErrorValue(value As UInteger) As String
        Return ValidateEnabledBits(_SupportedPortStatusErrors, value)
    End Function

    Public Function ValidateEnabledFaultRecoveryValue(value As UInteger) As String
        Return ValidateEnabledBits(_SupportedFaultRecovery, value)
    End Function

    Private Function ValidateEnabledBits(supportedValue As UInteger, enteredValue As UInteger) As String
        Dim msg As String = String.Empty
        If ((Not supportedValue) And enteredValue) <> 0 Then
            msg = String.Format("The entered value &H{0:X} is out of the range of supported bits", enteredValue)
        End If
        Return msg
    End Function
End Class
