﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the Interrupt project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.ComponentModel
Imports System.Runtime.InteropServices
Imports Interrupt.ALLDAQ.Boards

Public Class ModuleLocator
    Implements IDisposable, INotifyPropertyChanged
    Private _disposed As Boolean = False
    Private _adqOpenError As UInteger = AdqCommon.NotOpen

    Private Event PropertyChanged As PropertyChangedEventHandler _
        Implements INotifyPropertyChanged.PropertyChanged

    Public Sub New(handler As PropertyChangedEventHandler)
        AddHandler PropertyChanged, handler
    End Sub

    Public Sub InitModuleLocator()
        Try
            _adqOpenError = OpenAdqDriver()
            If _adqOpenError <> AdqErrors.ADQ_ERROR_NONE Then Return

            FindAdqModules()
        Catch ex As DllNotFoundException
            InfoText = "The device driver is not properly installed." & Environment.NewLine _
                & ex.Message
        End Try
    End Sub

    Private Function OpenAdqDriver() As UInteger
        Dim adqError As UInteger = AdqDriver.adqOpen()
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqOpen", adqError)
        End If
        Return adqError
    End Function

    Public Sub FindAdqModules()
        Dim numberOfBoards As UInteger = GetNumberOfBoards()
        Dim boardIdx As UInteger = 0

        While boardIdx < numberOfBoards
            Dim numberOfModules As UInteger
            Dim adqError As UInteger = AdqDriver.adqGetNumberOfModules(boardIdx, numberOfModules)
            If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                SetFunctionErrorInfo("adqGetNumberOfModules", adqError)
                boardIdx += 1UI
                Continue While
            End If

            Dim boardName As String = GetBoardName(boardIdx)
            GetIrqModules(boardName, boardIdx, numberOfModules)
            boardIdx += 1UI
        End While
    End Sub

    Private Function GetIrqModules(boardName As String, boardIndex As UInteger, numberOfModules As UInteger) As UInteger
        Dim adqError As UInteger = AdqErrors.ADQ_ERROR_NONE
        Dim moduleIdx As UInteger = 0

        While moduleIdx < numberOfModules
            Dim moduleType As UInteger
            adqError = AdqDriver.adqGetModuleType(boardIndex, moduleIdx, moduleType)
            If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                SetFunctionErrorInfo("adqGetModuleType", adqError)
                moduleIdx += 1UI
                Continue While
            End If

            If moduleType = AdqConstants.ADQ_MODULE_TYPE_DI Then
                adqError = GetDIModuleInfo(boardName, boardIndex, moduleIdx)
            ElseIf moduleType = AdqConstants.ADQ_MODULE_TYPE_DIO Then
                adqError = GetDIOModuleInfo(boardName, boardIndex, moduleIdx)
            ElseIf moduleType = AdqConstants.ADQ_MODULE_TYPE_DO Then
                adqError = GetDOModuleInfo(boardName, boardIndex, moduleIdx)
            ElseIf moduleType = AdqConstants.ADQ_MODULE_TYPE_EXTINT Then
                adqError = GetExtIntModuleInfo(boardName, boardIndex, moduleIdx)
            ElseIf moduleType = AdqConstants.ADQ_MODULE_TYPE_CNTR Then
                adqError = GetCounterModuleInfo(boardName, boardIndex, moduleIdx)
            ElseIf moduleType = AdqConstants.ADQ_MODULE_TYPE_AI Then
                adqError = GetAIModuleInfo(boardName, boardIndex, moduleIdx)
            End If
            If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                Exit While
            End If
            moduleIdx += 1UI
        End While
        Return adqError
    End Function

    Private Function GetDIModuleInfo(boardName As String, boardIndex As UInteger, moduleIndex As UInteger) As UInteger
        Dim adqGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqDiModuleInfo = New AdqDriver.ADQDIModuleInfo(AdqDriver.ADQDIModuleInfo_ID)

        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqDiModuleInfo.Size))
        Marshal.StructureToPtr(adqDiModuleInfo, moduleInfo, False)

        Dim infoSize As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetModuleInfo(boardIndex, moduleIndex, adqGeneralModuleInfo, adqDiModuleInfo.Size, moduleInfo, infoSize)
        adqDiModuleInfo = CType(Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQDIModuleInfo)), AdqDriver.ADQDIModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetModuleInfo", adqError)
            Return adqError
        End If

        Debug.Assert(adqDiModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqDiModuleInfo.Size, infoSize))

        If adqGeneralModuleInfo.IsIRQSource Then
            adqError = GetIrqInfo(
                    boardName,
                    boardIndex,
                    moduleIndex,
                    adqGeneralModuleInfo,
                    adqDiModuleInfo.PortWidthBits)
        End If
        Return adqError
    End Function

    Private Function GetDIOModuleInfo(boardName As String, boardIndex As UInteger, moduleIndex As UInteger) As UInteger
        Dim adqGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqDioModuleInfo = New AdqDriver.ADQDIOModuleInfo(AdqDriver.ADQDIOModuleInfo_ID)

        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqDioModuleInfo.Size))
        Marshal.StructureToPtr(adqDioModuleInfo, moduleInfo, False)

        Dim infoSize As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetModuleInfo(boardIndex, moduleIndex, adqGeneralModuleInfo, adqDioModuleInfo.Size, moduleInfo, infoSize)
        adqDioModuleInfo = CType(Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQDIOModuleInfo)), AdqDriver.ADQDIOModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetModuleInfo", adqError)
            Return adqError
        End If

        Debug.Assert(adqDioModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqDioModuleInfo.Size, infoSize))

        If adqGeneralModuleInfo.IsIRQSource Then
            adqError = GetIrqInfo(
                    boardName,
                    boardIndex,
                    moduleIndex,
                    adqGeneralModuleInfo,
                    adqDioModuleInfo.PortWidthBits)
        End If
        Return adqError
    End Function

    Private Function GetDOModuleInfo(boardName As String, boardIndex As UInteger, moduleIndex As UInteger) As UInteger
        Dim adqGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqDoModuleInfo = New AdqDriver.ADQDOModuleInfo(AdqDriver.ADQDOModuleInfo_ID)

        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqDoModuleInfo.Size))
        Marshal.StructureToPtr(adqDoModuleInfo, moduleInfo, False)

        Dim infoSize As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetModuleInfo(boardIndex, moduleIndex, adqGeneralModuleInfo, adqDoModuleInfo.Size, moduleInfo, infoSize)
        adqDoModuleInfo = CType(Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQDOModuleInfo)), AdqDriver.ADQDOModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetModuleInfo", adqError)
            Return adqError
        End If

        Debug.Assert(adqDoModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqDoModuleInfo.Size, infoSize))

        If adqGeneralModuleInfo.IsIRQSource Then
            adqError = GetIrqInfo(
                    boardName,
                    boardIndex,
                    moduleIndex,
                    adqGeneralModuleInfo,
                    adqDoModuleInfo.PortWidthBits)
        End If
        Return adqError
    End Function

    Private Function GetExtIntModuleInfo(boardName As String, boardIndex As UInteger, moduleIndex As UInteger) As UInteger
        Const moduleInfoSize As UInteger = 0
        Dim adqGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)

        Dim infoSize As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetModuleInfo(
                boardIndex,
                moduleIndex,
                adqGeneralModuleInfo,
                moduleInfoSize,
                IntPtr.Zero,
                infoSize)

        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetModuleInfo", adqError)
            Return adqError
        End If

        Debug.Assert(moduleInfoSize = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, moduleInfoSize, infoSize))

        If adqGeneralModuleInfo.IsIRQSource Then
            adqError = GetIrqInfo(
                    boardName,
                    boardIndex,
                    moduleIndex,
                    adqGeneralModuleInfo,
                    portWidthBits:=0)
        End If
        Return adqError
    End Function

    Private Function GetCounterModuleInfo(boardName As String, boardIndex As UInteger, moduleIndex As UInteger) As UInteger
        Const moduleInfoSize As UInteger = 0
        Dim adqGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)

        Dim infoSize As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetModuleInfo(
                boardIndex,
                moduleIndex,
                adqGeneralModuleInfo,
                moduleInfoSize,
                IntPtr.Zero,
                infoSize)

        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetModuleInfo", adqError)
            Return adqError
        End If

        Debug.Assert(moduleInfoSize = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, moduleInfoSize, infoSize))

        If adqGeneralModuleInfo.IsIRQSource Then
            adqError = GetIrqInfo(
                    boardName,
                    boardIndex,
                    moduleIndex,
                    adqGeneralModuleInfo,
                    portWidthBits:=0)
        End If
        Return adqError
    End Function

    Private Function GetAIModuleInfo(boardName As String, boardIndex As UInteger, moduleIndex As UInteger) As UInteger
        Dim adqGeneralModuleInfo = New AdqDriver.ADQGeneralModuleInfo(AdqDriver.ADQGeneralModuleInfo_ID)
        Dim adqAiModuleInfo = New AdqDriver.ADQAIModuleInfo(AdqDriver.ADQAIModuleInfo_ID)

        Dim moduleInfo As IntPtr = Marshal.AllocHGlobal(CInt(adqAiModuleInfo.Size))
        Marshal.StructureToPtr(adqAiModuleInfo, moduleInfo, False)

        Dim infoSize As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetModuleInfo(
                boardIndex,
                moduleIndex,
                adqGeneralModuleInfo,
                adqAiModuleInfo.Size,
                moduleInfo,
                 infoSize)
        adqAiModuleInfo = CType(Marshal.PtrToStructure(moduleInfo, GetType(AdqDriver.ADQAIModuleInfo)), AdqDriver.ADQAIModuleInfo)
        Marshal.FreeHGlobal(moduleInfo)

        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetModuleInfo", adqError)
            Return adqError
        End If

        Debug.Assert(adqAiModuleInfo.Size = infoSize,
                         String.Format("Board {0}, Module {1}: API-function adqGetModuleInfo info size does not match - expected {2}, returned {3}",
                            boardIndex, moduleIndex, adqAiModuleInfo.Size, infoSize))

        If adqGeneralModuleInfo.IsIRQSource Then
            adqError = GetIrqInfo(
                    boardName,
                    boardIndex,
                    moduleIndex,
                    adqGeneralModuleInfo,
                    portWidthBits:=0)
        End If
        Return adqError
    End Function

    Private Function GetIrqInfo(boardName As String, boardIndex As UInteger, moduleIndex As UInteger,
            ByRef generalInfo As AdqDriver.ADQGeneralModuleInfo, portWidthBits As UInteger) As UInteger
        Dim adqIrqInfo = New AdqDriver.ADQIRQInfo(AdqDriver.ADQIRQInfo_ID)

        Dim adqError As UInteger = AdqDriver.adqGetIRQInfo(boardIndex, moduleIndex, adqIrqInfo)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetIRQInfo", adqError)
            Return adqError
        End If

        Dim irqModule = New IrqModule(boardName, boardIndex, moduleIndex, generalInfo, adqIrqInfo, portWidthBits)
        IrqModules.Add(irqModule)

        Return adqError
    End Function

    Private Function GetNumberOfBoards() As UInteger
        Dim numBoards As UInteger
        Dim adqError As UInteger = AdqDriver.adqGetNumberOfBoards(numBoards)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqGetNumberOfBoards", adqError)
            Return 0
        End If
        Return numBoards
    End Function

    Private Function GetBoardName(boardIndex As UInteger) As String
        Dim s As String = String.Empty
        Dim boardInfo = New AdqDriver.ADQBoardInfo(AdqDriver.ADQBoardInfo_ID)

        Dim adqError As UInteger = AdqDriver.adqGetBoardInfo(boardIndex, boardInfo)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            s = boardInfo.BoardName
        End If
        Return s
    End Function

    Private Sub SetFunctionErrorInfo(functionName As String, adqError As UInteger)
        InfoText = String.Format("API-function {0} returned Error number: &H{1:X}", functionName, adqError)
    End Sub

    Public ReadOnly Property IrqModules As List(Of IrqModule) = New List(Of IrqModule)

    Private Sub CloseAdqDriver()
        If _adqOpenError = AdqErrors.ADQ_ERROR_NONE Then
            AdqDriver.adqClose()
            _adqOpenError = AdqCommon.NotOpen
        End If
    End Sub

    Dim _infoText As String = String.Empty
    Public Property InfoText As String
        Get
            Return _infoText
        End Get
        Set(value As String)
            _infoText = value
            NotifyPropertyChanged(NameOf(InfoText))
        End Set
    End Property

    Public Sub NotifyPropertyChanged(ByVal propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub

    Protected Overloads Overrides Sub Finalize()
        Dispose(False)
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
    End Sub

    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not _disposed Then
            _disposed = True
            CloseAdqDriver()
        End If
    End Sub
End Class
