﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the Interrupt project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.Collections.ObjectModel
Imports System.ComponentModel

Public Class ExternalIrqViewModel
    Inherits ErrorInfoViewModelBase

    Private _irqModule As IrqModule
    Public Sub UpdateIrqModule(irqModule As IrqModule)
        If SupportsIrqType Then
            RemoveHandler _irqModule.ExternalStatusChanged, AddressOf ExternalStatusChanged
            RemoveHandler _irqModule.PropertyChanged, AddressOf ExternalIrqViewModel_PropertyChanged
        End If

        _irqModule = irqModule
        SupportsIrqType = irqModule IsNot Nothing AndAlso irqModule.SupportedExternalIrqs <> 0

        If SupportsIrqType Then
            UpdateBitMaskViewModels()
            UpdateRisingEdgesText(irqModule.ExternalRisingEdge)
            UpdateRisingBitMaskEdges(irqModule.ExternalRisingEdge)
            UpdateFallingEdgesText(irqModule.ExternalFallingEdge)
            UpdateFallingBitMaskEdges(irqModule.ExternalFallingEdge)
            AddHandler _irqModule.ExternalStatusChanged, AddressOf ExternalStatusChanged
            AddHandler _irqModule.PropertyChanged, AddressOf ExternalIrqViewModel_PropertyChanged
        Else
            BitMaskEdgeItems.Clear()
            BitEdgeIndicatorItems.Clear()
            UpdateRisingEdgesText(0)
            UpdateFallingEdgesText(0)
        End If
        ClearStatusIndicators()
        NotifyPropertyChanged(NameOf(SupportsIrqType))
        UpdateModuleIdleStatus()
    End Sub

    Public Property SupportsIrqType As Boolean

    Public ReadOnly Property CanEditIrqSettings As Boolean
        Get
            Return SupportsIrqType AndAlso _irqModule IsNot Nothing AndAlso _irqModule.IsModuleIdle
        End Get
    End Property

    Private Sub UpdateBitMaskViewModels()
        BitMaskEdgeItems.Clear()
        BitEdgeIndicatorItems.Clear()

        Dim bitIdx As Integer = 0
        While bitIdx < 32 AndAlso ((_irqModule.SupportedExternalIrqs >> bitIdx) And 1) <> 0
            Dim bmItem As BitMaskEdgeItemViewModel = New BitMaskEdgeItemViewModel(bitIdx)
            AddHandler bmItem.PropertyChanged, AddressOf BmItem_PropertyChanged
            BitMaskEdgeItems.Add(bmItem)

            Dim indItem As BitEdgeIndicatorItemViewModel = New BitEdgeIndicatorItemViewModel(bitIdx)
            BitEdgeIndicatorItems.Add(indItem)

            bitIdx += 1
        End While
    End Sub

    Private Sub BmItem_PropertyChanged(sender As Object, e As PropertyChangedEventArgs)
        If e.PropertyName = NameOf(BitMaskEdgeItemViewModel.BitRisingEdge) Then
            UpdateRisingEdgeValue()
        ElseIf e.PropertyName = NameOf(BitMaskEdgeItemViewModel.BitFallingEdge) Then
            UpdateFallingEdgeValue()
        End If
    End Sub

    Private Sub UpdateRisingEdgeValue()
        Dim edgesValue As UInteger = 0
        Dim bit As UInteger = 1
        For Each bitMaskEdgeItem As BitMaskEdgeItemViewModel In BitMaskEdgeItems
            If bitMaskEdgeItem.BitRisingEdge Then
                edgesValue = edgesValue Or bit
            End If
            bit <<= 1
        Next
        _irqModule.ExternalRisingEdge = edgesValue
        UpdateRisingEdgesText(edgesValue)
    End Sub

    Private Sub UpdateRisingEdgesText(edgesValue As UInteger)
        _valueRisingEdgesText = AdqCommon.FormatHexValue(edgesValue)
        _errorInfo.Remove(NameOf(ValueRisingEdgesText))
        NotifyPropertyChanged(NameOf(ValueRisingEdgesText))
        NotifyPropertyChanged(NameOf(IsInputValid))
    End Sub

    Private Sub UpdateFallingEdgeValue()
        Dim edgesValue As UInteger = 0
        Dim bit As UInteger = 1
        For Each bitMaskEdgeItem As BitMaskEdgeItemViewModel In BitMaskEdgeItems
            If bitMaskEdgeItem.BitFallingEdge Then
                edgesValue = edgesValue Or bit
            End If
            bit <<= 1
        Next
        _irqModule.ExternalFallingEdge = edgesValue
        UpdateFallingEdgesText(edgesValue)
    End Sub

    Private Sub UpdateFallingEdgesText(edgesValue As UInteger)
        _valueFallingEdgesText = AdqCommon.FormatHexValue(edgesValue)
        _errorInfo.Remove(NameOf(ValueFallingEdgesText))
        NotifyPropertyChanged(NameOf(ValueFallingEdgesText))
        NotifyPropertyChanged(NameOf(IsInputValid))
    End Sub

    Public ReadOnly Property BitMaskEdgeItems As ObservableCollection(Of BitMaskEdgeItemViewModel) =
        New ObservableCollection(Of BitMaskEdgeItemViewModel)
    Public ReadOnly Property BitEdgeIndicatorItems As ObservableCollection(Of BitEdgeIndicatorItemViewModel) =
        New ObservableCollection(Of BitEdgeIndicatorItemViewModel)

    Public Sub ClearStatusIndicators()
        UpdateStatusRisingEdges(0)
        UpdateStatusFallingEdges(0)
    End Sub

    Private _valueRisingEdgesText As String = "0"
    Public Property ValueRisingEdgesText As String
        Get
            Return _valueRisingEdgesText
        End Get
        Set
            If _valueRisingEdgesText <> Value Then
                _valueRisingEdgesText = Value

                Dim edgesValue As UInteger
                Dim msg As String = InputValidation.ValidateHexValue(_valueRisingEdgesText, edgesValue)

                If Not String.IsNullOrEmpty(msg) Then
                    _errorInfo.AddOrUpdate(NameOf(ValueRisingEdgesText), msg)
                Else
                    msg = _irqModule.ValidateExtIrqValue(edgesValue)
                    If Not String.IsNullOrEmpty(msg) Then
                        _errorInfo.AddOrUpdate(NameOf(ValueRisingEdgesText), msg)
                    Else
                        _errorInfo.Remove(NameOf(ValueRisingEdgesText))
                        _irqModule.ExternalRisingEdge = edgesValue
                        _valueRisingEdgesText = AdqCommon.FormatHexValue(edgesValue)
                        UpdateRisingBitMaskEdges(edgesValue)
                    End If
                End If
                NotifyPropertyChanged(NameOf(ValueRisingEdgesText))
                NotifyPropertyChanged(NameOf(IsInputValid))
            End If
        End Set
    End Property

    Private _valueFallingEdgesText As String = "0"
    Public Property ValueFallingEdgesText As String
        Get
            Return _valueFallingEdgesText
        End Get
        Set
            If _valueFallingEdgesText <> Value Then
                _valueFallingEdgesText = Value

                Dim edgesValue As UInteger
                Dim msg As String = InputValidation.ValidateHexValue(Value, edgesValue)

                If Not String.IsNullOrEmpty(msg) Then
                    _errorInfo.AddOrUpdate(NameOf(ValueFallingEdgesText), msg)
                Else
                    msg = _irqModule.ValidateExtIrqValue(edgesValue)
                    If Not String.IsNullOrEmpty(msg) Then
                        _errorInfo.AddOrUpdate(NameOf(ValueFallingEdgesText), msg)
                    Else
                        _errorInfo.Remove(NameOf(ValueFallingEdgesText))
                        _irqModule.ExternalFallingEdge = edgesValue
                        _valueFallingEdgesText = AdqCommon.FormatHexValue(edgesValue)
                        UpdateFallingBitMaskEdges(edgesValue)
                    End If
                End If

                NotifyPropertyChanged(NameOf(ValueFallingEdgesText))
                NotifyPropertyChanged(NameOf(IsInputValid))
            End If
        End Set
    End Property

    Private Sub UpdateRisingBitMaskEdges(edgesValue As UInteger)
        Dim bit As Integer = 1
        For Each bitMaskEdgeItem As BitMaskEdgeItemViewModel In BitMaskEdgeItems
            bitMaskEdgeItem.BitRisingEdge = (bit And edgesValue) <> 0
            bit <<= 1
        Next
    End Sub

    Private Sub UpdateFallingBitMaskEdges(edgesValue As UInteger)
        Dim bit As Integer = 1
        For Each bitMaskEdgeItem As BitMaskEdgeItemViewModel In BitMaskEdgeItems
            bitMaskEdgeItem.BitFallingEdge = (bit And edgesValue) <> 0
            bit <<= 1
        Next
    End Sub


    Private Sub ExternalStatusChanged(sender As Object, e As ExternalStatusChangedEventArgs)
        UpdateStatusRisingEdges(e.ExternalStatusRisingEdge)
        UpdateStatusFallingEdges(e.ExternalStatusFallingEdge)
    End Sub

    Private Sub UpdateStatusRisingEdges(edgesValue As UInteger)
        If _statusExtIrqRisingEdges <> edgesValue Then
            _statusExtIrqRisingEdges = edgesValue
            NotifyPropertyChanged(NameOf(StatusExtIrqRisingEdges))
            UpdateRisingBitIndicatorsEdges(edgesValue)
        End If
    End Sub

    Private Sub UpdateStatusFallingEdges(edgesValue As UInteger)
        If _statusExtIrqFallingEdges <> edgesValue Then
            _statusExtIrqFallingEdges = edgesValue
            NotifyPropertyChanged(NameOf(StatusExtIrqFallingEdges))
            UpdateFallingBitIndicatorsEdges(edgesValue)
        End If
    End Sub

    Private _statusExtIrqRisingEdges As UInteger
    Public ReadOnly Property StatusExtIrqRisingEdges As UInteger
        Get
            Return _statusExtIrqRisingEdges
        End Get
    End Property

    Private _statusExtIrqFallingEdges As UInteger
    Public ReadOnly Property StatusExtIrqFallingEdges As UInteger
        Get
            Return _statusExtIrqFallingEdges
        End Get
    End Property

    Private Sub UpdateRisingBitIndicatorsEdges(edgesValue As UInteger)
        Dim bit As Integer = 1
        For Each bitIndicatorItem As BitEdgeIndicatorItemViewModel In BitEdgeIndicatorItems
            bitIndicatorItem.RisingEdge = (bit And edgesValue) <> 0
            bit <<= 1
        Next
    End Sub

    Private Sub UpdateFallingBitIndicatorsEdges(edgesValue As UInteger)
        Dim bit As Integer = 1
        For Each bitIndicatorItem As BitEdgeIndicatorItemViewModel In BitEdgeIndicatorItems
            bitIndicatorItem.FallingEdge = (bit And edgesValue) <> 0
            bit <<= 1
        Next
    End Sub

    Private Sub ExternalIrqViewModel_PropertyChanged(sender As Object, e As PropertyChangedEventArgs)
        If e.PropertyName = NameOf(IrqModule.IsModuleIdle) Then
            UpdateModuleIdleStatus()
        End If
    End Sub

    Private Sub UpdateModuleIdleStatus()
        NotifyPropertyChanged(NameOf(CanEditIrqSettings))
    End Sub
End Class
