﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the ReadBuffer project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.Runtime.InteropServices
Imports ReadBuffer.ALLDAQ.Boards

Public Class InputModule
    Implements IInputModule

    Public Property AdqError As UInteger Implements IInputModule.AdqError

    Public Event ModuleInfoTextChanged(ByVal Sender As Object, ByVal e As InfoTextChangedEventArgs) Implements IInputModule.ModuleInfoTextChanged

    Public Sub New(boardName As String, boardIndex As UInteger, moduleIndex As UInteger, ByRef generalInfo As AdqDriver.ADQGeneralModuleInfo)
        _numberOfValues = AdqCommon.DefaultNumberOfValues
        ModelName = AdqCommon.GetModelName(boardName)
        Me.BoardIndex = boardIndex
        Me.ModuleIndex = moduleIndex
        ModuleType = generalInfo.ModuleType
        ClockType = generalInfo.ClockType
        MaxfrequencyHz = generalInfo.ClockBaseFrequencyHz / generalInfo.ClockMinimumDivider
        _desiredSampleFrequency = MaxfrequencyHz
        _actualSampleFrequency = MaxfrequencyHz
        _sampleTime = 1.0 / MaxfrequencyHz
    End Sub

    Protected _moduleName As String
    Public ReadOnly Property ModuleName As String Implements IInputModule.ModuleName
        Get
            Return _moduleName
        End Get
    End Property

    Public ReadOnly Property ModelName As String Implements IInputModule.ModelName
    Public ReadOnly Property BoardIndex As UInteger Implements IInputModule.BoardIndex
    Public ReadOnly Property ModuleIndex As UInteger Implements IInputModule.ModuleIndex
    Public ReadOnly Property ModuleType As UInteger Implements IInputModule.ModuleType
    Public ReadOnly Property ClockType As UInteger Implements IInputModule.ClockType
    Public ReadOnly Property MaxfrequencyHz As Double Implements IInputModule.MaxfrequencyHz

    Private _desiredSampleFrequency As Double
    Public Property DesiredSampleFrequency As Double Implements IInputModule.DesiredSampleFrequency
        Get
            Return _desiredSampleFrequency
        End Get
        Set(value As Double)
            _desiredSampleFrequency = value
            SetSampleFrequency()
        End Set
    End Property

    Dim _actualSampleFrequency As Double
    Public ReadOnly Property ActualSampleFrequency As Double Implements IInputModule.ActualSampleFrequency
        Get
            Return _actualSampleFrequency
        End Get
    End Property

    Private _sampleTime As Double
    Public ReadOnly Property SampleTime As Double Implements IInputModule.SampleTime
        Get
            Return _sampleTime
        End Get
    End Property

    Public Function SetSampleFrequency() As UInteger Implements IInputModule.SetSampleFrequency
        Dim actualFrequency As Double
        Dim adqError As UInteger = AdqDriver.adqSetFrequency(BoardIndex, ModuleIndex, _desiredSampleFrequency, actualFrequency)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            ErrorTextChanged("adqSetFrequency", adqError)
            actualFrequency = 0.0
        Else
            _sampleTime = 1.0 / actualFrequency
        End If
        _actualSampleFrequency = actualFrequency
        Return adqError
    End Function

    Protected _numberOfValues As UInteger
    Public Property NumberOfValues As UInteger Implements IInputModule.NumberOfValues
        Get
            Return _numberOfValues
        End Get
        Set(value As UInteger)
            _numberOfValues = value
        End Set
    End Property

    Protected _values As UInteger()
    Public ReadOnly Property Values As UInteger() Implements IInputModule.Values
        Get
            Return _values
        End Get
    End Property

    Protected _numberOfValuesRead As UInteger
    Public ReadOnly Property NumberOfValuesRead As UInteger Implements IInputModule.NumberOfValuesRead
        Get
            Return _numberOfValuesRead
        End Get
    End Property

    Public Function GetSecondsToWait() As Double Implements IInputModule.GetSecondsToWait
        If _actualSampleFrequency <= 0.0 Then
            Return 0.0
        End If
        Return _numberOfValues / _actualSampleFrequency
    End Function

    Public Sub ValidateConfigurationValues() Implements IInputModule.ValidateConfigurationValues
        If _numberOfValues = 0 Then
            Throw New ApplicationException("Please enter a valid number of values.")
        End If
        If _actualSampleFrequency = 0.0 Then
            Throw New ApplicationException("Please enter a valid sample frequency.")
        End If
    End Sub

    Public Sub InitValueBuffer() Implements IInputModule.InitValueBuffer
        If _values Is Nothing OrElse _values.Length <> _numberOfValues Then
            _values = New UInteger(CInt(_numberOfValues - 1UI)) {}
        End If
        _numberOfValuesRead = 0
    End Sub

    Public Sub ReadValuesFromModule() Implements IInputModule.ReadValuesFromModule
        Dim gch As GCHandle = GCHandle.Alloc(Values, GCHandleType.Pinned)
        AdqError = AdqDriver.adqReadBuffer(
            BoardIndex,
            ModuleIndex,
            _numberOfValues,
            _values,
            _numberOfValuesRead)
        gch.Free()
    End Sub

    Public Function Reset() As UInteger Implements IInputModule.Reset
        AdqError = AdqDriver.adqResetModule(BoardIndex, ModuleIndex)
        If AdqError = AdqErrors.ADQ_ERROR_NONE Then
            InfoTextChanged(String.Format("Reset Board {0}, Module {1}",
                        BoardIndex, ModuleIndex))
        Else
            ErrorTextChanged("adqResetModule", AdqError)
        End If
        Return AdqError
    End Function

    Public Function StopAcquisition() As UInteger Implements IInputModule.StopAcquisition
        Return AdqDriver.adqStop(BoardIndex, ModuleIndex)
    End Function

    Public Function GetModuleStatus(ByRef isRunning As Boolean) As UInteger Implements IInputModule.GetModuleStatus
        Dim numValuesAvailable As UInteger
        Dim adqError = AdqDriver.adqReadContinuousStatus(BoardIndex, ModuleIndex, False, numValuesAvailable, isRunning)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            ErrorTextChanged("adqReadContinuousStatus", adqError)
        End If

        Return adqError
    End Function

    Public Sub ClearValueBuffer() Implements IInputModule.ClearValueBuffer
        _values = Nothing
    End Sub

    Protected Sub InfoTextChanged(text As String)
        Dim args = New InfoTextChangedEventArgs With {
            .InfoText = text
        }
        RaiseOnModuleInfoTextChanged(args)
    End Sub

    Protected Sub ErrorTextChanged(functionName As String, adqError As UInteger)
        Dim args = New InfoTextChangedEventArgs With {
            .InfoText = String.Format("Function {0} returned Error number: &H{1:X}", functionName, adqError)
        }
        RaiseOnModuleInfoTextChanged(args)
    End Sub

    Protected Sub RaiseOnModuleInfoTextChanged(e As InfoTextChangedEventArgs)
        RaiseEvent ModuleInfoTextChanged(Me, e)
    End Sub
End Class
