﻿'-----------------------------------------------------------------------------
' Messenger downloaded from StackOverflow and changed to Visual Basic with modifications
' http://stackoverflow.com/questions/23798425/wpf-mvvm-communication-between-view-model
'
' Purpose:
'       This file Is part of the WriteBufferAO project of the ALLDAQ-SDK.  
' 
'-----------------------------------------------------------------------------

Imports System.Collections.Concurrent

Public Class Messenger

    Private Shared ReadOnly CreationLock As Object = New Object()
    Private Shared ReadOnly Dictionary As ConcurrentDictionary(Of MessengerKey, Object) = New ConcurrentDictionary(Of MessengerKey, Object)()

    Private Shared _instance As Messenger

    ''' <summary>
    ''' Gets the single instance of the Messenger.
    ''' </summary>
    Public Shared ReadOnly Property Instance As Messenger
        Get
            If (_instance Is Nothing) Then
                SyncLock CreationLock
                    If (_instance Is Nothing) Then
                        _instance = New Messenger()
                    End If
                End SyncLock
            End If
            Return _instance
        End Get
    End Property

    ''' <summary>
    ''' Initializes a New instance of the Messenger class.
    ''' </summary>
    Private Sub New()
    End Sub

    ''' <summary>
    ''' Registers a recipient for a type of message T. The action parameter will be executed
    ''' when a corresponding message Is sent.
    ''' </summary>
    ''' <typeparam name="T"></typeparam>
    ''' <param name="recipient"></param>
    ''' <param name="action"></param>
    Public Sub Register(Of T)(recipient As Object, action As Action(Of T))
        Register(recipient, action, Nothing)
    End Sub

    ''' <summary>
    ''' Registers a recipient for a type of message T And a matching context. The action parameter will be executed
    ''' when a corresponding message Is sent.
    ''' </summary>
    ''' <typeparam name="T"></typeparam>
    ''' <param name="recipient"></param>
    ''' <param name="action"></param>
    ''' <param name="context"></param>
    Public Sub Register(Of T)(recipient As Object, action As Action(Of T), context As Object)
        Dim Key = New MessengerKey(recipient, context)
        Dictionary.TryAdd(Key, action)
    End Sub

    ''' <summary>
    ''' Unregisters a messenger recipient completely. After this method Is executed, the recipient will
    ''' no longer receive any messages.
    ''' </summary>
    ''' <param name="recipient"></param>
    Public Sub Unregister(recipient As Object)
        Unregister(recipient, Nothing)
    End Sub

    ''' <summary>
    ''' Unregisters a messenger recipient with a matching context completely. After this method Is executed, the recipient will
    ''' no longer receive any messages.
    ''' </summary>
    ''' <param name="recipient"></param>
    ''' <param name="context"></param>
    Public Sub Unregister(recipient As Object, context As Object)
        Dim action As Object = Nothing
        Dim Key = New MessengerKey(recipient, context)
        Dictionary.TryRemove(Key, action)
    End Sub

    ''' <summary>
    ''' Sends a message to registered recipients. The message will reach all recipients that are
    ''' registered for this message type.
    ''' </summary>
    ''' <typeparam name="T"></typeparam>
    ''' <param name="message"></param>
    Public Sub Send(Of T)(message As T)
        Send(message, Nothing)
    End Sub

    ''' <summary>
    ''' Sends a message to registered recipients. The message will reach all recipients that are
    ''' registered for this message type And matching context.
    ''' </summary>
    ''' <typeparam name="T"></typeparam>
    ''' <param name="message"></param>
    ''' <param name="context"></param>
    Public Sub Send(Of T)(message As T, context As Object)
        Dim result As IEnumerable(Of KeyValuePair(Of MessengerKey, Object))

        If (context Is Nothing) Then
            ' Get all recipients where the context Is null.
            result = From r In Dictionary Where r.Key.Context Is Nothing Select r
        Else
            ' Get all recipients where the context Is matching.
            result = From r In Dictionary Where r.Key.Context IsNot Nothing And r.Key.Context.Equals(context) Select r
        End If

        For Each act In result.Select(Function(x) x.Value).OfType(Of Action(Of T))()
            ' Send the message to all recipients.
            act(message)
        Next
    End Sub

    Protected Class MessengerKey
        Public ReadOnly Property Recipient As Object
        Public ReadOnly Property Context As Object

        ''' <summary>
        ''' Initializes a New instance of the MessengerKey class.
        ''' </summary>
        ''' <param name="recipient"></param>
        ''' <param name="context"></param>
        Public Sub New(recipient As Object, context As Object)
            Me.Recipient = recipient
            Me.Context = context
        End Sub

        ''' <summary>
        ''' Determines whether the specified MessengerKey Is equal to the current MessengerKey.
        ''' </summary>
        ''' <param name="other"></param>
        ''' <returns></returns>
        Protected Overloads Function Equals(other As MessengerKey) As Boolean
            If Me Is other Then
                Return True
            End If

            Return Equals(Recipient, other.Recipient) AndAlso Equals(Context, other.Context)
        End Function

        ''' <summary>
        ''' Determines whether the specified MessengerKey Is equal to the current MessengerKey.
        ''' </summary>
        ''' <param name="obj"></param>
        ''' <returns></returns>
        Public Overrides Function Equals(obj As Object) As Boolean
            If (obj Is Nothing) Then
                Return False
            End If
            If (ReferenceEquals(Me, obj)) Then
                Return True
            End If
            If (obj.GetType() <> [GetType]()) Then
                Return False
            End If
            Return MyBase.Equals(obj)
        End Function

        Public Overrides Function GetHashCode() As Integer
            Dim hash As Long = 17
            If Recipient IsNot Nothing Then
                hash = (hash * 23 + Recipient.GetHashCode()) Mod Integer.MaxValue
            End If
            If Context IsNot Nothing Then
                hash = (hash * 23 + Context.GetHashCode()) Mod Integer.MaxValue
            End If
            Return Convert.ToInt32(hash)
        End Function
    End Class
End Class
