﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the WriteBufferAO project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.ComponentModel
Imports System.Globalization
Imports System.Windows.Threading
Imports WriteBufferAO.ALLDAQ.Boards

Public Class AoModuleViewModel
    Implements INotifyPropertyChanged, IDataErrorInfo, IDisposable

    Private _disposed As Boolean = False
    Private ReadOnly _aoModule As AoModule
    Private ReadOnly _errorInfo As Dictionary(Of String, String) = New Dictionary(Of String, String)
    Private ReadOnly _defaultSignalLength As Double = 1.0
    Private ReadOnly _fmtStr As String = "{0:0.###}"
    Private ReadOnly _waveForm As WaveForm
    Private _dispatcherTimer As DispatcherTimer
    Private _worker As BackgroundWorker

    Private Event PropertyChanged As PropertyChangedEventHandler _
        Implements INotifyPropertyChanged.PropertyChanged

    Public Property StartStopCommand As RelayCommand

    Public Sub New(aoModule As AoModule, backgroundColor As Brush)
        _aoModule = aoModule
        Me.BackgroundColor = backgroundColor
        AddHandler _aoModule.PropertyChanged, AddressOf AoModuleViewModel_PropertyChanged

        _desiredOutputFrequency = String.Format(_fmtStr, _aoModule.DefaultOutputFrequency)

        _signalLength = _defaultSignalLength
        _signalLengthText = String.Format(_fmtStr, _signalLength)
        _repeatBufferCount = _aoModule.RepeatBufferCount.ToString()
        UpdateNumberOfValues()
        InitSignalViewModels()
        InitCommands()
        InitDispatcherTimer()
        _waveForm = New WaveForm()

        Messenger.Instance.Send(New InfoMessage(String.Format("Selected Board {0}, {1}",
            _aoModule.BoardIndex, _aoModule.ModuleName)), AdqCommon.MessengerContextInfo)
    End Sub

    Private Sub InitCommands()
        StartStopCommand = New RelayCommand(AddressOf StartStopWriteBuffer, AddressOf CanStartStopWriteBuffer)
    End Sub

    Private Sub InitDispatcherTimer()
        _dispatcherTimer = New DispatcherTimer With {
            .Interval = New TimeSpan(0, 0, 0, 0, 500)
        }
        AddHandler _dispatcherTimer.Tick, AddressOf DispatcherTimer_Tick
    End Sub

    Private Function CanStartStopWriteBuffer(obj As Object) As Boolean
        Return IsInputValid
    End Function

    Private Sub StartStopWriteBuffer(obj As Object)
        If _isModuleIdle Then
            GenerateWaveForm()
        Else
            _aoModule.StopOutput()
        End If
    End Sub

    Private _isModuleIdle As Boolean = True
    Public Property IsModuleIdle As Boolean
        Get
            Return _isModuleIdle
        End Get
        Set
            If _isModuleIdle <> Value Then
                _isModuleIdle = Value

                ButtonContent = If(_isModuleIdle, "Start", "Stop")
                RaisePropertyChanged(NameOf(IsModuleIdle))
            End If
        End Set
    End Property

    Private _buttonContent As String = "Start"
    Public Property ButtonContent As String
        Get
            Return _buttonContent
        End Get
        Set
            _buttonContent = Value
            RaisePropertyChanged(NameOf(ButtonContent))
        End Set
    End Property

    Private Sub GenerateWaveForm()
        _waveForm.InitWaveForm(_aoModule.ActualOutputFrequency, _signalLength, _numberOfValues)
        For Each signalViewModel In SignalViewModels
            signalViewModel.ApplySignalGenerator(_waveForm)
        Next
        _aoModule.SetWaveForm(_waveForm)

        StartBackgroundWorker()
    End Sub

    Public Sub StopModule()
        If Not _isModuleIdle Then
            _aoModule.StopOutput()
        End If
    End Sub

    Private Sub StartBackgroundWorker()
        IsModuleIdle = False
        Messenger.Instance.Send(New ModuleStatusMessage(Me, False), AdqCommon.MessengerContextStatus)
        _aoModule.AdqError = AdqErrors.ADQ_ERROR_NONE
        AppendInfoStartWriting()
        _worker = New BackgroundWorker With {
            .WorkerReportsProgress = False,
            .WorkerSupportsCancellation = False
        }
        AddHandler _worker.DoWork, AddressOf Worker_DoWork
        AddHandler _worker.RunWorkerCompleted, AddressOf Worker_RunWorkerCompleted
        CommandManager.InvalidateRequerySuggested()
        _worker.RunWorkerAsync(_aoModule)
    End Sub

    Private Sub Worker_DoWork(ByVal sender As Object, ByVal e As DoWorkEventArgs)
        Dim adqModule As AoModule = CType(e.Argument, AoModule)
        adqModule.WriteBuffer()
        e.Result = adqModule
    End Sub

    Private Sub Worker_RunWorkerCompleted(sender As Object, e As RunWorkerCompletedEventArgs)
        AppendInfoValuesWritten()
        _dispatcherTimer.Start()
    End Sub

    Private Sub DispatcherTimer_Tick(sender As Object, e As EventArgs)
        CheckModuleStatus()
    End Sub

    Private Sub CheckModuleStatus()
        Dim isModuleRunning As Boolean = _aoModule.GetModuleStatus()
        If Not isModuleRunning Then
            _dispatcherTimer.Stop()
            IsModuleIdle = True
            AppendInfoOutputEnded()
            Messenger.Instance.Send(New ModuleStatusMessage(Me, True), AdqCommon.MessengerContextStatus)
            CommandManager.InvalidateRequerySuggested()
        End If
    End Sub

    Private Sub AppendInfoStartWriting()
        Dim s As String = String.Format("Board {0}, Module {1}, output started, frequency: {2}, values: {3}, repeat: {4}",
                                _aoModule.BoardIndex,
                                _aoModule.ModuleIndex,
                                ActualOutputFrequency,
                                _numberOfValues,
                                _repeatBufferCount)
        Messenger.Instance.Send(New InfoMessage(s), AdqCommon.MessengerContextInfo)
    End Sub

    Private Sub AppendInfoValuesWritten()
        Dim s As String = String.Format("Board {0}, Module {1}, values written to internal data buffer",
                                _aoModule.BoardIndex,
                                _aoModule.ModuleIndex)
        Messenger.Instance.Send(New InfoMessage(s), AdqCommon.MessengerContextInfo)
    End Sub

    Private Sub AppendInfoOutputEnded()
        Dim s As String = String.Format("Board {0}, Module {1}, output ended, number of values written: {2}",
                                _aoModule.BoardIndex,
                                _aoModule.ModuleIndex,
                                _aoModule.NumberOfValuesWritten)
        Messenger.Instance.Send(New InfoMessage(s), AdqCommon.MessengerContextInfo)
    End Sub

    Public ReadOnly Property AoRanges As List(Of AoRange)
        Get
            Return _aoModule.AoRanges
        End Get
    End Property

    Public Property SelectedAoRange As AoRange
        Get
            Return _aoModule.SelectedAoRange
        End Get
        Set
            If _aoModule.SelectedAoRange IsNot Value Then
                _aoModule.SelectedAoRange = Value
                RaisePropertyChanged(NameOf(SelectedAoRange))
            End If
        End Set
    End Property

    Public ReadOnly Property ModuleName As String
        Get
            Return _aoModule.ModuleName
        End Get
    End Property

    Public ReadOnly Property BackgroundColor As Brush

    Private _desiredOutputFrequency As String
    Public Property DesiredOutputFrequency As String
        Get
            Return _desiredOutputFrequency
        End Get
        Set
            If _desiredOutputFrequency <> Value Then
                _desiredOutputFrequency = Value
                Dim frequency As Double
                Dim msg As String = InputValidation.ValidateValueIsPositive(_desiredOutputFrequency, frequency)
                If String.IsNullOrEmpty(msg) Then
                    msg = _aoModule.ValidateDesiredOutputFrequency(frequency)
                End If
                UpdateErrorInfo(NameOf(DesiredOutputFrequency), msg)

                If String.IsNullOrEmpty(msg) Then
                    _aoModule.DesiredOutputFrequency = frequency
                End If

                RaisePropertyChanged(NameOf(DesiredOutputFrequency))
                StartStopCommand.RaiseCanExecuteChanged()
            End If
        End Set
    End Property

    Public ReadOnly Property ActualOutputFrequency As String
        Get
            Return AdqCommon.FormatFrequency(_aoModule.ActualOutputFrequency)
        End Get
    End Property

    Private _signalLength As Double
    Public Property SignalLength As Double
        Get
            Return _signalLength
        End Get
        Set
            If _signalLength <> Value Then
                _signalLength = Value
                UpdateNumberOfValues()
            End If
        End Set
    End Property

    Private _signalLengthText As String
    Public Property SignalLengthText As String
        Get
            Return _signalLengthText
        End Get
        Set
            If _signalLengthText <> Value Then
                _signalLengthText = Value
                Dim signalLen As Double
                Dim msg As String = InputValidation.ValidateValueIsPositive(_signalLengthText, signalLen)
                UpdateErrorInfo(NameOf(SignalLengthText), msg)

                If String.IsNullOrEmpty(msg) Then
                    SignalLength = signalLen
                End If

                RaisePropertyChanged(NameOf(SignalLengthText))
                StartStopCommand.RaiseCanExecuteChanged()
            End If
        End Set
    End Property

    Private Sub UpdateNumberOfValues()
        _numberOfValues = CUInt(Math.Round(_signalLength * _aoModule.ActualOutputFrequency))
        UpdateNumberOfValuesInfo()
        RaisePropertyChanged(NameOf(NumberOfValues))
    End Sub

    Private Sub UpdateNumberOfValuesInfo()
        Try
            _numberOfValuesInfo = String.Format(CultureInfo.CurrentCulture, "Total values: {0:N0}, duration {1}",
                _numberOfValues * _aoModule.RepeatBufferCount,
                AdqCommon.FormatTimeSpan(_signalLength * _aoModule.RepeatBufferCount))
        Catch ovrflwEx As OverflowException
            _numberOfValuesInfo = "Overflow"
        End Try
        RaisePropertyChanged(NameOf(NumberOfValuesInfo))
    End Sub

    Private _numberOfValues As UInteger
    Public ReadOnly Property NumberOfValues As UInteger
        Get
            Return _numberOfValues
        End Get
    End Property

    Private _numberOfValuesInfo As String
    Public ReadOnly Property NumberOfValuesInfo() As String
        Get
            Return _numberOfValuesInfo
        End Get
    End Property

    Private _repeatBufferCount As String
    Public Property RepeatBufferCount As String
        Get
            Return _repeatBufferCount
        End Get
        Set
            If _repeatBufferCount <> Value Then
                _repeatBufferCount = Value
                Dim count As UInteger
                Dim msg As String = ValidateRepeatBufferCount(_repeatBufferCount, count)
                UpdateErrorInfo(NameOf(RepeatBufferCount), msg)

                If String.IsNullOrEmpty(msg) Then
                    _aoModule.RepeatBufferCount = count
                    UpdateNumberOfValuesInfo()
                End If

                RaisePropertyChanged(NameOf(RepeatBufferCount))
                StartStopCommand.RaiseCanExecuteChanged()
            End If
        End Set
    End Property

    Public ReadOnly Property IsInputValid As Boolean
        Get
            Return _errorInfo.Count = 0 AndAlso GetSignalsInputValid()
        End Get
    End Property

    Private Function GetSignalsInputValid() As Boolean
        Dim isValid As Boolean = True
        For Each sg In SignalViewModels
            If Not sg.IsGeneratorInputValid Then
                isValid = False
                Exit For
            End If
        Next
        Return isValid
    End Function

    Private Sub UpdateErrorInfo(propertyName As String, msg As String)
        If String.IsNullOrEmpty(msg) Then
            _errorInfo.Remove(propertyName)
        Else
            _errorInfo.AddOrUpdate(propertyName, msg)
        End If
    End Sub

    Default Public ReadOnly Property Item(propertyName As String) As String Implements IDataErrorInfo.Item
        Get
            Dim value As String = String.Empty
            If _errorInfo.TryGetValue(propertyName, value) Then
                Return value
            End If
            Return Nothing
        End Get
    End Property

    Public ReadOnly Property [Error] As String Implements IDataErrorInfo.Error
        Get
            Return "AoModuleViewModel.Error"
        End Get
    End Property

    Private Function ValidateRepeatBufferCount(textValue As String, ByRef count As UInteger) As String
        count = 0
        Dim msg As String
        If String.IsNullOrEmpty(textValue) Then
            msg = "The value cannot be empty"
        Else
            If Not UInteger.TryParse(textValue, count) Then
                msg = "The entered text """ & textValue & """ is not a valid number"
            Else
                msg = _aoModule.ValidateRepeatBufferCount(count)
            End If
        End If
        Return msg
    End Function

    Private Sub InitSignalViewModels()
        SignalViewModels = New List(Of SignalViewModel)

        Dim signalVmSettings = New SignalViewModelSettings With {
            .SignalName = "Main signal",
            .IsEnabled = True
        }

        Dim signalSettings As SignalSettings = New SignalSettings With {
            .PhaseShift = 0.0,
            .SignalBeginFrequency = 10.0,
            .SignalEndFrequency = 50.0,
            .SignalBeginPhysicalMin = Math.Round(_aoModule.SelectedAoRange.PhysicalMin / 2),
            .SignalBeginPhysicalMax = 0,
            .SignalEndPhysicalMin = 0,
            .SignalEndPhysicalMax = Math.Round(_aoModule.SelectedAoRange.PhysicalMax / 2),
            .AoRangePhysicalMin = _aoModule.SelectedAoRange.PhysicalMin,
            .AoRangePhysicalMax = _aoModule.SelectedAoRange.PhysicalMax
        }

        Dim signalViewModel = New SignalViewModel(signalVmSettings, signalSettings)
        AddHandler signalViewModel.PropertyChanged, AddressOf AoModuleViewModel_PropertyChanged
        SignalViewModels.Add(signalViewModel)

        signalVmSettings = New SignalViewModelSettings With {
            .SignalName = "Superimposed signal",
            .IsEnabled = False
        }

        signalSettings = New SignalSettings With {
            .PhaseShift = 0.0,
            .SignalBeginFrequency = 50.0,
            .SignalEndFrequency = 100.0,
            .SignalBeginPhysicalMin = Math.Round(_aoModule.SelectedAoRange.PhysicalMin / 5),
            .SignalBeginPhysicalMax = 0,
            .SignalEndPhysicalMin = 0,
            .SignalEndPhysicalMax = Math.Round(_aoModule.SelectedAoRange.PhysicalMax / 5),
            .AoRangePhysicalMin = _aoModule.SelectedAoRange.PhysicalMin,
            .AoRangePhysicalMax = _aoModule.SelectedAoRange.PhysicalMax
        }

        signalViewModel = New SignalViewModel(signalVmSettings, signalSettings)
        AddHandler signalViewModel.PropertyChanged, AddressOf AoModuleViewModel_PropertyChanged
        SignalViewModels.Add(signalViewModel)
    End Sub

    Public Property SignalViewModels As List(Of SignalViewModel)

    Public Sub RaisePropertyChanged(ByVal propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub

    Private Sub AoModuleViewModel_PropertyChanged(sender As Object, e As PropertyChangedEventArgs)
        If e.PropertyName = NameOf(ActualOutputFrequency) Then
            RaisePropertyChanged(NameOf(ActualOutputFrequency))
            UpdateNumberOfValues()
        ElseIf e.PropertyName = NameOf(RepeatBufferCount) Then
            RaisePropertyChanged(NameOf(RepeatBufferCount))
        ElseIf e.PropertyName = NameOf(SignalViewModel.IsGeneratorInputValid) Then
            RaisePropertyChanged(NameOf(IsInputValid))
            StartStopCommand.RaiseCanExecuteChanged()
        ElseIf e.PropertyName = NameOf(AoModule.ModuleErrorText) Then
            Messenger.Instance.Send(New InfoMessage(_aoModule.ModuleErrorText), AdqCommon.MessengerContextInfo)
        ElseIf e.PropertyName = NameOf(AoModule.ModuleInfoText) Then
            Messenger.Instance.Send(New InfoMessage(_aoModule.ModuleInfoText), AdqCommon.MessengerContextInfo)
        End If
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
    End Sub

    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not _disposed Then
            _disposed = True
            RemoveHandler _aoModule.PropertyChanged, AddressOf AoModuleViewModel_PropertyChanged
        End If
    End Sub
End Class
