﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the WriteBufferAO project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.Collections.ObjectModel
Imports System.ComponentModel

Public Class MainViewModel
    Implements INotifyPropertyChanged

    Private ReadOnly _boardLocator As BoardLocator
    Private ReadOnly _runningModules As List(Of AoModuleViewModel) = New List(Of AoModuleViewModel)

    Private Event PropertyChanged As PropertyChangedEventHandler _
        Implements INotifyPropertyChanged.PropertyChanged

    Public Property CloseCommand As ICommand

    Public Sub New()
        _isBoardIdle = True
        InitCommands()
        InitMessages()
        _boardLocator = New BoardLocator(AddressOf MainViewModel_PropertyChanged)
        _boardLocator.InitBoardLocator()
        If _boardLocator.AdqAoBoards.Count = 1 Then
            SelectedBoard = _boardLocator.AdqAoBoards(0)
        End If
    End Sub

    Private Sub InitCommands()
        CloseCommand = New RelayCommand(AddressOf CloseApplication)
    End Sub

    Private Sub InitMessages()
        Messenger.Instance.Register(Of InfoMessage)(Me, AddressOf OnInfoMessageReceived, AdqCommon.MessengerContextInfo)
        Messenger.Instance.Register(Of ModuleStatusMessage)(Me, AddressOf OnModuleStatusMessageReceived, AdqCommon.MessengerContextStatus)
    End Sub

    Private Sub OnInfoMessageReceived(obj As InfoMessage)
        AppendInfoText(obj.InfoText)
    End Sub

    Private Sub OnModuleStatusMessageReceived(obj As ModuleStatusMessage)
        If obj.IsIdle Then
            _runningModules.Remove(obj.ModuleViewModel)
            If _runningModules.Count = 0 Then
                IsBoardIdle = True
            End If
        Else
            If _runningModules.Count = 0 Then
                IsBoardIdle = False
            End If
            _runningModules.Add(obj.ModuleViewModel)
        End If
    End Sub

    Private Sub StopRunningModules()
        For Each aoModuleVm In AoModuleViewModels
            aoModuleVm.StopModule()
        Next
    End Sub

    Private _isBoardIdle As Boolean
    Public Property IsBoardIdle As Boolean
        Get
            Return _isBoardIdle
        End Get
        Set
            If _isBoardIdle <> Value Then
                _isBoardIdle = Value
                RaisePropertyChanged(NameOf(IsBoardIdle))
            End If
        End Set
    End Property

    Public ReadOnly Property AdqAoBoards As List(Of AdqAoBoard)
        Get
            Return _boardLocator.AdqAoBoards
        End Get
    End Property

    Private _selectedBoard As AdqAoBoard
    Public Property SelectedBoard As AdqAoBoard
        Get
            Return _selectedBoard
        End Get
        Set
            If _selectedBoard IsNot Value Then
                AoModuleViewModels.Clear()
                _selectedBoard = Value
                If (_selectedBoard IsNot Nothing) Then
                    InitAoModuleViewModels(_selectedBoard)
                End If
                RaisePropertyChanged(NameOf(SelectedBoard))
            End If
        End Set
    End Property

    Private Sub InitAoModuleViewModels(aoBoard As AdqAoBoard)
        Dim idx As Integer = 0
        For Each analogModule In aoBoard.AoModules
            Dim vm As AoModuleViewModel = New AoModuleViewModel(analogModule, AdqCommon.BackgroundColors(idx Mod AdqCommon.BackgroundColors.Length))
            AoModuleViewModels.Add(vm)
            idx += 1
        Next
    End Sub

    Public ReadOnly Property AoModuleViewModels As ObservableCollection(Of AoModuleViewModel) = 
        New ObservableCollection(Of AoModuleViewModel)

    Private _infoText As String = String.Empty
    Public ReadOnly Property InfoText As String
        Get
            Return _infoText
        End Get
    End Property

    Private Sub AppendInfoText(text As String)
        If Not String.IsNullOrEmpty(_infoText) Then
            _infoText &= Environment.NewLine
        End If
        _infoText += text
        RaisePropertyChanged(NameOf(InfoText))
    End Sub

    Public Sub RaisePropertyChanged(ByVal propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub

    Private Sub CloseApplication(obj As Object)
        StopRunningModules()
        Windows.Application.Current.MainWindow.Close()
    End Sub

    Private Sub MainViewModel_PropertyChanged(sender As Object, e As PropertyChangedEventArgs)
        If e.PropertyName = NameOf(AoModule.DesiredOutputFrequency) Then
            RaisePropertyChanged(e.PropertyName)
        ElseIf e.PropertyName = NameOf(AoModule.ActualOutputFrequency) Then
            RaisePropertyChanged(e.PropertyName)
        ElseIf e.PropertyName = NameOf(BoardLocator.InfoText) Then
            AppendInfoText(_boardLocator.InfoText)
        End If
    End Sub
End Class
