﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the WriteBufferAO project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.ComponentModel

Public Class SignalViewModel
    Implements INotifyPropertyChanged, IDataErrorInfo

    Private ReadOnly _fmtStr As String = "{0:0.###}"
    Private ReadOnly _errorInfo As Dictionary(Of String, String) = New Dictionary(Of String, String)
    Private _signalGenerators As List(Of ISignalGenerator)

    Private ReadOnly _signalViewModelSettings As SignalViewModelSettings
    Private ReadOnly _signalSettings As SignalSettings

    Public Property SetFrequencyEnd As ICommand
    Public Property SetAmplitudeMinEnd As ICommand
    Public Property SetAmplitudeMaxEnd As ICommand

    Public Event PropertyChanged As PropertyChangedEventHandler _
        Implements INotifyPropertyChanged.PropertyChanged

    Public Sub New(signalViewModelSettings As SignalViewModelSettings, signalSettings As SignalSettings)
        _signalViewModelSettings = signalViewModelSettings
        _signalSettings = signalSettings

        _isEnabled = _signalViewModelSettings.IsEnabled
        _phaseShift = _signalSettings.PhaseShift
        _phaseAngleText = String.Format(_fmtStr, _phaseShift * 360.0)
        _signalBeginFrequency = _signalSettings.SignalBeginFrequency
        _signalBeginFrequencyText = String.Format(_fmtStr, _signalBeginFrequency)
        _signalEndFrequency = _signalSettings.SignalEndFrequency
        _signalEndFrequencyText = String.Format(_fmtStr, _signalEndFrequency)
        _signalBeginAmplitudeMin = _signalSettings.SignalBeginPhysicalMin
        _signalBeginAmplitudeMinText = String.Format(_fmtStr, _signalBeginAmplitudeMin)
        _signalBeginAmplitudeMax = _signalSettings.SignalBeginPhysicalMax
        _signalBeginAmplitudeMaxText = String.Format(_fmtStr, _signalBeginAmplitudeMax)
        _signalEndAmplitudeMin = _signalSettings.SignalEndPhysicalMin
        _signalEndAmplitudeMinText = String.Format(_fmtStr, _signalEndAmplitudeMin)
        _signalEndAmplitudeMax = _signalSettings.SignalEndPhysicalMax
        _signalEndAmplitudeMaxText = String.Format(_fmtStr, _signalEndAmplitudeMax)
        InitSignalGenerators()
        InitCommands()
    End Sub

    Private Sub InitCommands()
        SetFrequencyEnd = New RelayCommand(AddressOf SetFrequencyEndExecute, AddressOf CanSetFrequencyEndExecute)
        SetAmplitudeMinEnd = New RelayCommand(AddressOf SetAmplitudeMinEndExecute, AddressOf CanSetAmplitudeMinEndExecute)
        SetAmplitudeMaxEnd = New RelayCommand(AddressOf SetAmplitudeMaxEndExecute, AddressOf CanSetAmplitudeMaxEndExecute)
    End Sub

    Private Function CanSetFrequencyEndExecute(obj As Object) As Boolean
        Return _isEnabled
    End Function

    Private Sub SetFrequencyEndExecute(obj As Object)
        SignalEndFrequencyText = _signalBeginFrequencyText
    End Sub

    Private Function CanSetAmplitudeMinEndExecute(obj As Object) As Boolean
        Return _isEnabled
    End Function

    Private Sub SetAmplitudeMinEndExecute(obj As Object)
        SignalEndAmplitudeMinText = _signalBeginAmplitudeMinText
    End Sub

    Private Function CanSetAmplitudeMaxEndExecute(obj As Object) As Boolean
        Return _isEnabled
    End Function

    Private Sub SetAmplitudeMaxEndExecute(obj As Object)
        SignalEndAmplitudeMaxText = _signalBeginAmplitudeMaxText
    End Sub

    Private Sub InitSignalGenerators()
        _signalGenerators = New List(Of ISignalGenerator) From {
            New SineWaveGenerator(),
            New SquareWaveGenerator(),
            New TriangleWaveGenerator(),
            New SawtoothFallingWaveGenerator(),
            New SawtoothRisingWaveGenerator(),
            New SincWaveGenerator(),
            New SineCutWaveGenerator(),
            New SineModSquareWaveGenerator(),
            New ChargeDischargeWaveGenerator(),
            New MexicanHatWaveGenerator(),
            New MorletWaveletGenerator(),
            New ShannonWaveletGenerator(),
            New PoissonWaveletGenerator(),
            New RandomRectangleWaveGenerator(),
            New NonReturnToZeroWaveGenerator(),
            New WhiteNoiseSignalGenerator(),
            New GaussianNoiseSignalGenerator()
        }

        _selectedSignalGenerator = _signalGenerators(0)
    End Sub

    Public ReadOnly Property SignalGenerators As List(Of ISignalGenerator)
        Get
            Return _signalGenerators
        End Get
    End Property

    Private _selectedSignalGenerator As ISignalGenerator
    Public Property SelectedSignalGenerator As ISignalGenerator
        Get
            Return _selectedSignalGenerator
        End Get

        Set
            If _selectedSignalGenerator IsNot Value Then
                _selectedSignalGenerator = Value
                RaisePropertyChanged(NameOf(SelectedSignalGenerator))
            End If
        End Set
    End Property

    Sub ApplySignalGenerator(waveForm As WaveForm)
        If _isEnabled Then
            Dim signalSettings As SignalSettings = New SignalSettings With {
                .PhaseShift = _phaseShift,
                .SignalBeginFrequency = _signalBeginFrequency,
                .SignalBeginPhysicalMin = _signalBeginAmplitudeMin,
                .SignalBeginPhysicalMax = _signalBeginAmplitudeMax,
                .SignalEndFrequency = _signalEndFrequency,
                .SignalEndPhysicalMin = _signalEndAmplitudeMin,
                .SignalEndPhysicalMax = _signalEndAmplitudeMax,
                .AoRangePhysicalMin = _signalSettings.AoRangePhysicalMin,
                .AoRangePhysicalMax = _signalSettings.AoRangePhysicalMax
            }

            waveForm.ApplySignalGenerator(_selectedSignalGenerator, signalSettings)
        End If
    End Sub

    Private _isEnabled As Boolean
    Public Property IsEnabled As Boolean
        Get
            Return _isEnabled
        End Get
        Set
            If _isEnabled <> Value Then
                _isEnabled = Value
                RaisePropertyChanged(NameOf(IsEnabled))
            End If
        End Set
    End Property

    Public ReadOnly Property SignalName As String
        Get
            Return _signalViewModelSettings.SignalName
        End Get
    End Property

    Private _phaseShift As Double
    Private _phaseAngleText As String
    Public Property PhaseAngleText As String
        Get
            Return _phaseAngleText
        End Get
        Set
            If _phaseAngleText <> Value Then
                _phaseAngleText = Value
                Dim phaseAngle As Double
                Dim msg As String = InputValidation.ValidateValueIsWithinRange(_phaseAngleText, 0.0, 360.0, phaseAngle)
                If String.IsNullOrEmpty(msg) Then
                    _phaseShift = phaseAngle / 360.0
                End If
                UpdateErrorInfo(NameOf(PhaseAngleText), msg)
                RaisePropertyChanged(NameOf(PhaseAngleText))
                RaisePropertyChanged(NameOf(IsGeneratorInputValid))
            End If
        End Set
    End Property

    Private _signalBeginFrequency As Double
    Private _signalBeginFrequencyText As String
    Public Property SignalBeginFrequencyText As String
        Get
            Return _signalBeginFrequencyText
        End Get
        Set
            If _signalBeginFrequencyText <> Value Then
                _signalBeginFrequencyText = Value

                Dim msg As String = InputValidation.ValidateValueIsPositive(_signalBeginFrequencyText, _signalBeginFrequency)
                UpdateErrorInfo(NameOf(SignalBeginFrequencyText), msg)
                RaisePropertyChanged(NameOf(SignalBeginFrequencyText))
                RaisePropertyChanged(NameOf(IsGeneratorInputValid))
            End If
        End Set
    End Property

    Private _signalEndFrequency As Double
    Private _signalEndFrequencyText As String
    Public Property SignalEndFrequencyText As String
        Get
            Return _signalEndFrequencyText
        End Get
        Set
            If _signalEndFrequencyText <> Value Then
                _signalEndFrequencyText = Value

                Dim msg As String = InputValidation.ValidateValueIsPositive(_signalEndFrequencyText, _signalEndFrequency)
                UpdateErrorInfo(NameOf(SignalEndFrequencyText), msg)
                RaisePropertyChanged(NameOf(SignalEndFrequencyText))
                RaisePropertyChanged(NameOf(IsGeneratorInputValid))
            End If
        End Set
    End Property

    Private _signalBeginAmplitudeMin As Double
    Private _signalBeginAmplitudeMinText As String
    Public Property SignalBeginAmplitudeMinText As String
        Get
            Return _signalBeginAmplitudeMinText
        End Get
        Set
            If _signalBeginAmplitudeMinText <> Value Then
                _signalBeginAmplitudeMinText = Value
                Dim msg As String = InputValidation.ValidateValueIsWithinRange(_signalBeginAmplitudeMinText,
                                                                              _signalSettings.AoRangePhysicalMin,
                                                                              _signalSettings.AoRangePhysicalMax,
                                                                              _signalBeginAmplitudeMin)
                UpdateErrorInfo(NameOf(SignalBeginAmplitudeMinText), msg)
                RaisePropertyChanged(NameOf(SignalBeginAmplitudeMinText))
                RaisePropertyChanged(NameOf(IsGeneratorInputValid))
            End If
        End Set
    End Property

    Private _signalBeginAmplitudeMax As Double
    Private _signalBeginAmplitudeMaxText As String
    Public Property SignalBeginAmplitudeMaxText As String
        Get
            Return _signalBeginAmplitudeMaxText
        End Get
        Set
            If _signalBeginAmplitudeMaxText <> Value Then
                _signalBeginAmplitudeMaxText = Value

                Dim msg As String = InputValidation.ValidateValueIsWithinRange(_signalBeginAmplitudeMaxText,
                                                                             _signalSettings.AoRangePhysicalMin,
                                                                             _signalSettings.AoRangePhysicalMax,
                                                                             _signalBeginAmplitudeMax)
                UpdateErrorInfo(NameOf(SignalBeginAmplitudeMaxText), msg)
                RaisePropertyChanged(NameOf(SignalBeginAmplitudeMaxText))
                RaisePropertyChanged(NameOf(IsGeneratorInputValid))
            End If
        End Set
    End Property

    Private _signalEndAmplitudeMin As Double
    Private _signalEndAmplitudeMinText As String
    Public Property SignalEndAmplitudeMinText As String
        Get
            Return _signalEndAmplitudeMinText
        End Get
        Set
            If _signalEndAmplitudeMinText <> Value Then
                _signalEndAmplitudeMinText = Value

                Dim msg As String = InputValidation.ValidateValueIsWithinRange(_signalEndAmplitudeMinText,
                                                                             _signalSettings.AoRangePhysicalMin,
                                                                             _signalSettings.AoRangePhysicalMax,
                                                                             _signalEndAmplitudeMin)
                UpdateErrorInfo(NameOf(SignalEndAmplitudeMinText), msg)
                RaisePropertyChanged(NameOf(SignalEndAmplitudeMinText))
                RaisePropertyChanged(NameOf(IsGeneratorInputValid))
            End If
        End Set
    End Property

    Private _signalEndAmplitudeMax As Double
    Private _signalEndAmplitudeMaxText As String
    Public Property SignalEndAmplitudeMaxText As String
        Get
            Return _signalEndAmplitudeMaxText
        End Get
        Set
            If _signalEndAmplitudeMaxText <> Value Then
                _signalEndAmplitudeMaxText = Value

                Dim msg As String = InputValidation.ValidateValueIsWithinRange(_signalEndAmplitudeMaxText,
                                                                             _signalSettings.AoRangePhysicalMin,
                                                                             _signalSettings.AoRangePhysicalMax,
                                                                             _signalEndAmplitudeMax)
                UpdateErrorInfo(NameOf(SignalEndAmplitudeMaxText), msg)
                RaisePropertyChanged(NameOf(SignalEndAmplitudeMaxText))
                RaisePropertyChanged(NameOf(IsGeneratorInputValid))
            End If
        End Set
    End Property

    Private Sub UpdateErrorInfo(propertyName As String, msg As String)
        If String.IsNullOrEmpty(msg) Then
            _errorInfo.Remove(propertyName)
        Else
            _errorInfo.AddOrUpdate(propertyName, msg)
        End If
    End Sub

    Public ReadOnly Property IsGeneratorInputValid As Boolean
        Get
            Return _errorInfo.Count = 0
        End Get
    End Property

    Default Public ReadOnly Property Item(propertyName As String) As String Implements IDataErrorInfo.Item
        Get
            Dim value As String = String.Empty
            If _errorInfo.TryGetValue(propertyName, value) Then
                Return value
            End If
            Return Nothing
        End Get
    End Property

    Public ReadOnly Property [Error] As String Implements IDataErrorInfo.Error
        Get
            Return "SignalViewModel.Error"
        End Get
    End Property

    Public Sub RaisePropertyChanged(ByVal propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub
End Class
