﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the WriteBufferDO project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.ComponentModel
Imports System.Runtime.InteropServices
Imports WriteBufferDO.ALLDAQ.Boards

Public Class DigitalModule
    Implements INotifyPropertyChanged

    Public Values As UInteger()
    Public Property NumberOfValuesWritten As UInteger

    Public Event PropertyChanged As PropertyChangedEventHandler _
        Implements INotifyPropertyChanged.PropertyChanged

    Public Sub New(boardName As String, boardIndex As UInteger, moduleIndex As UInteger,
                   ByRef moduleInfo As AdqDriver.ADQGeneralModuleInfo, portWidthBits As UInteger)
        _numberOfValues = portWidthBits
        Me.BoardIndex = boardIndex
        Me.ModuleIndex = moduleIndex
        ModuleType = moduleInfo.ModuleType
        ClockType = moduleInfo.ClockType
        Me.PortWidthBits = portWidthBits
        MaxfrequencyHz = moduleInfo.ClockBaseFrequencyHz / moduleInfo.ClockMinimumDivider
        _desiredOutputFrequency = MaxfrequencyHz
        _actualOutputFrequency = MaxfrequencyHz
        ModuleName = GetModuleName(boardName)
    End Sub

    Public Property AdqError As UInteger
    Public ReadOnly Property ModuleName As String
    Public ReadOnly Property BoardIndex As UInteger
    Public ReadOnly Property ModuleIndex As UInteger
    Public ReadOnly Property ModuleType As UInteger
    Public ReadOnly Property ClockType As UInteger
    Public ReadOnly Property PortWidthBits As UInteger
    Public ReadOnly Property MaxfrequencyHz As Double

    Dim _desiredOutputFrequency As Double
    Public Property DesiredOutputFrequency As Double
        Get
            Return _desiredOutputFrequency
        End Get
        Set(ByVal value As Double)
            _desiredOutputFrequency = value
            NotifyPropertyChanged(NameOf(DesiredOutputFrequency))
            SetOutputFrequency()
        End Set
    End Property

    Dim _actualOutputFrequency As Double
    Public ReadOnly Property ActualOutputFrequency As Double
        Get
            Return _actualOutputFrequency
        End Get
    End Property

    Private Function SetOutputFrequency() As Boolean
        Dim actualFrequency As Double = 0.0
        If _desiredOutputFrequency > 0.0 Then

            Dim adqError As UInteger = AdqDriver.adqSetFrequency(BoardIndex, ModuleIndex, _desiredOutputFrequency, actualFrequency)
            If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                SetErrorText("adqSetFrequency", adqError)
                actualFrequency = 0.0
            End If
        End If
        SetActualOutputFrequency(actualFrequency)
        Return actualFrequency > 0.0
    End Function

    Private Sub SetActualOutputFrequency(value As Double)
        If _actualOutputFrequency <> value Then
            _actualOutputFrequency = value
            NotifyPropertyChanged(NameOf(ActualOutputFrequency))
        End If
    End Sub

    Public Function GetSecondsToWait() As Double
        If _actualOutputFrequency <= 0.0 Then
            Return 0.0
        End If
        Return _numberOfValues / _actualOutputFrequency
    End Function

    Dim _numberOfValues As UInteger
    Public Property NumberOfValues As UInteger
        Get
            Return _numberOfValues
        End Get
        Set(ByVal value As UInteger)
            If value <> _numberOfValues Then
                _numberOfValues = value
                NotifyPropertyChanged(NameOf(NumberOfValues))
            End If
        End Set
    End Property

    Dim _repeatBufferCount As UInteger = AdqCommon.DefaultRepeatBufferCount
    Public Property RepeatBufferCount As UInteger
        Get
            Return _repeatBufferCount
        End Get
        Set(ByVal value As UInteger)
            If value <> _repeatBufferCount Then
                _repeatBufferCount = value
                NotifyPropertyChanged(NameOf(RepeatBufferCount))
            End If
        End Set
    End Property

    Private Function GetModuleName(boardName As String) As String
        Return String.Format("{0}, Board {1}, Module {2}, Type {3}, {4} Bit, Max. {5}",
            AdqCommon.GetModelName(boardName),
            BoardIndex,
            ModuleIndex,
            AdqCommon.GetModuleTypeName(ModuleType),
            PortWidthBits,
            AdqCommon.FormatFrequency(MaxfrequencyHz))
    End Function

    Public Sub ResetModule()
        Dim adqError As UInteger = AdqDriver.adqResetModule(BoardIndex, ModuleIndex)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetErrorText("adqResetModule", adqError)
        Else
            ModuleInfoText = String.Format("Reset board {0}, module {1}", BoardIndex, ModuleIndex)
        End If
    End Sub

    Public Function ConfigureModule() As Boolean
        If Not SetOutputFrequency() Then Return False

        Return ConfigureDioModuleToOutput()
    End Function

    Public Sub WriteBuffer()
        NumberOfValuesWritten = 0
        Dim numValuesWritten As UInteger
        Dim gch As GCHandle = GCHandle.Alloc(Values, GCHandleType.Pinned)
        AdqError = AdqDriver.adqWriteBuffer(BoardIndex, ModuleIndex,
                NumberOfValues, Values, _repeatBufferCount, numValuesWritten)
        NumberOfValuesWritten = numValuesWritten
        gch.Free()
    End Sub

    Private Function ConfigureDioModuleToOutput() As Boolean
        If ModuleType = AdqConstants.ADQ_MODULE_TYPE_DIO Then
            Dim adqError As UInteger = AdqDriver.adqDIOSetDirection(BoardIndex, ModuleIndex, AdqConstants.ADQ_DIO_OUTPUT)
            If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                SetErrorText("adqDIOSetDirection", adqError)
                Return False
            End If
        End If
        Return True
    End Function

    Public Function GetModuleStatus() As Boolean
        Dim numAccept As UInteger
        Dim isModuleRunning As Boolean
        Dim adqError As UInteger = AdqDriver.adqWriteContinuousStatus(BoardIndex, ModuleIndex, False, numAccept, isModuleRunning)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetErrorText("adqWriteContinuousStatus", adqError)
        End If
        Return isModuleRunning
    End Function

    Public Function StopOutput() As UInteger
        Return AdqDriver.adqStop(BoardIndex, ModuleIndex)
    End Function

    Public Function ValidateDesiredOutputFrequency(frequency As Double) As String
        Dim msg As String = String.Empty
        If frequency <= 0.0 Then
            msg = "The output frequency must be greater than 0"
        End If
        Return msg
    End Function

    Public Function ValidateNumberOfValues(numValues As UInteger) As String
        Return ValidateValueIsPositive(numValues)
    End Function

    Public Function ValidateRepeatBufferCount(count As UInteger) As String
        Return ValidateValueIsPositive(count)
    End Function

    Private Function ValidateValueIsPositive(value As UInteger) As String
        Dim msg As String = String.Empty
        If value = 0 Then
            msg = "The number must be greater than 0"
        End If
        Return msg
    End Function

    Private Sub SetErrorText(functionName As String, adqError As UInteger)
        ModuleInfoText = AdqCommon.GetFunctionErrorText(functionName, adqError)
    End Sub

    Dim _infoText As String = String.Empty
    Public Property ModuleInfoText As String
        Get
            Return _infoText
        End Get
        Set(ByVal value As String)
            _infoText = value
            If _infoText <> String.Empty Then
                NotifyPropertyChanged(NameOf(ModuleInfoText))
            End If
        End Set
    End Property

    Public Sub NotifyPropertyChanged(ByVal propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub
End Class
