﻿'-----------------------------------------------------------------------------
' adqDevice.vb - Constants, types and function prototypes
'      Copyright (c) ALLDAQ, a division of ALLNET GmbH
'      https://www.alldaq.com
'
' Purpose:
'      This file has defines for constants, types and function prototypes
'      used by ALLDAQ measurement boards
' Class AdqDriver:
'      types used by unmanaged library
'      import function prototypes
' Class AdqConstants:
'      constants used by the API
' Class AdqErrors:
'      error codes returned by the API functions
'
' Note:
'      Native import library 32bit: adqDevice.dll
'                            64bit: adqDevice_64.dll
'-----------------------------------------------------------------------------

Option Strict Off
Option Explicit On

Imports System.Runtime.InteropServices

Namespace ALLDAQ.Boards

    Public NotInheritable Class AdqDriver

#If WIN64 Then
        Public Const ADQDEVICE_DLL As String = "adqDevice_64.dll"
#Else
        Public Const ADQDEVICE_DLL As String = "adqDevice.dll"
#End If

#Region "Types"
        '---------------------------------------------------------------------
        '  ADQ System information
        '---------------------------------------------------------------------
        <StructLayout(LayoutKind.Sequential, Pack:=4)>
        Public Structure ADQSystemInfo
            Public Size As UInteger
            Public ID As UInteger
            Public DLLVersion As UInteger
            Public NumberOfBoards As UInteger

            Public Sub New(ByVal ID As UInteger)
                Size = Marshal.SizeOf(GetType(ADQSystemInfo))
                Me.ID = ID
                DLLVersion = 0
                NumberOfBoards = 0
            End Sub
        End Structure

        Public Const ADQSystemInfo_ID As UInteger = &H10000

        Public Shared Function SizeOfSystemInfo() As UInteger
            Return Marshal.SizeOf(GetType(ADQSystemInfo))
        End Function


        '---------------------------------------------------------------------
        '  Board information
        '---------------------------------------------------------------------
        <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Unicode, Pack:=4)>
        Public Structure ADQBoardInfo
            Public Size As UInteger
            Public ID As UInteger
            Public VendorID As UShort
            Public DeviceID As UShort
            Public SerialNumber As UInteger
            <MarshalAs(UnmanagedType.ByValTStr, SizeConst:=64)>
            Public BoardName As String
            <MarshalAs(UnmanagedType.ByValTStr, SizeConst:=256)>
            Public BoardDescription As String
            <MarshalAs(UnmanagedType.ByValTStr, SizeConst:=16)>
            Public DriverName As String
            Public DriverVersion As UInteger
            Public TTLVoltageLevel As UInteger
            Public BusType As UInteger
            <MarshalAs(UnmanagedType.U1)>
            Public Remote As Boolean
            <MarshalAs(UnmanagedType.ByValTStr, SizeConst:=256)>
            Public RemoteIPAddressOrHostName As String
            Public FirmwareVersion As UInteger
            Public NumberOfModules As UInteger

            Public Sub New(ByVal ID As UInteger)
                Size = Marshal.SizeOf(GetType(ADQBoardInfo))
                Me.ID = ID
                VendorID = 0
                DeviceID = 0
                SerialNumber = 0
                BoardName = ""
                BoardDescription = ""
                DriverName = ""
                DriverVersion = 0
                TTLVoltageLevel = 0
                BusType = 0
                Remote = False
                RemoteIPAddressOrHostName = ""
                FirmwareVersion = 0
                NumberOfModules = 0
            End Sub
        End Structure

        Public Const ADQBoardInfo_ID As UInteger = &H20001

        Public Shared Function SizeOfBoardInfo() As UInteger
            Return Marshal.SizeOf(GetType(ADQBoardInfo))
        End Function


        '---------------------------------------------------------------------
        '  Board PCI information structure, passed to adqGetBoardPCIInfo
        '---------------------------------------------------------------------
        <StructLayout(LayoutKind.Sequential, Pack:=4)>
        Public Structure ADQBoardPCIInfo
            Public Size As UInteger
            Public ID As UInteger
            Public BusNumber As UInteger
            Public DeviceNumber As UInteger
            Public FunctionNumber As UInteger

            Public Sub New(ByVal ID As UInteger)
                Size = Marshal.SizeOf(GetType(ADQBoardPCIInfo))
                Me.ID = ID
                BusNumber = 0
                DeviceNumber = 0
                FunctionNumber = 0
            End Sub
        End Structure

        Public Const ADQBoardPCIInfo_ID As UInteger = &HD0000

        Public Shared Function SizeOfBoardPCIInfo() As UInteger
            Return Marshal.SizeOf(GetType(ADQBoardPCIInfo))
        End Function


        '---------------------------------------------------------------------
        '  General module information
        '---------------------------------------------------------------------
        <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Unicode, Pack:=4)>
        Public Structure ADQGeneralModuleInfo
            Public Size As UInteger
            Public ID As UInteger
            <MarshalAs(UnmanagedType.ByValTStr, SizeConst:=1024)>
            Public ModuleDescription As String
            Public ModuleType As UInteger
            <MarshalAs(UnmanagedType.U1)>
            Public IsIRQSource As Boolean
            Public ClockType As UInteger
            Public ClockBaseFrequencyHz As Double
            Public ClockMinimumDivider As UInteger
            Public InternalDataBufferSizeValues As UInteger

            Public Sub New(ByVal ID As UInteger)
                Size = Marshal.SizeOf(GetType(ADQGeneralModuleInfo))
                Me.ID = ID
                ModuleDescription = ""
                ModuleType = 0
                IsIRQSource = False
                ClockType = AdqConstants.ADQ_CLOCK_TYPE_NONE
                ClockBaseFrequencyHz = 0
                ClockMinimumDivider = 0
                InternalDataBufferSizeValues = 0
            End Sub
        End Structure

        Public Const ADQGeneralModuleInfo_ID As UInteger = &H30001

        Public Shared Function SizeOfGeneralModuleInfo() As UInteger
            Return Marshal.SizeOf(GetType(ADQGeneralModuleInfo))
        End Function


        '---------------------------------------------------------------------
        '  Structures for trigger routines
        '---------------------------------------------------------------------
        <StructLayout(LayoutKind.Sequential, Pack:=4)>
        Public Structure ADQHWTriggerInfo
            Public Size As UInteger
            Public ID As UInteger
            Public HWTriggerStartSource As UInteger
            Public HWTriggerStopSource As UInteger
            <MarshalAs(UnmanagedType.U1)>
            Public SupportsChannelListPerTrigger As Boolean

            Public Sub New(ByVal ID As UInteger)
                Size = Marshal.SizeOf(GetType(ADQHWTriggerInfo))
                Me.ID = ID
                HWTriggerStartSource = 0
                HWTriggerStopSource = 0
                SupportsChannelListPerTrigger = False
            End Sub
        End Structure

        Public Const ADQHWTriggerInfo_ID As UInteger = &HB0000

        Public Shared Function SizeOfHWTriggerInfo() As UInteger
            Return Marshal.SizeOf(GetType(ADQHWTriggerInfo))
        End Function


        <StructLayout(LayoutKind.Sequential, Pack:=4)>
        Public Structure ADQHWTriggerEnable
            Public Size As UInteger
            Public ID As UInteger
            Public HWTriggerStartRisingEdge As UInteger
            Public HWTriggerStartFallingEdge As UInteger
            Public HWTriggerStopRisingEdge As UInteger
            Public HWTriggerStopFallingEdge As UInteger
            <MarshalAs(UnmanagedType.U1)>
            Public ChannelListPerTrigger As Boolean
            <MarshalAs(UnmanagedType.U1)>
            Public AutomaticRearm As Boolean

            Public Sub New(ByVal ID As UInteger)
                Size = Marshal.SizeOf(GetType(ADQHWTriggerEnable))
                Me.ID = ID
                HWTriggerStartRisingEdge = 0
                HWTriggerStartFallingEdge = 0
                HWTriggerStopRisingEdge = 0
                HWTriggerStopFallingEdge = 0
                ChannelListPerTrigger = False
                AutomaticRearm = False
            End Sub
        End Structure

        Public Const ADQHWTriggerEnable_ID As UInteger = &HC0000

        Public Shared Function SizeOfHWTriggerEnable() As UInteger
            Return Marshal.SizeOf(GetType(ADQHWTriggerEnable))
        End Function


        '---------------------------------------------------------------------
        '  Module information for DI, DIO and DO modules
        '---------------------------------------------------------------------
        Public Const ADQ_MAX_DI_FILTERS As UInteger = 4

        <StructLayout(LayoutKind.Sequential, Pack:=4)>
        Public Structure ADQDIModuleInfo
            Public Size As UInteger
            Public ID As UInteger
            Public PortWidthBits As UInteger
            <MarshalAs(UnmanagedType.U1)>
            Public IsOptoIsolated As Boolean
            <MarshalAs(UnmanagedType.U1)>
            Public IsTTL As Boolean
            Public NumberOfFilters As UInteger
            <MarshalAs(UnmanagedType.ByValArray, SizeConst:=ADQ_MAX_DI_FILTERS)>
            Public FilterTimePicoseconds As ULong()

            Public Sub New(ByVal ID As UInteger)
                Size = Marshal.SizeOf(GetType(ADQDIModuleInfo))
                Me.ID = ID
                PortWidthBits = 0
                IsOptoIsolated = False
                IsTTL = False
                NumberOfFilters = 0
                Dim FilterTimePicoseconds(ADQ_MAX_DI_FILTERS) As ULong
            End Sub
        End Structure

        Public Const ADQDIModuleInfo_ID As UInteger = &H40001

        Public Shared Function SizeOfDIModuleInfo() As UInteger
            Return Marshal.SizeOf(GetType(ADQDIModuleInfo))
        End Function


        <StructLayout(LayoutKind.Sequential, Pack:=4)>
        Public Structure ADQDOModuleInfo
            Public Size As UInteger
            Public ID As UInteger
            Public PortWidthBits As UInteger
            <MarshalAs(UnmanagedType.U1)>
            Public IsOptoIsolated As Boolean
            <MarshalAs(UnmanagedType.U1)>
            Public IsTTL As Boolean

            Public Sub New(ByVal ID As UInteger)
                Size = Marshal.SizeOf(GetType(ADQDOModuleInfo))
                Me.ID = ID
                PortWidthBits = 0
                IsOptoIsolated = False
                IsTTL = False
            End Sub
        End Structure

        Public Const ADQDOModuleInfo_ID As UInteger = &H40000

        Public Shared Function SizeOfDOModuleInfo() As UInteger
            Return Marshal.SizeOf(GetType(ADQDOModuleInfo))
        End Function


        <StructLayout(LayoutKind.Sequential, Pack:=4)>
        Public Structure ADQDIOModuleInfo
            Public Size As UInteger
            Public ID As UInteger
            Public PortWidthBits As UInteger
            <MarshalAs(UnmanagedType.U1)>
            Public IsOptoIsolated As Boolean
            <MarshalAs(UnmanagedType.U1)>
            Public IsTTL As Boolean

            Public Sub New(ByVal ID As UInteger)
                Size = Marshal.SizeOf(GetType(ADQDIOModuleInfo))
                Me.ID = ID
                PortWidthBits = 0
                IsOptoIsolated = False
                IsTTL = False
            End Sub
        End Structure

        Public Const ADQDIOModuleInfo_ID As UInteger = &H40000

        Public Shared Function SizeOfDIOModuleInfo() As UInteger
            Return Marshal.SizeOf(GetType(ADQDIOModuleInfo))
        End Function


        '---------------------------------------------------------------------
        '  Module information for Counter modules
        '---------------------------------------------------------------------
        <StructLayout(LayoutKind.Sequential, Pack:=4)>
        Public Structure ADQCNTRModuleInfo
            Public Size As UInteger
            Public ID As UInteger
            Public CounterType As UInteger
            Public WidthBits As UInteger
            Public ClockSources As UInteger

            Public Sub New(ByVal ID As UInteger)
                Size = Marshal.SizeOf(GetType(ADQCNTRModuleInfo))
                Me.ID = ID
                CounterType = 0
                WidthBits = 0
                ClockSources = 0
            End Sub
        End Structure

        Public Const ADQCNTRModuleInfo_ID As UInteger = &H50000

        Public Shared Function SizeOfADQCNTRModuleInfo() As UInteger
            Return Marshal.SizeOf(GetType(ADQCNTRModuleInfo))
        End Function


        '---------------------------------------------------------------------
        '  Module information for Analog input modules
        '---------------------------------------------------------------------
        <StructLayout(LayoutKind.Sequential, Pack:=4)>
        Public Structure ADQAIRange
            Public DigitalMin As UInteger
            Public DigitalMax As UInteger
            Public PhysicalUnit As UInteger
            Public PhysicalMin As Double
            Public PhysicalMax As Double
        End Structure

        Public Const ADQ_MAX_AI_RANGES As UInteger = 8


        <StructLayout(LayoutKind.Sequential, Pack:=4)>
        Public Structure ADQAIModuleInfo
            Public Size As UInteger
            Public ID As UInteger
            Public NumberOfChannels As UInteger
            <MarshalAs(UnmanagedType.U1)>
            Public SharedDataRate As Boolean
            <MarshalAs(UnmanagedType.U1)>
            Public RangeInfoPerChannel As Boolean
            Public NumberOfRanges As UInteger
            <MarshalAs(UnmanagedType.ByValArray, SizeConst:=ADQ_MAX_AI_RANGES)>
            Public Ranges() As ADQAIRange

            Public Sub New(ByVal ID As UInteger)
                Size = Marshal.SizeOf(GetType(ADQAIModuleInfo))
                Me.ID = ID
                NumberOfChannels = 0
                SharedDataRate = False
                RangeInfoPerChannel = False
                NumberOfRanges = 0
                Dim Ranges(ADQ_MAX_AI_RANGES) As ADQAIRange
            End Sub
        End Structure

        Public Const ADQAIModuleInfo_ID As UInteger = &H60000

        Public Shared Function SizeOfADQAIModuleInfo() As UInteger
            Return Marshal.SizeOf(GetType(ADQAIModuleInfo))
        End Function


        <StructLayout(LayoutKind.Sequential, Pack:=4)>
        Public Structure ADQAIChannel
            <MarshalAs(UnmanagedType.U1)>
            Public acquireChannel As Boolean
            Public rangeIndex As UInteger
        End Structure


        '---------------------------------------------------------------------
        '  Module information for Analog output modules
        '---------------------------------------------------------------------
        <StructLayout(LayoutKind.Sequential, Pack:=4)>
        Public Structure ADQAORange
            Public DigitalMin As UInteger
            Public DigitalMax As UInteger
            Public PhysicalUnit As UInteger
            Public PhysicalMin As Double
            Public PhysicalMax As Double
        End Structure

        Public Const ADQ_MAX_AO_RANGES As UInteger = 8

        <StructLayout(LayoutKind.Sequential, Pack:=4)>
        Public Structure ADQAOModuleInfo
            Public Size As UInteger
            Public ID As UInteger
            Public NumberOfRanges As UInteger
            <MarshalAs(UnmanagedType.ByValArray, SizeConst:=ADQ_MAX_AO_RANGES)>
            Public Ranges() As ADQAORange

            Public Sub New(ByVal ID As UInteger)
                Size = Marshal.SizeOf(GetType(ADQAOModuleInfo))
                Me.ID = ID
                NumberOfRanges = 0
                Dim Ranges(ADQ_MAX_AO_RANGES) As ADQAORange
            End Sub
        End Structure

        Public Const ADQAOModuleInfo_ID As UInteger = &HA0000

        Public Shared Function SizeOfADQAOModuleInfo() As UInteger
            Return Marshal.SizeOf(GetType(ADQAOModuleInfo))
        End Function


        '---------------------------------------------------------------------
        '  Module information for I2C modules
        '---------------------------------------------------------------------
        <StructLayout(LayoutKind.Sequential, Pack:=4)>
        Public Structure ADQI2CModuleInfo
            Public Size As UInteger
            Public ID As UInteger
            Public SpeedBitsPerSecond As UInteger
            <MarshalAs(UnmanagedType.U1)>
            Public Supports7BitAddresses As Boolean
            <MarshalAs(UnmanagedType.U1)>
            Public Supports10BitAddresses As Boolean

            Public Sub New(ByVal ID As UInteger)
                Size = Marshal.SizeOf(GetType(ADQI2CModuleInfo))
                Me.ID = ID
                SpeedBitsPerSecond = 0
                Supports7BitAddresses = False
                Supports10BitAddresses = False
            End Sub
        End Structure

        Public Const ADQI2CModuleInfo_ID As UInteger = &HE0000

        Public Shared Function SizeOfADQI2CModuleInfo() As UInteger
            Return Marshal.SizeOf(GetType(ADQI2CModuleInfo))
        End Function


        '---------------------------------------------------------------------
        '  Module information for PWM modules
        '---------------------------------------------------------------------
        <StructLayout(LayoutKind.Sequential, Pack:=4)>
        Public Structure ADQPWMModuleInfo
            Public Size As UInteger
            Public ID As UInteger
            Public BaseFrequencyHz As Double
            Public MinimumDivider As UInteger

            Public Sub New(ByVal ID As UInteger)
                Size = Marshal.SizeOf(GetType(ADQPWMModuleInfo))
                Me.ID = ID
                BaseFrequencyHz = 0.0
                MinimumDivider = 0
            End Sub
        End Structure

        Public Const ADQPWMModuleInfo_ID As UInteger = &HF0000

        Public Shared Function SizeOfADQPWMModuleInfo() As UInteger
            Return Marshal.SizeOf(GetType(ADQPWMModuleInfo))
        End Function


        '---------------------------------------------------------------------
        '  Structures for IRQ routines
        '---------------------------------------------------------------------
        <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Unicode, Pack:=4)>
        Public Structure ADQIRQInfo
            Public Size As UInteger
            Public ID As UInteger
            <MarshalAs(UnmanagedType.ByValTStr, SizeConst:=1024)>
            Public IRQDescription As String
            <MarshalAs(UnmanagedType.U1)>
            Public SupportsBitChange As Boolean
            <MarshalAs(UnmanagedType.U1)>
            Public SupportsBitPatternCompare As Boolean
            Public SupportsExternal As UInteger
            Public SupportsPortStatusError As UInteger
            Public SupportsFaultRecovery As UInteger

            Public Sub New(ByVal ID As UInteger)
                Size = Marshal.SizeOf(GetType(ADQIRQInfo))
                Me.ID = ID
                IRQDescription = ""
                SupportsBitChange = False
                SupportsBitPatternCompare = False
                SupportsExternal = 0
                SupportsPortStatusError = 0
                SupportsFaultRecovery = 0
            End Sub
        End Structure

        Public Const ADQIRQInfo_ID As UInteger = &H70001

        Public Shared Function SizeOfADQIRQInfo() As UInteger
            Return Marshal.SizeOf(GetType(ADQIRQInfo))
        End Function


        <StructLayout(LayoutKind.Sequential, Pack:=4)>
        Public Structure ADQIRQEnable
            Public Size As UInteger
            Public ID As UInteger
            Public PortBitChangeRisingEdge As UInteger
            Public PortBitChangeFallingEdge As UInteger
            <MarshalAs(UnmanagedType.U1)>
            Public PortBitPatternCompare As Boolean
            Public PortBitPatternValue As UInteger
            Public ExternalRisingEdge As UInteger
            Public ExternalFallingEdge As UInteger
            Public PortStatusError As UInteger
            Public FaultRecovery As UInteger
            <MarshalAs(UnmanagedType.U1)>
            Public ResetIRQCount As Boolean

            Public Sub New(ByVal ID As UInteger)
                Size = Marshal.SizeOf(GetType(ADQIRQEnable))
                Me.ID = ID
                PortBitChangeRisingEdge = 0
                PortBitChangeFallingEdge = 0
                PortBitPatternCompare = False
                PortBitPatternValue = 0
                ExternalRisingEdge = 0
                ExternalFallingEdge = 0
                PortStatusError = 0
                FaultRecovery = 0
                ResetIRQCount = False
            End Sub
        End Structure

        Public Const ADQIRQEnable_ID As UInteger = &H80001

        Public Shared Function SizeOfADQIRQEnable() As UInteger
            Return Marshal.SizeOf(GetType(ADQIRQEnable))
        End Function


        <StructLayout(LayoutKind.Sequential, Pack:=4)>
        Public Structure ADQIRQStatus
            Public Size As UInteger
            Public ID As UInteger
            Public IRQCount As UInteger
            Public PortBitChangeRisingEdge As UInteger
            Public PortBitChangeFallingEdge As UInteger
            <MarshalAs(UnmanagedType.U1)>
            Public PortBitPatternCompare As Boolean
            Public ExternalRisingEdge As UInteger
            Public ExternalFallingEdge As UInteger
            Public PortStatusError As UInteger
            Public FaultRecovery As UInteger
            Public TriggerValue As UInteger

            Public Sub New(ByVal ID As UInteger)
                Size = Marshal.SizeOf(GetType(ADQIRQStatus))
                Me.ID = ID
                IRQCount = 0
                PortBitChangeRisingEdge = 0
                PortBitChangeFallingEdge = 0
                PortBitPatternCompare = False
                ExternalRisingEdge = 0
                ExternalFallingEdge = 0
                PortStatusError = 0
                FaultRecovery = 0
                TriggerValue = 0
            End Sub
        End Structure

        Public Const ADQIRQStatus_ID As UInteger = &H90001

        Public Shared Function SizeOfADQIRQStatus() As UInteger
            Return Marshal.SizeOf(GetType(ADQIRQStatus))
        End Function


        '---------------------------------------------------------------------
        '  Module configuration for counter modules of type 8254 (ADQ_CNTR_TYPE_8254)
        '---------------------------------------------------------------------
        <StructLayout(LayoutKind.Sequential, Pack:=4)>
        Public Structure ADQConfigCNTR8254
            Public Size As UInteger
            Public ID As UInteger
            Public ClockSource As UInteger
            Public CNTR8254Mode As UInteger
            Public StartValue As UInteger

            Public Sub New(ByVal ID As UInteger)
                Size = Marshal.SizeOf(GetType(ADQConfigCNTR8254))
                Me.ID = ID
                ClockSource = 0
                CNTR8254Mode = 0
                StartValue = 0
            End Sub
        End Structure

        Public Const ADQConfigCNTR8254_ID As UInteger = &H100000

        Public Shared Function SizeOfADQConfigCNTR8254() As UInteger
            Return Marshal.SizeOf(GetType(ADQConfigCNTR8254))
        End Function


        '---------------------------------------------------------------------
        '  Module configuration for counter modules of type count down 32-Bit (ADQ_CNTR_TYPE_COUNT_DOWN_32_BIT)
        '---------------------------------------------------------------------
        <StructLayout(LayoutKind.Sequential, Pack:=4)>
        Public Structure ADQConfigCNTRCountDown32Bit
            Public Size As UInteger
            Public ID As UInteger
            Public ClockSource As UInteger
            Public StartValue As UInteger
            Public ThresholdValue As UInteger
            <MarshalAs(UnmanagedType.U1)>
            Public OneShot As Boolean
            Public StrobeValue As UInteger
            <MarshalAs(UnmanagedType.U1)>
            Public StrobeInvert As Boolean
            <MarshalAs(UnmanagedType.U1)>
            Public HardwareGate As Boolean
            <MarshalAs(UnmanagedType.U1)>
            Public HardwareTrigger As Boolean

            Public Sub New(ByVal ID As UInteger)
                Size = Marshal.SizeOf(GetType(ADQConfigCNTRCountDown32Bit))
                Me.ID = ID
                ClockSource = 0
                StartValue = 0
                ThresholdValue = 0
                OneShot = False
                StrobeValue = 0
                StrobeInvert = False
                HardwareGate = False
                HardwareTrigger = False
            End Sub
        End Structure

        Public Const ADQConfigCNTRCountDown32Bit_ID As UInteger = &H200000

        Public Shared Function SizeOfADQConfigCNTRCountDown32Bit() As UInteger
            Return Marshal.SizeOf(GetType(ADQConfigCNTRCountDown32Bit))
        End Function


        '---------------------------------------------------------------------
        ' Board synchron start information structure, passed to adqGetBoardSynchronStartInfo
        '---------------------------------------------------------------------
        <StructLayout(LayoutKind.Sequential, Pack:=4)>
        Public Structure ADQBoardSynchronStartInfo
            Public Size As UInteger
            Public ID As UInteger
            Public NumberOfGroups As UInteger
            Public SizeOfGroup As UInteger
            Public ModuleMap As UInteger

            Public Sub New(ByVal ID As UInteger)
                Size = Marshal.SizeOf(GetType(ADQConfigCNTRCountDown32Bit))
                Me.ID = ID
                NumberOfGroups = 0
                SizeOfGroup = 0
                ModuleMap = 0
            End Sub
        End Structure

        Public Const SADQBoardSynchronStartInfo_ID As UInteger = &H300000

        Public Shared Function SizeOfADQBoardSynchronStartInfo() As UInteger
            Return Marshal.SizeOf(GetType(ADQBoardSynchronStartInfo))
        End Function


        '---------------------------------------------------------------------
        '  Callback routines
        '---------------------------------------------------------------------

        ''' <summary>
        ''' Delegate for the function passed to adqEnableIRQ
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="irqStatus">
        ''' Reference to a structure ADQIRQStatus
        ''' </param>
        ''' <param name="adqError">
        ''' Error code
        ''' </param>
        ''' <param name="userContext">
        ''' Pointer to user defined context that was passed to function adqEnableIRQ
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <UnmanagedFunctionPointer(CallingConvention.StdCall)>
        Public Delegate Function adqIrqCallback(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByRef irqStatus As ADQIRQStatus,
            ByVal adqError As UInteger,
            ByVal userContext As IntPtr
        ) As UInteger


        ''' <summary>
        ''' Delegate for the function passed to adqReadContinuousStart
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="numberOfValuesAvailable">
        ''' Number of values that can be read
        ''' </param>
        ''' <param name="values">
        ''' Pointer to values buffer
        ''' </param>
        ''' <param name="adqError">
        ''' Error code
        ''' </param>
        ''' <param name="userContext">
        ''' Pointer to user defined context that was passed to function adqReadContinuousStart
        ''' </param>
        ''' <param name="reasonForCall">
        ''' One of the constants for start, data or stop of the operation
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <UnmanagedFunctionPointer(CallingConvention.StdCall)>
        Public Delegate Function adqReadContinuousCallback(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal numberOfValuesAvailable As UInteger,
            ByRef values As UInteger(),
            ByVal adqError As UInteger,
            ByVal userContext As IntPtr,
            ByVal reasonForCall As UInteger
        ) As UInteger



        ''' <summary>
        ''' Delegate for the function passed to adqWriteContinuousStart
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="numberOfValuesAcceptable">
        ''' Number of values that can be written to the buffer
        ''' </param>
        ''' <param name="values">
        ''' Pointer to values buffer
        ''' </param>
        ''' <param name="numberOfValuesWritten">
        ''' Number of values written to the module
        ''' </param>
        ''' <param name="adqError">
        ''' Error code
        ''' </param>
        ''' <param name="userContext">
        ''' Pointer to user defined context that was passed to function adqWriteContinuousStart
        ''' </param>
        ''' <param name="reasonForCall">
        ''' One of the constants for start, data or stop of the operation
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <UnmanagedFunctionPointer(CallingConvention.StdCall)>
        Public Delegate Function adqWriteContinuousCallback(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal numberOfValuesAcceptable As UInteger,
            ByVal values As IntPtr,
            ByRef numberOfValuesWritten As UInteger,
            ByVal adqError As UInteger,
            ByVal userContext As IntPtr,
            ByVal reasonForCall As UInteger
        ) As UInteger


        ''' <summary>
        ''' Delegate for the function passed to adqAIReadContinuousStart
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="numberOfFramesAvailable">
        ''' Number of frames that can be read
        ''' </param>
        ''' <param name="frames">
        ''' Pointer to frames in data buffer
        ''' </param>
        ''' <param name="adqError">
        ''' Error code
        ''' </param>
        ''' <param name="userContext">
        ''' Pointer to user defined context that was passed to function adqAIReadContinuousStart
        ''' </param>
        ''' <param name="reasonForCall">
        ''' One of the constants for start, data or stop of the operation
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <UnmanagedFunctionPointer(CallingConvention.StdCall)>
        Public Delegate Function adqAIReadContinuousCallback(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal numberOfFramesAvailable As UInteger,
            ByRef frames As UInteger(),
            ByVal adqError As UInteger,
            ByVal userContext As IntPtr,
            ByVal reasonForCall As UInteger
        ) As UInteger


#End Region

#Region "Import Functions"
#Disable Warning IDE1006 ' Naming Styles

        '---------------------------------------------------------------------
        '  API routines
        '---------------------------------------------------------------------
        ''' <summary>
        ''' Opens the adqDriver system
        ''' Call adqOpen before accessing the boards using other functions
        ''' </summary>
        ''' <returns>
        ''' Error code
        ''' If the function succeeds, the return value is ADQ_ERROR_NONE.
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqOpen() As UInteger
        End Function


        ''' <summary>
        ''' Opens the adqDriver system by given names or IP-addresses
        ''' </summary>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall,
            CharSet:=CharSet.Unicode)>
        Public Shared Function adqOpenEx(
            nameOrIPAddress As String()
        ) As UInteger
        End Function


        ''' <summary>
        ''' Closes an open adqDriver system
        ''' Call adqClose when the application is finished using the adqDriver system
        ''' </summary>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqClose() As UInteger
        End Function


        ''' <summary>
        ''' Get information about an internal error. For internal use.
        ''' </summary>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqGetInternalError(
            ByRef component As UInteger,
            ByRef file As UInteger,
            ByRef lineNumber As UInteger,
            ByRef aType As UInteger,
            ByRef errorParameter1 As UInteger,
            ByRef errorParameter2 As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Get Info about the ADQ driver system
        ''' </summary>
        ''' <param name="systemInfo">
        ''' Struct of type ADQSystemInfo to receive the information
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqGetSystemInfo(
            ByRef systemInfo As ADQSystemInfo
        ) As UInteger
        End Function


        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqQuery(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal Id As UInteger,
            ByVal subId As UInteger,
            ByVal bufferSize As UInteger,
            ByRef buffer As Byte(),
            ByRef infoSize As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Determines the number of boards recognised by the driver system
        ''' </summary>
        ''' <param name="numberOfBoards">
        ''' Contains the number of boards
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqGetNumberOfBoards(
            ByRef numberOfBoards As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Gets info about the given board
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="boardInfo">
        ''' Struct that receives the information about the board
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqGetBoardInfo(
            ByVal boardIndex As UInteger,
            ByRef boardInfo As ADQBoardInfo
        ) As UInteger
        End Function


        ''' <summary>
        ''' Get PCI location information information about a board if BusType Is ADQ_BUS_TYPE_PCI
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="boardPCIInfo">
        ''' Struct that contains the PCI location information
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqGetBoardPCIInfo(
            ByVal boardIndex As UInteger,
            ByRef boardPCIInfo As ADQBoardPCIInfo
        ) As UInteger
        End Function


        ''' <summary>
        ''' Get the voltage level of all TTL ports on the given board
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="voltage">
        ''' Contains the current voltage level as one of the defined constants
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqGetBoardTTLVoltage(
            ByVal boardIndex As UInteger,
            ByRef voltage As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Set the voltage level of all TTL ports on the given board
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="voltage">
        ''' The new voltage level as one of the defined constants
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqSetBoardTTLVoltage(
            ByVal boardIndex As UInteger,
            ByVal voltage As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Determine the number of modules of the given board
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="numberOfModules">
        ''' Contains the number of modules
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqGetNumberOfModules(
            ByVal boardIndex As UInteger,
            ByRef numberOfModules As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Retrieve the type of the module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module to be queried
        ''' </param>
        ''' <param name="moduleType">
        ''' Contains the type of the module
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqGetModuleType(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByRef moduleType As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Get Info about given Module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module to be queried
        ''' </param>
        ''' <param name="generalInfo">
        ''' Struct of type ADQGeneralModuleInfo to receive the information
        ''' </param>
        ''' <param name="moduleInfoSize">
        ''' Size of moduleInfo in bytes
        ''' </param>
        ''' <param name="moduleInfo">
        ''' Pointer to a structure that receives the information about the module
        ''' This parameter can be null
        ''' </param>
        ''' <param name="infoSize">
        ''' Contains the size in bytes of the information in moduleInfo
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqGetModuleInfo(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByRef generalInfo As ADQGeneralModuleInfo,
            ByVal moduleInfoSize As UInteger,
            ByVal moduleInfo As IntPtr,
            ByRef infoSize As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Reads a single value from the given module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module to read from
        ''' </param>
        ''' <param name="value">
        ''' Contains the value read from the module
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqReadSingle(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByRef value As UInteger
        ) As UInteger
        End Function


        '' <summary>
        '' Read a predefined number of values from a module
        '' </summary>
        '' <param name="boardIndex">
        '' Index of the board
        '' </param>
        '' <param name="moduleIndex">
        '' Index of the module to read from
        '' </param>
        '' <param name="numberOfValuesRequired">
        '' Number of values to be read
        '' </param>
        '' <param name="values">
        '' Buffer that holds the data
        '' </param>
        '' <param name="numberOfValuesReturned">
        '' Number of values that can be read from buffer
        '' </param>
        '' <returns>
        '' Error code
        '' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqReadBuffer(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal numberOfValuesRequired As UInteger,
            values As UInteger(),
            ByRef numberOfValuesReturned As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Start a conitinuous data acquisition from a module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module to read from
        ''' </param>
        ''' <param name="notifyAfterValues">
        ''' Number of values after the application will get notified
        ''' </param>
        ''' <param name="stopAfterValues">
        ''' Number of values after which the acquisition will stop automatically
        ''' </param>
        ''' <param name="readContinuousCallback">
        ''' Callback function, called on start, new data available, and stop of the acquisition
        ''' </param>
        ''' <param name="readContinuousUserContext">
        ''' Pointer to user defined context
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqReadContinuousStart(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal notifyAfterValues As UInteger,
            ByVal stopAfterValues As ULong,
            readContinuousCallback As adqReadContinuousCallback,
            ByVal readContinuousUserContext As IntPtr
        ) As UInteger
        End Function


        ''' <summary>
        ''' Read values during a conitinuous data acquisition from a module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module to read from
        ''' </param>
        ''' <param name="numberOfValuesRequired">
        ''' Number of values required
        ''' </param>
        ''' <param name="values">
        ''' Reference to the data buffer to read from
        ''' </param>
        ''' <param name="waitForData">
        ''' True, if the function should wait till the number of required values are acquired
        ''' False, if the function should return immediately
        ''' </param>
        ''' <param name="numberOfValuesReturned">
        ''' Number of values available in values buffer
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqReadContinuous(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal numberOfValuesRequired As UInteger,
            values As UInteger(),
            ByVal waitForData As Boolean,
            ByRef numberOfValuesReturned As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Read the status of a continuous data acquisition
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module to read the status
        ''' </param>
        ''' <param name="waitForIdle">
        ''' True, if the function should wait till the module will be in idle state
        ''' False, if the function should return immediately
        ''' </param>
        ''' <param name="numberOfValuesAvailable">
        ''' Number of values that can be read
        ''' </param>
        ''' <param name="running">
        ''' True, if the module is still acquiring data
        ''' False, if the module is in idle state
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqReadContinuousStatus(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal waitForIdle As Boolean,
            ByRef numberOfValuesAvailable As UInteger,
            ByRef running As Boolean
        ) As UInteger
        End Function


        ''' <summary>
        ''' Writes a single value to the given module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module to write to
        ''' </param>
        ''' <param name="value">
        ''' The value to be written
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqWriteSingle(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal value As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Write a predefined number of values to a module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module to write to
        ''' </param>
        ''' <param name="bufferSizeValues">
        ''' Size of the data buffer in values
        ''' </param>
        ''' <param name="values">
        ''' Reference to buffer that contains the values to be written
        ''' </param>
        ''' <param name="repeatBuffer">
        ''' Number of repetitions to write the values
        ''' </param>
        ''' <param name="numberOfValuesWritten">
        ''' Number of values written to the module
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqWriteBuffer(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal bufferSizeValues As UInteger,
            values As UInteger(),
            ByVal repeatBuffer As UInteger,
            ByRef numberOfValuesWritten As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Start a continuous output
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module to write to
        ''' </param>
        ''' <param name="bufferSizeValues">
        ''' Size of the data buffer in values
        ''' </param>
        ''' <param name="values">
        ''' Reference to buffer that contains the values to be written
        ''' </param>
        ''' <param name="repeatBuffer">
        ''' Number of repetitions to write the values
        ''' </param>
        ''' <param name="waitForWrite">
        ''' True, if the function should wait till the values are written to the buffer of the module
        ''' False, if the function should return immediately
        ''' </param>
        ''' <param name="notifyAfterValues">
        ''' Number of values after the application will get notified
        ''' </param>
        ''' <param name="numberOfValuesWritten">
        ''' Number of values written to the module
        ''' </param>
        ''' <param name="writeContinuousCallback">
        ''' Delegate of the callback function, called on start, data written, and stop of the output operation
        ''' </param>
        ''' <param name="writeContinuousUserContext">
        ''' Pointer to user defined context
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqWriteContinuousStart(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal bufferSizeValues As UInteger,
            values As UInteger(),
            ByVal repeatBuffer As UInteger,
            ByVal waitForWrite As Boolean,
            ByVal notifyAfterValues As UInteger,
            ByRef numberOfValuesWritten As UInteger,
            writeContinuousCallback As adqWriteContinuousCallback,
            ByVal writeContinuousUserContext As IntPtr
        ) As UInteger
        End Function


        ''' <summary>
        ''' Write values for a continuous output operation
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="numberOfValuesToWrite">
        ''' Number of values to write to the data buffer
        ''' </param>
        ''' <param name="values">
        ''' Reference to buffer that contains the values to be written
        ''' </param>
        ''' <param name="waitForWrite">
        ''' True, if the function should wait till the values are written to the buffer of the module
        ''' False, if the function should return immediately
        ''' </param>
        ''' <param name="numberOfValuesWritten">
        ''' Number of values written to the module
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqWriteContinuous(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal numberOfValuesToWrite As UInteger,
            values As UInteger(),
            ByVal waitForWrite As Boolean,
            ByRef numberOfValuesWritten As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Read the status of a continuous output operation
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="waitForIdle">
        ''' True, if the function should wait till the module will be in idle state
        ''' False, if the function should return immediately
        ''' </param>
        ''' <param name="numberOfValuesAcceptable">
        ''' Number of values that can be be written
        ''' </param>
        ''' <param name="running">
        ''' True, if the module is still writing data
        ''' False, if the module is in idle state
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqWriteContinuousStatus(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal waitForIdle As Boolean,
            ByRef numberOfValuesAcceptable As UInteger,
            ByRef running As Boolean
        ) As UInteger
        End Function


        ''' <summary>
        ''' Stop a clock controlled acquisition or output
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqStop(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Get the current configured direction of a DIO module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="direction">
        ''' Contains the direction of the module (Input, Output)
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqDIOGetDirection(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByRef direction As UInteger
        ) As UInteger
        End Function

        ''' <summary>
        ''' Sets the direction of a DIO port
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="direction">
        ''' Direction of the module (Input, Output)
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqDIOSetDirection(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal direction As UInteger
        ) As UInteger
        End Function

        ''' <summary>
        ''' Get the current digital filter value in picoseconds
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="filterPicoseconds">
        ''' Contains the current digital filter value
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqDIOGetFilterPicoseconds(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByRef filterPicoseconds As ULong
        ) As UInteger
        End Function


        ''' <summary>
        ''' Set the digital filter value in picoseconds
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="filterPicoseconds">
        ''' The digital filter value to be set
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqDIOSetFilterPicoseconds(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal filterPicoseconds As ULong
        ) As UInteger
        End Function

        ''' <summary>
        ''' Reset all modules of the board
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqResetBoard(
            ByVal boardIndex As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Reset the specified module of the board
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqResetModule(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Set the frequency for a module for an acquisition or output
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="nominalFrequency">
        ''' Desired frequency in Hertz
        ''' </param>
        ''' <param name="actualFrequency">
        ''' Actual frequency in Hertz
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        ''' <remarks>
        ''' For analog input modules use the function adqAISetChannelListAndFrequency if more than one channel Is used
        ''' </remarks>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqSetFrequency(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal nominalFrequency As Double,
            ByRef actualFrequency As Double
        ) As UInteger
        End Function


        ''' <summary>
        ''' Get the maximum sample frequency in Hertz for a given number of channels for analog input modules
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="numberOfChannels">
        ''' The number of channels
        ''' </param>
        ''' <param name="maximumSampleFrequency">
        ''' The maximum sample frequency per channel in Hertz
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqGetMaximumSampleFrequency(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal numberOfChannels As UInteger,
            ByRef maximumSampleFrequency As Double
        ) As UInteger
        End Function


        ''' <summary>
        ''' Get info about supported hardware triggers of a module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="hwTriggerInfo">
        ''' Struct ADQHWTriggerInfo to receive the information
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqGetHWTriggerInfo(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByRef hwTriggerInfo As ADQHWTriggerInfo
        ) As UInteger
        End Function


        ''' <summary>
        ''' Enable hardware trigger for a module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="hwTriggerEnable">
        ''' Struct ADQHWTriggerEnable with enabled trigger settings
        ''' </param>
        ''' <param name="waitForTrigger">
        ''' True, if the function should wait till the trigger event occurs
        ''' False, if the function should return immediately
        ''' </param>
        ''' <param name="timeoutMilliseconds">
        ''' Timeout value in milliseconds until the function returns.
        ''' If the constant ADQ_HW_START_TRIGGER_TIMEOUT_INFINITE Is passed the function waits infinitely.
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqEnableHWTrigger(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByRef hwTriggerEnable As ADQHWTriggerEnable,
            ByVal waitForTrigger As Boolean,
            ByVal timeoutMilliseconds As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Disable the hardware trigger for a module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqDisableHWTrigger(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger
        ) As UInteger
        End Function

        ''' <summary>
        ''' Get info about supported IRQ-sources of a module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="irqInfo">
        ''' Struct ADQIRQInfo to receive the information
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqGetIRQInfo(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByRef irqInfo As ADQIRQInfo
        ) As UInteger
        End Function


        ''' <summary>
        ''' Intall a callback function and enable an Interrupt
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="irqEnable">
        ''' Struct ADQIRQEnable with IRQ enabled settings
        ''' </param>
        ''' <param name="irqCallback">
        ''' Delegate of the callback function, can be Nothing
        ''' </param>
        ''' <param name="adqIRQStatusID">
        ''' ID of the struct ADQIRQEnable
        ''' </param>
        ''' <param name="userContext">
        ''' user defined reference, will be passed to delegate adqIrqCallback, can be Nothing
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqEnableIRQ(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByRef irqEnable As ADQIRQEnable,
            irqCallback As adqIrqCallback,
            ByVal adqIRQStatusID As UInteger,
            ByVal userContext As IntPtr
        ) As UInteger
        End Function


        ''' <summary>
        ''' Wait for an Interrupt to occur
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="irqStatus">
        ''' Struct ADQIRQStatus that contains the status of the
        ''' </param>
        ''' <param name="timeoutMilliseconds">
        ''' Timeout value in milliseconds until the function returns.
        ''' If the constant ADQ_TIMEOUT_INFINITE is passed the function waits infinitely
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqWaitIRQ(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByRef irqStatus As ADQIRQStatus,
            ByVal timeoutMilliseconds As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Get the number of Interrupts that occured since the IRQ functionality was enabled
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="irqCount">
        ''' Number of interrupts
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqGetIRQCount(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByRef irqCount As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Disable IRQ
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqDisableIRQ(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Configures the counter with the specified mode
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="counterMode">
        ''' Mode of the counter
        ''' </param>
        ''' <param name="startValue">
        ''' Counter start value
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqCNTRConfig(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal clockSource As UInteger,
            ByVal counterMode As UInteger,
            ByVal startValue As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Configures a counter module with the given settings
        ''' This function does Not enable the counter at the same time
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="counterConfigSize">
        ''' Size of counterConfig in bytes
        ''' </param>
        ''' <param name="counterConfig">
        ''' Pointer to the structure that contains the configuration
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqCNTRConfigEx(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal counterConfigSize As UInteger,
            ByVal counterConfig As IntPtr
        ) As UInteger
        End Function


        ''' <summary>
        ''' Enables the counter module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqCNTREnable(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Disables the counter module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqCNTRDisable(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Configures the channel and range of an analog input module.
        ''' Use the functions adqReadSingle, adqReadBuffer, adqReadContinuous to read values.
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="channelIndex">
        ''' Index of the channel
        ''' </param>
        ''' <param name="rangeIndex">
        ''' Index of the range
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqAISetChannelAndRange(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal channelIndex As UInteger,
            ByVal rangeIndex As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Get the range for analog input modules that Is currently set on the hardware
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="channelIndex">
        ''' Index of the channel
        ''' </param>
        ''' <param name="rangeIndex">
        ''' Contains the index of the range currently set
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        ''' <remarks>
        ''' Use this function for modules that have a safety mechanism to switch to the highest range on excessive range overflow e.g. ADQ-412
        ''' </remarks>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqAIGetChannelHardwareRange(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal channelIndex As UInteger,
            ByRef rangeIndex As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Set the channel list for an analog input module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="channels">
        ''' Reference to an array of ADQAIChannel structures
        ''' </param>
        ''' <param name="numberOfChannels">
        ''' Number of elements in parameter channels
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <Obsolete("adqAISetChannelList is deprecated, please use adqAISetChannelListAndFrequency instead.")>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqAISetChannelList(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            channels As ADQAIChannel(),
            ByVal numberOfChannels As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Set the channel list And sample frequency for an analog input module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="channels">
        ''' Array of structures of type ADQAIChannel
        ''' </param>
        ''' <param name="numberOfChannels">
        ''' Number of elements in parameter channels
        ''' </param>
        ''' <param name="nominalFrequency">
        ''' Desired sample frequency in Hertz
        ''' </param>
        ''' <param name="actualFrequency">
        ''' Actual sample frequency in Hertz
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqAISetChannelListAndFrequency(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            channels As ADQAIChannel(),
            ByVal numberOfChannels As UInteger,
            ByVal nominalFrequency As Double,
            ByRef actualFrequency As Double
        ) As UInteger
        End Function


        ''' <summary>
        ''' Read a single frame from an analog input module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="frameLength">
        ''' Number of elements in one frame
        ''' </param>
        ''' <param name="frame">
        ''' Reference to the data buffer to read from
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqAIReadSingle(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal frameLength As UInteger,
            ByRef frame As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Read a predefined number of frames from an analog input module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="frameLength">
        ''' Number of elements in one frame
        ''' </param>
        ''' <param name="numberOfFramesRequired">
        ''' Number of frames to read
        ''' </param>
        ''' <param name="frames">
        ''' Reference to the data buffer to read from
        ''' </param>
        ''' <param name="numberOfFramesReturned">
        ''' Number of frames successfully read
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqAIReadBuffer(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal frameLength As UInteger,
            ByVal numberOfFramesRequired As UInteger,
            frames As UInteger(),
            ByRef numberOfFramesReturned As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Start a continuous acquisition of frames on an analog input module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="notifyAfterFrames">
        ''' Number of values after the application will get notified
        ''' </param>
        ''' <param name="stopAfterFrames">
        ''' Number of frames after which the acquisition will stop automatically
        ''' </param>
        ''' <param name="aiReadContinuousCallback">
        ''' Delegate of the callback function, called on start, data read, and stop of the acquisition
        ''' </param>
        ''' <param name="readContinuousUserContext">
        ''' Pointer to user defined context
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqAIReadContinuousStart(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal notifyAfterFrames As UInteger,
            ByVal stopAfterFrames As ULong,
            aiReadContinuousCallback As adqAIReadContinuousCallback,
            ByVal readContinuousUserContext As IntPtr
        ) As UInteger
        End Function


        ''' <summary>
        ''' Read frames during a conitinuous data acquisition from a module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="frameLength">
        ''' Number of elements in one frame
        ''' </param>
        ''' <param name="numberOfFramesRequired">
        ''' Number of frames to read
        ''' </param>
        ''' <param name="frames">
        ''' Array of the data buffer to read from
        ''' </param>
        ''' <param name="waitForData">
        ''' True, if the function should wait till the number of required frames are acquired
        ''' False, if the function should return immediately
        ''' </param>
        ''' <param name="numberOfFramesReturned">
        ''' Number of frames successfully read
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqAIReadContinuous(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal frameLength As UInteger,
            ByVal numberOfFramesRequired As UInteger,
            frames As UInteger(),
            ByRef waitForData As Boolean,
            ByRef numberOfFramesReturned As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Read the status of a continuous data acquisition
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="waitForIdle">
        ''' True, if the function should wait till the module will be in idle state
        ''' False, if the function should return immediately
        ''' </param>
        ''' <param name="numberOfFramesAvailable">
        ''' Number of frames that can be read
        ''' </param>
        ''' <param name="running">
        ''' True, if the module is still acquiring data
        ''' False, if the module is in idle state
        ''' </param>
        ''' <returns></returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqAIReadContinuousStatus(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal waitForIdle As Boolean,
            ByRef numberOfFramesAvailable As UInteger,
            ByRef running As Boolean
        ) As UInteger
        End Function


        ''' <summary>
        ''' Get the currently selected calibration of the given module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="CalibrationID">
        ''' Contains one of the calibration constants ADQ_CALIBRATION_...
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqAnalogGetSelectedCalibration(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByRef calibrationID As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Set a calibration to be used for the given module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="CalibrationID">
        ''' One of the calibration constants ADQ_CALIBRATION_...
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqAnalogSelectCalibration(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal calibrationID As UInteger
        ) As UInteger
        End Function

        ''' <summary>
        ''' Set the temporary calibration for a single channel 'channelIndex' of module 'moduleIndex' on board 'boardIndex' at range 'rangeIndex'.
        ''' The temporary calibration gets lost after a driver newstart. 
        ''' It also gets lost when the function itself Or the function 'adqAnalogSelectCalibration' is called again.
        ''' </summary>
        ''' <param name="boardIndex">Index of the board</param>
        ''' <param name="moduleIndex">Index of the module</param>
        ''' <param name="channelIndex">Index of the channel</param>
        ''' <param name="rangeIndex">Index of the range</param>
        ''' <param name="nominalPhysicalValue1">nominal physical value 1</param>
        ''' <param name="actualPhysicalValue1">actual physical value 1</param>
        ''' <param name="nominalPhysicalValue2">nominal physical value 2</param>
        ''' <param name="actualPhysicalValue2">actual physical value 2</param>
        ''' <returns>Error code</returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqAnalogSetTemporaryCalibration(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal channelIndex As UInteger,
            ByVal rangeIndex As UInteger,
            ByVal nominalPhysicalValue1 As Double,
            ByVal actualPhysicalValue1 As Double,
            ByVal nominalPhysicalValue2 As Double,
            ByVal actualPhysicalValue2 As Double
            ) As UInteger
        End Function

        ''' <summary>
        ''' Set the temporary calibration for a single channel 'channelIndex' of module 'moduleIndex' on board 'boardIndex' at range 'rangeIndex'.
        ''' The temporary calibration gets lost after a driver newstart. 
        ''' It also gets lost when the function itself Or the function 'adqAnalogSelectCalibration' is called again.
        ''' </summary>
        ''' <param name="boardIndex">Index of the board</param>
        ''' <param name="moduleIndex">Index of the module</param>
        ''' <param name="channelIndex">Index of the channel</param>
        ''' <param name="rangeIndex">Index of the range</param>
        ''' <returns>Error code</returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqAISetCalibrationZero(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal channelIndex As UInteger,
            ByVal rangeIndex As UInteger
            ) As UInteger
        End Function

        ''' <summary>
        ''' Gets the number of user calibrations available for the actual module.
        ''' Actually gives back '1' for all modules except AI-Module of ADQ-3XX.
        ''' </summary>
        ''' <param name="boardIndex">Index of the board</param>
        ''' <param name="moduleIndex">Index of the module</param>
        ''' <param name="numberOfUserCalibrations">Number of user calibrations available for the actual module.</param>
        ''' <returns></returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqAnalogGetNumberOfUserCalibrations(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByRef numberOfUserCalibrations As UInteger
            ) As UInteger
        End Function



        ''' <summary>
        ''' Get the factor (calibrationFactor) And the offset (calibrationConstant) for a certian channel at a certian range.
        ''' The function Is used to calibrate 'raw values' as result of a measurement done without calibration.
        ''' To get a correct value first multiply the raw value with 'calibrationFactor' and then add 'calibrationConstant'.
        ''' (A module can be configured for measuring without calibration by calling adqAnalogSelectCalibration() with calibrationId = AdqConstants.ADQ_CALIBRATION_NONE.)
        ''' </summary>
        ''' <param name="boardIndex">Index of the board</param>
        ''' <param name="moduleIndex">Index of the module</param>
        ''' <param name="channelIndex">Index of the channel</param>
        ''' <param name="rangeIndex">Index of the range</param>
        ''' <param name="calibrationID">The id of the calibration to get the values from</param>
        ''' <param name="calibrationFactor">The calibration factor</param>
        ''' <param name="calibrationConstant">The calibration constant (offset)</param>
        ''' <returns></returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention = CallingConvention.StdCall)>
        Public Shared Function adqAnalogGetCalibrationFactorAndConstant(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal channelIndex As UInteger,
            ByVal rangeIndex As UInteger,
            ByVal calibrationID As UInteger,
            ByRef calibrationFactor As Double,
            ByRef calibrationConstant As Double) As UInteger
        End Function


        ''' <summary>
        ''' Set the address of the slave device for a write Or read operation
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="slaveAddress">
        ''' The address of the slave device
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqI2CSetSlaveAddress(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal slaveAddress As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Read a given number of values from an I2C module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module to read from
        ''' </param>
        ''' <param name="numberOfBytesRequired">
        ''' Number of bytes to read
        ''' </param>
        ''' <param name="data">
        ''' Data buffer that receives the values
        ''' </param>
        ''' <param name="numberOfBytesReturned">
        ''' Number of bytes successfully read
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqI2CRead(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal numberOfBytesRequired As UInteger,
            data As Byte(),
            ByRef numberOfBytesReturned As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Write a given number of bytes to an I2C module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module to write to
        ''' </param>
        ''' <param name="numberOfBytesToWrite">
        ''' The number of bytes to write to the module
        ''' </param>
        ''' <param name="data">
        ''' Reference to the data buffer that contains the values
        ''' </param>
        ''' <param name="numberOfBytesWritten">
        ''' The number of bytes successfully written to the module
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqI2CWrite(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal numberOfBytesToWrite As UInteger,
            data As Byte(),
            ByRef numberOfBytesWritten As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Performs a write operation of a byte array, followed by a read operation
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module to write to
        ''' </param>
        ''' <param name="numberOfBytesToWrite">
        ''' The number of bytes to write to the module
        ''' </param>
        ''' <param name="writeData">
        ''' The data buffer that contains the output values
        ''' </param>
        ''' <param name="numberOfBytesWritten">
        ''' The number of bytes successfully written to the module
        ''' </param>
        ''' <param name="numberOfBytesRequired">
        ''' Number of bytes to read
        ''' </param>
        ''' <param name="readData">
        ''' Data buffer that receives the values
        ''' </param>
        ''' <param name="numberOfBytesReturned">
        ''' Number of bytes successfully read
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqI2CWriteThenRead(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal numberOfBytesToWrite As UInteger,
            writeData As Byte(),
            ByRef numberOfBytesWritten As UInteger,
            ByVal numberOfBytesRequired As UInteger,
            readData As Byte(),
            ByRef numberOfBytesReturned As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Get the maximum signal frequency in Hertz that can be generated by the PWM module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="maximumFrequency">
        ''' The maximum frequency the module can generate
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqPWMGetMaximumFrequency(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByRef maximumFrequency As Double
        ) As UInteger
        End Function


        ''' <summary>
        ''' Configure a PWM module for the output operation
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="nominalFrequency">
        ''' The desired output frequency of the signal in Hertz
        ''' </param>
        ''' <param name="nominalOnTime">
        ''' The part of one period the output signal will have high level in seconds
        ''' </param>
        ''' <param name="invert">
        ''' True, if the output signal will be inverted, False otherwise
        ''' By default (False) the signal starts with the high level part
        ''' </param>
        ''' <param name="actualFrequency">
        ''' The actual output frequency in Hertz
        ''' </param>
        ''' <param name="actualOnTime">
        ''' The actual time of the high level part of one period in seconds
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqPWMConfig(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal nominalFrequency As Double,
            ByVal nominalOnTime As Double,
            ByVal invert As Boolean,
            ByRef actualFrequency As Double,
            ByRef actualOnTime As Double
        ) As UInteger
        End Function


        ''' <summary>
        ''' Enables the output of the PWM module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqPWMEnable(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal timeoutMilliseconds As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Disables the output of the PWM module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqPWMDisable(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Measures a digital signal period
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="frequency">
        ''' The frequency of the signal period
        ''' </param>
        ''' <param name="onTime">
        ''' The high-time of the signal period in seconds
        ''' </param>
        ''' <param name="timeoutMilliseconds">
        ''' Timeout value in milliseconds until the function returns.
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqFINRead(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByRef frequency As Double,
            ByRef onTime As Double,
            ByVal timeoutMilliseconds As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Configures the output of a sine wave signal for an I2S output module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="nominalOutputFrequency">
        ''' The desired output frequency of the module in Hertz
        ''' </param>
        ''' <param name="signalLeftPrecisionBits">
        ''' The resolution in bits of the signal for the left channel
        ''' </param>
        ''' <param name="signalLeftTwosComplement">
        ''' True, if the output signal values for the left channel should be generated as two's complement
        ''' </param>
        ''' <param name="signalLeftSignalFrequency">
        ''' The signal frequency of the left channel in Hertz
        ''' </param>
        ''' <param name="signalLeftAmplitude">
        ''' The signal amplitude of the left channel as value between 0 And 1
        ''' </param>
        ''' <param name="signalRightPrecisionBits">
        ''' The resolution in bits of the signal for the right channel
        ''' </param>
        ''' <param name="signalRightTwosComplement">
        ''' True, if the output signal values for the right channel should be generated as two's complement
        ''' </param>
        ''' <param name="signalRightSignalFrequency">
        ''' The signal frequency of the right channel in Hertz
        ''' </param>
        ''' <param name="signalRightAmplitude">
        ''' The signal amplitude of the right channel as value between 0 And 1
        ''' </param>
        ''' <param name="actualOutputFrequency">
        ''' Actual output frequency in Hertz
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqI2SOSineConfig(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal nominalOutputFrequency As Double,
            ByVal signalLeftPrecisionBits As UInteger,
            ByVal signalLeftTwosComplement As Boolean,
            ByVal signalLeftSignalFrequency As Double,
            ByVal signalLeftAmplitude As Double,
            ByVal signalRightPrecisionBits As UInteger,
            ByVal signalRightTwosComplement As Boolean,
            ByVal signalRightSignalFrequency As Double,
            ByVal signalRightAmplitude As Double,
            ByRef actualOutputFrequency As Double
        ) As UInteger
        End Function


        ''' <summary>
        ''' Starts the output operation on an I2S module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqI2SOSineStart(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Modify the sine wave signal for an I2SO module during an output operation
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <param name="signalLeftSignalFrequency">
        ''' The New signal frequency of the left channel in Hertz
        ''' </param>
        ''' <param name="signalLeftAmplitude">
        ''' The New signal amplitude of the left channel as value between 0 And 1
        ''' </param>
        ''' <param name="signalRightSignalFrequency">
        ''' The New signal frequency of the right channel in Hertz
        ''' </param>
        ''' <param name="signalRightAmplitude">
        ''' The New signal amplitude of the right channel as value between 0 And 1
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqI2SOSineModify(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger,
            ByVal signalLeftSignalFrequency As Double,
            ByVal signalLeftAmplitude As Double,
            ByVal signalRightSignalFrequency As Double,
            ByVal signalRightAmplitude As Double
        ) As UInteger
        End Function


        ''' <summary>
        ''' Stops the output operation on an I2S module
        ''' </summary>
        ''' <param name="boardIndex">
        ''' Index of the board
        ''' </param>
        ''' <param name="moduleIndex">
        ''' Index of the module
        ''' </param>
        ''' <returns>
        ''' Error code
        ''' </returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqI2SOSineStop(
            ByVal boardIndex As UInteger,
            ByVal moduleIndex As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Get the board synchron start information, means get to know which combinations of channels can be started synchronously.
        ''' </summary>
        ''' <param name="boardIndex">Board index</param>
        ''' <param name="boardSynchronStartInfo">Board synchron start information</param>
        ''' <returns>Error code</returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqGetBoardSynchronStartInfo(
            ByVal boardIndex As UInteger,
            ByRef boardSynchronStartInfo As ADQBoardSynchronStartInfo
        ) As UInteger
        End Function


        ''' <summary>
        ''' Configure the start of a synchronous read operation.
        ''' </summary>
        ''' <param name="uiBoardIndex">Board index</param>
        ''' <param name="uiGroupIndex">Synchron group index</param>
        ''' <param name="uiModuleMap">Module map</param>
        ''' <returns>Error code</returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqConfigBoardSynchronStart(
            ByVal uiBoardIndex As UInteger,
            ByVal uiGroupIndex As UInteger,
            ByVal uiModuleMap As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Start a synchronous read operation.
        ''' </summary>
        ''' <param name="uiBoardIndex">Board index</param>
        ''' <param name="uiGroupIndex">Synchron group index</param>
        ''' <returns></returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqBoardSynchronStart(
            ByVal uiBoardIndex As UInteger,
            ByVal uiGroupIndex As UInteger
        ) As UInteger
        End Function


#Enable Warning IDE1006 ' Naming Styles
#End Region
    End Class

    Public NotInheritable Class AdqConstants
        ''' <summary>
        '''  Used for any parameter in adqQuery which is not required in a particular context
        ''' </summary>
        Public Const ADQ_QUERY_UNUSED_PARAMETER As UInteger = &HFFFFFFFFUI

        Public Const ADQ_QUERY_SYSTEM_NUMBER_OF_BOARDS As UInteger = &H1
        Public Const ADQ_QUERY_SYSTEM_BOARD_HANDLE As UInteger = &H2

        ''' <summary>
        ''' Board constants for the parameter 'ID' in adqQuery.
        ''' Used with the board handle as the first parameter.
        ''' The second parameter 'IndexModule' should always be ADQ_QUERY_UNUSED_PARAMETER.
        ''' </summary>
        Public Const ADO_QUERY_BOARD_NUMBER_OF_MODULES As UInteger = &H10001
        Public Const ADQ_QUERY_BOARD_INFO As UInteger = &H10002

        ''' <summary>
        ''' Module constants for the parameter 'ID' in adqQuery.
        ''' Used with the board handle as the first parameter.
        ''' The second parameter 'IndexModule' is the 0 based module index.
        ''' </summary>
        Public Const ADO_QUERY_MODULE_TYPE As UInteger = &H20001
        Public Const ADQ_QUERY_MODULE_INFO As UInteger = &H20002

        ''' <summary>
        ''' Module types
        ''' </summary>
        Public Const ADQ_MODULE_TYPE_DI As UInteger = &H30000       ' Digital input
        Public Const ADQ_MODULE_TYPE_DO As UInteger = &H30001       ' Digital output
        Public Const ADQ_MODULE_TYPE_DIO As UInteger = &H30002      ' Digital input / output
        Public Const ADQ_MODULE_TYPE_CNTR As UInteger = &H30003     ' Counter
        Public Const ADQ_MODULE_TYPE_EXTINT As UInteger = &H30004   ' External interrupt
        Public Const ADQ_MODULE_TYPE_AI As UInteger = &H30005       ' Analog input
        Public Const ADQ_MODULE_TYPE_AO As UInteger = &H30006       ' Analog output
        Public Const ADQ_MODULE_TYPE_I2C As UInteger = &H30007      ' I2C
        Public Const ADQ_MODULE_TYPE_PWM As UInteger = &H30008      ' PWM
        Public Const ADQ_MODULE_TYPE_FIN As UInteger = &H30009      ' FIN
        Public Const ADQ_MODULE_TYPE_I2SO As UInteger = &H3000A     ' I2S output

        ''' <summary>
        ''' Direction of DIO modules
        ''' </summary>
        Public Const ADQ_DIO_INPUT As UInteger = &H40000
        Public Const ADQ_DIO_OUTPUT As UInteger = &H40001

        ''' <summary>
        ''' Clock source used by function adqCNTRConfig and structure ADQCNTRModuleInfo
        ''' </summary>
        Public Const ADQ_CNTR_CLOCK_SOURCE_DEFAULT As UInteger = &H50000
        Public Const ADQ_CNTR_CLOCK_SOURCE_EXTERNAL As UInteger = &H50001
        Public Const ADQ_CNTR_CLOCK_SOURCE_INTERNAL_10_MHZ As UInteger = &H50002
        Public Const ADQ_CNTR_CLOCK_SOURCE_INTERNAL_66_MHZ As UInteger = &H50004

        ''' <summary>
        ''' Counter type 8254 and modes of the 8254 counter
        ''' </summary>
        Public Const ADQ_CNTR_8254_MODE_0 As UInteger = &H60000  ' Mode 0: Change state at zero
        Public Const ADQ_CNTR_8254_MODE_1 As UInteger = &H60001  ' Mode 1: Retriggerable "One Shot"
        Public Const ADQ_CNTR_8254_MODE_2 As UInteger = &H60002  ' Mode 2: Asymmetric divider
        Public Const ADQ_CNTR_8254_MODE_3 As UInteger = &H60003  ' Mode 3: Symmetric divider
        Public Const ADQ_CNTR_8254_MODE_4 As UInteger = &H60004  ' Mode 4: Counter start by software trigger
        Public Const ADQ_CNTR_8254_MODE_5 As UInteger = &H60005  ' Mode 5: Counter start by hardware trigger

        ''' <summary>
        ''' System constants for the parameter CalibbrationID
        ''' returned by adqAnalogGetSelectedCalibration and passed to adqAnalogSelectCalibration
        ''' </summary>
        Public Const ADQ_CALIBRATION_FACTORY As UInteger = &H70000
        Public Const ADQ_CALIBRATION_USER_1 As UInteger = &H70001
        Public Const ADQ_CALIBRATION_USER_2 As UInteger = &H70002
        Public Const ADQ_CALIBRATION_USER_3 As UInteger = &H70003
        Public Const ADQ_CALIBRATION_USER_4 As UInteger = &H70004


        ''' <summary>
        ''' System constants for the variable TTLVoltageLevel returned in the structure ADQBoardInfo
        ''' One of these constants should also passed to the routine adqSetBoardTTLVoltage in the parameter 'Voltage'
        ''' </summary>
        Public Const ADQ_TTL_VOLTAGE_LEVEL_3_3_VOLTS As UInteger = &H1
        Public Const ADQ_TTL_VOLTAGE_LEVEL_5_0_VOLTS As UInteger = &H2

        ''' <summary>
        ''' System constants for the variable ClockType in the structure ADQGeneralModuleInfo below
        ''' </summary>
        Public Const ADQ_CLOCK_TYPE_NONE As UInteger = &H80000
        Public Const ADQ_CLOCK_TYPE_SOFTWARE As UInteger = &H80001
        Public Const ADQ_CLOCK_TYPE_HARDWARE As UInteger = &H80002

        ''' <summary>
        ''' System constants for the variable CounterType in the structure ADQCNTRModuleInfo
        ''' </summary>
        Public Const ADQ_CNTR_TYPE_8254 As UInteger = &H90000               ' Counter type 8254
        Public Const ADQ_CNTR_TYPE_COUNT_DOWN_32_BIT As UInteger = &H90001  ' Counter type Count Down 32-Bit

        ''' <summary>
        ''' System constants for the variable PhysicalUnit in the structure ADQAIRange
        ''' </summary>
        Public Const ADQ_PHYSICAL_UNIT_VOLT As UInteger = &HA0000
        Public Const ADQ_PHYSICAL_UNIT_AMPERE As UInteger = &HA0001
        Public Const ADQ_PHYSICAL_UNIT_OHM As UInteger = &HA0002

        ''' <summary>
        ''' System constants for the variable BusType in the structure ADQBoardInfo
        ''' </summary>
        Public Const ADQ_BUS_TYPE_PCI As UInteger = &HB0000
        Public Const ADQ_BUS_TYPE_USB As UInteger = &HB0001

        ''' <summary>
        ''' System constants for parameter ReasonForCall in callback functions
        ''' </summary>
        Public Const ADQ_CALLED_FOR_START As UInteger = &H1
        Public Const ADQ_CALLED_FOR_DATA As UInteger = &H2
        Public Const ADQ_CALLED_FOR_STOP As UInteger = &H4

        ''' <summary>
        ''' System constants for using with adqWaitIRQ
        ''' </summary>
        Public Const ADQ_TIMEOUT_INFINITE As UInteger = 0

        ''' <summary>
        ''' System constants for using with continuous functions
        ''' </summary>
        Public Const ADQ_NUMBER_UNKNOWN As UInteger = 0

        ''' <summary>
        ''' System constants for using with adqEnableHWTrigger
        ''' </summary>
        Public Const ADQ_HW_START_TRIGGER_TIMEOUT_INFINITE As UInteger = &HFFFFFFFFUI
    End Class

    Public NotInheritable Class AdqErrors
        ''' <summary>
        ''' Bits 28 to 31 of the error code contain the error type.
        ''' </summary>
        Public Const ADQ_ERROR_TYPE_MASK As UInteger = &HF0000000UI
        Public Const ADQ_ERROR_TYPE_FATAL As UInteger = &H80000000UI
        Public Const ADQ_ERROR_TYPE_ERROR As UInteger = &H40000000UI
        Public Const ADQ_ERROR_TYPE_WARNING As UInteger = &H20000000UI
        Public Const ADQ_ERROR_TYPE_HINT As UInteger = &H10000000UI
        Public Const ADQ_ERROR_TYPE_SUCCESS As UInteger = &H0

        ''' <summary>
        ''' If Bit 25 is set, then internal information is available via adqGetInternalError
        ''' and the returned information should be sent to ALLDAQ
        ''' </summary>
        Public Const ADQ_ERROR_INTERNAL_INFORMATION As UInteger = &H2000000

        ''' <summary>
        ''' If Bit 24 is set, then more information is available via adqGetExtendedError
        ''' </summary>
        Public Const ADQ_ERROR_MORE_INFORMATION As UInteger = &H1000000

        ''' <summary>
        ''' Bits 16 to 19 contain the index of the parameter which has caused the error, 1 based.
        ''' </summary>
        Public Const ADQ_ERROR_PARAMETER_MASK As UInteger = &HF0000
        Public Const ADQ_ERROR_PARAMETER_NONE As UInteger = &H0
        Public Const ADQ_ERROR_PARAMETER_1 As UInteger = &H10000
        Public Const ADQ_ERROR_PARAMETER_2 As UInteger = &H20000
        Public Const ADQ_ERROR_PARAMETER_3 As UInteger = &H30000
        Public Const ADQ_ERROR_PARAMETER_4 As UInteger = &H40000
        Public Const ADQ_ERROR_PARAMETER_5 As UInteger = &H50000
        Public Const ADQ_ERROR_PARAMETER_6 As UInteger = &H60000
        Public Const ADQ_ERROR_PARAMETER_7 As UInteger = &H70000
        Public Const ADQ_ERROR_PARAMETER_8 As UInteger = &H80000
        Public Const ADQ_ERROR_PARAMETER_9 As UInteger = &H90000
        Public Const ADQ_ERROR_PARAMETER_10 As UInteger = &H100000
        Public Const ADQ_ERROR_PARAMETER_11 As UInteger = &H110000

        ''' <summary>
        ''' The error codes themselves are in the lower 16 bits.
        ''' </summary>
        Public Const ADQ_ERROR_NONE As UInteger = &H0
        Public Const ADQ_ERROR_INTERNAL As UInteger = &H1
        Public Const ADQ_ERROR_INVALID_PARAMETER As UInteger = &H2
        Public Const ADQ_ERROR_NO_DEVICES As UInteger = &H3
        Public Const ADQ_ERROR_ALREADY_OPEN As UInteger = &H4
        Public Const ADQ_ERROR_NOT_OPEN As UInteger = &H5
        Public Const ADQ_ERROR_INVALID_BOARD_INDEX As UInteger = &H6
        Public Const ADQ_ERROR_CANNOT_OPEN_BOARD As UInteger = &H7
        Public Const ADQ_ERROR_INVALID_MODULE_INDEX As UInteger = &H8
        Public Const ADQ_ERROR_UNKNOWN_SYSTEM_QUERY As UInteger = &H9
        Public Const ADQ_ERROR_BUFFER_TOO_SMALL As UInteger = &HA
        Public Const ADQ_ERROR_UNSUPPORTED As UInteger = &HB
        Public Const ADQ_ERROR_IRQ_ALREADY_ENABLED As UInteger = &HC
        Public Const ADQ_ERROR_IRQ_NOT_ENABLED As UInteger = &HD
        Public Const ADQ_ERROR_TIMEOUT As UInteger = &HE
        Public Const ADQ_ERROR_MODULE_BUSY As UInteger = &HF
        Public Const ADQ_ERROR_KERNEL_BUFFER_OVERFLOW As UInteger = &H10
        Public Const ADQ_ERROR_AI_INVALID_CHANNEL_INDEX As UInteger = &H11
        Public Const ADQ_ERROR_AI_INVALID_RANGE_INDEX As UInteger = &H12
        Public Const ADQ_ERROR_AI_INVALID_FRAME_LENGTH As UInteger = &H13
        Public Const ADQ_ERROR_WRITE_OP_NOT_RUNNING As UInteger = &H14
        Public Const ADQ_ERROR_KERNEL_BUFFER_UNDERFLOW As UInteger = &H15
        Public Const ADQ_ERROR_STRUCTURE_SIZE_OR_ID_FALSE As UInteger = &H16
        Public Const ADQ_ERROR_STRUCTURE_ID_FALSE As UInteger = &H17
        Public Const ADQ_ERROR_NETWORK_COMMUNICATION As UInteger = &H18
        Public Const ADQ_ERROR_NETWORK_TIMEOUT As UInteger = &H19
        Public Const ADQ_ERROR_FREQUENCY_INVALID As UInteger = &H1A
        Public Const ADQ_ERROR_SYNCHRON_START_INVALID_GROUP_INDEX As UInteger = &H1B
        Public Const ADQ_ERROR_SYNCHRON_START_INVALID_MODULES = As UInteger = &H1C
        Public Const ADQ_ERROR_SYNCHRON_START_TOO_MANY_MODULES = As UInteger = &H1D

    End Class
End Namespace
