'-----------------------------------------------------------------------------
' adqUtility.vb - AdqUtility, types and function prototypes
'      Copyright (c) ALLDAQ, a division of ALLNET GmbH
'      https://www.alldaq.com
'
' Purpose:
'      This file has defines for constants and function prototypes
'      used by the ALLDAQ Utility library
' class AdqUtility:
'      import function prototypes
'      error codes returned by the API functions
'
' Note:
'      Native import library 32bit: adqUtility.dll
'      Native import library 64bit: adqUtility_64.dll
'-----------------------------------------------------------------------------

Imports System.Runtime.InteropServices

Namespace ALLDAQ.Utility

    Public NotInheritable Class AdqUtility

#If WIN64 Then
        Public Const ADQUTILITY_DLL as String = "adqUtility_64.dll"
#Else
        Public Const ADQUTILITY_DLL As String = "adqUtility.dll"
#End If

#Disable Warning IDE1006 ' Naming Styles
        ''' <summary>
        ''' Calculate the RMS value. The buffer can contain values of several channels interleaved.
        ''' </summary>
        ''' <param name="buffer">Buffer with values [in]</param>
        ''' <param name="offset">Offset of the first value in buffer [in]</param>
        ''' <param name="channelStep">Step to the next value of the same channel [in]</param>
        ''' <param name="numberOfValues">Number of values of the channel [in]</param>
        ''' <param name="digitalMin">Digital minimum value [in]</param>
        ''' <param name="digitalMax">Digital maximum value [in]</param>
        ''' <param name="physicalMin">Physical minimum value of the signal [in]</param>
        ''' <param name="physicalMax">Physical maximum value of the signal [in]</param>
        ''' <param name="RMSValue">The calculated RMS value [out]</param>
        ''' <returns>Error code</returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqUtilityCalculateRMSValue(
            ByVal buffer As UInteger(),
            ByVal offset As UInteger,
            ByVal channelStep As UInteger,
            ByVal numberOfValues As UInteger,
            ByVal digitalMin As UInteger,
            ByVal digitalMax As UInteger,
            ByVal physicalMin As Double,
            ByVal physicalMax As Double,
            ByRef RMSValue As Double
        ) As UInteger
        End Function


        ''' <summary>
        ''' Calculate minimum, maximum, average, And P2P value. The buffer can contain values of several channels interleaved.
        ''' </summary>
        ''' <param name="buffer">Buffer with values [in]</param>
        ''' <param name="offset">Offset of the first value in buffer [in]</param>
        ''' <param name="channelStep">Step to the next value of the same channel [in]</param>
        ''' <param name="numberOfValues">Number of values of the channel [in]</param>
        ''' <param name="digitalMin">Digital minimum value [in]</param>
        ''' <param name="digitalMax">Digital maximum value [in]</param>
        ''' <param name="physicalMin">Physical minimum value of the signal [in]</param>
        ''' <param name="physicalMax">Physical maximum value of the signal [in]</param>
        ''' <param name="minValue">The minimum value of the signal in physical units [out]</param>
        ''' <param name="maxValue">The maximum value of the signal in physical units [out]</param>
        ''' <param name="averageValue">The arithmetic mean value of the signal in physical units [out]</param>
        ''' <param name="P2PValue">The peak to peak value of the signal, the difference of maxValue And minValue [out]</param>
        ''' <returns></returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqUtilityCalculateMinMaxAverageAndP2PValue(
            ByVal buffer As UInteger(),
            ByVal offset As UInteger,
            ByVal channelStep As UInteger,
            ByVal numberOfValues As UInteger,
            ByVal digitalMin As UInteger,
            ByVal digitalMax As UInteger,
            ByVal physicalMin As Double,
            ByVal physicalMax As Double,
            ByRef minValue As Double,
            ByRef maxValue As Double,
            ByRef averageValue As Double,
            ByRef P2PValue As Double
        ) As UInteger
        End Function


        ''' <summary>
        ''' Calculate the frequency spectrum of an input signal. The buffer can contain values of several channels interleaved.
        ''' </summary>
        ''' <param name="samplingRateHz">Sampling rate of the acquired values in Hertz [in]</param>
        ''' <param name="buffer">Buffer with values [in]</param>
        ''' <param name="offset">Offset of the first value in buffer [in]</param>
        ''' <param name="channelStep">Step to the next value of the same channel [in]</param>
        ''' <param name="numberOfValues">Number of values of the channel [in]</param>
        ''' <param name="digitalMin">Digital minimum value [in]</param>
        ''' <param name="digitalMax">Digital maximum value [in]</param>
        ''' <param name="physicalMin">Physical minimum value of the signal [in]</param>
        ''' <param name="physicalMax">Physical maximum value of the signal [in]</param>
        ''' <param name="frequencySpectrum">Array of frequencies [out]</param>
        ''' <param name="frequencyStep">The frequency channelStep of the frequency spectrum. Steps always start with 0. [out]</param>
        ''' <param name="frequencySpectrumSize">The number of values of the array frequencySpectrum [in]</param>
        ''' <param name="windowFunction">One of the defined constants FFT_WINDOW_FUNCTION... [in]</param>
        ''' <param name="overlap">The amount of overlapping of the window for the following calculation. The value can be from 0 to 0.98 [in]</param>
        ''' <returns>Error code</returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqUtilityFrequencySpectrum(
            ByVal samplingRateHz As Double,
            ByVal buffer As UInteger(),
            ByVal offset As UInteger,
            ByVal channelStep As UInteger,
            ByVal numberOfValues As UInteger,
            ByVal digitalMin As UInteger,
            ByVal digitalMax As UInteger,
            ByVal physicalMin As Double,
            ByVal physicalMax As Double,
            ByVal frequencySpectrum As Double(),
            ByRef frequencyStep As Double,
            ByVal frequencySpectrumSize As UInteger,
            ByVal windowFunction As UInteger,
            ByVal overlap As Double
        ) As UInteger
        End Function


        ''' <summary>
        ''' Calculate the frequency spectrum of a known sine signal. The buffer can contain values of several channels interleaved.
        ''' </summary>
        ''' <param name="samplingRateHz">Sampling rate of the acquired values in Hertz [in]</param>
        ''' <param name="sinusFrequencyHz">The value of the known sine frequency in Hertz [in]</param>
        ''' <param name="buffer">Buffer with values [in]</param>
        ''' <param name="offset">Offset of the first value in buffer [in]</param>
        ''' <param name="step">Step to the next value of the same channel [in]</param>
        ''' <param name="numberOfValues">Number of values of the channel [in]</param>
        ''' <param name="digitalMin">Digital minimum value [in]</param>
        ''' <param name="digitalMax">Digital maximum value [in]</param>
        ''' <param name="physicalMin">Physical minimum value of the signal [in]</param>
        ''' <param name="physicalMax">Physical maximum value of the signal [in]</param>
        ''' <param name="frequencySpectrum">Array of frequencies [out]</param>
        ''' <param name="frequencyStep">The frequency step of the frequency spectrum. Steps always start with 0. [out]</param>
        ''' <param name="frequencySpectrumSize">The number of values of the array frequencySpectrum [in]</param>
        ''' <param name="windowFunction">One of the defined constants FFT_WINDOW_FUNCTION... [in]</param>
        ''' <param name="overlap">The amount of overlapping of the window for the following calculation. The value can be from 0 to 0.98 [in]</param>
        ''' <returns>Error code</returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqUtilityFrequencySpectrumCoherentSampling(
            ByVal samplingRateHz As Double,
            ByVal sinusFrequencyHz As Double,
            ByVal buffer As UInteger(),
            ByVal offset As UInteger,
            ByVal channelStep As UInteger,
            ByVal numberOfValues As UInteger,
            ByVal digitalMin As UInteger,
            ByVal digitalMax As UInteger,
            ByVal physicalMin As Double,
            ByVal physicalMax As Double,
            ByVal frequencySpectrum As Double(),
            ByRef frequencyStep As Double,
            ByVal frequencySpectrumSize As UInteger,
            ByVal windowFunction As UInteger,
            ByVal overlap As Double
        ) As UInteger
        End Function


        ''' <summary>
        ''' Calculate characteristic signal values of a given frequency spectrum (main frequency, THD, THDr+N, THDf+N)
        ''' </summary>
        ''' <param name="frequencySpectrum">Array of Frequencies, calculated with function adqUtilityFrequencySpectrum [in]</param>
        ''' <param name="frequencyStep">The frequency step of the frequency spectrum [in]</param>
        ''' <param name="frequencySpectrumSize">Size of the array frequencySpectrum in values. [in]</param>
        ''' <param name="frequencyIgnoreBelow">Filter lower frequencies up to this value [in]</param>
        ''' <param name="frequencyIgnoreAbove">Filter higher frequencies from this value [in]</param>
        ''' <param name="mainFrequencyHz">The main frequency of the signal in Hertz [out]</param>
        ''' <param name="proportionMainFrequency">The proportion of the main frequency as normalised value from 0 to 1.0 [out]</param>
        ''' <param name="THD">The total harmonic distortion [out]</param>
        ''' <param name="THDrPlusNoise">The total harmonic distortion + noise (root mean square) [out]</param>
        ''' <param name="THDfPlusNoise">The total harmonic distortion + noise (fundamental) [out]</param>
        ''' <param name="RMSValue">The calculated RMS value [out]</param>
        ''' <returns>Error code</returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqUtilityCalculateCharacteristicValuesFromFrequencySpectrum(
            ByVal frequencySpectrum As Double(),
            ByVal frequencyStep As Double,
            ByVal frequencySpectrumSize As UInteger,
            ByVal frequencyIgnoreBelow As Double,
            ByVal frequencyIgnoreAbove As Double,
            ByRef mainFrequencyHz As Double,
            ByRef proportionMainFrequency As Double,
            ByRef THD As Double,
            ByRef THDrPlusNoise As Double,
            ByRef THDfPlusNoise As Double,
            ByRef RMSValue As Double
        ) As UInteger
        End Function


        ''' <summary>
        ''' Apply a digital filter on acquired values using generated filter coefficients
        ''' </summary>
        ''' <param name="buffer">Buffer with values [in, out]</param>
        ''' <param name="offset">Offset of the first value in buffer [in]</param>
        ''' <param name="channelStep">Step to the next value of the same channel [in]</param>
        ''' <param name="numberOfValues">Number of values of the channel [in]</param>
        ''' <param name="digitalMin">Digital minimum value [in]</param>
        ''' <param name="digitalMax">Digital maximum value [in]</param>
        ''' <param name="physicalMin">Physical minimum value of the signal [in]</param>
        ''' <param name="physicalMax">Physical maximum value of the signal [in]</param>
        ''' <param name="firCoefficients">Array of filter coefficients generated with one of the four filter functions [in]</param>
        ''' <param name="firOrder">The number of coefficients in firCoefficients [in]</param>
        ''' <returns>Error code</returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqUtilityFilterFIR(
            ByRef buffer As UInteger(),
            ByVal offset As UInteger,
            ByVal channelStep As UInteger,
            ByVal numberOfValues As UInteger,
            ByVal digitalMin As UInteger,
            ByVal digitalMax As UInteger,
            ByVal physicalMin As Double,
            ByVal physicalMax As Double,
            ByVal firCoefficients As Double(),
            ByVal firOrder As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Generate filter coefficients for a high pass filter
        ''' </summary>
        ''' <param name="samplingRateHz">The sampling rate of the signal in Hertz [in]</param>
        ''' <param name="filterOutToHz">The frequency of the high pass filter in Hertz [in]</param>
        ''' <param name="filterCoefficients">The array that receives the generated filter coefficients [in, out]</param>
        ''' <param name="filterOrder">The number of coefficients to generate [in]</param>
        ''' <returns>Error code</returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqUtilityFilterFIRGenerateHighPassCoefficients(
            ByVal samplingRateHz As Double,
            ByVal filterOutToHz As Double,
            ByRef filterCoefficients As Double(),
            ByVal filterOrder As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Generate filter coefficients for a low pass filter
        ''' </summary>
        ''' <param name="samplingRateHz">The sampling rate of the signal in Hertz [in]</param>
        ''' <param name="filterOutFromHz">The frequency of the low pass filter in Hertz [in]</param>
        ''' <param name="filterCoefficients">The array that receives the generated filter coefficients [in, out]</param>
        ''' <param name="filterOrder">The number of coefficients to generate [in]</param>
        ''' <returns>Error code</returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqUtilityFilterFIRGenerateLowPassCoefficients(
            ByVal samplingRateHz As Double,
            ByVal filterOutFromHz As Double,
            ByRef filterCoefficients As Double(),
            ByVal filterOrder As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Generate filter coefficients for a band-pass filter
        ''' </summary>
        ''' <param name="samplingRateHz">The sampling rate of the signal in Hertz [in]</param>
        ''' <param name="filterOutToHz">The lower frequency of the band-pass filter in Hertz [in]</param>
        ''' <param name="filterOutFromHz">The upper frequency of the band-pass filter in Hertz [in]</param>
        ''' <param name="filterCoefficients">The array that receives the generated filter coefficients [in, out]</param>
        ''' <param name="filterOrder">The number of coefficients to generate [in]</param>
        ''' <returns>Error code</returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqUtilityFilterFIRGenerateBandPassCoefficients(
            ByVal samplingRateHz As Double,
            ByVal filterOutToHz As Double,
            ByVal filterOutFromHz As Double,
            ByRef filterCoefficients As Double(),
            ByVal filterOrder As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Generate filter coefficients for a band-stop filter
        ''' </summary>
        ''' <param name="samplingRateHz">The sampling rate of the signal in Hertz [in]</param>
        ''' <param name="filterOutFromHz">The lower frequency of the band-stop filter in Hertz [in]</param>
        ''' <param name="filterOutToHz">The upper frequency of the band-stop filter in Hertz [in]</param>
        ''' <param name="filterCoefficients">The array that receives the generated filter coefficients [in, out]</param>
        ''' <param name="filterOrder">The number of coefficients to generate [in]</param>
        ''' <returns>Error code</returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqUtilityFilterFIRGenerateBandStopCoefficients(
            ByVal samplingRateHz As Double,
            ByVal filterOutFromHz As Double,
            ByVal filterOutToHz As Double,
            ByRef filterCoefficients As Double(),
            ByVal filterOrder As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Generate filter coefficients for an A-weighted filter
        ''' </summary>
        ''' <param name="samplingRateHz">The sampling rate of the signal in Hertz [in]</param>
        ''' <param name="filterCoefficients">The array that receives the generated filter coefficients [in, out]</param>
        ''' <param name="filterOrder">The number of coefficients to generate [in]</param>
        ''' <returns>Error code</returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqUtilityFilterFIRGenerateAWeightedCoefficients(
            ByVal samplingRateHz As Double,
            ByRef filterCoefficients As Double(),
            ByVal filterOrder As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Generate filter coefficients for a ITUR-468 weighted filter
        ''' </summary>
        ''' <param name="samplingRateHz">The sampling rate of the signal in Hertz [in]</param>
        ''' <param name="filterCoefficients">The array that receives the generated filter coefficients [in, out]</param>
        ''' <param name="filterOrder">The number of coefficients to generate [in]</param>
        ''' <returns>Error code</returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqUtilityFilterFIRGenerateITUR468WeightedCoefficients(
            ByVal samplingRateHz As Double,
            ByRef filterCoefficients As Double(),
            ByVal filterOrder As UInteger
        ) As UInteger
        End Function


        ''' <summary>
        ''' Calculate the ratio in decibel for a linear input value as 20 * log(linearValue / referenceValue)
        ''' </summary>
        ''' <param name="referenceValue">Reference value, e.g. for dBu 0.775, for dbV 1.0 [in]</param>
        ''' <param name="linearValue">The linear value, e.g. calculated by one of the FFT functions [in]</param>
        ''' <param name="dBuValue">The calculated value in decibel [out]</param>
        ''' <returns>Error code</returns>
        <DllImport(ADQDEVICE_DLL, CallingConvention:=CallingConvention.StdCall)>
        Public Shared Function adqUtilityConvertLinearTodBu(
            ByVal referenceValue As Double,
            ByVal linearValue As Double,
            ByRef dBuValue As Double
        ) As UInteger
        End Function

#Enable Warning IDE1006 ' Naming Styles
    End Class
    
    Public NotInheritable Class AdqUtilityConstants
        ''' <summary>
        ''' Constants used with utility functions to calculate the frequency spectrum.
        ''' </summary>
        Public Const FFT_WINDOW_FUNCTION_RECTANGULAR As UInteger = 0
        Public Const FFT_WINDOW_FUNCTION_HANN As UInteger = 1
        Public Const FFT_WINDOW_FUNCTION_HAMMING As UInteger = 2
        Public Const FFT_WINDOW_FUNCTION_BARTLETT As UInteger = 3
        Public Const FFT_WINDOW_FUNCTION_FLAT_TOP As UInteger = 4
        Public Const FFT_WINDOW_FUNCTION_HARRIS_NUTALL As UInteger = 5
        Public Const FFT_WINDOW_FUNCTION_BLACKMAN_HARRIS_7_TERM As UInteger = 6
    End Class

    Public NotInheritable Class AdqUtilityErrors
        ''' <summary>
        ''' Error constants returned from utility functions.
        ''' </summary>
        Public Const ADQ_UTILITY_ERROR_NONE As UInteger = 0
        Public Const ADQ_UTILITY_ERROR_DIGITAL_LIMITS_INVALID As UInteger = 1
        Public Const ADQ_UTILITY_ERROR_INVALID_PARAMETER_1 As UInteger = 100
        Public Const ADQ_UTILITY_ERROR_INVALID_PARAMETER_2 As UInteger = 101
    End Class
End Namespace


