//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the adqWriteContinuousCallback project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

#include "stdafx.h"
#include "AoModule.h"
#include "AdqCommon.h"

#include <sstream>
#include <algorithm>

using std::vector;
using std::wstring;
using std::unique_ptr;
using std::shared_ptr;

//This callback function is called in a separate thread created by the DLL of the driver system
unsigned int __stdcall adqWriteContinuousCallback(
    unsigned int /*uiBoardIndex*/,
    unsigned int /*uiModuleIndex*/,
    unsigned int uiNumberOfValuesAcceptable,
    unsigned int* puiValues,
    unsigned int& uiNumberOfValuesWritten,
    unsigned int uiError,
    void* pvUserContext,
    unsigned int uiReasonForCall)
{
    AoModule* aoModule = static_cast<AoModule*>(pvUserContext);
    if (uiError == ADQ_ERROR_NONE)
    {
        if ((uiReasonForCall & ADQ_CALLED_FOR_DATA) == ADQ_CALLED_FOR_DATA)
        {
            if (uiNumberOfValuesAcceptable > 0)
            {
                uiNumberOfValuesWritten = aoModule->WriteValues(uiNumberOfValuesAcceptable, puiValues);
            }
        }
    }
    else
    {
        aoModule->m_AdqError = uiError;
    }

    return uiError;
}


AoModule::AoModule(const wstring &boardName, unsigned int boardIndex, unsigned int moduleIndex, 
    const SADQGeneralModuleInfo &generalInfo, const SADQAOModuleInfo &aoModuleInfo)
    : m_boardIndex(boardIndex),
    m_moduleIndex(moduleIndex),
    m_AdqError(ADQ_ERROR_NONE),
    m_outputBufferLength(0),
    m_notifyAfterValues(0),
    m_signalAmplitude(0),
    m_signalFrequency(1),
    m_actualOutputFrequency(0),
    m_valueOffset(0),
    m_numValuesPerPeriod(0),
    m_outputValueBuffer(nullptr),
    m_physicalValueBuffer(nullptr)
{
    m_maxFrequency = generalInfo.m_dClockBaseFrequencyHz / generalInfo.m_uiClockMinimumDivider;
    AddRanges(aoModuleInfo);
    BuildModuleName(boardName);
    m_selectedRange = m_ranges[0];
    m_waveForm = std::make_unique<WaveForm>();
}

AoModule::~AoModule()
{
    DeleteValueBuffers();
}

void AoModule::DeleteValueBuffers()
{
    delete[] m_outputValueBuffer;
    m_outputValueBuffer = nullptr;
    delete[] m_physicalValueBuffer;
    m_physicalValueBuffer = nullptr;
    m_outputBufferLength = 0;
}

wstring AoModule::GetModuleName()
{
    return m_moduleName;
}

double AoModule::GetMaxFrequency()
{
    return m_maxFrequency;
}

shared_ptr<AoRange> AoModule::GetSelectedRange()
{
    return m_selectedRange;
}

unsigned int AoModule::SetFrequency(double nominalFrequency)
{
    double actFrequency;
    unsigned int adqError = adqSetFrequency(m_boardIndex, m_moduleIndex, nominalFrequency, actFrequency);
    if (adqError == ADQ_ERROR_NONE)
    {
        AddErrorInfo(L"adqSetFrequency", adqError);
        m_actualOutputFrequency = actFrequency;
    }
    return adqError;
}

double AoModule::GetActualOutputFrequency()
{
    return m_actualOutputFrequency;
}

void AoModule::SetSignalGenerator(shared_ptr<ISignalGenerator>& signalGenerator)
{
    m_selectedSignalGenerator = signalGenerator;
}

std::shared_ptr<ISignalGenerator> AoModule::GetSelectedSignalGenerator()
{
    return m_selectedSignalGenerator;
}

void AoModule::SetNumberOfValuesPerSignalPeriod(int numValuesPerPeriod)
{
    m_numValuesPerPeriod = numValuesPerPeriod;

    m_signalFrequency = m_actualOutputFrequency / m_numValuesPerPeriod;
}

double AoModule::GetSignalFrequency()
{
    return m_signalFrequency;
}

void AoModule::SetSignalAmplitude(double signalAmplitude)
{
    m_signalAmplitude = signalAmplitude;
}

double AoModule::GetSignalAmplitude()
{
    return m_signalAmplitude;
}

unsigned int AoModule::StartOutput()
{
    m_AdqError = ADQ_ERROR_NONE;
    SetNotifyAfterValues();
    InitOutputBuffer();
    InitWaveForm();
    return StartContinuousOutput();
}

void AoModule::InitOutputBuffer()
{
    unsigned int outputBufferLength = EvaluateOutputBufferLength(m_notifyAfterValues);
    
    if (m_outputBufferLength != outputBufferLength)
    {
        if (m_outputBufferLength > 0)
        {
            DeleteValueBuffers();
        }

        m_outputValueBuffer = new unsigned int[outputBufferLength];
        m_physicalValueBuffer = new double[outputBufferLength];
        m_outputBufferLength = outputBufferLength;
    }

    m_valueOffset = 0;
}

void AoModule::SetNotifyAfterValues()
{
    const int numberOfNotificationsPerSecond = 10;
    m_notifyAfterValues = (std::max)((unsigned int)(m_actualOutputFrequency / numberOfNotificationsPerSecond), 2U);
}

unsigned int AoModule::EvaluateOutputBufferLength(unsigned int notifyAfterValues)
{
    unsigned int reminder = notifyAfterValues % m_numValuesPerPeriod;
    unsigned int length = notifyAfterValues;

    if (reminder > 0)
    {
        length += (m_numValuesPerPeriod - reminder);
    }

    return (std::max)(length, (unsigned int)m_numValuesPerPeriod);
}

void AoModule::InitWaveForm()
{
    m_waveForm->InitWaveForm(m_actualOutputFrequency, m_physicalValueBuffer, m_outputBufferLength, 
        m_selectedRange->GetPhysicalMin(), m_selectedRange->GetPhysicalMax());
    CalculateValueBuffers();
}

unsigned int AoModule::StartContinuousOutput()
{
    const unsigned int repeatBuffer = 1; //for parameter uiRepeatBuffer only the value 1 is supported, see help file: ALLDAQ-DriverAPI.chm

    unsigned int numValuesWritten;
    unsigned int adqError = adqWriteContinuousStart(m_boardIndex, m_moduleIndex, m_outputBufferLength, m_outputValueBuffer, repeatBuffer, 
        false, m_notifyAfterValues, numValuesWritten, adqWriteContinuousCallback, this);
    if (adqError == ADQ_ERROR_NONE)
    {
        m_valueOffset = numValuesWritten;
    }
    else
    {
        AddErrorInfo(L"adqWriteContinuousStart", adqError);
    }
    return adqError;
}

unsigned int AoModule::WriteValues(unsigned int numberOfValuesAcceptable, unsigned int * valueBuffer)
{
    unsigned int numCount = 0;

    while (numCount < numberOfValuesAcceptable)
    {
        if (m_valueOffset == m_outputBufferLength)
        {
            m_valueOffset = 0;
        }

        valueBuffer[numCount] = m_outputValueBuffer[m_valueOffset];

        m_valueOffset++;
        numCount++;
    }

    return numCount;
}

void AoModule::CalculateValueBuffers()
{
    m_waveForm->ApplySignalGenerator(m_selectedSignalGenerator, m_signalFrequency, m_signalAmplitude);
    
    CalculateDigitalValueBuffer();
}

void AoModule::CalculateDigitalValueBuffer()
{
    for (unsigned int i = 0; i < m_outputBufferLength; i++)
    {
        m_outputValueBuffer[i] = m_selectedRange->PhysicalToDigital(m_physicalValueBuffer[i]);
    }
}

unsigned int AoModule::StopOutput()
{
    unsigned int adqError = adqStop(m_boardIndex, m_moduleIndex);
    if (adqError != ADQ_ERROR_NONE)
    {
        AddErrorInfo(L"adqStop", adqError);
    }

    return adqError;
}


void AoModule::AddRanges(const SADQAOModuleInfo &aoModuleInfo)
{
    for (unsigned int rangeIdx = 0; rangeIdx < aoModuleInfo.m_uiNumberOfRanges; rangeIdx++)
    {
        SADQAORange adqRange = aoModuleInfo.m_arrRanges[rangeIdx];
        m_ranges.push_back(std::make_shared<AoRange>(rangeIdx, adqRange.m_uiDigitalMin, adqRange.m_uiDigitalMax, 
            adqRange.m_uiPhysicalUnit, adqRange.m_dPhysicalMin, adqRange.m_dPhysicalMax));
    }
}

void AoModule::BuildModuleName(const wstring &boardName)
{
    std::wstringstream wss;
    wss << ALLDAQ::GetModelName(boardName) 
        << L" Board " << m_boardIndex
        << L" Module " << m_moduleIndex
        << L" max. " << ALLDAQ::FormatPhysicalValue(m_maxFrequency, L"Hz");
    
    m_moduleName = wss.str(); 
}

vector<wstring> AoModule::GetErrorInfo()
{
    return m_errorInfo;
}

void AoModule::AddErrorInfo(const wchar_t * functionName, unsigned int adqError)
{
    std::wstringstream wss;
    wss << L"API-function " << functionName
        << L" returned error number 0x" << std::hex << std::uppercase << adqError;
    m_errorInfo.push_back(wss.str());
}

void AoModule::ClearErrorInfo()
{
    m_errorInfo.clear();
}


