//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the adqWriteContinuousCallback project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

#include "stdafx.h"
#include "ModuleLocator.h"
#include "AoModule.h"

#include "..\\..\\..\\adqDevice.h"

#include <sstream> 

#ifdef WIN64
#pragma comment(lib, "..\\..\\adqDevice_64.lib")
#else
#pragma comment(lib, "..\\..\\adqDevice.lib")
#endif

using std::vector;
using std::shared_ptr;
using std::wstring;

static const unsigned int NotOpen = ADQ_ERROR_TYPE_FATAL | ADQ_ERROR_NOT_OPEN;

ModuleLocator::ModuleLocator(vector<shared_ptr<AoModule>> &aoModules)
    : m_adqOpenError(NotOpen),
    m_aoModules(aoModules)
{
}

ModuleLocator::~ModuleLocator()
{
    CloseAdqDriver();
}

void ModuleLocator::InitModuleLocator()
{
    m_errorInfo.clear();
    m_adqOpenError = OpenAdqDriver();
    if (m_adqOpenError != ADQ_ERROR_NONE) return;

    FindAoModules();
}

void ModuleLocator::FindAoModules()
{
    unsigned int numberOfBoards = GetNumberOfBoards();
    for (unsigned int boardIdx = 0; boardIdx < numberOfBoards; boardIdx++)
    {
        unsigned int numberOfModules;
        unsigned int adqError = adqGetNumberOfModules(boardIdx, numberOfModules);
        if (adqError != ADQ_ERROR_NONE)
        {
            AddErrorInfo(L"adqGetNumberOfModules", adqError);
            continue;
        }

        GetModules(boardIdx, numberOfModules);
    }
}

unsigned int ModuleLocator::GetModules(unsigned int boardIndex, unsigned int numberOfModules)
{
    unsigned int adqError = ADQ_ERROR_NONE;
    wstring boardName = L"";
    for (unsigned int moduleIdx = 0; moduleIdx < numberOfModules; moduleIdx++)
    {
        unsigned int currentModuleType;
        adqError = adqGetModuleType(boardIndex, moduleIdx, currentModuleType);
        if (adqError != ADQ_ERROR_NONE)
        {
            AddErrorInfo(L"adqGetModuleType", adqError);
            continue;
        }

        if (currentModuleType == ADQ_MODULE_TYPE_AO)
        {
            if (boardName == L"")
            {
                boardName = GetBoardName(boardIndex);
            }            
            
            GetModuleInfo(boardName, boardIndex, moduleIdx);
        }
    }

    return adqError;
}

unsigned int ModuleLocator::GetModuleInfo(const wstring &boardName, unsigned int boardIndex, unsigned int moduleIndex)
{
    SADQGeneralModuleInfo adqGeneralModuleInfo = {};
    adqGeneralModuleInfo.m_uiSize = sizeof(SADQGeneralModuleInfo);
    adqGeneralModuleInfo.m_uiID = SADQGeneralModuleInfo_ID;

    SADQAOModuleInfo adqAoModuleInfo = {};
    adqAoModuleInfo.m_uiSize = sizeof(SADQAOModuleInfo);
    adqAoModuleInfo.m_uiID = SADQAOModuleInfo_ID;

    unsigned int infoSize;
    unsigned int adqError = adqGetModuleInfo(boardIndex, moduleIndex, adqGeneralModuleInfo, adqAoModuleInfo.m_uiSize, (unsigned char*)&adqAoModuleInfo, infoSize);
    if (adqError != ADQ_ERROR_NONE)
    {
        AddErrorInfo(L"adqGetModuleInfo", adqError);
        return adqError;
    }

    if (IsClockControlledModule(adqGeneralModuleInfo.m_uiClockType))
    {
        m_aoModules.push_back(std::make_shared<AoModule>(boardName, boardIndex, moduleIndex, adqGeneralModuleInfo, adqAoModuleInfo));
    }
    return adqError;
}

bool ModuleLocator::IsClockControlledModule(unsigned int clockType)
{
    return clockType == ADQ_CLOCK_TYPE_HARDWARE || clockType == ADQ_CLOCK_TYPE_SOFTWARE;
}

unsigned int ModuleLocator::GetNumberOfBoards()
{
    unsigned int numBoards;
    unsigned int adqError = adqGetNumberOfBoards(numBoards);
    if (adqError != ADQ_ERROR_NONE)
    {
        AddErrorInfo(L"adqGetNumberOfBoards", adqError);
        return 0;
    }
    return numBoards;
}

wstring ModuleLocator::GetBoardName(unsigned int boardIndex)
{
    SADQBoardInfo adqBoardInfo = {};
    adqBoardInfo.m_uiID = SADQBoardInfo_ID;
    adqBoardInfo.m_uiSize = sizeof(SADQBoardInfo);

    unsigned int adqError = adqGetBoardInfo(boardIndex, adqBoardInfo);
    if (adqError != ADQ_ERROR_NONE)
    {
        AddErrorInfo(L"adqGetBoardInfo", adqError);
        return L"";
    }

    return adqBoardInfo.m_wszBoardName;
}

unsigned int ModuleLocator::OpenAdqDriver()
{
    unsigned int adqError = adqOpen();
    if (adqError != ADQ_ERROR_NONE)
    {
        AddErrorInfo(L"adqOpen", adqError);
    }
    return adqError;
}

void ModuleLocator::CloseAdqDriver()
{
    if (m_adqOpenError == ADQ_ERROR_NONE)
    {
        adqClose();
        m_adqOpenError = NotOpen;
    }
}

vector<wstring> ModuleLocator::GetErrorInfo()
{
    return m_errorInfo;
}

void ModuleLocator::AddErrorInfo(const wchar_t * functionName, unsigned int adqError)
{
    std::wstringstream wss;
    wss << L"API-function " << functionName
        << L" returned error number 0x" << std::hex << std::uppercase << adqError;

    m_errorInfo.push_back(wss.str());
}
